<?php

	if (!defined("ENV")) exit();
	
	require_once("content/classes/class.doba.php");
	require_once("content/classes/class.doba_product.php");
	require_once("content/classes/class.doba_order.php");
	require_once("content/classes/class.doba_retailer.php");
	require_once("content/classes/class.doba_mapper.php");
	
	$dobaProductAPI = new DobaProductAPI($settings, $db);
	$dobaOrderAPI = new DobaOrderAPI($settings, $db);
	$dobaRetailerAPI = new DobaRetailerAPI($settings, $db);
	$action = isset($_GET["action"]) ? $_GET["action"] : "";
	
	$result = array("status" => 0, "message" => "Wrong request", "data" => false);
	
	switch ($action)
	{
		/**
		 * Get mapped categories
		 */
		case "getMappedCategories" :
		{
			$mapper = new DobaMapper($db, $settings);
			$result = array("status"=>1, "data"=>$mapper->getDobaCategory($_POST["doba_cid"]));
			break;
		}
		
		/**
		 * Set mapped categories
		 */
		case "setMappedCategories" :
		{
			$mapper = new DobaMapper($db, $settings);
			$mapper->mapCategories(
				$_POST["doba_cid"],
				$_POST["primary_cid"],
				isset($_POST["subcats"]) && $_POST["subcats"] ? true : false,
				trim($_POST["categories"]) != '' ? explode(',', $_POST["categories"]) : null
			);
			$result = array("status"=>1);
			break;
		}
		
		/**
		 * Remap products
		 */
		case "remapProducts" :
		{
			$mapper = new DobaMapper($db, $settings);
			$mapper->remapProducts($_REQUEST["mapping_option"] == "all", $_REQUEST["mapping_keep_categories"] == "yes");
			$result = array("status"=>1);
			break;
		}
		
		/**
		 * Save prices settings
		 */
		case "savePriceSettings" :
		{
			$s = array(
				"doba_price_option" => $_GET["price_option"],
				"doba_price_option99" => $_GET["price_option99"],
				"doba_price_modifier" => $_GET["price_modifier"],
			);
			foreach ($s as $key=>$value)
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$db->escape($key)."'");
			}
			$result = array("status"=>1);
			break;				
		}
			
		/**
		 * Get watchlist previously created by user
		 */
		case "getWatchlists" : 
		{
			$dobaProductAPI->initDobaTable();
			if (($watchLists = $dobaProductAPI->getWatchlists()) != false)
			{
				$result = array("status"=>1, "count"=>count($watchLists["watchlist"]), "data"=>$watchLists);
			}
			else
			{
				$result = array("status" => 0, "message" => "You do not have watchlists yet or something wrong with a server", "data" => false);
			}
			break;
		}
		
		/**
		 * Get watchlist products from doba and cache in database
		 */
		case "getWatchlistProducts" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			$page = isset($_GET["page"]) ? intval($_GET["page"]) : 1;
			$limit = isset($_GET["limit"]) ? intval($_GET["limit"]) : 20;

			if ($page == 1)
			{
				$finish = false;
				$wl_page = 1;
				while (!$finish)
				{
					if (($pr = $dobaProductAPI->getWatchlistProducts($id, $wl_page)) != false)
					{
						$dobaProductAPI->cacheItems($id, $pr);
						$wl_page++;
						unset($pr);
					}
					else
					{
						$finish = true;
					}
				}
			}
			$items_count = $dobaProductAPI->getCachedItemsCount($id);
			$items = $dobaProductAPI->getCachedItems($id, $page, $limit);
			$result = array("status"=>1, "items"=>$items, "items_count"=>$items_count);
			break;
		}
		
		/**
		 * Get items details from cache
		 */
		case "getItemDetailFromCache" : 
		{
			$item_id = isset($_GET["item_id"]) ? intval($_GET["item_id"]) : 0;
			$watchlist_id = isset($_GET["watchlist_id"]) ? intval($_GET["watchlist_id"]) : null;
			if (($item = $dobaProductAPI->getItemDetailFromCache($item_id, $watchlist_id)) != false)
			{
				$result = array("status"=>1, "item"=>$item);
				unset($pr);
			}
			break;				
		}
		
		/**
		 * Update product (synk with cart database)
		 */
		case "updateProductsSet" :
		{
			$id = isset($_POST["id"]) ? intval($_POST["id"]) : 0;
			$page = isset($_POST["page"]) ? intval($_POST["page"]) : 0;
			$limit = isset($_POST["limit"]) ? floatval($_POST["limit"]) : 0;

			if ($page == 1)
			{
				$db->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."doba_cache WHERE watchlist_id='".intval($id)."'");
				$c = $db->moveNext();
				if ($c["c"] == 0)
				{
					if ($dobaProductAPI->saveWatchlistProducts($id))
					{
						$dobaProductAPI->cacheItemsFromFile($id);
					}	
				}
			}
			
			if (($items = $dobaProductAPI->getCachedItems($id, $page, $limit, false)) != false)
			{
				if (!DEMO_LOCK)
				{
					foreach ($items as $item)
					{
						$a = $dobaProductAPI->itemToDb($item['p'], $item['i']);

						/**
						 * Do images here too
						 */
						if (!in_array("images", $dobaProductAPI->lastProductLockedFields))
						{
							if ($settings["doba_images"] != "No")
							{
								$defaultImageURL = $dobaProductAPI->getItemDefaultImageUrl($item['p']);
								$dobaProductAPI->getItemDefaultImage($defaultImageURL, $item['i']['item_id'], $settings["doba_images"] == "All");
								if ($settings["doba_images_secondary"] == "Yes")
								{
									$secondaryImages = $dobaProductAPI->getItemSecondaryImagesUrl($item['p']);
									
									if ($secondaryImages)
									{
										$dobaProductAPI->getItemSecondaryImages(
											$secondaryImages,
											$item['i']['item_id'],
											$settings["doba_images"] == "All"
										);
									}
								}
							}
							else
							{
								$imageFile = "images/products/".imageFileName($item['i']['item_id']).".jpg";
								$previewFile = "images/products/preview/".imageFileName($item['i']['item_id']).".jpg";
								$thumbFile = "images/products/thumbs/".imageFileName($item['i']['item_id']).".jpg";
								if (is_file($imageFile)) @unlink($imageFile);
								if (is_file($previewFile)) @unlink($previewFile);
								if (is_file($thumbFile)) @unlink($thumbFile);
							}
						}
					}
					updateSeoURLs($db, $settings, true, true, false, true);
				}
				$result = array("status"=>1,  "ignored"=>$a==0?1:0, "added" =>$a==2?1:0, "updated"=>$a==1?1:0);
			}
			break;		
		}
		
		/**
		 * Update import settings
		 */
		case "updateImportSettings" : 
		{
			$data = isset($_POST["data"]) ? $_POST["data"] : array();
			if (!DEMO_LOCK)
			{
				foreach ($data as $key => $value)
				{
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$db->escape($key)."'");
					$settings[$key] = $value;
				}
			}
			$result = array("status"=>1);
			
			break;
		}
		
		/**
		 * Request Partner Permission
		 */
		case "requestPartnerPermission":
		{
			if ($dobaRetailerAPI->requestPartnerPermission())
			{
				$result = array("status"=>1);
			}
			
			break;
		}
		
		/**
		 * Has Partner Permission
		 */
		case "hasPartnerPermission":
		{
			if ($r = $dobaRetailerAPI->hasPartnerPermission())
			{
				$result = array("status"=>1, "message"=> $r['response']['has_permission']);
			}
			
			break;
		}
		
		/**
		 * Remove Partner Permission
		 */
		case "removePartnerPermission":
		{
			if ($dobaRetailerAPI->removePartnerPermission())
			{
				$result = array("status"=>1);
			}
			
			break;
		}
		
		/**
		 * Send order to doba
		 */
		case "sendOrder" : 
		{
			$oid = isset($_GET["oid"]) ? intval($_GET["oid"]) : 0;
			$db->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".$oid."' AND is_doba='Yes'");
			$doba_items = array();
			
			while ($item = $db->moveNext())
			{
				$doba_items[] = array("item_id"=>$item["product_id"], "quantity"=>$item["admin_quantity"]);	
			}
			
			if (count($doba_items) > 0)
			{
				$db->query("
					SELECT 
						".DB_PREFIX."orders.order_num AS order_num,
						".DB_PREFIX."orders.shipping_address_type AS address_type,
						".DB_PREFIX."orders.shipping_name AS name,
						".DB_PREFIX."orders.shipping_address1 AS address1,
						".DB_PREFIX."orders.shipping_address2 AS address2,
						".DB_PREFIX."orders.shipping_city AS city,
						".DB_PREFIX."orders.shipping_zip AS zip,
						".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
						".DB_PREFIX."states.short_name AS state_abbr
					FROM ".DB_PREFIX."orders 
					LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
					LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."orders.shipping_state
					WHERE ".DB_PREFIX."orders.oid='".$oid."'
				");

				if ($db->moveNext())
				{
					$data = $db->col;
					$doba_order_response = $dobaOrderAPI->createOrder($data, $doba_items, $data["order_num"]);
				
				 	if ($doba_order_response)
		 			{
		 				if (!DEMO_LOCK)
		 				{
	 						$db->query("UPDATE ".DB_PREFIX."orders SET doba_order_sent=1, doba_order_id='".$db->escape($doba_order_response["response"]["order_id"])."' WHERE oid='".$oid."'");
		 				}
	 					$result = array("status"=>1, "doba_order_id"=>trim($doba_order_response["response"]["order_id"]));
	 				}
	 				else
	 				{
	 					$result = array("status"=>0, "message"=>$dobaOrderAPI->last_error, "data" => false);
	 				}
				}
			}
			
			break;
		}
		
		/**
		 * Get order details
		 */
		case "getOrderDetail" :
		{
			if (($doba_order_id = isset($_GET["doba_order_id"]) ? $_GET["doba_order_id"] : false) != false)
			{
				$doba_order_response = $dobaOrderAPI->getOrderDetail($doba_order_id);
				if ($doba_order_response)
				{
					$result = array("status"=>1, "order"=>$doba_order_response);
				}
			}
			break;
		}
		
		/**
		 * Fund order 
		 */
		case "fundOrder" :
		{
			if (($doba_order_id = isset($_GET["doba_order_id"]) ? $_GET["doba_order_id"] : false) != false)
			{
				$doba_order_response = $dobaOrderAPI->fundOrder($doba_order_id, $_GET["cvv2"]);
				if ($doba_order_response)
			 	{
		 			if (!DEMO_LOCK)
		 			{
	 					$db->query("UPDATE ".DB_PREFIX."orders SET doba_order_funded=1 WHERE doba_order_id='".$db->escape($doba_order_id)."'");
			 		}
		 			$result = array("status"=>1);
	 			}
	 			else
	 			{
	 				$result = array("status"=>0, "message"=>$dobaOrderAPI->last_error);
	 			}
			}
			break;
		}
	}

	if ($dobaProductAPI->last_error)
	{
		$result = array("status" => 0, "message" => $dobaProductAPI->last_error, "data" => false);
	}
	
	if ($dobaRetailerAPI->last_error)
	{
		$result = array("status" => 0, "message" => $dobaRetailerAPI->last_error, "data" => false);	
	}
	
	echo json_encode($result);