<?php

	if (!defined("ENV")) exit();
	
	require_once("content/classes/class.doba.php");
	require_once("content/classes/class.doba_product.php");
	require_once("content/classes/class.doba_order.php");
	require_once("content/classes/class.doba_retailer.php");
	
	$dobaProductAPI = new DobaProductAPI($settings, $db);
	$dobaOrderAPI = new DobaOrderAPI($settings, $db);
	$dobaRetailerAPI = new DobaRetailerAPI($settings, $db);
	$action = isset($_GET["action"]) ? $_GET["action"] : "";
	
	$result = array("status" => 0, "message" => "Wrong request", "data" => false);
	
	switch ($action)
	{
		/**
		 * Get watchlist previously created by user
		 */
		case "getWatchlists" : 
		{
			$wl = $dobaProductAPI->getWatchlists();
			
			if (is_array($wl))
			{	
				$result = array("status"=>1, "count"=>count($wl), "data"=>$wl);
			}
			break;
		}
		
		/**
		 * Get watchlist products
		 */
		case "getWatchlistProducts" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			
			if(($pr = $dobaProductAPI->getWatchlistProducts($id)) != false)
			{
				$result = array("status"=>1, "data"=>$pr);
			}
			
			break;		
		}
		
		/**
		 * Get more details on watchlist products
		 */
		case "getWatchlistProductsExtended" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			
			if(($pr = $dobaProductAPI->getWatchlistProductsExtended($id)) != false)
			{
				$result = array("status"=>1, "data"=>$pr);
			}
			break;		
		}
		
		/**
		 * Return item-spcific details
		 */
		case "getItemDetail" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			
			if(($pr = $dobaProductAPI->getItemDetail($id)) != false)
			{
				$result = array("status"=>1, "data"=>$pr);
			}
			
			break;		
		}
		
		/**
		 * Update product (synk with cart database)
		 */
		case "updateProduct" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			$price = isset($_GET["price"]) ? floatval($_GET["price"]) : 0;
			if (($item = $dobaProductAPI->getItemDetail($id)) != false)
			{
				$a = $dobaProductAPI->itemToDb($item, $price);
				//TODO : optimize flat url generation after doba product import
				//no reasons to update URLs on each product change
				//should be run after import completed
				//add one more ajax/json call
				updateSeoURLs($db, $settings, true, true, false);
				$result = array("status"=>1,  "ignored"=>$a==0?1:0, "added" =>$a==1?1:0, "updated"=>$a==2?1:0);
			}
			break;		
		}
		
		/**
		 * Update import settings
		 */
		case "updateImportSettings" : 
		{
			$data = isset($_POST["data"]) ? $_POST["data"] : array();

			foreach ($data as $key => $value)
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$db->escape($key)."'");
				$settings[$key] = $value;
			}
			
			$result = array("status"=>1);
			
			break;
		}
		
		/**
		 * Request Partner Permission
		 */
		case "requestPartnerPermission":
		{
			if ($dobaRetailerAPI->requestPartnerPermission() !== false)
			{
				$result = array("status"=>1);
			}
			
			break;
		}
		
		/**
		 * Has Partner Permission
		 */
		case "hasPartnerPermission":
		{
			if ($dobaRetailerAPI->hasPartnerPermission() !== false)
			{
				$dobaRetailerAPI->last_error = (String) $dobaRetailerAPI->response_xml->response->has_permission;
			}
			
			break;
		}
		
		/**
		 * Remove Partner Permission
		 */
		case "removePartnerPermission":
		{
			if ($dobaRetailerAPI->removePartnerPermission() !== false)
			{
				$result = array("status"=>1);
			}
			
			break;
		}
		
		
		
		
		/**
		 * Send order to doba
		 */
		case "sendOrder" : 
		{
			$oid = isset($_GET["oid"]) ? intval($_GET["oid"]) : 0;
			$db->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".$oid."'");
			$doba_items = array();
			while ($item = $db->moveNext())
			{
				if ($item["is_doba"] == "Yes")
				{
					$doba_items[] = array("item_id"=>$item["product_id"], "quantity"=>$item["admin_quantity"]);	
				}
				
				if (count($doba_items) > 0)
				{
					$db->query("
						SELECT 
							".DB_PREFIX."orders.shipping_name AS name,
							".DB_PREFIX."orders.shipping_address1 AS address1,
							".DB_PREFIX."orders.shipping_address2 AS address2,
							".DB_PREFIX."orders.shipping_city AS city,
							".DB_PREFIX."orders.shipping_zip AS zip,
							".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
							".DB_PREFIX."states.short_name AS state_abbr
						FROM ".DB_PREFIX."orders 
						LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
						LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."orders.shipping_state
						WHERE ".DB_PREFIX."orders.oid='".$oid."'
					");
					if (($address = $db->moveNext()) != false)
					{
						$doba_order_response = $dobaOrderAPI->createOrder($address, $doba_items);
					 	if ($doba_order_response)
			 			{
		 					$db->query("UPDATE ".DB_PREFIX."orders SET doba_order_sent=1, doba_order_id='".$db->escape($doba_order_response["order_id"][0])."' WHERE oid='".$oid."'");
		 					$result = array("status"=>1, "doba_order_id"=>$doba_order_response["order_id"][0]);
		 				}
					}
				}
			}
			break;
		}
		
		/**
		case "getOrders" :
		{
			$orders = $dobaOrderAPI->getOrders();
			
			if (is_array($orders))
			{	
				$result = array("status"=>1, "count"=>count($orders), "data"=>$orders);
			}
			break;
		}
		
		case "getOrderItems" :
		case "getOrderDetail" :
		{
			$id = isset($_GET["id"]) ? intval($_GET["id"]) : 0;
			
			if(($order = $dobaOrderAPI->getOrderDetail($id)) != false)
			{
				$result = array("status"=>1, "data"=>$order);
			}
			break;
		}
		**/
	}

	if ($dobaProductAPI->last_error)
	{
		$result = array("status" => 0, "message" => $dobaProductAPI->last_error, "data" => false);
	}
	
	if ($dobaRetailerAPI->last_error)
	{
		$result = array("status" => 0, "message" => $dobaRetailerAPI->last_error, "data" => false);	
	}
	
//	print_r($result);
	echo json_encode($result);