<?php

	if (!defined('ENV')) exit();

	require_once('content/engine/engine_notifications.php');
	require_once('content/engine/engine_user.php');
	require_once('content/engine/engine_order.php');

	/**
	 * @param $db
	 * @param $order
	 * @param $order_data
	 * @param $post_form
	 * @return array
	 */
	function handlePaymentCustomMethod($db, $order, $order_data, $post_form)
	{
		if ($order_data['payment_gateway_id'] != '' && file_exists('content/engine/payment/payment_'.$order_data['payment_gateway_id'].'.php'))
		{
			$gateway_file = escapeFileName($order_data['payment_gateway_id']);
			
			require_once('content/engine/payment/payment_'.$gateway_file.'.php');

			$payment_gateway_class_name = 'PAYMENT_'.strtoupper($order_data['payment_gateway_id']);

			$payment_gateway = new $payment_gateway_class_name();
			
			if ($payment_gateway)
			{
				$payment_gateway->enableLog = defined('DEVMODE') && DEVMODE;
				$payment_gateway->getSettingsData($db);
				$payment_gateway->isTestMode();

				// TODO: looks like never used
				/**
				$show_refund = $payment_gateway->supportsRefund($order_data);
				$show_void = $payment_gateway->supportsVoid($order_data);
				$show_capture = $payment_gateway->supportsCapture($order_data);
				**/

				$custom_payment_status = $post_form['custom_payment_status'];
				
				switch ($custom_payment_status)
				{
					case 'Capture':
					{
						$capture_amount = number_format($post_form['custom_capture_amt'], 2, '.', '');
						$payment_gateway->capture($order_data, $capture_amount);
						Admin_Log::log('Order #'.$order->order_num.' - funds captured', Admin_Log::LEVEL_IMPORTANT);
						break;
					}
					
					case 'Refund':
					{
						$refund_amount = number_format($post_form['custom_refund_amt'], 2, '.', '');
						$payment_gateway->refund($order_data, $refund_amount);
						Admin_Log::log('Order #'.$order->order_num.' - funds refunded', Admin_Log::LEVEL_IMPORTANT);
						break;
					}
					
					case 'Void':
					{
						$payment_gateway->void($order_data);
						Admin_Log::log('Order #'.$order->order_num.' - transaction void', Admin_Log::LEVEL_IMPORTANT);
						break;
					}
				}

				return array(
					'success' => !$payment_gateway->is_error,
					'error_message' => $payment_gateway->error_message,
				);
			}
		}

		return array();
	}

	/**
	 * @param $db
	 * @param $oid
	 * @param $order_status
	 * @param $payment_status
	 * @param $order
	 * @param null $placed_date
	 * @param null $shipping_tracking_number
	 * @param null $shipping_tracking_number_type
	 */
	function updateOrderStatus($db, $oid, $order_status, $payment_status, $order, $placed_date = null, $shipping_tracking_number = null, $shipping_tracking_number_type = null)
	{
		/**
		 * Update orders status
		 */
		if (!DEMO_LOCK)
		{
			$shouldUpdate = false;
			$db->reset();
			if ($order_status != '' || $payment_status != '')
			{
				$shouldUpdate = true;
				if ($order_status != '')
				{
					$order->status = $order_status;

					$db->assignStr('status', $order_status);

					if ($order_status == ORDER::STATUS_COMPLETED && !is_null($placed_date) && $placed_date == '0000-00-00 00:00:00')
					{
						$db->assign('placed_date', 'NOW()');
					}
				}

				if ($payment_status != '')
				{
					$order->payment_status = $payment_status;
					$db->assignStr('payment_status', $payment_status);
				}
			}

			if (!is_null($shipping_tracking_number) && !is_null($shipping_tracking_number_type))
			{
				$shouldUpdate = true;
				$db->assignStr('shipping_tracking_number', $shipping_tracking_number);
				$db->assignStr('shipping_tracking_number_type', $shipping_tracking_number_type);
			}

			if ($shouldUpdate)
			{
				$db->assign('status_date', 'NOW()');
				$db->update(DB_PREFIX.'orders', 'WHERE oid='.intval($oid));

				Admin_Log::log('Order #'.$order->order_num.' - status changed', Admin_Log::LEVEL_IMPORTANT);
			}
		}
	}

	/**
 	 * @param $db
 	 * @param $order_data
 	 * @param $post_form
 	 */
	function googleNotification($db, $order_data, $post_form)
	{
		if ($order_data['payment_gateway_id'] == 'googlecheckout')
		{
			$post_form['carrier'] = $order_data['shipping_cm_name'];
			$post_form['tracking_id'] = $order_data['shipping_tracking_number'];

			require_once('content/engine/payment/payment_googlecheckout.php');

			$payment_gateway = new PAYMENT_GOOGLECHECKOUT();
			$payment_gateway->enableLog = defined('DEVMODE') && DEVMODE;
			$payment_gateway->getSettingsData($db);
			$payment_gateway->isTestMode();
			$payment_gateway->processCustomAdvance($db, 4, $order_data['oid'], $post_form);
		}
	}

	/**
	 * @param $db
	 * @param $order_data
	 */
	function sendOrderNotificationEmail($db, $order_data)
	{
		require_once('content/admin/admin_init_smarty.php');
		// *** GIFT CERTIFICATE
		view()->assign('payment_method_name', 'Gift Certificate');
		view()->assign('gift_cert_amount', $order_data['gift_cert_amount']);
		// *** GIFT CERTIFICATE
		emailOrderCompleted($db, $order_data['oid']);
	}

	$mode = isset($mode) ? trim($mode) : '';
	
	$allow_linkpoint_extension = file_exists('content/engine/payment/payment_linkpoint.php');
	
	if (($mode =='globalstatus' || $mode == 'globaldelete') && !DEMO_LOCK)
	{
		switch ($mode)
		{
			/**
			 * CHANGE ORDER AND PAYMENT STATUS FOR FEW ORDERS AT ONCE
			 */
			case 'globalstatus' :
			{
				$_payment_status = isset($payment_status) ? trim($payment_status) : '';
				$_order_status = isset($order_status) ? trim($order_status) : '';
				$orders = isset($orders) ? trim($orders) : '';
				$orders = explode(';', $orders);
				
				if ($_order_status != '' || $_payment_status != '')
				{
					//create linkpoint instance to batch process orders
					if ($order_status == ORDER::STATUS_COMPLETED && $allow_linkpoint_extension)
					{
						require_once('content/engine/payment/payment_linkpoint.php');

						$payment_gateway = new PAYMENT_LINKPOINT();

						if ($payment_gateway)
						{
							$payment_gateway->enableLog = defined('DEVMODE') && DEVMODE;
							$payment_gateway->getSettingsData($db);
						}
					}

					//walk selected orders
					foreach ($orders as $i=>$oid)
					{
						$oid = intval($oid);

						//check is order id correct
						$_result = $db->query('SELECT * FROM '.DB_PREFIX.'orders WHERE oid='.$oid);
						
						if (($order_data = $db->moveNext($_result, MYSQL_ASSOC)) != false)
						{
							/**
							 * Get user and order objects
							 */
							$user = USER::getById($db, $settings, $order_data['uid']);
							$order = ORDER::getById($db, $settings, $user, $oid);

							$user->getUserData();
							$order->getOrderData();
							$order->getOrderItems();

							//reset status vars
							$order_status = $_order_status;
							$payment_status = $_payment_status;
							$current_payment_status = $order_data["payment_status"];
							$current_custom_value = $order_data["custom1"];
							$order_num = $order_data["order_num"];
							$placed_date = $order_data['placed_date'];
							//process linkpoint batch transactions for PRE/POST AUTH

							// TODO: probably should not be here
							if (($order_status == ORDER::STATUS_COMPLETED) && $allow_linkpoint_extension)
							{
								// TODO: !!!! WE DO NOT Awaiting PAYMENT STATUS !!!!
								if (($current_payment_status == 'Awaiting') && ($current_custom_value == 'PREAUTH'))
								{
									if (!safe_mode()) set_time_limit(180);

									$payment_gateway->processCustom($db, 1, $oid);

									if ($payment_gateway->is_error)
									{
										pageNote(array(array('class' => 'ic-cross', 'text' => '<strong>Error processing transaction for order # '.$order_num.'. Error message: '.$payment_gateway->error_message.'</b>')));
										$payment_status = '';
										$order_status = '';
									}
									else
									{
										$payment_status = 'Received';
									}
								}
							}

							$post_form = isset($post_data) ? $post_data : array();

							if ($order_status == ORDER::STATUS_COMPLETED && $order_data['status'] != ORDER::STATUS_COMPLETED)
							{
								if (isset($send_notification_email))
								{
									sendOrderNotificationEmail($db, $order_data);
								}

								googleNotification($db, $order_data, $post_form);
							}

							updateOrderStatus($db, $oid, $order_status, $payment_status, $order, $placed_date);

							$orderStatusChanged = $order_status != '' && $order_data['status'] != $order_status;
							$paymentStatusChanged = $payment_status != '' && $order_data['payment_status'] != $payment_status;

							if ($orderStatusChanged || $paymentStatusChanged)
							{
								/**
								 * Handle on status change event
								 */
								$event = new Events_OrderEvent(Events_OrderEvent::ON_STATUS_CHANGE);

								$event
									->setOrder($order)
									->setData('oldOrderStatus', $order_data['status'])
									->setData('oldPaymentStatus', $order_data['payment_status'])
									->setData('newOrderStatus', $order_status)
									->setData('newPaymentStatus', $payment_status);

								Events_EventHandler::handle($event);

								Admin_Log::log(
									'Order #'.$order_num.' - '.
									($orderStatusChanged ? ' status changed from '.$order_data['status'].' to '.$order_status : '').
									($paymentStatusChanged ? ($orderStatusChanged ? ', ' : '').' payment status changed from '.$order_data['payment_status'].' to '.$payment_status : ''),
									Admin_Log::LEVEL_IMPORTANT
								);
							}
						}
					}
				}

				pageHeader('Orders Status Updated', '', 'icb-list2');
				pageNote(array(array('class' => 'ic-info', 'text' => '<strong>Orders status has been successfully updated</strong>')));
				pageWhat(array(array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')));
				break;
			}
			
			/**
			 * Soft-delete orders
			 */
			case 'globaldelete' :
			{
				$orders = isset($orders) ? trim($orders) : '';
				$orders = explode(';', $orders);

				$ordersData = $db->selectAll('SELECT oid, uid FROM '.DB_PREFIX.'orders WHERE oid IN ('.trim(implode(',', $orders),',').')');

				foreach ($ordersData as $orderData)
				{
					$user = USER::getById($db, $settings, $orderData['uid']);
					$order = ORDER::getById($db, $settings, $user, $orderData['oid']);

					$db->query('UPDATE '.DB_PREFIX.'orders SET removed="Yes", status_date = NOW() WHERE oid='.$order->getId());

					$event = new Events_OrderEvent(Events_OrderEvent::ON_DELETED);
					$event->setOrder($order);
					Events_EventHandler::handle($event);
					Admin_Log::log('Order #'.$order->order_num.' - deleted', Admin_Log::LEVEL_IMPORTANT);
				}
				
				pageHeader('Orders Removed', '', 'icb-list2');
				pageNote(array(array('class' => 'ic-info', 'text' => '<strong>Selected orders has been successfully removed</strong>')));
				pageWhat(array(array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')));

				break;
			}
		}
	}
	else
	{
		$oid = isset($oid) ? intval($oid) : 0;
		
		//SELECT ORDER DATA FROM DATABASE
		$_result = $db->query('
			SELECT '.DB_PREFIX.'orders.*,
				'.DB_PREFIX.'countries.name AS country_name,
				'.DB_PREFIX.'countries.iso_a2 AS country_iso_a2,
				'.DB_PREFIX.'countries.iso_a3 AS country_iso_a3,
				'.DB_PREFIX.'countries.iso_number AS country_iso_number
			FROM '.DB_PREFIX.'orders
			LEFT JOIN '.DB_PREFIX.'countries ON '.DB_PREFIX.'countries.coid = '.DB_PREFIX.'orders.shipping_country
			WHERE 
				'.DB_PREFIX.'orders.oid='.$oid
		);
		
		if (($order_data = $db->moveNext($_result, MYSQL_ASSOC)) != false)
		{
			/**
			 * Get user and order objects
			 */
			$user = USER::getById($db, $settings, $order_data['uid']);
			$order = ORDER::getById($db, $settings, $user, $order_data['oid']);
			$user->getUserData();
			$order->getOrderData();
			$order->getOrderItems();

			switch ($mode)
			{
				/**
				 * Process custom
				 */
				case 'process_custom' :
				{
					$custom_id = isset($custom_id) ? intval($custom_id) : 0;
					$post_form = isset($post_data) ? $post_data : array();
					
					$gateway_file = escapeFileName($order_data['payment_gateway_id']);
					
					require_once('content/engine/payment/payment_'.$gateway_file.'.php');

					$payment_gateway_class_name = 'PAYMENT_'.strtoupper($order_data['payment_gateway_id']);

					$payment_gateway = new $payment_gateway_class_name();
					
					if ($payment_gateway)
					{
						$payment_gateway->enableLog = defined('DEVMODE') && DEVMODE;
						$payment_gateway->getSettingsData($db);
						$payment_gateway->isTestMode();
					}
					
					if ($order_data['payment_gateway_id'] == 'googlecheckout')
					{
						$payment_gateway->processCustomAdvance($db, $custom_id, $oid, $post_form);
					}
					else
					{
						$payment_gateway->processCustom($db, $custom_id, $oid);
					}
						
					if ($payment_gateway->is_error)
					{
						pageHeader('Error Updating Order Payment Status');
						pageNote(array(
							array('class' => 'ic-cross', 'text' => '<strong>Error processing transaction. Error message: '.$payment_gateway->error_message.'</strong>')
						));
					}
					else
					{
						pageHeader('Order Payment Status Changed', '', 'icb-list2');
						pageNote(array(
							array('class' => 'ic-info', 'text' => '<strong>Order payment status has been changed</strong>')
						));
						
						echo '<script type="text/javascript">document.location = "admin.php?p=order&oid='.$oid.';</script>';
					}
					
					pageWhat(array(
						array('text' => 'View edited order', 'url' => 'admin.php?p=order&oid='.$oid),
						array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')
					));

					break;
				}
				
				/**
			 	 * Insure ship
			 	 */
				case 'insureship_cancel' :
				{
					require_once('content/classes/class.insureship.php');
					$insureShip = new InsureShip($settings['InsureShipAccountId'], $settings['InsureShipApiURL']);
					$insureShip->cancel($_REQUEST['oid'], $db);
					echo '<script type="text/javascript">document.location = "admin.php?p=order&oid='.$oid.'";</script>';
					break;
				}
				
				/**
				 * Update order data
				 */
				case 'update' :
				{
					if (!DEMO_LOCK)
					{
						$order_num = $order_data['order_num'];

						$payment_status = isset($_REQUEST['payment_status']) ? trim($_REQUEST['payment_status']) : '';
						$order_status = isset($_REQUEST['order_status']) ? trim($_REQUEST['order_status']) : '';
						$order_editable = isset($_REQUEST['order_editable']) ? trim($_REQUEST['order_editable']) : false;
						$order_editable = $order_editable == 'yes' ? true : false;

						$shipping_error_message = '';
						$shipping_tracking_number = isset($_REQUEST['shipping_tracking_number']) ? trim($_REQUEST['shipping_tracking_number']) : '';
						$shipping_tracking_number_type = isset($_REQUEST['shipping_tracking_number_type']) ? trim($_REQUEST['shipping_tracking_number_type']) : '';

						if ($order_data['shipping_tracking_number'] != $shipping_tracking_number ||	$order_data['shipping_tracking_number_type'] != $shipping_tracking_number_type)
						{
							if ($order_data['payment_gateway_id'] == 'bongocheckout' && isset($_REQUEST['bongoOrderTrackingUpdate']) && trim($order_data['custom2']) != '')
							{
								$bongoOrderData = @unserialize($order_data['custom2']);
								
								if (is_array($bongoOrderData) && isset($bongoOrderData['trackingLink']) && trim($bongoOrderData['trackingLink']) != '')
								{
									$result = $db->query('SELECT product_id, product_sub_id, admin_quantity FROM '.DB_PREFIX.'orders_content WHERE oid='.$oid);
									$bongoOrderItems = array();

									while (($productData = $db->moveNext($result, MYSQL_ASSOC)) != false)
									{
										$bongoOrderItems[] = array(
											'id' => $productData['product_id'].(isset($productData['product_sub_id']) && trim($productData['product_sub_id']) != '' ? '-'.$productData['product_sub_id'] : ''),
											'quantity' => $productData['admin_quantity']
										);
									}

									require_once('content/classes/Payment/Bongo/BongoConnect.php');
									
									$bongoConnect = new BongoConnect($settings['bongocheckout_Partner_Key']);

									$bongoOrderTrackingUpdateResult = $bongoConnect->orderTrackingUpdate(
										$bongoOrderItems, $bongoOrderData['bongoOrderId'],
										$shipping_tracking_number_type, $shipping_tracking_number
									);

									if (!$bongoOrderTrackingUpdateResult)
									{
										$bongoError = $bongoConnect->getLastError().'<br/>'.$bongoConnect->getLastErrorDetails();
									}
								}
							}
						}

						/**
						 * Check are there any changes to be saved
						 */
						if ($order_editable)
						{
							AdminOrderProvider::editOrder($order_data, $order, $shipping_error_message);
						}

						updateOrderStatus($db, $oid, $order_status, $payment_status, $order, null, $shipping_tracking_number, $shipping_tracking_number_type);
					}

					/**
					 * Handle payment methods actions
					 */
					if (isset($_POST['handle_custom_method']) && $_POST['handle_custom_method'] == '1')
					{
						$paymentCustomRet = handlePaymentCustomMethod($db, $order, $order_data, $_POST);

						if (!empty($paymentCustomRet))
						{
							pageHeader('Updating Order Payment Status', '', 'icb-list2');

							pageNote(array(
								array('class' => 'ic-info', 'text' => '<strong>Payment status has been successfully updated.</strong>', 'visible' => $paymentCustomRet['success']),
								array('class' => 'ic-cross', 'text' => '<strong>An error occurred processing payment status: '.$paymentCustomRet['error_message'].'</strong>', 'visible' => !$paymentCustomRet['success'])
							));
						}
					}

					if ($order_status == ORDER::STATUS_COMPLETED && $order_data['status'] != ORDER::STATUS_COMPLETED)
					{
						if (isset($send_notification_email))
						{
							sendOrderNotificationEmail($db, $order_data);
						}

						googleNotification($db, $order_data, $_POST);
					}

					//Send admin note email 
					$send_email =  isset($send_email) ? trim($send_email) : 'No';
					$note_text = isset($note_text) ? $note_text : '';
					$note_title = isset($note_title) ? $note_title : '';
					$send_email_to = isset($send_email_to) ? $send_email_to : '';
					$order_invoice = isset($order_invoice) ? $order_invoice : '0';
					
					if ($send_email == 'Yes' && $send_email_to != '')
					{
						require_once('content/admin/admin_init_smarty.php');
						emailToCustomer($db, $oid, $order_invoice, $send_email_to, $note_text, $note_title);
					}
					
					if (!DEMO_LOCK)
					{
						if ($note_text != '')
						{
							$db->reset();
							$db->assign('oid', $oid);
							$db->assignStr('note_for', 'order');
							$db->assignStr('send_email', $send_email);
							$db->assign('created_at', 'NOW()');
							$db->assignStr('note_title', $note_title);
							$db->assignStr('note_text', $note_text);
							$anid = $db->insert(DB_PREFIX.'admin_notes');
						}
					}
					
					//End of Send admin note email
					//PayPal changes for DoCapture and DoAuthorization	
					$paypal_payment_status = isset($paypal_payment_status) ? $paypal_payment_status : '';
					
					if (trim($paypal_payment_status) != '')
					{
						require_once('admin_order_paypal.php');
						
						if (!$paypal_error)
						{
							pageHeader('Updating Order Payment Status for PayPal', '', 'icb-list2');
							pageNote(array(array('class' => 'ic-info', 'text' => '<storng>PayPal payment status has been successfully updated.</strong>')));
						}
					}

					$datacash_payment_status = isset($datacash_payment_status) ? $datacash_payment_status : '';
					
					if (trim($datacash_payment_status) != '')
					{
						require_once('admin_order_datacash.php');
						
						if (!$datacash->is_error)
						{
							pageHeader('Updating Order Payment Status for DataCash', '', 'icb-list2');
							pageNote(array(
								array('class' => 'ic-info', 'text' => '<strong>DataCash payment status has been successfully updated.</strong>')
							));
						}
					}

					$orderStatusChanged = $order_status != '' && $order_data['status'] != $order_status;
					$paymentStatusChanged = $payment_status != '' && $order_data['payment_status'] != $payment_status;

					if ($orderStatusChanged || $paymentStatusChanged)
					{
						/**
						 * Handle on status change event
						 */
						$event = new Events_OrderEvent(Events_OrderEvent::ON_STATUS_CHANGE);

						$event
							->setOrder($order)
							->setData('oldOrderStatus', $order_data['status'])
							->setData('oldPaymentStatus', $order_data['payment_status'])
							->setData('newOrderStatus', $order_status)
							->setData('newPaymentStatus', $payment_status);

						Events_EventHandler::handle($event);

						Admin_Log::log(
							'Order #'.$order_num.' - '.
							($orderStatusChanged ? ' status changed from '.$order_data['status'].' to '.$order_status : '').
							($paymentStatusChanged ? ($orderStatusChanged ? ', ' : '').' payment status changed from '.$order_data['payment_status'].' to '.$payment_status : ''),
							Admin_Log::LEVEL_IMPORTANT
						);
					}

					pageHeader('Order Updated', '', 'icb-list2');
					
					pageNote(array(
						array('class' => 'ic-info', 'text' => '<strong>Order has been successfully updated</strong>'),
						array('class' => 'ic-cross', 'text' => '<strong>PLEASE NOTE: UNABLE TO UPDATE SHIPPING COSTS.<br>Shipping Server response is: '.$shipping_error_message.'</strong>', 'visible' => ($shipping_error_message != '')),
						array('class' => 'ic-cross', 'text' => '<strong>PLEASE NOTE: UNABLE TO UPDATE BONGO TRACKING NUMBER.<br>Returned error:<br/>'.(isset($bongoError) ? $bongoError : false).'</strong>', 'visible' => isset($bongoError))
					));
					
					pageWhat(array(
						array('text' => 'View edited order', 'url' => 'admin.php?p=order&oid='.$oid),
						array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')
					));
					
					break;
				}
				
				/**
				 * Soft-delete order
				 */
				case 'delete' :
				{
					if (!DEMO_LOCK)
					{
						$db->query('UPDATE '.DB_PREFIX.'orders SET removed="Yes", status_date = NOW() WHERE oid='.$oid);

						/**
						 * Handle on order delete even
						 */
						$event = new Events_OrderEvent(Events_OrderEvent::ON_DELETED);
						$event->setOrder($order);
						Events_EventHandler::handle($event);

						Admin_Log::log('Order #'.$order->order_num.' - deleted', Admin_Log::LEVEL_IMPORTANT);
					}
					
					pageHeader('Order Deleted', '', 'icb-list2');
					pageNote(array(array('class' => 'ic-info', 'text' => '<strong>Selected order has been successfully deleted</strong>')));
					pageWhat(array(array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')));
					break;
				}
				
				/**
				 * Incorrect action
				 */
				default : 
				{
					pageHeader('Incorrect Action', '', 'icb-list2');
					pageNote(array(array('class' => 'ic-cross', 'text' => '<strong>Can\'t apply changes by specified action ID</strong>')));
					pageWhat(array(array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')));
					break;
				}
			}
		}
		else
		{
			pageHeader('Incorrect Order ID', '', 'icb-list2');
			pageNote(array(array('class' => 'ic-cross', 'text' => '<strong>Can\'t select order by specified ID</strong>')));
			pageWhat(array(array('text' => 'Browse orders', 'url' => 'admin.php?p=orders')));
		}
	}
