<?php
	if (!defined("ENV")) exit();

	$tooltips = array(
		'product_id'=>'This is used for internal referencing purposes.  No special characters other than _ or – should be entered here.',
		'image'=>'The image size that you upload here will be used as the "enlarged image". The "enlarged image" will be displayed when someone clicks on a thumbnail image of a product.',
		'is_visible'=>'This controls whether or not the product shows up on the store front.',
		'overview'=>'A short (80 characters max) description of the product. This shows up on the category page where multiple products are listed.',
		'description'=>'Enter any information about the product you\'d like to display.  This description will show up on the product page itself.'
	);
	
	$adminView = Ddm_AdminView::getInstance();

	$category_options = array();
	$tree->collection[TREE_ROOT]->flatten($category_options);
	
	/**
	 * Main container
	 */
	$generalFormBuilder = new core_Form_FormBuilder('general'); 
	
	/**
	 * Basic properties group
	 */
	$basicPropertiesGroup = new core_Form_FormBuilder('basic', array('label'=>'Basic Properties', 'class'=>'ic-product', 'first'=>true));
	$generalFormBuilder->add($basicPropertiesGroup);
	
	$basicPropertiesGroup->add('title', 'text', array('label'=>'Product name (title)', 'required'=>true, 'lockable'=>true, 'lockName'=>'name', 'value'=>$title));
	$basicPropertiesGroup->add('product_id', 'text', array('label'=>'Product ID', 'required'=>true, 'lockable'=>true, 'value'=>$product_id));
	$basicPropertiesGroup->add('price', 'money', array('label'=>'Item Price', 'required'=>true, 'lockable'=>true, 'value'=>$price));
	$basicPropertiesGroup->add('call_for_price', 'checkbox', array('label'=>'Call For Price', 'value'=>'Yes', 'current_value'=>$call_for_price));
	$basicPropertiesGroup->add('cid', 'choice', array(
		'label'=>'Main Category (primary)', 'required'=>true,	'lockable'=>true,	'lockName'=>'category', 
		'options'=>$category_options, 'empty_value'=>array('0'=>'Select Category'),	'value'=>$cid
	));
	$basicPropertiesGroup->add('secondary_categories[]', 'choice', array('label'=>'Secondary Categories', 'required'=>false, 'lockable'=>true,
		'lockName'=>'secondary-categories', 'options'=>$category_options, 'empty_value'=>array('0'=>'None Selected'), 'multiple'=>true, 'value'=>$categories,
		'note' => 'Hold down the control key to select multiple categories.',
	));
	$basicPropertiesGroup->add('is_visible', 'checkbox', array('label'=>'Is this product available?', 'value'=>'Yes', 'current_value'=>$is_visible));
	$basicPropertiesGroup->add('image', 'file', array('label'=>'Primary image', 'required'=>false, 'lockable'=>true, 'lockName'=>'images'));
	
	/**
	 * Primary image
	 */
	if ($mode != 'insert')
	{
		if ($image_location == 'Local')
		{
			$productImageFile = ProductHasImage($product_id);
			
			if ($productImageFile)
			{
				$info = getimagesize($productImageFile);
				$basicPropertiesGroup->add(
					'product_image_file', 'imagepreview', 
					array(
						'label'=>'Current product image', 'value'=>$productImageFile, 'imageInfo'=>imageInfo($productImageFile),
						'note' => '<input type="checkbox" name="cb_delete_image" id="cb_delete_image"/><label for="cb_delete_image">Check to delete</label>'.
							' / <a onclick="return ConfirmDeleteProductImage('.$pid.');" class="delete_image"><strong>Delete selected images</strong></a>'
					)
				);
			}
			$basicPropertiesGroup->add('nonce-product-delete-image', 'hidden', array('value'=>create_nonce('product_delete_image')));
		}
		else if ($image_url != '')
		{
			$basicPropertiesGroup->add(
				'product_image_file', 'imagepreview', 
				array('label'=>'Current product image', 'value' => $image_url, 'note'=>'Click over image to see it real size')
			);
		}
	}
	
	/**
	 * Product overview
	 */
	$quickOverviewGroup = new core_Form_FormBuilder('QuickOverview', array('label'=>'Quick Overview', 'class'=>'ic-pencil', 'collapsable'=>true));
	$generalFormBuilder->add($quickOverviewGroup);
	$quickOverviewGroup->add('overview', 'textarea', array('label'=>'Please enter quick overview', 'value'=>$overview, 'lockable'=>true));
	
	/**
	 * Product description
	 */
	$descriptionGroup = new core_Form_FormBuilder('description', array('label'=>'Product Description', 'class'=>'ic-pencil', 'hasWysiwyg'=>true, 'collapsable'=>true));
	$generalFormBuilder->add($descriptionGroup);
	$descriptionGroup->add('description', 'textarea', array('label'=>'Please enter product description', 'value'=>$description, 'lockable'=>true));
	
	/**
	 * More Images
	 */
	$imagesGroup = new core_Form_FormBuilder('Image', array('label'=>'Image-Related Properties', 'class'=>'ic-file-format-jpg', 'collapsable'=>true));
	$generalFormBuilder->add($imagesGroup);

	$imagesGroup->add('image_alt_text', 'text', array('label'=>'Image Alt Text', 'value'=>$image_alt_text));
	
	$zoomOptions = array(
		new core_Form_Option('global', 'Use Global Settings'),
		new core_Form_Option('none', 'None'),
		new core_Form_Option('zoom', 'Zoom'),
		new core_Form_Option('magnify', 'Magnify'),
		new core_Form_Option('magicthumb', 'Magic Thumb'),
		new core_Form_Option('imagelayover', 'Image Layover'),
	);
	
	$imagesGroup->add('zoom_option', 'choice', array('label'=>'Image zooming', 'value'=>$zoom_option, 'options'=>$zoomOptions));
	$imagesGroup->add('image_url', 'text', array('label'=>'Primary Image Url', 'value'=>$image_url, 'lockable' => true, 'lockName' => 'images'));

	if ($settings['CatalogThumbType'] == 'Manual')
	{
		$imagesGroup->add('image_thumb', 'file', array('label'=>'Thumbnail Image'));
	}
	
	if ($settings['CatalogThumb2Type'] == 'Manual')
	{
		$imagesGroup->add('image_thumb2', 'file', array('label'=>'Product Page Image'));
	}

	
	
	/**
	 * Check for current images
	 */
	if ($mode != 'insert')
	{
		if ($image_location == 'Local')
		{
			if (isset($productImageFile) && $productImageFile)
			{
				/**
				 * Thumb image preview
				 */
				$productImageThumb = ProductHasThumb($product_id);
				if ($productImageThumb)
				{
					$imagesGroup->add(
						'product_image_thumb', 'imagepreview', 
						array(
							'label'=>'Product thumbnail', 'value'=>$productImageThumb, 'imageInfo'=>imageInfo($productImageThumb),
							'note' => '<input type="checkbox" name="cb_delete_thumb" id="cb_delete_thumb" alt="Image thumbnail" title="Image thumbnail"/><label for="cb_delete_thumb">Check to delete</label>'.
								' / <a onclick="return ConfirmDeleteProductImage('.$pid.');" class="delete_image"><strong>Delete selected images</strong></a>'
						)
					);
				}
				else
				{
					$imagesGroup->add(
						'product_image_thumb', 'static', array('label'=>'Product thumbnail', 'value'=>'There is not a thumbnail for this image')
					);
				}
		
				/**
				 * Product page iage preview
				 */
				$productPageImage = ProductHasThumb2($product_id);
				if ($productPageImage)
				{
					$imagesGroup->add(
						'product_page_image', 'imagepreview', 
						array(
							'label'=>'Product page image', 'value'=>$productPageImage, 'imageInfo'=>imageInfo($productPageImage),
							'note' => '<input type="checkbox" name="cb_delete_thumb2" id="cb_delete_thumb2"><label for="cb_delete_thumb2">Check to delete</label>'.
								' / <a onclick="return ConfirmDeleteProductImage('.$pid.');" class="delete_image"><strong>Delete selected images</strong></a>'
						)
					);
				}
				else
				{
					$imagesGroup->add(
						'product_page_image', 'static', array('label'=>'Product page image', 'value'=>'There is not an image for the product page')
					);
				}
			}	
		}
	}
	
	/**
	 * Get form
	 */
	$generalForm = $generalFormBuilder->getForm();
	$generalForm->lockForm($lock_fields);
	$generalForm->setTooltips($tooltips);
	
	$adminView->assign('generalForm', $generalForm);

	/**
	 * Assign other data
	 */
	$adminView->assign('lock_fields', $lock_fields);
	
	$adminView->assign('mode', $mode);
	
	$adminView->assign('pid', $pid);

	$adminView->assign('rand_num', rand(1, 9999999));

	$adminView->assign('prev_pid', $prev_pid);
	$adminView->assign('next_pid', $next_pid);

	/**
	 * Advanced properties
	 */

	$tooltips = array(
		'manufacturer_id' => 'This is where you can chose a the manufacturer of the product so it shows up in searches. Manufacturers need to be setup prior to assigning them.',
		'product_mpn' => 'This is a number used for Google Base.',
		'products_location_id'=>
			'This feature allows you to automatically send shipping notification of products purchased to manufacturer. '.
			'<a href="admin.php?p=products_locations">Manage products locations</a>',
		'priority' => 'This is for sorting your list of products. The lower the number you assign a product, the higher it will show in your store. 0 will show first, then 1, then 2, etc.',
		'cost' => 'This is for your reference only, to help you track profit.  It will not be visible anywhere on your site.',
		'price2' => 'Want to put a product on sale? This is the area you enter the discounted price. Make sure it\'s lower than the item\'s original price!',
		'is_taxable' => 'If this is unchecked tax will not be charged for the product.',
		'tax_class_id' => 'Tax classes need to be setup first prior to assigning them.',
		'tax_rate' => 'This will override any tax class you have defined.',
		'weight' => 'Enter the total shipping weight. This must be defined if you\'re using real time shipping rates.',
		'free_shipping' => 'Check this box if you\'re not going to charge extra to ship the product. ',
		'shipping_price' => 'Want to charge a flat rate for shipping? Enter it here.',
		'estimated_shipping_price' => 'This value will be used in product feeds.',
		'inter_case_pack' => 'This is used for bulk items. Interpack is the number of products per pack, Case Pack is the number of packs per case.',
		'order' => 'Set the minimum and maximum number of this product that can be purchased by your customer. The default minimum is 1.',
		'digital_product' => 'If you select yes, the product will be classified as a digital product and customers may download it from your store. Use a FTP application to upload products to your store. Please upload the file/application into /content/download folder and enter the relative path below. You may organize any directory structure you like in this folder.',
		'digital_product_file' =>
			'Please enter the relative path to the file. '.
			'For example, if full file path is <u>/content/download/books/story.pdf</u>, '.
			'you have to enter <u>books/story.pdf</u> only (exclude "/content/download"). '.
			'Do not place "/" (slash) symbol before file path',
		'digital_product_file_upload' =>
			'File will be uploaded into content/download folder. If file with the same name already exist, it will be replaced. '.
			'Max. upload file size: '.ini_get('upload_max_filesize'),
		'image_url' => 'Overrides local image if Url is not empty',
		'url_custom' => 'Enter a custom Url to be used for search engine optimization.',
		'image_url' => 'This allows you to manually define the url of the product image. Only used if you\'re hosting the product image someplace else.',
		'image_alt_text' => 'Enter the alternate text for the image. If left blank, the Product Title text will be used.',
		'is_hotdeal' => 'This controls whether or not the "hot deal" image shows up on store front.',
		'zoom_option' => 'Enter any information about the product you\'d like to display.  This description will show up on the product page itself.'
	);
	
	$adminView = Ddm_AdminView::getInstance();

	$advancedFormBuilder = new core_Form_FormBuilder('advanced_form');

	$additionalInfoGroup = new core_Form_FormBuilder('additional_info', array('label'=>'Additional Settings', 'class'=>'ic-pen', 'collapsable'=>true));
	$advancedFormBuilder->add($additionalInfoGroup);

	$additionalInfoGroup->add('product_sku', 'text', array('label'=>'Product SKU', 'value'=>$product_sku));
	$additionalInfoGroup->add('product_upc', 'text', array('label'=>'UPC', 'value'=>$product_upc));
	$additionalInfoGroup->add('product_gtin', 'text', array('label'=>'Global Trade Item Number', 'value'=>$product_gtin));

	$manufacturers = array();
	if ($settings['CatalogManufacturers'] == 'YES')
	{
		$db->query("SELECT * FROM ".DB_PREFIX."manufacturers ORDER BY manufacturer_name");
		
		while ($manufacturer = $db->moveNext())
		{
			$manufacturers[] = new core_Form_Option($manufacturer['manufacturer_id'], $manufacturer['manufacturer_name']);
		}

		if (count($manufacturers) > 0)
		{
			$additionalInfoGroup->add('manufacturer_id', 'choice', array(
				'label'=>'Product Manufacturer',
				'value'=>$manufacturer_id,
				'empty_value'=>array('0'=>'Do not assign manufacturer'),
				'options'=>$manufacturers,
			));
		}
	}
	$additionalInfoGroup->add('product_mpn', 'text', array('label'=>'Manufacturer Product Number', 'value'=>$product_mpn));

	$product_locations = array();
	foreach ($productsLocations->getAll() as $product_location)
	{
		$product_locations[] = new core_Form_Option($product_location['products_location_id'], $product_location['name'].' ('.$product_location['products_location_id'].')');
	}

	if (count($product_locations) > 0)
	{
		$additionalInfoGroup->add('products_location_id', 'choice', array('label'=>'Product Location', 'options'=>$product_locations, 'empty_value'=>array('0'=>'Do not assign location'), 'value'=>$products_location_id));
	}

	$additionalInfoGroup->add('priority', 'integer', array('label'=>'Priority', 'value'=>$priority));

	$taxPropertiesGroup = new core_Form_FormBuilder('tax-properties', array('label'=>'Sales Tax Properties','class'=>'ic-checkmark','collapsable'=>true));
	$advancedFormBuilder->add($taxPropertiesGroup);

	$taxService = Tax_ProviderFactory::getTaxService();

	if ($taxService instanceof Tax_CustomProvider)
	{
		$taxPropertiesGroup->add('is_taxable', 'checkbox', array('label'=>'Is this a taxable item?', 'value'=>'Yes', 'current_value'=>$is_taxable));

		$taxes = new Taxes($db);
		$taxes->demo = defined('DEMO_LOCK') && DEMO_LOCK;

		$taxClassOptions = array();
		$taxClassOptions[] = new core_Form_Option(0, 'Tax Class is not assigned');
		$taxClasses = $taxes->getTaxClassesList();

		foreach ($taxClasses as $taxClass)
		{
			$taxClassOptions[] = new core_Form_Option($taxClass['class_id'], $taxClass['class_name']);
		}

		$taxPropertiesGroup->add('tax_class_id', 'choice', array('label'=>'Select tax class', 'value'=>$tax_class_id, 'options'=>$taxClassOptions));
		$taxPropertiesGroup->add('tax_rate', 'money', array('label'=>'Tax rate at product level', 'value'=>($tax_rate > -1 ? $tax_rate : null)));

	}
	else if ($taxService instanceof Tax_Avalara_AvalaraService)
	{
		$avalaraTaxCodesOptions = array(new core_Form_Option(0,'None'));
		$avalara_tax_codes = $db->selectAll("select * from ".DB_PREFIX."avalara_tax_codes");

		foreach ($avalara_tax_codes as $code_item)
		{
			$avalaraTaxCodesOptions[] = new core_Form_Option($code_item['code'], $code_item['code'].': '.$code_item['name']);
		}
		$taxPropertiesGroup->add('avalara_tax_code', 'choice', array('label'=>'Avalara tax code', 'value'=> $avalara_tax_code, 'options'=>$avalaraTaxCodesOptions));
	}
	else if ($taxService instanceof Tax_Exactor_ExactorService)
	{
		$exactorTaxCodesOptions = array(new core_Form_Option(0,'None'));
		$exactor_tax_codes = $db->selectAll("select * from ".DB_PREFIX."exactor_tax_codes");

		foreach ($exactor_tax_codes as $code_item)
		{
			$exactorTaxCodesOptions[] = new core_Form_Option($code_item['code'], $code_item['code'].': '.$code_item['name']);
		}

		$taxPropertiesGroup->add('exactor_euc_code', 'choice', array('label'=>'Exactor tax code', 'value'=> $exactor_euc_code, 'options'=>$exactorTaxCodesOptions));
	}

	$productPropertiesGroup = new core_Form_FormBuilder('properties', array('label'=>'Product Properties','class'=>'ic-checkmark','collapsable'=>true));
	$advancedFormBuilder->add($productPropertiesGroup);

	$productPropertiesGroup->add('cost', 'money', array('label'=>'Cost', 'value'=>$cost, 'emptyValue' => -1));
	$productPropertiesGroup->add('price2', 'money', array('label'=>'Sale price', 'value'=>$price2, 'lockable'=>true, 'lockName'=>'sale-price'));

	if ($settings['WholesaleDiscountType'] == 'PRODUCT')
	{
		$fieldOptions = array(
			'label'=>'Wholesale Prices (Level 1/Level 2/Level 3)',
			'levels'=>$settings['WholesaleMaxLevels'],
			'price_level_1'=>$price_level_1,
		);
		if ($settings['WholesaleMaxLevels'] != '1')
			$fieldOptions['price_level_2'] = $price_level_2;
		if ($settings['WholesaleMaxLevels'] != '2')
			$fieldOptions['price_level_3'] = $price_level_3;

		$productPropertiesGroup->add('price_level', 'wholesalepricing', $fieldOptions);
	}

	$productPropertiesGroup->add('free_shipping', 'checkbox', array('label'=>'Free shipping for this product?', 'value'=>'Yes', 'current_value'=>$free_shipping));
	$productPropertiesGroup->add('weight', 'numeric', array('label'=>'Unit Weight ('.$settings['LocalizationWeightUnits'].')', 'value'=>$weight));

	$db->query("
		SELECT
			".DB_PREFIX."shipping_selected.carrier_name,
			".DB_PREFIX."shipping_selected.ssid,
			".DB_PREFIX."products_shipping_price.is_price,
			IF(".DB_PREFIX."products_shipping_price.price IS NULL, 0.00000, ".DB_PREFIX."products_shipping_price.price) AS shipping_price
		FROM
			".DB_PREFIX."shipping_selected
		LEFT JOIN ".DB_PREFIX."products_shipping_price ON
			".DB_PREFIX."products_shipping_price.ssid = ".DB_PREFIX."shipping_selected.ssid AND
			".DB_PREFIX."products_shipping_price.pid = ".intval($pid)."
		WHERE
			carrier_id ='custom' AND
			method_id='product_level' AND
			hidden='No'
		ORDER BY
			".DB_PREFIX."shipping_selected.ssid 
		LIMIT 1
	");
	while ($provider = $db->moveNext())
	{
		$productPropertiesGroup->add('shipping_price['.$provider['ssid'].']', 'money', array(
				'label'=>'Shipping price for '.gs($provider['carrier_name']),
				'value'=>$provider['shipping_price'], 'emptyValue' => 0
			)
		);
	}
	$productPropertiesGroup->add('estimated_shipping_price', 'money', array('label'=>'Estimated Shipping Price', 'value'=>($estimated_shipping_price > -1 ? $estimated_shipping_price : null)));

	if ($settings['WholesaleInterPack'] == 'YES' || $settings['WholesaleCasePack'] == 'YES')
	{
		$fieldOptions = array();
		$label = '';
		if ($settings['WholesaleInterPack'] == 'YES')
		{
			$label = 'Inter Pack';
			$fieldOptions['interPackEnabled'] = true;
		}
		if ($settings['WholesaleInterPack'] == 'YES' && $settings['WholesaleCasePack'] == 'YES')
		{
			$label .= ' / ';
		}
		if ($settings['WholesaleCasePack'] == 'YES')
		{
			$label .= 'Case Pack';
			$fieldOptions['casePackEnabled'] = true;
		}
		$fieldOptions['casePackValue'] = $case_pack >= 0 ? $case_pack : null;
		$fieldOptions['interPackValue'] = $inter_pack >= 0 ? $inter_pack : null;
		$fieldOptions['label'] = $label;

		$productPropertiesGroup->add('inter_case_pack', 'intercasepack', $fieldOptions);
	}

	$productPropertiesGroup->add('order', 'minmaxnumeric', array('label'=>'Minimum / Maximum quantity in order', 'minValue'=>$min_order, 'maxValue'=>$max_order));

	$productPropertiesGroup->add('dimension_width', 'numeric', array('label'=>'Package width ('.($settings["LocalizationLengthUnits"]=="cm"?"cm":"inches").')', 'value'=>$dimension_width));
	$productPropertiesGroup->add('dimension_length', 'numeric', array('label'=>'Package length ('.($settings["LocalizationLengthUnits"]=="cm"?"cm":"inches").')', 'value'=>$dimension_length));
	$productPropertiesGroup->add('dimension_height', 'numeric', array('label'=>'Package height ('.($settings["LocalizationLengthUnits"]=="cm"?"cm":"inches").')', 'value'=>$dimension_height));
	
	$productPropertiesGroup->add('is_hotdeal', 'checkbox', array('label'=>'Mark product as a "Hot Deal"?', 'value'=>'Yes', 'current_value'=>$is_hotdeal));
	$productPropertiesGroup->add('is_home', 'checkbox', array('label'=>'Show product on home page?', 'value'=>'Yes', 'current_value'=>$is_home));

	if ($google_co_active)
	{
		$productPropertiesGroup->add('is_google_co', 'checkbox', array('label'=>'Is this product available for Google Checkout?', 'value'=>'Yes', 'current_value'=>$is_google_co));
	}
	else
	{
		$productPropertiesGroup->add('is_google_co', 'hidden', array('value'=>'Yes'));
	}

	$seoGroup = new core_Form_FormBuilder('seo_properties', array('label'=>'SEO Properties', 'class'=>'ic-zoom', 'collapsable'=>true));
	$advancedFormBuilder->add($seoGroup);

	$seoGroup->add('meta_keywords', 'text', array('label'=>'Meta Keywords', 'value'=>$meta_keywords, 'lockable'=>true, 'lockName'=>'meta-keywords'));
	$seoGroup->add('meta_title', 'text', array('label'=>'Meta Title', 'value'=>$meta_title, 'lockable'=>true, 'lockName'=>'meta-title'));
	$seoGroup->add('meta_description', 'textarea', array('label'=>'Meta Description', 'value'=>$meta_description, 'lockable'=>true, 'lockName'=>'meta-description'));
	$seoGroup->add('url_default', 'static', array('label'=>'Product Url (pre-generated)', 'value'=>'<span style="color:blue;text-decoration:underline;">'.$url_default.'</span>'));
	$seoGroup->add('url_custom', 'text', array('label'=>'Product Url (custom)', 'value'=>$url_custom));

	if ($settings['DigitalProductsActive'] == 'YES' && AccessManager::checkAccess('DOWNLOADS'))
	{
	 	$digitalGroup = new core_Form_FormBuilder('Digital', array('label'=>'Digital Product Properties', 'class'=>'ic-bit', 'collapsable'=>true));
	 	$advancedFormBuilder->add($digitalGroup);

	 	$digitalGroup->add('digital_product', 'checkbox', array('label'=>'Is this a digital download?', 'value'=>'Yes', 'current_value'=>$digital_product));
	 	$digitalGroup->add('digital_product_file', 'text', array('label'=>'Enter path to digital product file', 'value'=>$digital_product_file));
	 	$digitalGroup->add('digital_product_file_upload', 'file', array('label'=>'OR choose file you would like to upload'));
	}
	else
	{
		$advancedFormBuilder->add('digital_product', 'hidden', array('value'=>'No'));
	}


	$advancedForm = $advancedFormBuilder->getForm();
	$advancedForm->lockForm($lock_fields);

	$advancedForm->setTooltips($tooltips);

	$adminView->assign('advancedForm', $advancedForm);
	
	/**
	 * Render view
	 */
	
	$adminView->render('pages/product/product-info-tab');
