<?php
	if (!defined("ENV")) exit();

	pageHeader("CCS Settings", "", "icb-safety-box");
	
	if ($_SERVER["SERVER_PORT"] != 443 && (!defined('DEVMODE') || (defined('DEVMODE') && !DEVMODE)))
	{
		pageNote(array(
			array("class"=>"ic-cross", "text"=>"<b>You can't use CCS without SSL connection", "url"=>"",  "visible"=>true)
		));
	}
	else
	{
		require_once("content/classes/class.openssl.php");

		if (AccessManager::checkAccess('CCS'))
		{
			$action = isset($action) ? $action : "";
			
			if ($action != '')
			{
				$nonce = isset($nonce) ? $nonce : '';
				$nonce_valid = verify_nonce($nonce, 'settings_css_'.gs($action));
				if (!$nonce_valid)
				{
					invalid_nonce_message();
					exit;
				}
			}
			
			pageNote(array(
				array(
					"class"=>"ic-info", 
					"text"=>"This will enable credit card storage within the application.<br/><br/>".
					"<strong>Please Note:</strong><br/>".
					"It's strongly recommended that you do not store credit cards on your system unless you feel it is absolutely necessary.<br/>".	
					"CISP complaint payment gateways are the only recommended repository for customer payment information.<br/>".
					"While we provide this feature as a service to our customers, we strongly advise against using it.<br/>".
					"Since no encryption method is completely secure, we cannot be held responsible for any issues surrounding ".
					"the storage of your customers payment data.<br/>".
					"Please be advised that for compliance reasons the CVV2 number for any credit card WILL NOT be saved and therefore cannot be retrieved.", 
					"url"=>"", "visible"=>true),		
				array("class"=>"ic-info", "text"=>"<b>To activate CCS, please generate a certificate first.</b>", "url"=>"", "visible"=>($settings["SecurityCCSCertificate"] == "" || $settings["SecurityCCSPrivateKey"] == ""))
			));
	
			$dn = isset($_POST["dn"])?$_POST["dn"]:array(
				"countryName" => $settings["CompanyCountry"],
				"stateOrProvinceName" => $settings["CompanyState"],
				"localityName" => $settings["CompanyCity"],
				"organizationName" => $settings["CompanyName"],
				"organizationalUnitName" => $settings["CompanyName"]." Unit",
				"commonName" => $settings["CompanyName"],
				"emailAddress" => $settings["CompanyEmail"]
			);
			
			if ($action == "update" && !DEMO_LOCK)
			{
				$form = isset($form) ? $form : array();
				
				foreach ($form as $name=>$value)
				{
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$name."'");
					$settings[$name] = $value;
				
					if ($name = "SecurityCCSActive")
					{
						adminLog("CCS is turned ".($value == "1" ? "on":"off"));
					}
				}
			}
		
			$error_message = false;
			$message = false;
			
			if ($action == "generate" && !DEMO_LOCK)
			{
				adminLog("Attempt to generate CCS certificate");
				if (strlen($ccs_username) < 8)
				{
					$error_message = "Unable to generate certificate. CCS Username is too short. Minimal length is 8 chars";
				}
				elseif ($ccs_username != $ccs_username2)
				{
					$error_message = "Unable to generate certificate. Username and confirmation are different";
				}
				elseif ($ccs_username == $ccs_password)
				{
					$error_message = "Unable to generate certificate. Username and password are the same";
				}
				elseif (pwdStrength($ccs_password) < 3)
				{
					$error_message = "Unable to generate certificate. CCS Password is not strong enough";
				}
				elseif ($ccs_password != $ccs_password2)
				{
					$error_message = "Unable to generate certificate. Password and confirmation are different";
				}
				else
				{
					$ds = DIRECTORY_SEPARATOR;
					$openSSL = new openSSL(4096, OPENSSL_KEYTYPE_RSA, 3650, $settings["GlobalServerPath"].$ds."content".$ds."ssl".$ds."openssl.cnf");
				
					$pwd = $ccs_username.$ccs_password;
				
					if (!$openSSL->generate($dn, $pwd))
					{
						$error_message = "Error occurred while generating CA certificate.";
					}
					else
					{
						adminLog("CCS certificate updated");
						$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape(base64_encode($openSSL->csr_pem))."' WHERE name='SecurityCCSCertificate'");
						$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape(base64_encode($openSSL->private_key_pem))."' WHERE name='SecurityCCSPrivateKey'");
					
						$settings["SecurityCCSCertificate"] = base64_encode($openSSL->csr_pem);
						$settings["SecurityCCSPrivateKey"] = base64_encode($openSSL->private_key_pem);
				
						$message = "<b>New CCS Certificate has been successfully created.</b>";
					}
				}
			}
			if (($settings["SecurityCCSCertificate"] != "" && $settings["SecurityCCSPrivateKey"] != "") || 1)
			{
?>
<div class="admin-tabs admin-tabs-framed" id="tabs-ccs-settings">
	<div class="clearfix admin-tabs-items">
		<ul class="clearfix">
			<li class="admin-tab first admin-tab-active" onclick="setActiveTab('tabs-ccs-settings', 'main')" id="admin-tab-main">
				<span>CCS Settings</span>
			</li>
		</ul>
	</div>
	<div id="admin-tab-sheet-main" class="admin-tab-sheet" style="">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first">Enable / Disable CCS</div>
			
			<form name="frmSettings" action="admin.php?p=settings_ccs" method="post">
				<input type="hidden" name="action" value="update" />
				<input type="hidden" name="nonce" value="<?=create_nonce('settings_ccs_update')?>" />
				<div class="field field-text clearfix ">
					<label for="field-title" class="caption">Do you want to enable the storage of card holder data?</label>
					<div class="control">
						<select name="form[SecurityCCSActive]">
							<option value="0">Disable CCS Features</option>
							<option value="1" <?=$settings["SecurityCCSActive"]=="1"?'selected="selected"':''?>>Enable CCS Features</option>
						</select>
					</div>
				</div>
				<div class="field field-text clearfix ">
					<label for="field-title" class="caption">Length of time you would like card holder information stored? (Max 90 days)</label>
					<div class="control">
						<select name="form[SecurityCCSStoredFor]" class="tiny">			
<?php
							for ($i=1; $i<=90; $i++)
							{
?>
						<option value="<?php echo $i;?>" <?php echo $i==$settings["SecurityCCSStoredFor"]?"selected":"";?>><?php echo $i;?></option>
<?php
							}
?>
						</select>
					</div>
				</div>
				<div class="admin-form-buttons">
					<input type="submit" value="Save changes"/>
				</div>
			</form>
		</div>
	</div>
</div>
<?php
			}
			else
			{
				echo "<br/>";
			}
			echo "<div class=\"clear:both;\"></div>";
			pageNote(array(
				array("class"=>"ic-info", "text"=>
					"Entering in the information below will create a certificate.<br/><br/>".
 					"Complete the information below to generate a certificate for encrypting payment information. ".
 					"The password you enter will be used to view payment data on the order page.<br/>".
	 				"PLEASE NOTE - Any payment data encrypted with this certificate can only be accessed by the password you enter.<br/>".
	 				"While you can generate a new certificate at anytime, you will not be able to access payment information ".
 					"encrypted with an older certificate.<br/>Therefore, it is very that important you do not lose this password. ".
 					"If lost, the only solution is to generate a new certificate.", "url"=>"", "visible"=>true),
				array("class"=>"ic-cross", "text"=>'<span style="color:#CC0000"><b>'.$error_message.'</b></span>', "url"=>"", "visible"=>$error_message),
				array("class"=>"ic-info", "text"=>$message, "url"=>"", "visible"=>$message)
			));
?>
<form name="frmSettings" action="admin.php?p=settings_ccs" method="post">
	<input type="hidden" name="action" value="generate"/>
	<input type="hidden" name="nonce" value="<?=create_nonce('settings_ccs_generate')?>"/>
	<div class="admin-tabs admin-tabs-framed" id="tabs-ccs">
<?php 
	pageTabs(array(
		array("title"=>"Certificate Data", "id"=>"main"),
	), "tabs-ccs", false);
?>
		<div id="admin-tab-sheet-main" class="admin-tab-sheet">
			<div class="admin-tab-sheet-wrap">
				<div class="admin-form-header first"><span class="icon ic-icon"></span>Generate New Certificate</div>
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">CCS Username (8 chars min)</td>
						<td class="formItemControl">
							<input type="text" name="ccs_username" value=""/>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">CCS Username Confirmation</td>
						<td class="formItemControl"><input type="text" name="ccs_username2" value=""></td>
					</tr>
					
					<tr valign="top">
						<td class="formItemCaption">CCS Password (8 chars min)</td>
						<td class="formItemControl">
							<input type="password" name="ccs_password" value="" onKeyUp="pwdStrength(this.value)" />
							<div id="pwdDescription"></div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">CCS Password Confirmation</td>
						<td class="formItemControl"><input type="password" name="ccs_password2" value=""></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Country Name</td>
						<td class="formItemControl">
							<select name="dn[countryName]">
<?php
				$db->query("SELECT * FROM ".DB_PREFIX."countries ORDER BY name");
				while ($db->moveNext())
				{
					echo '<option value="'.$db->col["iso_a2"].'" '.($db->col["iso_a2"] == $dn["countryName"] ? "selected":"").'>'.htmlspecialchars($db->col["name"]).'</option>'."\n";
				}		
?>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">State / Province Name</td>
						<td class="formItemControl"><input type="text" name="dn[stateOrProvinceName]" value="<?=htmlspecialchars($dn["stateOrProvinceName"])?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Locality Name</td>
						<td class="formItemControl"><input type="text" name="dn[localityName]" value="<?=htmlspecialchars($dn["localityName"])?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Organization Name</td>
						<td class="formItemControl"><input type="text" name="dn[organizationName]" value="<?=htmlspecialchars($dn["organizationName"])?>"></td>
					</tr>	
					<tr valign="top">
						<td class="formItemCaption">Organization Unit Name</td>
						<td class="formItemControl"><input type="text" name="dn[organizationalUnitName]" value="<?=htmlspecialchars($dn["organizationalUnitName"])?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Common Name</td>
						<td class="formItemControl"><input type="text" name="dn[commonName]" value="<?=htmlspecialchars($dn["commonName"])?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Email Address</td>
						<td class="formItemControl"><input type="text" name="dn[emailAddress]" value="<?=htmlspecialchars($dn["emailAddress"])?>"></td>
					</tr>
				</table>
			</div>
			<div class="admin-form-buttons">
				<input type="submit" value="Save changes"/>
				<input type="reset" value="Reset form"/>
			</div>
		</div>
	</div>
</form>
<?
		}
		else{
			pageNote(array(
				array("class"=>"ic-info", "text"=>"<b>This feature is not available under your license agreement. Please contact your provider to upgrade your application.</b>", "url"=>"", "visible"=>true)
			));
		}
	}
?>