<?php
	if (!defined("ENV")) exit();

	if (!AccessManager::checkAccess('QR'))
	{
		AccessManager::notAllowed();
		return false;
	}
	
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : "";
	$nonce = isset($_REQUEST["nonce"]) ? $_REQUEST["nonce"] : "";
	
	require_once('content/classes/class.qr.php');
	require_once('content/classes/class.offsite_commerce_campaigns.php');
	
	$qrCodes = new QR($settings);
	$offsite_commerce_campaigns = new OffsiteCommerceCampaigns($db, $settings);
	
	switch ($action)
	{
		/**
		 * Update ajax status
		 */
		case "update_status" :
		{
			$qr_active = isset($_REQUEST["qr_active"]) ? $_REQUEST["qr_active"] : "";
			if (in_array($qr_active, array("Yes", "No")))
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($qr_active)."' WHERE name='qr_active'");
			}
			_session_write_close();
			$db->done();
			die();
			break;
		}
		
		/**
		 * Update settings
		 */
		case "update_settings" :
		{
			if (!verify_nonce($nonce, "qr_update_settings"))
			{
				invalid_nonce_message();
				exit;
			}
			$form = isset($_POST["form"]) ? $_REQUEST["form"] : array();
			$form = is_array($form) ? $form : array();
			
			if (isset($form["qr_media"]) && $form["qr_media"] != "0")
			{
				//TODO: convert to array
				switch ($form["qr_media"])
				{
					case "1" : $e = "L"; $s = "4"; break;
					case "2" : $e = "L"; $s = "8"; break;
					case "3" : $e = "M"; $s = "8"; break;
					case "4" : $e = "M"; $s = "12"; break;
					case "5" : $e = "Q"; $s = "4"; break;
					case "6" : $e = "H"; $s = "4"; break;
				}
				$form["qr_error_check"] = $e;
				$form["qr_default_size"] = $s;
			}
			
			foreach ($form as $name=>$value)
			{
				$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($value)."' WHERE name='".$db->escape($name)."'");
				$settings[$name] = $value;
			}
			$message = "Settings are updated";
			break;
		}
		
		/**
		 * Add a new campaign
		 */
		case "add_campaign" :
		{
			if (!verify_nonce($nonce, "qr_add_campaign"))
			{
				invalid_nonce_message();
				exit;
			}
			$campaign = isset($_POST["campaign"]) ? $_REQUEST["campaign"] : array();
			$campaign = is_array($campaign) ? $campaign : array();
			$offsite_commerce_campaigns->addCampaign($campaign, "qr");
			$message = "Campaign has been added";
			break;
		}
		
		/**
		 * Update campaigns
		 */
		case "update_campaigns" :
		{
			if (!verify_nonce($nonce, "qr_update_campaigns"))
			{
				invalid_nonce_message();
				exit;
			}
			$campaigns = isset($_POST["campaigns"]) ? $_REQUEST["campaigns"] : array();
			$campaigns = is_array($campaigns) ? $campaigns : array();
			$offsite_commerce_campaigns->updateCampaigns($campaigns);
			$message = "Campaigns have been update";
			break;
		}
		
		/**
		 * Delete campaign
		 */
		case "delete_campaign" :
		{
			$campaign_id = isset($_REQUEST["campaign_id"]) ? $_REQUEST["campaign_id"] : 0;
			$offsite_commerce_campaigns->deleteCampaign($campaign_id);
			echo ""; die();
			break;
		}
	}

	pageHeader("QR Codes Settings", "", "icb-raster-gear");
	
	pageNote(array(
		array("class"=>"ic-info", "text"=>"<b>Enable / disable QR Codes, set default settings, etc</b>", "url"=>"", "visible"=>true),
		array("class"=>"ic-info", "text"=>isset($message) ? $message : "", "url"=>"", "visible"=>isset($message))
	));
	
	loadJS("qr-settings.js");
	loadJS("qr-marketing.js");
	
	$campaigns = $offsite_commerce_campaigns->getCampaigns("qr");
?>
<div class="admin-tabs admin-tabs-framed" id="tabs-qr">
<?php
	$last_active_tab = isset($_REQUEST["last_active_tab"]) ? $_REQUEST["last_active_tab"] : "main";
	pageTabs(array(
		array("title"=>"Enable / Disable", "id"=>"main"),
		array("title"=>"Campaigns", "id"=>"campaigns"),
		array("title"=>"Settings", "id"=>"settings"),
		array("title"=>"Create Custom QR Code", "id"=>"custom"),
	), "tabs-qr", $last_active_tab);
?>

	<div id="admin-tab-sheet-main" class="admin-tab-sheet">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>What are QR Codes?</div>
			<table cellpadding="0" cellspacing="0" border="0" width="100%">
				<tr valign="top">
					<td>
						<div style="padding-left:30px;padding-bottom:10px;">
							With <?php echo $label_app_name;?>, QR codes are used to engage potential customers with your store 
							from mobile devices such as smart phones or tablet computers. Additionally, they can be used to instantly 
							direct customers to a specific item or make a purchase by simply taking a picture (Sometimes called 
							scanning) of the code. QR codes work with just about every type of smart phone, and are being used by many 
							top retailers worldwide.
							<div style="padding:10px 0px;"><img src="images/admin/qr-demo.png"/></div>
							By default, our QR code generator will create two separate codes for every item in your cart. 
							The first code will take a visitor to a specific product, the second code will automatically add 
							that item to their cart. Once activated, you'll find these codes within the "QR Code" tab on the 
							"Edit Product" page of the cart.
							<br/>
							<br/>
							<iframe width="560" height="345" src="<?php echo (isSslMode() ? "https" : "http");?>://www.cartmanual.com/url/qrcode-promo-video" frameborder="0" allowfullscreen></iframe>
						</div>
					</td>
					<td style="min-width:400px;width:400px;" width="300">
						<div class="div-qr-activate">
							Activate QR codes and<br/>Off-Site Commerce for your store?<br/>
							<input id="button-qr-activate" class="<?php echo $settings["qr_active"] == "Yes" ? "disable" : "";?>" type="button" value="<?php echo $settings["qr_active"] == "Yes" ? "Disable" : "Activate";?>">
						</div>
						<ol>
							<li>Click the ACTIVATE button above</li>
							<li>Click on the "Campaign" tab</li>
							<li>Create a new QR Campaign</li>
							<li>Go to "edit product" to grab QR code specific product / campaign</li>
							<li>
								To view results from a campaign, click the To view results from a campaign, click the
								<a href="admin.php?p=reports&report_type=qr_campaigns">Statistics &amp; Reports / Reports By Type / QR Campaigns</a>
								in the main menu.
							</li>
						</ol>
					</td>
				</tr>
			</table>
		</div>
	</div>
		
	<div id="admin-tab-sheet-settings" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>QR Code Settings</div>
			<form name="form-qr-settings" id="form-qr-settings" action="admin.php?p=settings_qr&last_active_tab=settings" method="post">
				<input type="hidden" name="action" value="update_settings" />
				<input type="hidden" name="nonce" value="<?php echo create_nonce('qr_update_settings');?>" />
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Default image type</td>
						<td class="formItemControl">
							<select name="form[qr_image_type]" class="short">
								<option value="<?php echo IMAGETYPE_PNG;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_PNG ? "selected" : "";?>>PNG</option>
								<option value="<?php echo IMAGETYPE_JPG;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_JPG ? "selected" : "";?>>JPG</option>
								<option value="<?php echo IMAGETYPE_GIF;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_GIF ? "selected" : "";?>>GIF</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Default media</td>
						<td class="formItemControl">
							<select name="form[qr_media]" id="qr-settings-media">
								<option value="1" <?php echo $settings["qr_media"] == "1" ? "selected" : "";?>>Web - Small</option>
								<option value="2" <?php echo $settings["qr_media"] == "2" ? "selected" : "";?>>Web - Big</option>
								<option value="3" <?php echo $settings["qr_media"] == "3" ? "selected" : "";?>>Print Ad - Small</option>
								<option value="4" <?php echo $settings["qr_media"] == "4" ? "selected" : "";?>>Print Ad - Big</option>
								<option value="5" <?php echo $settings["qr_media"] == "5" ? "selected" : "";?>>Parcel Label</option>
								<option value="6" <?php echo $settings["qr_media"] == "6" ? "selected" : "";?>>Factory Environment</option>
								<option value="0" <?php echo $settings["qr_media"] == "0" ? "selected" : "";?>>Custom</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Default error correction level</td>
						<td class="formItemControl">
							<select name="form[qr_error_check]" class="short" id="qr-settings-error-check">
								<option value="L">Low</option>
								<option value="M" <?php echo $settings["qr_error_check"] == "M" ? "selected" : "";?>>Medium</option>
								<option value="Q" <?php echo $settings["qr_error_check"] == "Q" ? "selected" : "";?>>Good</option>
								<option value="H" <?php echo $settings["qr_error_check"] == "H" ? "selected" : "";?>>The best</option>
							</select>
							<div class="formItemComment">
								Low - approx. 7% of codewords can be restored.<br/>
								Medium - approx. 15% of codewords can be restored.<br/>
								Good - approx. 25% of codewords can be restored.<br/>
								The best - approx. 30% of codewords can be restored.
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Default pixels per QR Code dot</td>
						<td class="formItemControl">
							<select name="form[qr_default_size]" class="short" id="qr-settings-default-size">
<?php
								for($i=4; $i<=12; $i++)
								{
?>
								<option value="<?php echo $i;?>" <?php if($settings["qr_default_size"] == $i) echo "selected";?>><?php echo $i;?></option>
<?php
								}
?>
							</select>
							<div class="formHelp">QR Code dot size in pixels</div>
						</td>
					</tr>
				</table>
				<div class="admin-form-buttons">
					<input type="submit" value="Save changes"/>
					<input type="reset" value="Reset form"/>
				</div>
			</form>	
		</div>
	</div>
	
	<div id="admin-tab-sheet-campaigns" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>QR Code and Off-Site Commerce Campaigns</div>		
			<table cellpadding="0" cellspacing="0" border="0" width="100%">
				<tr valign="top">
					<td>
						<form name="form-qr-campaign-new"  action="admin.php?p=settings_qr&last_active_tab=campaigns" method="post">
							<input type="hidden" name="action" value="add_campaign" />
							<input type="hidden" name="nonce" value="<?php echo create_nonce('qr_add_campaign');?>" />
							<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table" <?php if ($campaigns) echo 'style="border-radius:5px 5px 0px 0px;border-bottom:none;"'?>>
								<thead>
									<tr>
										<th class="icon">&nbsp;</th>
										<th style="width:200px;min-width:200px;">Name</th>
										<th style="min-width:200px;">Description</th>
										<th style="width:200px;min-width:200px;">Date Added</th>
										<th style="width:100px;">&nbsp;</th>
									</tr>
								</thead>
								<tbody>
									<tr>
										<td>&nbsp;</td>
										<td colspan="4">
											<a href="#" id="link-qr-campaign-add">Add a new campaign</a>
										</td>
									</tr>
									<tr id="form-qr-campaign-new" <?php echo $campaigns ? 'class="hidden"' : '';?> style="background:none;">
										<td class="icon"><img src="images/icons/a/add.png"/></td>
										<td><input type="text" name="campaign[name]" style="width:180px;"/></td>
										<td><input type="text" style="width:95%;" name="campaign[description]"/></td>
										<td>&nbsp;</td>
										<td style="text-align:right;"><input type="submit" value="Add" style="width:80px;min-width:80px;padding:3px;"/></td>
									</tr>
								</tbody>
							</table>
						</form>
<?php
				if ($campaigns)
				{
?>
						<form name="form-qr-campaigns" id="form-qr-campaigns" action="admin.php?p=settings_qr&last_active_tab=campaigns" method="post">
							<input type="hidden" name="action" value="update_campaigns" />
							<input type="hidden" name="nonce" value="<?php echo create_nonce('qr_update_campaigns');?>" />
							<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table" style="border-radius:0px 0px 5px 5px;border-top:0px;">
								<tbody>
<?php
					foreach ($campaigns as $c)
					{
?>									<tr id="qr-campaign-<?php echo $c["campaign_id"];?>">
										<td class="icon">
											<input type="hidden" name="campaigns[<?php echo $c["campaign_id"];?>][campaign_id]" value="<?php echo $c["campaign_id"];?>"/>
											<img src="images/icons/a/document.png"/>
										</td>
										<td style="width:200px;">
											<input type="text" class="campaign-name" name="campaigns[<?php echo $c["campaign_id"];?>][name]" value="<?php echo htmlspecialchars($c["name"]);?>" style="width:180px;"/>
										</td>
										<td style="min-width:200px;">
											<input type="text" class="campaign-description" name="campaigns[<?php echo $c["campaign_id"];?>][description]" value="<?php echo htmlspecialchars($c["description"]);?>" style="width:95%;"/>
										</td>
										<td style="width:200px;min-width:200px;">
											<?php echo $c["created"];?>
										</td>
										<td style="width:100px;text-align:right;">
											<a href="#" class="link-qr-campaign-delete" id="link-delete-<?php echo $c["campaign_id"];?>"><img src="images/icons/a/delete.png" border="0" title="Delete Campaign"></a>
											<a href="admin.php?p=reports&report_type=qr_campaigns"><img src="images/icons/a/list.png" border="0" title="Campaign Statistics"></a>
										</td>
									</tr>
			<?php
					}
			?>

								</tbody>
							</table>
							<div class="admin-form-buttons">
								<input type="submit" value="Save changes"/>
								<input type="reset" value="Reset form"/>
							</div>
						</form>
<?php
				}
?>	
					</td>
					<td style="min-width:250px;width:250px;" width="250">
						<div class="div-qr-ideas">
							<strong>QR Campaign Ideas</strong>
							<ol>
								<li>Business cards</li>
								<li>Printed advertisements</li>
								<li>T-shirts</li>
								<li>Linked-in profile</li>
								<li>Various other print media</li>
							</ol>
						</div>
					</td>
				</tr>
			</table>
		</div>
	</div>

	<div id="admin-tab-sheet-custom" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>Generate Custom QR Code</div>		
			<table width="100%" cellpadding="0" cellspacing="0" border="0">
				<tr valign="top">
					<td class="formItemCaption">QR Code URL or Text</td>
					<td class="formItemControl">
						<textarea id="qr-generate-text" style="width:400px;height:65px;"><?php echo htmlspecialchars($settings["GlobalHttpUrl"]);?></textarea>
					</td>
				</tr>
				<tr valign="top">
					<td class="formItemCaption">Image type</td>
					<td class="formItemControl">
						<select class="short" id="qr-generate-image-type">
							<option value="<?php echo IMAGETYPE_PNG;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_PNG ? "selected" : "";?>>PNG</option>
							<option value="<?php echo IMAGETYPE_JPG;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_JPG ? "selected" : "";?>>JPG</option>
							<option value="<?php echo IMAGETYPE_GIF;?>" <?php echo $settings["qr_image_type"] == IMAGETYPE_GIF ? "selected" : "";?>>GIF</option>
						</select>
					</td>
				</tr>
				<tr valign="top">
					<td class="formItemCaption">Media</td>
					<td class="formItemControl">
						<select id="qr-generate-media">
							<option value="1" <?php echo $settings["qr_media"] == "1" ? "selected" : "";?>>Web - Small</option>
							<option value="2" <?php echo $settings["qr_media"] == "2" ? "selected" : "";?>>Web - Big</option>
							<option value="3" <?php echo $settings["qr_media"] == "3" ? "selected" : "";?>>Print Ad - Small</option>
							<option value="4" <?php echo $settings["qr_media"] == "4" ? "selected" : "";?>>Print Ad - Big</option>
							<option value="5" <?php echo $settings["qr_media"] == "5" ? "selected" : "";?>>Parcel Label</option>
							<option value="6" <?php echo $settings["qr_media"] == "6" ? "selected" : "";?>>Factory Environment</option>
							<option value="0" <?php echo $settings["qr_media"] == "0" ? "selected" : "";?>>Custom</option>
						</select>
					</td>
				</tr>
				<tr valign="top">
					<td class="formItemCaption">Error correction level</td>
					<td class="formItemControl">
						<select class="short" id="qr-generate-error-check">
							<option value="L">Low</option>
							<option value="M" <?php echo $settings["qr_error_check"] == "M" ? "selected" : "";?>>Medium</option>
							<option value="Q" <?php echo $settings["qr_error_check"] == "Q" ? "selected" : "";?>>Good</option>
							<option value="H" <?php echo $settings["qr_error_check"] == "H" ? "selected" : "";?>>The best</option>
						</select>
						<div class="formItemComment">
							Low - approx. 7% of codewords can be restored.<br/>
							Medium - approx. 15% of codewords can be restored.<br/>
							Good - approx. 25% of codewords can be restored.<br/>
							The best - approx. 30% of codewords can be restored.
						</div>
					</td>
				</tr>
				<tr valign="top">
					<td class="formItemCaption">Pixels per QR Code dot</td>
					<td class="formItemControl">
						<select class="short" id="qr-generate-default-size">
<?php
							for($i=4; $i<=12; $i++)
							{
?>
							<option value="<?php echo $i;?>" <?php if($settings["qr_default_size"] == $i) echo "selected";?>><?php echo $i;?></option>
<?php
							}
?>
						</select>
						<div class="formHelp">QR Code dot size in pixels</div>
					</td>
				</tr>
			</table>
			<div class="admin-form-buttons">
				<input type="button" value="Generate" id="qr-generate-button"/>
			</div>
		</div>
	</div>
</div>

<div id="dialog-qr-generated" style="display:none;">
	<div>
		<img id="dialog-qr-generated-image" src="images/icons/a/raster.png"/>
	</div>
</div>
