<?php
	if (!defined('ENV')) exit();

	if (!isset($adminView))
	{
		$adminView = Ddm_AdminView::getInstance();
	}

	$action = empty($action) ? '' : $action;

	$last_active_tab = isset($_GET['last_active_tab']) ? $_GET['last_active_tab'] : 'main';

	if (in_array($action, array('update', 'force-htaccess', 'force-url', 'force-unique')) && !DEMO_LOCK)
	{
		$nonce = isset($nonce) ? $nonce : '';
		$nonce_valid = verify_nonce($nonce, 'settings_search_'.$action);

		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
		
		/**
		 * Save robots.txt file
		 */
		if ($action == 'update')
		{
			if (!@file_put_contents('robots.txt', $robots))
			{
				Admin_Flash::setMessage('Please note: Error writing "robots.txt" file. Please check file permissions.', Admin_Flash::TYPE_ERROR);
			}
		}
		
		/**
		 * Update settings in DB
		 */
		if ($action == 'update' && !DEMO_LOCK)
		{
			foreach($form as $var=>$value)
			{
				$db->reset();
				$db->assignStr('value', $value);
				$db->update(DB_PREFIX.'settings', 'WHERE name="'.$db->escape($var).'"');
				$settings[$var] = $value;
			}
		}
		
		/**
		 * Create .htaccess file
		 */
		if (in_array($action, array('update', 'force-htaccess')) && !DEMO_LOCK)
		{
			try
			{
				$form['USE_MOD_REWRITE'] = isset($form['USE_MOD_REWRITE']) ? $form['USE_MOD_REWRITE'] : $settings['USE_MOD_REWRITE'];
				Seo::generateHtaccessFile(($form['USE_MOD_REWRITE'] == 'YES' ? 'On' : 'Off'));
			}
			catch (Exception $e)
			{
				Admin_Flash::setMessage('Please note: Error writing ".htaccess" file. Please check file permissions.', Admin_Flash::TYPE_ERROR);
			}
		}
		
		/**
		 * Generate flat URLs
		 */
		if (in_array($action, array('update', 'force-url', 'force-unique')) && !DEMO_LOCK)
		{
			//check are there changes in URL templates
			if (isset($_SESSION['seo-settings']))
			{
				//category template changed
				$updateCategories = (
					$_SESSION['seo-settings']['SearchURLCategoryTemplate'] != $settings['SearchURLCategoryTemplate'] ||
					$_SESSION['seo-settings']['SearchAutoGenerateLowercase'] != $settings['SearchAutoGenerateLowercase'] ||
					$_SESSION['seo-settings']['SearchURLJoiner'] != $settings['SearchURLJoiner'] || 
					$_SESSION['seo-settings']['USE_MOD_REWRITE'] != $settings['USE_MOD_REWRITE'] ||
					$action == 'force-url'
				);
			
				//product template changed
				$updateProducts = (
					$_SESSION['seo-settings']['SearchURLProductTemplate'] != $settings['SearchURLProductTemplate'] ||
					$_SESSION['seo-settings']['SearchAutoGenerateLowercase'] != $settings['SearchAutoGenerateLowercase'] ||
					$_SESSION['seo-settings']['SearchURLJoiner'] != $settings['SearchURLJoiner'] ||
					$_SESSION['seo-settings']['USE_MOD_REWRITE'] != $settings['USE_MOD_REWRITE'] ||
					$action == 'force-url'
				);
			
				//page template changed
				$updatePages = (
					$_SESSION['seo-settings']['SearchURLPageTemplate'] != $settings['SearchURLPageTemplate'] ||
					$_SESSION['seo-settings']['SearchAutoGenerateLowercase'] != $settings['SearchAutoGenerateLowercase'] ||
					$_SESSION['seo-settings']['SearchURLJoiner'] != $settings['SearchURLJoiner'] ||
					$_SESSION['seo-settings']['USE_MOD_REWRITE'] != $settings['USE_MOD_REWRITE'] ||
					$action == 'force-url'
				);
				
				//manufacturer template changed
				$updateManufacturers  = (
					$_SESSION['seo-settings']['SearchURLManufacturerTemplate'] != $settings['SearchURLManufacturerTemplate'] ||
					$_SESSION['seo-settings']['SearchAutoGenerateLowercase'] != $settings['SearchAutoGenerateLowercase'] ||
					$_SESSION['seo-settings']['SearchURLJoiner'] != $settings['SearchURLJoiner'] ||
					$_SESSION['seo-settings']['USE_MOD_REWRITE'] != $settings['USE_MOD_REWRITE'] ||
					$action == 'force-url'
				);
				
				updateSeoURLs($db, $settings, $updateCategories, $updateProducts, $updatePages, $updateManufacturers);
			}
		}
		
		/**
		 * Generate unique URL's
		 */
		if ($action == 'force-unique' && !DEMO_LOCK)
		{
			$unique_count = Seo::generateUniqueUrls();
			
			$note_message = ($unique_count > 0)
				? 'There was '.$unique_count.' duplicate URL(s) to process.'
				: 'There were '.$unique_count.' duplicate URLs to process.';

			$adminView->assign('forceUniqueMessage', $note_message);
		}

		Admin_Flash::setMessage('Search engine settings have been updated');
	}

	$f = @fopen('robots.txt', 'r');
	if ($f)
	{
		$robots = '';
		while(($s = fgets($f, 1024)) != false) $robots.=$s;
		fclose($f);
	}
	
	$_SESSION['seo-settings'] = array(
		'SearchURLCategoryTemplate' => $settings['SearchURLCategoryTemplate'],
		'SearchAutoGenerateLowercase' => $settings['SearchAutoGenerateLowercase'],
		'SearchURLProductTemplate' => $settings['SearchURLProductTemplate'],
		'SearchURLPageTemplate' => $settings['SearchURLPageTemplate'],
		'SearchURLManufacturerTemplate' => $settings['SearchURLManufacturerTemplate'],
		'SearchURLJoiner' => $settings['SearchURLJoiner'],
		'USE_MOD_REWRITE' => $settings['USE_MOD_REWRITE']
	);

	$settingsFormBuilder = new core_Form_FormBuilder('settings');
	$basicGroup = new core_Form_FormBuilder('basic', array('label' => 'Search Engine Optimization Settings', 'first' => true));
	$settingsFormBuilder->add($basicGroup);

	$yesnoOptions = array('YES' => 'Yes', 'NO' => 'No');

	$basicGroup->add('form[SearchTitle]', 'text', array('label' => 'Page (Site) Title', 'value' => $settings['SearchTitle'], 'attr' => array('class' => 'long'), 'note' => 'seo.notes.metatitle'));

	if (AccessManager::checkAccess('FLAT_URL'))
	{
		$basicGroup->add('form[USE_MOD_REWRITE]', 'choice', array('label' => 'Enable flat URLs generation?', 'value' => $settings['USE_MOD_REWRITE'], 'options' => $yesnoOptions, 'note' => 'Please note: this feature requires MOD_REWRITE Apache Web server Module and does not work on all servers.'));
	}
	else
	{
		$basicGroup->add('modrewrite', 'static', array('label' => 'Enable flat URLs generation?', 'value' => '<strong>This feature is not available under your license agreement. Please contact your provider to upgrade your application.</strong>'));
	}

	$basicGroup->add('form[SearchAutoGenerateLowercase]', 'choice', array('label' => 'Lowercase generated URLs?', 'value' => $settings['SearchAutoGenerateLowercase'], 'options' => $yesnoOptions));

	$urlJoinerOptions = array(
		'-' => '- (dash)',
		'_' => '_ (underscore)',
		'' => 'none',
	);
	$basicGroup->add('form[SearchURLJoiner]', 'choice', array('label' => 'URL words joiner', 'value' => $settings['SearchURLJoiner'], 'options' => $urlJoinerOptions));
	$basicGroup->add('form[SearchURLCategoryTemplate]', 'text', array('label' => 'Category page URL template', 'value' => $settings['SearchURLCategoryTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template Variables:<br/>%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath% - at least one variable required<br/>'));
	$basicGroup->add('form[SearchURLCategoryPaginatorTemplate]', 'text', array('label' => 'Category page pagination', 'value' => $settings['SearchURLCategoryPaginatorTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template Variables:<br/>%PageNumber% - required<br/>Will be added before extension (if exists) or closing slash symbol. Resulting URL will look like:<br/>[category-url][page-number].html<br/>[category-url][page-number]<br/>[category-url][page-number]/<br/>'));
	$basicGroup->add('form[SearchURLProductTemplate]', 'text', array('label' => 'Product page URL template', 'value' => $settings['SearchURLProductTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template variables:<br/>%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath%, %CategoryURL%<br/>%ProductId%, %ProductName%, %ProductDbId% - at least one of product variables is required<br/>'));
	$basicGroup->add('form[SearchURLManufacturerTemplate]', 'text', array('label' => 'Manufacturer page URL template', 'value' => $settings['SearchURLManufacturerTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template Variables:<br/>%ManufacturerKey%, %ManufaturerName%, %ManufacturerDbId% - at least one variable required<br/>'));
	$basicGroup->add('form[SearchURLManufacturerPaginatorTemplate]', 'text', array('label' => 'Manufacturer page pagination', 'value' => $settings['SearchURLManufacturerPaginatorTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template Variables:<br/>%PageNumber% - required<br/>Will be added before extension (if exists) or closing slash symbol. Resulting URL will look like:<br/>[manufacturer-url][page-number].html<br/>[manufacturer-url][page-number]<br/>[manufacturer-url][page-number]/<br/>'));
	$basicGroup->add('form[SearchURLPageTemplate]', 'text', array('label' => 'Text page URL template', 'value' => $settings['SearchURLPageTemplate'], 'attr' => array('class' => 'long'), 'note' => 'Template variables:<br/>%PageName%, %PageTitle%'));
	$basicGroup->add('form[SearchAutoGenerate]', 'choice', array('label' => 'Allow keyword auto-generation?', 'value' => $settings['SearchAutoGenerate'], 'options' => $yesnoOptions));
	$basicGroup->add('form[SearchHtaccessOverrides]', 'textarea', array('label' => '.htaccess Overrides', 'value' => $settings['SearchHtaccessOverrides'], 'note' => '<span style="color:red">Note: Use extreme caution when customizing the .htaccess file. A misconfiguration may cause your site to be inaccessible.</span><br/>These changes will be added to the .htaccess file prior to the default RewriteRules.'));
	$basicGroup->add('form[SearchMetaKeywords]', 'textarea', array('label' => 'Meta Keywords', 'value' => $settings['SearchMetaKeywords'], 'note' => 'Recommended length is 200-1000 characters. Separate keywords and phrases by using a comma.'));
	$basicGroup->add('form[SearchMetaDescription]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['SearchMetaDescription'], 'note' => 'seo.notes.metadescription'));
	$basicGroup->add('robots', 'textarea', array('label' => 'File "robots.txt"', 'value' => $robots, 'note' => 'This file creates a robots.txt file that will be located in your main directory. This file makes it easier for the search engines to index your website.'));

	$sitePagesFormBuilder = new core_Form_FormBuilder('sitepages');

	//testimonials
	//search
	//login
	//register
	//sitemap
	//404
	//cart
	//subscribe
	//unsubscribe
	//password_reset
	//international
	//gift_certificate
	//shipping_quote

	$searchPropertiesGroup = new core_Form_FormBuilder('catsearch', array('label' => 'Category Search Page', 'class' => 'ic-document', 'first'=>true));
	$sitePagesFormBuilder->add($searchPropertiesGroup);

	$searchPropertiesGroup->add('form[search_page_title]', 'text', array('label' => 'Page Title', 'value' => $settings['search_page_title'], 'note' => 'seo.notes.metatitle'));
	$searchPropertiesGroup->add('form[search_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['search_meta_description'], 'note' => 'seo.notes.metadescription'));
	$searchPropertiesGroup->add('form[search_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['search_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	if ($settings['testimonials_Enabled'] == 'Yes')
	{
		$testimonialsPropertiesGroup = new core_Form_FormBuilder('testimonials', array('label' => 'Testimonials Page', 'class' => 'ic-document'));
		$sitePagesFormBuilder->add($testimonialsPropertiesGroup);

		$testimonialsPropertiesGroup->add('form[testimonials_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['testimonials_page_title'], 'note' => 'seo.notes.metatitle'));
		$testimonialsPropertiesGroup->add('form[testimonials_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['testimonials_meta_description'], 'note' => 'seo.notes.metadescription'));
		$testimonialsPropertiesGroup->add('form[testimonials_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['testimonials_meta_keywords'], 'note' => 'seo.notes.metakeywords'));
	}

	$loginPropertiesGroup = new core_Form_FormBuilder('login', array('label' => 'Login Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($loginPropertiesGroup);

	$loginPropertiesGroup->add('form[login_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['login_page_title'], 'note' => 'seo.notes.metatitle'));
	$loginPropertiesGroup->add('form[login_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['login_meta_description'], 'note' => 'seo.notes.metadescription'));
	$loginPropertiesGroup->add('form[login_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['login_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$registerPropertiesGroup = new core_Form_FormBuilder('register', array('label' => 'Register Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($registerPropertiesGroup);

	$registerPropertiesGroup->add('form[register_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['register_page_title'], 'note' => 'seo.notes.metatitle'));
	$registerPropertiesGroup->add('form[register_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['register_meta_description'], 'note' => 'seo.notes.metadescription'));
	$registerPropertiesGroup->add('form[register_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['register_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$sitemapPropertiesGroup = new core_Form_FormBuilder('sitemap', array('label' => 'Site-map Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($sitemapPropertiesGroup);

	$sitemapPropertiesGroup->add('form[sitemap_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['sitemap_page_title'], 'note' => 'seo.notes.metatitle'));
	$sitemapPropertiesGroup->add('form[sitemap_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['sitemap_meta_description'], 'note' => 'seo.notes.metadescription'));
	$sitemapPropertiesGroup->add('form[sitemap_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['sitemap_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$notFoundPropertiesGroup = new core_Form_FormBuilder('notfound', array('label' => '404 Not Found Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($notFoundPropertiesGroup);

	$notFoundPropertiesGroup->add('form[notfound_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['notfound_page_title'], 'note' => 'seo.notes.metatitle'));
	$notFoundPropertiesGroup->add('form[notfound_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['notfound_meta_description'], 'note' => 'seo.notes.metadescription'));
	$notFoundPropertiesGroup->add('form[notfound_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['notfound_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$cartPropertiesGroup = new core_Form_FormBuilder('cart', array('label' => 'Cart Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($cartPropertiesGroup);

	$cartPropertiesGroup->add('form[cart_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['cart_page_title'], 'note' => 'seo.notes.metatitle'));
	$cartPropertiesGroup->add('form[cart_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['cart_meta_description'], 'note' => 'seo.notes.metadescription'));
	$cartPropertiesGroup->add('form[cart_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['cart_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$subscribePropertiesGroup = new core_Form_FormBuilder('subscribe', array('label' => 'Subscribe Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($subscribePropertiesGroup);

	$subscribePropertiesGroup->add('form[subscribe_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['subscribe_page_title'], 'note' => 'seo.notes.metatitle'));
	$subscribePropertiesGroup->add('form[subscribe_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['subscribe_meta_description'], 'note' => 'seo.notes.metadescription'));
	$subscribePropertiesGroup->add('form[subscribe_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['subscribe_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$unsubscribePropertiesGroup = new core_Form_FormBuilder('unsubscribe', array('label' => 'Unsubscribe Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($unsubscribePropertiesGroup);

	$unsubscribePropertiesGroup->add('form[unsubscribe_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['unsubscribe_page_title'], 'note' => 'seo.notes.metatitle'));
	$unsubscribePropertiesGroup->add('form[unsubscribe_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['unsubscribe_meta_description'], 'note' => 'seo.notes.metadescription'));
	$unsubscribePropertiesGroup->add('form[unsubscribe_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['unsubscribe_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$passwordresetPropertiesGroup = new core_Form_FormBuilder('reset', array('label' => 'Password Reset Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($passwordresetPropertiesGroup);

	$passwordresetPropertiesGroup->add('form[reset_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['reset_page_title'], 'note' => 'seo.notes.metatitle'));
	$passwordresetPropertiesGroup->add('form[reset_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['reset_meta_description'], 'note' => 'seo.notes.metadescription'));
	$passwordresetPropertiesGroup->add('form[reset_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['reset_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	$internationalPropertiesGroup = new core_Form_FormBuilder('international', array('label' => 'International Page', 'class' => 'ic-document'));
	$sitePagesFormBuilder->add($internationalPropertiesGroup);

	$internationalPropertiesGroup->add('form[international_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['international_page_title'], 'note' => 'seo.notes.metatitle'));
	$internationalPropertiesGroup->add('form[international_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['international_meta_description'], 'note' => 'seo.notes.metadescription'));
	$internationalPropertiesGroup->add('form[international_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['international_meta_keywords'], 'note' => 'seo.notes.metakeywords'));

	if ($settings['enable_gift_cert'] == 'Yes')
	{
		$passwordresetPropertiesGroup = new core_Form_FormBuilder('giftcert', array('label' => 'Gift Certificate Page', 'class' => 'ic-document'));
		$sitePagesFormBuilder->add($passwordresetPropertiesGroup);

		$passwordresetPropertiesGroup->add('form[giftcert_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['giftcert_page_title'], 'note' => 'seo.notes.metatitle'));
		$passwordresetPropertiesGroup->add('form[giftcert_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['giftcert_meta_description'], 'note' => 'seo.notes.metadescription'));
		$passwordresetPropertiesGroup->add('form[giftcert_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['giftcert_meta_keywords'], 'note' => 'seo.notes.metakeywords'));
	}

	if ($settings['ShippingAllowGetQuote'] == 'YES')
	{
		$shippingquotePropertiesGroup = new core_Form_FormBuilder('shipquote', array('label' => 'Shipping Quote Page', 'class' => 'ic-document'));
		$sitePagesFormBuilder->add($shippingquotePropertiesGroup);

		$shippingquotePropertiesGroup->add('form[shippingquote_page_title]', 'text', array('label' => 'Page Title', 'attr' => array('class' => 'long'), 'value' => $settings['shippingquote_page_title'], 'note' => 'seo.notes.metatitle'));
		$shippingquotePropertiesGroup->add('form[shippingquote_meta_description]', 'textarea', array('label' => 'Meta Description', 'value' => $settings['shippingquote_meta_description'], 'note' => 'seo.notes.metadescription'));
		$shippingquotePropertiesGroup->add('form[shippingquote_meta_keywords]', 'text', array('label' => 'Meta Keywords', 'attr' => array('class' => 'long'), 'value' => $settings['shippingquote_meta_keywords'], 'note' => 'seo.notes.metakeywords'));
	}

	$adminView->assign('settingsForm', $settingsFormBuilder->getForm());
	$adminView->assign('sitePagesForm', $sitePagesFormBuilder->getForm());

	$adminView->assign('htaccessNonce', create_nonce('settings_search_force-htaccess'));
	$adminView->assign('forceurlNonce', create_nonce('settings_search_force-url'));
	$adminView->assign('regenerateNonce', create_nonce('settings_search_force-unique'));

	$adminView->assign('nonce', create_nonce('settings_search_update'));

	$tabs = new Admin_Tabs('tabs-search', $last_active_tab, array(
		array('title' => 'Search Engine Settings', 'id' => 'main'),
		array('title' => 'Site Pages SEO', 'id' => 'site')
	));

	$adminView->assign('tabs', $tabs);

	echo $adminView->render('pages/settings/search');
