<?php

	if (!defined('ENV')) exit();

	$mode = isset($_REQUEST['mode']) ? trim($_REQUEST['mode']) : 'about';
	$action = isset($_REQUEST['action']) ? trim($_REQUEST['action']) : '';
	$last_active_tab = isset($_REQUEST['last_active_tab']) ? trim($_REQUEST['last_active_tab']) : $mode;

	$stamps = new Stampscom($db, $settings);

	$postage_purchase_status = '';
	$postage_purchase_transactionid = 0;

	$stamps_scanform_ids = isset($stamps_scanform_ids) ? $stamps_scanform_ids : array();
	$stamps_scanform_iderrs = array();
	$stamps_scanform_is_valid = 0;
	$stamps_scanform_urls = '';

	$stamps_account_is_valid = false;
	$stamps_account_capability_purchase_postage = false;
	$stamps_account_may_create_scanform = false;
	$stamps_account_may_carrier_pickup = false;

	$stamps_account_postage_balance = null;
	$stamps_account_postage_control_total = null;

	$stamps_required_settings = array(
		'ShippingOriginName', 'ShippingOriginAddressLine1', 'ShippingOriginCity', 'ShippingOriginState',
		'ShippingOriginZip', 'ShippingOriginCountry', 'ShippingOriginPhone'
	);

	$stamps_required_settings_error = false;

	/**
 	 * Check for required settings
 	 */
	foreach ($stamps_required_settings as $settings_name)
	{
		if (!isset($settings[$settings_name]) || trim($settings[$settings_name]) == '')
		{
			$stamps_required_settings_error = true;
		}
	}

	if ($stamps_required_settings_error)
	{
		Admin_Flash::setMessage(
			'To create a label through Stamps.com, please make sure that you have filled in all required fields '.
            'under <a href="admin.php?p=shipping&last_active_tab=settings">Shipping Origin Settings</a> (real-time) on the Manage Shipping page. '.
            'Please note: Stamps.com may not work with PO Box addresses',
			Admin_Flash::TYPE_ERROR
		);
	}

	/**
	 * Save settings
	 */
	if ($action == 'manage_settings')
	{
		$stamps_settings_new = isset($_REQUEST["stamps_settings_new"]) ? is_array($_REQUEST["stamps_settings_new"]) ? $_REQUEST["stamps_settings_new"] : array() : array();
		$stamps->setSettings($stamps_settings_new);
		$stamps_validation_successful = false;

		if ($stamps_settings_new['StampscomActive'] === 'Yes')
		{
			// validate username and password
			if (strlen($stamps_settings_new['StampscomUsername']) && strlen($stamps_settings_new['StampscomPassword']))
			{
				// validating settings
				$stamps_auth_result = $stamps->apiCall('AuthenticateUser');

				if ($stamps_auth_result && !isset($stamps_auth_result['faultcode']) && isset($stamps_auth_result['LoginBannerText']))
				{
					Admin_Flash::setMessage('Stamps.com: User authentication test succeeded');
					$stamps_validation_successful = true;
				}
				else
				{
					$stampsMsg = 'Stamps.com: User authentication test failed';

					if (isset($stamps_auth_result['faultstring']))
					{
						$stampsMsg .= ': '.$stamps_auth_result['faultstring'];
					}
					Admin_Flash::setMessage($stampsMsg, Admin_Flash::TYPE_ERROR);
				}
			}
			else
			{
				Admin_Flash::setMessage('Stamps.com: Please make sure to fill in all information when enabling Stamps.com.', Admin_Flash::TYPE_ERROR);
			}
		}

		// forcing to turn stamps off.
		if (!$stamps_validation_successful)
		{
			$stamps_settings_new['StampscomActive'] = 'No';
		}

		$settings = array_merge($settings, $stamps_settings_new);
		$stamps->saveSettings();
	}

	/**
	 * Validate account / settings
	 */
	if ($settings['StampscomActive'] === 'Yes')
	{
		$stamps_return = $stamps->apiMethodGetAccountInfo();
		$stamps_account_may_carrier_pickup = true;

		if (is_array($stamps_return) && isset($stamps_return['AccountInfo']))
		{
			$stamps_account_is_valid = true;
			$stamps_account_postage_balance = $stamps_return['AccountInfo']['PostageBalance']['AvailablePostage'];
			$stamps_account_postage_control_total = $stamps_return['AccountInfo']['PostageBalance']['ControlTotal'];

			if ($stamps_return['AccountInfo']['Capabilities']['CanPurchasePostage'] == 'true')
			{
				$stamps_account_capability_purchase_postage = true;
			}

			if ($stamps_return['AccountInfo']['Capabilities']['CanCreateSCANForm'] == 'true')
			{
				$stamps_account_may_create_scanform = true;
			}
		}
		else
		{
			Admin_Flash::setMessage('Stamps.com: Couldn\'t connect to the Stamps.com API. Please check your account settings and mode', Admin_Flash::TYPE_ERROR);
		}
	}

	/**
	 * Purchase the given amount of postage
	 */
	if ($action == 'postage_purchase' && $stamps_account_is_valid)
	{
		$postage_purchase_status = '';
		$postage_purchase_transactionid = 0;

		if (isset($stamps_purchase) && isset($stamps_purchase['postage_amount']) && isset($stamps_purchase['postage_control_total']))
		{
			$purchase_amount = trim($stamps_purchase['postage_amount']);
			$purchase_control_total = $stamps_purchase['postage_control_total'];

			if (!preg_match('/^\d+(\.\d+)?$/', $purchase_amount))
			{
				$purchase_amount = 0;
			}
			else
			{
				$purchase_amount = floatval($purchase_amount);
			}

			if ($purchase_amount <= 0)
			{
				if (trim($stamps_purchase['postage_amount']) === '')
				{
					Admin_Flash::setMessage('Stamps.com: No amount was entered. Please enter a valid amount', Admin_Flash::TYPE_ERROR);
				}
				else
				{
					Admin_Flash::setMessage('Stamps.com: The amount \''.gs($stamps_purchase['postage_amount']).'\' was not understood. Please enter a valid amount', Admin_Flash::TYPE_ERROR);
				}
			}
			else
			{
				$stamps_arguments = array( 'PurchaseAmount'=>$purchase_amount, 'ControlTotal'=>$purchase_control_total );
				$stamps_return = $stamps->apiMethodPurchasePostage($stamps_arguments);

				if (!$stamps_return)
				{
					Admin_Flash::setMessage('There was an error connecting to the Stamps.com API', Admin_Flash::TYPE_ERROR);
				}
				elseif (isset($stamps_return['faultcode']))
				{
					$stampsMsg = 'Stamps.com: There was an error connecting to the Stamps.com API';

					if ($stamps_return['faultcode'] === 'soap:Server' && isset($stamps_return['detail']) && isset($stamps_return['detail']['sdcerror']) && isset($stamps_return['detail']['sdcerror']['!code']) && intval($stamps_return['detail']['sdcerror']['!code']) === 450116)
					{
						$stampsMsg = 'Stamps.com: Your current postage balance does not match that of Stamps.com. This can be caused by something that changes your postage balance, such as another postage purchase or generating a label. Please reload this page to update the local balance and try your purchase again';
					}
					elseif (isset($stamps_return['faultstring']))
					{
						$stampsMsg .= ': '.$stamps_return['faultstring'];
					}

					Admin_Flash::setMessage($stampsMsg, Admin_Flash::TYPE_ERROR);
				}
				else
				{
					if (isset($stamps_return['TransactionID']))
					{
						$postage_purchase_transactionid = $stamps_return['TransactionID'];
					}

					if (isset($stamps_return['PurchaseStatus']) && $stamps_return['PurchaseStatus'] !== 'Rejected')
					{
						$postage_purchase_status = $stamps_return['PurchaseStatus'];
					}

					if (!isset($stamps_return['PurchaseStatus']) || $stamps_return['PurchaseStatus'] === 'Rejected')
					{
						$stampsMsg = 'Stamps.com: There was an error purchasing the requested postage';
						if (isset($stamps_return['RejectionReason']))
						{
							$stampsMsg .= ': '.gs($stamps_return['RejectionReason']);
						}

						Admin_Flash::setMessage($stampsMsg, Admin_Flash::TYPE_ERROR);
					}
				}
			}

			$_SESSION['StampscomPurcasePostageTransactionID'] = 0;
		}

		if ($settings['StampscomActive'] !== 'Yes')
		{
			Admin_Flash::setMessage('Stamps.com is disabled', Admin_Flash::TYPE_INFO);
		}
		else
		{
			$stamps_return = $stamps->apiMethodGetAccountInfo();

			if (!$stamps_account_is_valid)
			{
				Admin_Flash::setMessage('Stamps.com: Couldn\'t connect to the Stamps.com API. Please check your account settings and mode', Admin_Flash::TYPE_ERROR);
			}
			else
			{
				if ($postage_purchase_transactionid == 0 && isset($_SESSION['StampscomPurcasePostageTransactionID']))
				{
					$postage_purchase_transactionid = $_SESSION['StampscomPurcasePostageTransactionID'];
				}

				if ($postage_purchase_transactionid > 0)
				{
					if ($postage_purchase_status == '')
					{
						$stamps_arguments = array('TransactionID'=>$postage_purchase_transactionid);
						$stamps_return = $stamps->apiMethodGetPurchaseStatus($stamps_arguments);

						if ($stamps_return && isset($stamps_return['PurchaseStatus']))
						{
							$postage_purchase_status = $stamps_return['PurchaseStatus'];
						}
					}
				}

				if ($postage_purchase_status != '' && $postage_purchase_status === 'Success')
				{
					$postage_purchase_transactionid = 0;
				}

				$_SESSION['StampscomPurcasePostageTransactionID'] = $postage_purchase_transactionid;
			}
		}
	}

	/**
	 * Generate the SCAN Form
	 */
	if ($action == 'scanform_generate')
	{
		$request = Framework_Request::createFromGlobals();

		$scanShipmentDate = $request->request->get('shipment_date', date_parse(date('Y-m-d h:i:s')));
		$scanShipmentDateYear = intval($scanShipmentDate['year']);
		$scanShipmentDateMonth = intval($scanShipmentDate['month']);
		$scanShipmentDateDay = intval($scanShipmentDate['day']);

		$scanShipmentDate =
			$scanShipmentDateYear.'-'.
			($scanShipmentDateMonth < 10 ? '0' : '').$scanShipmentDateMonth.'-'.
			($scanShipmentDateDay < 10 ? '0' : '').$scanShipmentDateDay;

		$stamps_tmp_arguments_state = '';
		$db->query('SELECT short_name FROM '.DB_PREFIX.'states WHERE coid='.intval($settings['ShippingOriginCountry']).' AND stid='.intval($settings['ShippingOriginState']));

		if ($db->moveNext())
		{
			$stamps_tmp_arguments_state = $db->col['short_name'];
		}
		else
		{
			$stamps_tmp_arguments_state = $settings['ShippingOriginProvince'];
		}

		$stamps_tmp_arguments['ShipDate'] = $scanShipmentDate;

		$stamps_tmp_arguments['FromAddress'] = array(
			'name_' => $settings['ShippingOriginName'],
			'address_1_' => $settings['ShippingOriginAddressLine1'],
			'address_2_' => $settings['ShippingOriginAddressLine2'],
			'city_' => $settings['ShippingOriginCity'],
			'state_' => $stamps_tmp_arguments_state,
			'zip_code_' => $settings['ShippingOriginZip'],
		);
		$stamps_tmp_arguments['ImageType'] = 'Png';
		$stamps_tmp_arguments['PrintInstructions'] = 'true';

		$stamps_return = $stamps->apiCall('CreateScanForm', $stamps_tmp_arguments);

		if (isset($stamps_return['faultcode']))
		{
			$stamps_msg_scanform = 'Error generating SCAN form';

			if (isset($stamps_return['faultstring']))
			{
				$stamps_msg_scanform .= ": ".$stamps_return['faultstring'];
			}

			if (isset($stamps_return['detail']) && is_array($stamps_return['detail']) && isset($stamps_return['detail']['sdcerror']) && is_array($stamps_return['detail']['sdcerror']) && isset($stamps_return['detail']['sdcerror'][0]))
			{
				foreach ($stamps_return['detail']['sdcerror'] as $v)
				{
					if (isset($v['!context']) && preg_match('/^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/',$v['!context']))
					{
						foreach ($stamps_scanform_ids as $tmp_i=>$tmp_id)
						{
							if (trim($tmp_id) == $v['!context'])
							{
								$stamps_scanform_iderrs[$tmp_i] = isset($v['!']) ? $v['!'] : '';
							}
						}
					}
				}
			}

			Admin_Flash::setMessage($stamps_msg_scanform, Admin_Flash::TYPE_ERROR);
		}
		elseif (isset($stamps_return['Url']))
		{
			/** @var DataAccess_StampsRepository $stampsRepository */
			$stampsRepository = DataAccess_Repository::get('StampsRepository');

			$stamps_scanform_urls = explode(' ', $stamps_return['Url']);

			$stampsRepository->saveScanForm(array(
				'date' => $scanShipmentDate,
				'code' => $stamps_return['ScanFormId'],
				'urls' => $stamps_scanform_urls
			));
?>
<div id="dialog_stamps_scan_result" style="display:none;">
	<div id="dialog_stamps_scan_result_preview" style="overflow:auto;">
<?php
			foreach ($stamps_scanform_urls as $stamps_scanform_url)
			{
				if (trim($stamps_scanform_url) != '')
				{
					echo '<img src="'.$stamps_scanform_url.'" style="display:none;">';
				}
			}
?>
	</div>
</div>
<?php
		}
	}

	/**
	 * Search the SCAN Form
	 */
	if ($action == 'scanform_search')
	{
		$request = Framework_Request::createFromGlobals();

		$scanSearchDate = $request->request->get('search_date', date_parse(date('Y-m-d h:i:s')));
		$scanSearchDateYear = intval($scanSearchDate['year']);
		$scanSearchDateMonth = intval($scanSearchDate['month']);
		$scanSearchDateDay = intval($scanSearchDate['day']);

		$scanSearchDate =
			$scanSearchDateYear.'-'.
			($scanSearchDateMonth < 10 ? '0' : '').$scanSearchDateMonth.'-'.
			($scanSearchDateDay < 10 ? '0' : '').$scanSearchDateDay;

		/** @var DataAccess_StampsRepository $stampsRepository */
		$stampsRepository = DataAccess_Repository::get('StampsRepository');

		$scanFormsSearchResult = $stampsRepository->getScanFormsByDate($scanSearchDate);

		if (!$scanFormsSearchResult)
		{
			Admin_Flash::setMessage('stamps.settings.scan.no_forms_for_date', Admin_Flash::TYPE_INFO);
		}
	}

	/**
	 * Now display the actual page
	 */
	pageHeader("Stamps.com", '', 'icb-truck');
?>
&nbsp;
<div class="admin-tabs admin-tabs-framed" id="tabs-stampscom">
<?php
	loadJS('stampscom_label.js');
	pageTabs(array(
		array('title' => 'About Stamps.com', 'id' => 'about'),
		array('title' => 'Manage Stamps.com', 'id' => 'manage'),
		array('title' => 'Purchase Postage', 'id' => 'postage', 'visible' => ($settings['StampscomActive'] == 'Yes')),
		array(
			'title' => 'SCAN Forms', 'id' => 'scanform',
			'visible' => ($settings['StampscomActive'] == 'Yes' && $settings['StampscomSCANFormEnable'] == 'Yes' && $stamps_account_may_create_scanform),
		),
		array(
			'title' => 'Carrier Pickup', 'id' => 'pickup',
			'visible' => ($settings['StampscomActive'] == 'Yes' && $stamps_account_may_carrier_pickup)
		),
	), 'tabs-stampscom', $last_active_tab);
?>

	<div id="admin-tab-sheet-about" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon ic-info"></span>About Stamps.com</div>
			<table cellpadding="0" cellspacing="0" border="0" width="100%">
				<tr valign="top">
					<td>
						<div style="padding:10px 30px;">
							<p style="margin-top:0px;"><strong>Stamps.com - the smartest way to ship with the USPS</strong></p>
							<p>Stamps.com has been the leading provider of USPS shipping services since 1998.</p>

							<p>With Stamps.com, you can perform USPS shipping functions right from your Pinnacle Cart eCommerce. No costly changes to your existing infrastructure. No disruptions to your current workflow.</p>

							<p>Whether you ship one package a day or 1000, from your garage or from a giant warehouse, Stamps.com has the tools to make your USPS shipping process faster, more efficient and more reliable than ever before.</p>
							<div>
								<strong>Highlights:</strong>
								<ul style="margin-top:5px;">
									<li>Over 2.5 billion letters and packages sent using Stamps.com</li>
									<li>Over 450,000 customers</li>
									<li>50,000 professional shipping customers</li>
									<li>Award-winning shipping support with two gold medals to prove it</li>
									<li>99.9% server uptime</li>
									<li>Official PC Postage Provider of Amazon.com Marketplace</li>
									<li>Official USPS postage provider for QuickBooks Shipping Manager</li>
									<li>Official technology provider of USPS Click-N-Ship&reg;</li>
								</ul>
							</div>
							<div>
								<strong>Get the most powerful USPS features with Stamps.com</strong>
								<ul style="margin-top:5px;">
									<li>Flexible, competitive pricing</li>
									<li>Discounted rates
										<ol>
											<li>Up to 62% off Express Mail, Priority Mail, and First Class Package Service</li>
											<li>Up to 8% off international rates for Express Mail and Priority Mail</li>
										</ol>
									</li>
									<li>Discounted Stamps.com insurance
										<ol>
											<li>Up to 40% discount on package insurance</li>
											<li>Insurance coverage for values up to $10,000</li>
											<li>Claims processed electronically and paid within 7-10 business days</li>
											<li>No Post office drop-off</li>
											<li>No special forms needed</li>
										</ol>
									</li>
								</ul>
							</div>

							<p><a href="http://www.stamps.com/<?php if (IS_LABEL) { ?>welcome<?php } else { ?>pinnaclecart<?php } ?>" target="_stampscom">Sign up</a> for a Stamps.com account today and gain access to these industry proven USPS features.</p>
						</div>
					</td>
					<td style="min-width:400px;width:400px;" width="300">
						<div class="div-stampscom-activate" style="background-color:#ffffff;border:solid #efeded 5px;border-radius:10px 10px 10px 10px;font-size:14px;margin-left:10px;padding:20px;text-align:center;">
							<p><img src="images/admin/stampscom-logo.png" /></p>
<?php
	if ($settings['StampscomActive'] !== 'Yes')
	{
?>
							<p style="margin:0 auto;width:250px;">
								<a href="http://www.stamps.com/<?php if (IS_LABEL) { ?>welcome<?php } else { ?>pinnaclecart<?php } ?>" target="_stampscom" style="margin-top:10px;display:block;width:250px;border-radius:7px;padding:10px;border:2px solid orange;text-align:center;color:#3399ff;background-color:#f7f7f7;font-weight:bold;">Sign Up for a Stamps.com Account</a>
							</p>
<?php
	}
	else
	{
?>
							<p style="margin:0 auto;width:220px;">
								<span style="margin-top:10px;display:block;width:200px;border-radius:7px;padding:10px;border:2px solid orange;text-align:center;color:#3399ff;background-color:#f7f7f7;font-weight:bold;">Enabled</span>
							</p>
<?php
	}
?>
						</div>
						<ol>
							<li>Click the Sign Up button above</li>
							<li>Enter your account <a href="admin.php?p=settings_stampscom&amp;last_active_tab=manage&amp;last_active_tab=manage">credentials</a></li>
							<li>Change your Passphrase</li>
							<li>Purchase Postage</li>
							<li>Print labels</li>
						</ol>
					</td>
				</tr>
			</table>
		</div>
	</div>

	<div id="admin-tab-sheet-manage" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon ic-gear"></span>Stamps.com Account Settings</div>
			<form action="admin.php?p=settings_stampscom&amp;last_active_tab=manage" method="post">
				<input type="hidden" name="mode" value="manage" />
				<input type="hidden" name="action" value="manage_settings" />
				<input type="hidden" name="last_active_tab" value="manage" />

				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Stamps.com enabled</td>
						<td class="formItemControl">
							<select name="stamps_settings_new[StampscomActive]" class="short">
								<option value="No"<?php if($settings['StampscomActive']!=='Yes')echo(' selected="selected"'); ?>>Disabled</option>
								<option value="Yes"<?php if($settings['StampscomActive']==='Yes')echo(' selected="selected"'); ?>>Enabled</option>
							</select>
						</td>
					</tr>

					<tr valign="top">
						<td class="formItemCaption required">Stamps.com user account</td>
						<td class="formItemControl">
							<input type="text" name="stamps_settings_new[StampscomUsername]" value="<?php echo gs($settings['StampscomUsername']); ?>" />
						</td>
					</tr>

					<tr valign="top">
						<td class="formItemCaption required">Stamps.com user password</td>
						<td class="formItemControl">
							<input type="password" name="stamps_settings_new[StampscomPassword]" value="<?php echo gs($settings['StampscomPassword']); ?>" />
						</td>
					</tr>
<?php
	if (defined('DEVMODE') && DEVMODE)
	{
?>
					<tr valign="top">
						<td class="formItemCaption">Generate labels using</td>
						<td class="formItemControl">
							<select name="stamps_settings_new[StampscomTestMode]" class="short">
								<option value="No"<?php if($settings['StampscomTestMode']!=='Yes')echo(' selected="selected"'); ?>>Live Mode</option>
								<option value="Yes"<?php if($settings['StampscomTestMode']==='Yes')echo(' selected="selected"'); ?>>Test Mode</option>
							</select>
							<div class="formItemComment">
								Test labels can be generated for free to verify that the system is working.
							</div>
						</td>
					</tr>
<?php
	}
?>
					<tr valign="top">
						<td class="formItemCaption">Enable SCAN form generation</td>
						<td class="formItemControl">
							<select name="stamps_settings_new[StampscomSCANFormEnable]" class="short">
								<option value="No"<?php if($settings['StampscomSCANFormEnable']!=='Yes')echo(' selected="selected"'); ?>>Disabled</option>
								<option value="Yes"<?php if($settings['StampscomSCANFormEnable']==='Yes')echo(' selected="selected"'); ?>>Enabled</option>
							</select>
							<div class="formItemComment">
<?php
	if ($stamps_account_is_valid && !$stamps_account_may_create_scanform)
	{
		echo '<span style="color:#990000;">Your account is not configured to allow creating SCAN Forms</span><br/>';
	}
?>
								Shipment Confirmation Acceptance Notice (SCAN) forms can be generated for previously-created labels.
							</div>
						</td>
					</tr>

				</table>
				<div class="admin-form-buttons">
					<input type="submit" value="Save changes" />
					<input type="reset" value="Reset form" />
				</div>
			</form>
		</div>
	</div>

	<div id="admin-tab-sheet-postage" class="admin-tab-sheet hidden">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon ic-currency-sign-dollar2"></span>Purchase Postage</div>
			<form id="form_stampscom_purchase_postage" name="form_stampscom_purchase_postage" action="admin.php?p=settings_stampscom&amp;last_active_tab=postage" method="post">
				<input type="hidden" name="action" value="postage_purchase" />
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
<?php
	if ($stamps_account_postage_balance !== null)
	{
		if ($postage_purchase_status != '' && $postage_purchase_status !== 'Success')
		{
?>
					<tr valign="top">
						<td class="formItemCaption">Postage Purchase Status</td>
						<td class="formItemControl" style="padding-top:10px;">
							<strong><?php echo gs($postage_purchase_status); ?></strong>
						</td>
					</tr>
<?php
		}
?>
					<tr valign="top">
						<td class="formItemCaption">Current balance</td>
						<td class="formItemControl" style="padding-top:10px;"><strong><?php echo getAdminPrice($stamps_account_postage_balance); ?></strong></td>
					</tr>
<?php
		if ($stamps_account_capability_purchase_postage)
		{
			if ($postage_purchase_status != 'Pending')
			{
?>
					<tr valign="top">
						<td class="formItemCaption required">Purchase amount</td>
						<td class="formItemControl">
							<input type="hidden" name="stamps_purchase[postage_control_total]" value="<?php echo gs($stamps_account_postage_control_total); ?>" />
							<input type="text" name="stamps_purchase[postage_amount]" class="short" />
						</td>
					</tr>
<?php
			}
		}
		else
		{
			Admin_Flash::setMessage('Stamps.com: Your account is not allowed to purchase postage', Admin_Flash::TYPE_INFO);
		}
	}
?>
				</table>
<?php
	if ($stamps_account_postage_balance !== null)
	{
?>
				<div class="admin-form-buttons">
<?php
		if ($postage_purchase_status != 'Pending')
		{
?>
					<input id="button-save-purchase-postage" type="submit" value="Purchase" />
					<input type="reset" value="Reset form" />
<?php
		}
		else
		{
?>
					<input type="button" value="Refresh" onclick="document.location='admin.php?p=settings_stampscom&last_active_tab=postage';" />
<?php
		}
?>
				</div>
<?php
	}
?>
			</form>
		</div>
	</div>
<?php
	if ($stamps_account_may_create_scanform)
	{
		$scanFormGenerateBuilder = new core_Form_FormBuilder('scan-generate');
		$scanFormGenerateDateGroup = new core_Form_FormBuilder('scan-generate-basic', array('label' => 'stamps.settings.scan.title_generate', 'class'=>'ic-checkmark', 'first' => true));
		$scanFormGenerateBuilder->add($scanFormGenerateDateGroup);
		$scanFormGenerateDateGroup->add('shipment_date', 'date', array('label' => 'stamps.settings.scan.shipment_date', 'value' => isset($scanShipmentDate) ? $scanShipmentDate : ''));

		$scanFormSearchBuilder = new core_Form_FormBuilder('scan-search');
		$scanFormSearchDateGroup = new core_Form_FormBuilder('scan-search-basic', array('label' => 'stamps.settings.scan.title_search', 'class'=>'ic-preview', 'first' => true));
		$scanFormSearchBuilder->add($scanFormSearchDateGroup);
		$scanFormSearchDateGroup->add('search_date', 'date', array('label' => 'stamps.settings.scan.search_date', 'value' => isset($scanSearchDate) ? $scanSearchDate : ''));

		$adminView = Ddm_AdminView::getInstance();
		$adminView->assign('scanFormGenerate', $scanFormGenerateBuilder->getForm());
		$adminView->assign('scanFormSearch', $scanFormSearchBuilder->getForm());

		if (isset($scanFormsSearchResult))
		{
			$adminView->assign('scanFormsSearchResult', $scanFormsSearchResult);
		}

		$adminView->render('pages/stamps/settings/scan-form');
	}

	if ($stamps_account_may_carrier_pickup)
	{
		$request = Framework_Request::createFromGlobals();
		$settingsRepository = DataAccess_SettingsRepository::getInstance();

		$pickupDate = $request->request->get('pickup_date', date_parse(date('Y-m-d h:i:s')));
		$pickupDateYear = intval($pickupDate['year']);
		$pickupDateMonth = intval($pickupDate['month']);
		$pickupDateDay = intval($pickupDate['day']);
		$pickupDate = $pickupDateYear.'-'.$pickupDateMonth.'-'.$pickupDateDay;

		$numberOfExpressMailPieces = $request->get('pickup_express_mail_pieces', 0);
		$numberOfPriorityMailPieces = $request->get('pickup_priority_mail_pieces', 0);
		$numberOfInternationalPieces = $request->get('pickup_international_mail_pieces', 0);
		$numberOfOtherPieces = $request->get('pickup_other_mail_pieces', 0);
		$totalWeightOfPackagesLbs = ceil($request->get('pickup_total_weight', 0));

		$pickupFirstName = $request->get('pickup_first_name', '');
		$pickupLastName = $request->get('pickup_last_name', '');
		$pickupCompany = $request->get('pickup_company', $settingsRepository->get('ShippingOriginName'));
		$pickupAddress = $request->get('pickup_address', $settingsRepository->get('ShippingOriginAddressLine1'));
		$pickupSuite = $request->get('pickup_suite', $settingsRepository->get('ShippingOriginAddressLine2'));
		$pickupCity = $request->get('pickup_city', $settingsRepository->get('ShippingOriginCity'));
		$pickupState = $request->get('pickup_state', '');
		$pickupZip = $request->get('pickup_zip', $settingsRepository->get('ShippingOriginZip'));
		$pickupZip4 = $request->get('pickup_zi4', '');
		$pickupPhone = preg_replace('/[^0-9]/', '', $request->get('pickup_phone', $settingsRepository->get('ShippingOriginPhone')));

		if (strlen($pickupPhone) > 10)
		{
			if ($pickupPhone[0] == '1') $pickupPhone = substr($pickupPhone, 1);
			$pickupPhone = substr($pickupPhone, 0, 10);
		}

		$pickupPhoneExt = $request->get('pickup_phone_ext', '');
		$pickupLocation = $request->get('pickup_location', 'Office');
		$pickupInstructions = $request->get('pickup_instructions', '');

		if ($pickupFirstName == '' && $pickupLastName == '')
		{
			$a = explode(' ', trim($admin['name']));
			if (count($a) > 0)
			{
				$pickupFirstName = $a[0]; unset($a[0]);
				$pickupLastName = implode(' ', $a);
			}
		}

		if ($action == 'pickup_autofill_piecec')
		{
			$shipmentsForPickup = $db->selectAll($q = 'SELECT * FROM '.DB_PREFIX.'orders_labels WHERE type="stampscom" AND shipping_date="'.$db->escape($pickupDate).'" AND status="Created" AND tracking_number <> ""');

			if ($shipmentsForPickup)
			{
				foreach ($shipmentsForPickup as $shipment)
				{
					switch ($shipment['shipping_service_type'])
					{
						// express mail
						case 'US-XM' : $numberOfExpressMailPieces++; break;

						// priority mail
						case 'US-PM' : $numberOfPriorityMailPieces++; break;

						// international mail
						case 'US-EMI' :
						case 'US-PMI' :
						case 'US-FCI' : $numberOfInternationalPieces++; break;

						// other mail pieces
						default : $numberOfOtherPieces++; break;
					}

					$totalWeightOfPackagesLbs += $shipment['shipping_weight'];
				}

				$totalWeightOfPackagesLbs = ceil($totalWeightOfPackagesLbs);
			}
			else
			{
				Admin_Flash::setMessage('stamps.settings.pickup.no_results_for_date', Admin_Flash::TYPE_ERROR);
			}
		}

		if ($action == 'pickup_send_request')
		{
			$arguments = array(
				'FirstName' => $pickupFirstName,
				'LastName' => $pickupLastName,
				'Company' => $pickupCompany,
				'Address' => $pickupAddress,
				'SuiteOrApt' => $pickupSuite,
				'City' => $pickupCity,
				'State' => $pickupState,
				'ZIP' => $pickupZip,
				'ZIP4' => $pickupZip4,
				'PhoneNumber' => $pickupPhone,
				'PhoneExt' => $pickupPhoneExt,
				'NumberOfExpressMailPieces' => $numberOfExpressMailPieces,
				'NumberOfPriorityMailPieces' => $numberOfPriorityMailPieces,
				'NumberOfInternationalPieces' => $numberOfInternationalPieces,
				'NumberOfOtherPieces' => $numberOfOtherPieces,
				'TotalWeightOfPackagesLbs' => ceil($totalWeightOfPackagesLbs),
				'PackageLocation' => $pickupLocation,
				'SpecialInstruction' => $pickupInstructions
			);

			$result = $stamps->apiMethodCarrierPickup($arguments);

			if ($result)
			{
				if (isset($result['faultstring']) && trim($result['faultstring']) != '')
				{
					Admin_Flash::setMessage($result['faultstring'], Admin_Flash::TYPE_ERROR);
				}
				else if (isset($result['ErrorMsg']) && trim($result['ErrorMsg']) != '')
				{
					Admin_Flash::setMessage($result['faultstring'], Admin_Flash::TYPE_ERROR);
				}
				else
				{
					Admin_Flash::setMessage(
						'stamps.settings.pickup.request_sent'.
						isset($result['ConfirmationNumber']) && trim($result['ConfirmationNumber']) != '' ? $result['ConfirmationNumber'] : 'n/a'
					);
				}
			}
			else
			{
				Admin_Flash::setMessage('stamps.settings.pickup.error_cannot_get_data', Admin_Flash::TYPE_ERROR);
			}
		}

		$pickupFormBuilder = new core_Form_FormBuilder('pickup');

		$pickupOptionsGroup = new core_Form_FormBuilder('pickup_options', array('label' => 'stamps.settings.pickup.title_options', 'class'=>'ic-truck', 'first' => true));
		$pickupFormBuilder->add($pickupOptionsGroup);

		$pickupOptionsGroup->add('pickup_date', 'date', array('label' => 'stamps.settings.pickup.pickup_date', 'value' => $pickupDate, 'note' => 'stamps.settings.pickup.pickup_date_note'));

		$pickupOptionsGroup->add('autofill_button', 'static', array('label' => '', 'value' => '<input type="button" id="pickup_autofill_pieces" style="font-size:14px;" value="Prepopulate fields for selected date">'));

		$pickupOptionsGroup->add('pickup_express_mail_pieces', 'text', array('label' => 'stamps.settings.pickup.express_mail_pieces', 'value' => $numberOfExpressMailPieces, 'attr' => array('class' => 'tiny mail-piece')));
		$pickupOptionsGroup->add('pickup_priority_mail_pieces', 'text', array('label' => 'stamps.settings.pickup.priority_mail_pieces', 'value' => $numberOfPriorityMailPieces, 'attr' => array('class' => 'tiny mail-piece')));
		$pickupOptionsGroup->add('pickup_international_mail_pieces', 'text', array('label' => 'stamps.settings.pickup.international_mail_pieces', 'value' => $numberOfInternationalPieces, 'attr' => array('class' => 'tiny mail-piece')));
		$pickupOptionsGroup->add(
			'pickup_other_mail_pieces', 'text',
			array('label' => 'stamps.settings.pickup.other_mail_pieces', 'note'=>'stamps.settings.pickup.other_mail_pieces_note', 'value' => $numberOfOtherPieces, 'attr' => array('class' => 'tiny mail-piece'))
		);
		$pickupOptionsGroup->add(
			'pickup_total_weight', 'text',
			array('label' => 'stamps.settings.pickup.total_weight', 'required' => true, 'note'=>'stamps.settings.pickup.total_weight_note', 'value' => $totalWeightOfPackagesLbs, 'attr' => array('class' => 'tiny'))
		);

		$pickupAddressGroup = new core_Form_FormBuilder('pickup_address', array('label' => 'stamps.settings.pickup.title_address', 'class'=>'ic-location'));
		$pickupFormBuilder->add($pickupAddressGroup);

		$pickupAddressGroup->add('pickup_first_name', 'text', array('label' => 'stamps.settings.pickup.first_name', 'value' => $pickupFirstName, 'required' => true, 'attr' => array('maxlength' => 50)));
		$pickupAddressGroup->add('pickup_last_name', 'text', array('label' => 'stamps.settings.pickup.last_name', 'value' => $pickupLastName, 'required' => true, 'attr' => array('maxlength' => 50)));
		$pickupAddressGroup->add('pickup_company', 'text', array('label' => 'stamps.settings.pickup.company', 'value' => $pickupCompany, 'attr' => array('maxlength' => 50)));
		$pickupAddressGroup->add('pickup_address', 'text', array('label' => 'stamps.settings.pickup.address', 'value' => $pickupAddress, 'required' => true, 'attr' => array('maxlength' => 50)));
		$pickupAddressGroup->add('pickup_suite', 'text', array('label' => 'stamps.settings.pickup.suite', 'value' => $pickupSuite, 'attr' => array('maxlength' => 10, 'class' => 'short')));
		$pickupAddressGroup->add('pickup_city', 'text', array('label' => 'stamps.settings.pickup.city', 'value' => $pickupCity, 'required' => true, 'attr' => array('maxlength' => 30)));

		$usStates = $db->selectAll('SELECT stid, name, short_name FROM '.DB_PREFIX.'states WHERE coid=1 ORDER BY name');
		$statesOptions = array();
		foreach ($usStates as $usState)
		{
			$statesOptions[] = new core_Form_Option($usState['short_name'], $usState['name']);
			if ($pickupState == '' && $usState['stid'] == $settings['ShippingOriginState'])
			{
				$pickupState = $usState['short_name'];
			}
		}

		$pickupAddressGroup->add('pickup_state', 'choice', array('label' => 'stamps.settings.pickup.state', 'value' => $pickupState, 'options' => $statesOptions, 'required' => true));
		$pickupAddressGroup->add('pickup_zip', 'text', array('label' => 'stamps.settings.pickup.zip', 'value' => $pickupZip, 'required' => true, 'attr' => array('class' => 'short', 'maxlength' => 5)));
		$pickupAddressGroup->add('pickup_zip4', 'text', array('label' => 'stamps.settings.pickup.zip4', 'value' => $pickupZip4, 'attr' => array('class' => 'short', 'maxlength' => 4)));
		$pickupAddressGroup->add('pickup_phone', 'text', array('label' => 'stamps.settings.pickup.phone', 'value' => $pickupPhone, 'required' => true, 'attr' => array('class' => 'short', 'maxlength' => 10), 'note' => 'stamps.settings.pickup.phone_note'));
		$pickupAddressGroup->add('pickup_phone_ext', 'text', array('label' => 'stamps.settings.pickup.phone_ext', 'value' => $pickupPhoneExt, 'attr' => array('class' => 'short', 'maxlength' => 6)));

		$pickupAddressGroup->add('pickup_location', 'choice', array(
			'label' => 'stamps.settings.pickup.pickup_location',
			'value' => $pickupLocation,
			'options' => array(
				new core_Form_Option('FrontDoor', 'stamps.settings.pickup.pickup_location_options.FrontDoor'),
				new core_Form_Option('BackDoor', 'stamps.settings.pickup.pickup_location_options.BackDoor'),
				new core_Form_Option('SideDoor', 'stamps.settings.pickup.pickup_location_options.SideDoor'),
				new core_Form_Option('KnockOnDoorOrRingBell', 'stamps.settings.pickup.pickup_location_options.KnockOnDoorOrRingBell'),
				new core_Form_Option('MailRoom', 'stamps.settings.pickup.pickup_location_options.MailRoom'),
				new core_Form_Option('Office', 'stamps.settings.pickup.pickup_location_options.Office'),
				new core_Form_Option('Reception', 'stamps.settings.pickup.pickup_location_options.Reception'),
				new core_Form_Option('InOrAtMailbox', 'stamps.settings.pickup.pickup_location_options.InOrAtMailbox'),
				new core_Form_Option('Other', 'stamps.settings.pickup.pickup_location_options.Other')
			),
			'note' => 'stamps.settings.pickup.pickup_location_options_note',
			'attr' => array('class' => 'large')
		));

		$pickupAddressGroup->add('pickup_instructions', 'textarea', array('label' => 'stamps.settings.pickup.instructions', 'value' => $pickupInstructions));

		$adminView = Ddm_AdminView::getInstance();
		$adminView->assign('pickupForm', $pickupFormBuilder->getForm());
		$adminView->render('pages/stamps/settings/carrier-pickup');
	}
?>
</div>

<div id="dialog-stampscom-view" style="display:none;">
	<div>
		<iframe id="iframe-stampscom-view-label" src="" scrolling="no" style="overflow:hidden;border:none;width:100%;height:580px;"></iframe>
	</div>
</div>

<?php

	if (isset($stamps_msg_postage) && strlen(trim($stamps_msg_postage)) > 0)
	{
		Admin_Flash::setMessage('Stamps.com: '.$stamps_msg_postage);
	}
