$(document).ready(function(){
	/**
	 * The onclick event handler for the quick start guide link on the top menu
	 */
	$('#quick-start-guide-link').click(function(){
		QuickStartGuide_show();
		return false;
	});

	$('#QuickStartGuide').data('loader', false);

	if (autoOpenQuickStartGuide) QuickStartGuide_show();
});

/**
 * Load and show HTML from server
 */
function QuickStartGuide_show()
{
	if ($('#QuickStartGuide').data('loaded'))
	{
		QuickStartGuide_goToSection(1);
		$("#QuickStartGuide").dialog("open");
	}
	else
	{
		$.post(
			'admin.php?p=quick_start_guide&triggerHelper=true',
			{
				'action': 'getDialogHTML',
				'data': true
			},
			function(response)
			{
				var resp = JSON.parse(response);

				if (!resp.result)
				{
					alert('There was a problem rendering quick start guide dialog');
				}
				else
				{
					$("#QuickStartGuideInner").css("min-width", "500px");
					$("#QuickStartGuideInner").html(resp.result);
					$('#QuickStartGuide').dialog({
						modal: true,
						width: 675,
						maxWidth: 675,
						showTitle: false,
						resizable: false,
						dialogClass: 'quick-start-dialog',
						autoOpen: false,
						closeOnEscape: false,
						title: 'Quick Start Guide',
						open: function(ev, ui)
						{
							correctDialogPosition("#QuickStartGuide");
						}
					}); 
					QuickStartGuide_init();
					QuickStartGuide_goToSection(1);
					$('#QuickStartGuide').data('loaded', true);
					$("#QuickStartGuide").dialog("open");
					
					$("#QuickStartGuide").height($("#QuickStartGuideWrapper").height());

					correctDialogPosition("#QuickStartGuide");
				}
			}
		);
	}
}

/**
 * Save information
 * @param step
 */
function QuickStartGuide_saveInformation(step)
{
	var saveAction = '';
	switch (step)
	{
		// Company Information
		case 1: saveAction = 'saveCompanyInformation'; break;
		case 3: saveAction = 'saveCategoryLayout'; break;
		case 4: saveAction = 'saveProductLayout'; break;
		case 5: saveAction = 'saveCheckoutProcess'; break;
		case 6: saveAction = 'saveSeoInformation'; break;
	}

	if (saveAction != '')
	{
		$.post('admin.php?p=quick_start_guide&triggerHelper=true', {
			'action': saveAction,
			'data': $('#quick-start-guide-form').serialize()
		});
	}
}

/**
 * Displays the requested section on the quick start guide
 * @param step
 */
function QuickStartGuide_goToSection(step)
{
	// store the current section and the new requested section
	var oldSection = parseInt($('#QuickStartGuide').data('selectedSection'));
	var newSection = parseInt(step);

	// if old and new do not match, continue with changing sections
	if (oldSection != newSection)
	{
		$('.quick-start-guide-footer ul li.selected').removeClass('selected');
		$('.quick-start-guide-footer ul li[section="'+newSection+'"]').addClass('selected');

		$('#QuickStartGuide').data('selectedSection', newSection);
		$('.quick-start-guide-sections div[section="'+oldSection+'"]').hide();
		$('.quick-start-guide-sections div[section="'+newSection+'"]').show();
	}

	// always show the buttons by default
	$('#quick-start-guide-previous-step-link').show();
	$('#quick-start-guide-next-step-link').show();
	$('.turn-off-quick-start').show();

	// hide the complete link by default
	$('#quick-start-guide-complete-link').hide();

	// hide specific buttons based on current section
	switch (newSection)
	{
		case 1:
		{
			$('#quick-start-guide-previous-step-link').hide();
			break;
		}
		case 8:
		{
			$('#quick-start-guide-next-step-link').hide();
			$('#quick-start-guide-previous-step-link').hide();
			$('#quick-start-guide-complete-link').show();
			break;
		}
		default:
		{
			// hide the turn off flag on every other page but the first and last
			$('.turn-off-quick-start').hide();
		}
	}
}

/**
 * Goes to the next section from the current position
 */
function QuickStartGuide_goToNextSection()
{
	var step = $('#QuickStartGuide').data('selectedSection') + 1;
	QuickStartGuide_goToSection(step);
}

/**
 * Goes to the previous section from the current position
 */
function QuickStartGuide_goToPreviousSection()
{
	var step = $('#QuickStartGuide').data('selectedSection') - 1;
	QuickStartGuide_goToSection(step);
}

/**
 * Init dialog after loading
 */
function QuickStartGuide_init()
{
	/**
	 * The onclick event handler for the step links
	 */
	$('.quick-start-guide-footer ul li, .quick-start-guide-footer a').click(function(){
		QuickStartGuide_goToSection(parseInt($(this).attr('section')));
		return false;
	});

	/**
	 * The onclick event handler for the next step button
	 * this event will hide the button if it has reached the last step
	 */
	$('#quick-start-guide-next-step-link').click(function(){
		QuickStartGuide_saveInformation($('#QuickStartGuide').data('selectedSection'));
		QuickStartGuide_goToNextSection();
		return false;
	});

	/**
	 * The onclick event handler for the next step button
	 */
	$('#quick-start-guide-previous-step-link').click(function(){
		QuickStartGuide_goToPreviousSection();
		return false;
	});

	/**
	 * The onclick event handler for the complete button
	 */
	$('#quick-start-guide-complete-link').click(function(){
		$('#QuickStartGuide').dialog('close');
	});

	/**
	 * The onchange event for the display on startup checkbox
	 */
	$('#turn-off-quick-start').change(function(){
		$.post('admin.php?p=quick_start_guide&triggerHelper=true', {
			'action': 'setDisplayOnStartup',
			'data': this.checked
		});
	});

	/**
	 * The onchange event for the display on startup checkbox
	 */
	$('#activate-seo').change(function(){
		$.post('admin.php?p=quick_start_guide&triggerHelper=true', {
			'action': 'setActivateSearchEngineUrl',
			'data': this.checked
		});
	});

	/**
	 * The onclick event for the remove test products/categories button
	 */
	$('#quick-start-guide-remove-items-button').click(function(){
		if (confirm('Are you sure you want to remove test products & categories?'))
		{
			$.post('admin.php?p=quick_start_guide&triggerHelper=true', {
				'action': 'removeTestItems',
				'data': true
			}, function(response){
				resp = JSON.parse(response);

				if (!resp.result)
				{
					alert('There was a problem removing the test products.');
				}
				else
				{
					alert('You have successfully removed all test products.');
					$('div.add-test-items').show();
					$('div.remove-test-items').hide();
				}
			});
		}
	});

	/**
	 * The onclick event for the add test products/categories button
	 */
	$('#quick-start-guide-add-items-button').click(function(){
		if (confirm('Are you sure you want to add all test products & categories?'))
		{
			$.post('admin.php?p=quick_start_guide&triggerHelper=true', {
				'action': 'addTestItems',
				'data': null
			}, function(response){
				resp = JSON.parse(response);

				if (!resp.result)
				{
					alert('There was a problem importing test products.');
				}
				else
				{
					alert('You now have test products in your shopping cart.');
					$('div.add-test-items').hide();
					$('div.remove-test-items').show();
				}
			});
		}
	});

	/**
	 * The onclick event handler for the skip import/remove buttons
	 */
	$('#quick-start-guide-skip-add-items-button, #quick-start-guide-skip-remove-items-button').click(function(){
		QuickStartGuide_goToNextSection();
	});

	// set the category layout
	var category_layout = qsg_categoryLayout;
	// loop through each input field and see if it has been selected
	$('#QuickStartGuide .catalog-page-views input[type="checkbox"]').each(function(){
		var checked = (category_layout[$(this).attr('name')] == 'YES');
		if (checked) $(this).attr('checked', 'checked');
	});
	var category_default_view = qsg_categoryDefaultView;
	$('#CatalogDefaultView'+category_default_view).attr('checked', 'checked');
	// set the product layout
	var product_layout = qsg_productLayout;
	$('#CatalogProductPage'+product_layout).attr('checked', 'checked');
	// set the checkout process
	var checkout_process = '{/literal}{$quickStartGuide.checkoutProcess}{literal}';
	$('#checkout-'+checkout_process).attr('checked', 'checked');
	// hide the checkboxes for catalog view
	$('#QuickStartGuide .catalog-page-views input:checkbox').hide();
	if (autoOpenQuickStartGuide) $('#QuickStartGuide').dialog('open');
}