<?php
/**
 * Testimonials
 */
class plugin_testimonials extends cartPlugin 
{
	var $plugin_id = "testimonials";
	var $plugin_group = "misc";
	var $plugin_caption = "Testimonials";
	var $plugin_description = "Users can write testimonials, admins can verify / accept them";
	var $totalTestimonials = '0';
	var $imageDirectory = 'content/admin/plugins/testimonials/images/';
	
	function plugin_testimonials()
	{	
		$this->interface = array();
		$this->interface["title"] = $this->plugin_caption;
		$this->interface["url"] = 'admin.php?p=plugin&plugin_id='.$this->plugin_id;
		$this->interface["icon"] = "ic_plugin";
		$this->interface["group"] = "misc";
		return $this;	
	}
	
	function setupPlugin(&$db)
	{	
		// used so frontend can pass db and settings
		$this->db = $db;	
	}
	
	function runPlugin(&$db, &$settings, &$request)
	{	
		parent::runPlugin($db, $settings, $request);
		extract($request);	
		
		//process ajax action
		$action = isset($_POST["action"]) ? $_POST["action"] : false;
		if ($action && !DEMO_LOCK)
		{
			switch ($action)
			{
				case "deleteTestimonial" : $this->delete($_POST["id"]); break;
				case "setTestimonialStatus" : $this->setStatus($_POST["id"], $_POST["status"]); break;
			}
			session_write_close();
			die();
		}
		
		//save testimonial
		$saveTestimonial = isset($_REQUEST["saveTestimonial"]) ? $_REQUEST["saveTestimonial"] : false;
		if ($saveTestimonial == 1)
		{	
			if (!DEMO_LOCK)
			{
				$saveData = array();
				$saveData['id'] = intval($testimonial_id);
				$saveData['name'] = $testimonial_name;
				$saveData['company'] = $testimonial_company;
				$saveData['testimonial'] = $testimonial_testimonial;
				$saveData['rating'] = $testimonial_rating;
				$saveData['status'] = $testimonial_status;
				$this->saveTestimonial($saveData, true);
			}
		}
		
		//save settings
		$saveSettings = isset($_REQUEST["saveSettings"]) ? $_REQUEST["saveSettings"] : false;
		if ($saveSettings == 1 && !DEMO_LOCK)
		{
			if (!empty($settings_testimonial))
			{
				foreach ($settings_testimonial as $key=>$value)
				{
					$this->setSetting($key,$value);
					$settings[$key] = $value;
				}
			}
			
			// images
			if (!empty($_FILES['full'])) $this->uploadImage($_FILES['full'],'full');
			if (!empty($_FILES['half'])) $this->uploadImage($_FILES['half'],'half');
			if (!empty($_FILES['empty'])) $this->uploadImage($_FILES['empty'],'empty');
		}
		
		
		$last_active_tab = isset($_REQUEST["last_active_tab"]) ? $_REQUEST["last_active_tab"] : 'testimonials';
		pageHeader($this->plugin_caption);
		echo '<br/><div class="admin-tabs admin-tabs-framed" id="tabs-testimonials">';
		pageTabs(array(
			array("title"=>"Testimonials", "id"=>"testimonials"),
			array("title"=>"Write", "id"=>"create"),
			array("title"=>"Settings", "id"=>"settings")
		), "tabs-testimonials", $last_active_tab);
	
		
		include 'layouts/tab_testimonials.php';
		include 'layouts/tab_create.php';
		include 'layouts/tab_settings.php';
		
		echo '</div>';
	}
	
	function delete($id)
	{
		$this->db->query("DELETE FROM ".DB_PREFIX."testimonials WHERE id='".intval($id)."'");
	}
	
	function getTestimonials($start, $howMany, $mode='')
	{	
		$start = isset($start) ? intval($start) : 0;
		$howMany = isset($howMany) ? intval($howMany) : 10;
		$whereClause = "";
		if ($mode != '') $whereClause = "WHERE `status` = '".$this->db->escape($mode)."'";
		$sql = "SELECT SQL_CALC_FOUND_ROWS * FROM ".DB_PREFIX."testimonials ".$whereClause." ORDER BY `date_created` DESC LIMIT ".$start.",".$howMany;
		$this->db->query($sql);
		$testimonials = $this->db->getRecords();
		$this->db->query("SELECT FOUND_ROWS() as 'howmany'");
		$this->db->moveNext();
		$this->totalTestimonials = $this->db->col['howmany'];
		return $testimonials;
		
	}
	
	function getTestimonial($id)
	{		
		$sql = "SELECT * FROM ".DB_PREFIX."testimonials WHERE `id` = ".intval($id)." LIMIT 1";
		$this->db->query($sql);
		$results = $this->db->getRecords();
		
		if (!empty($results)) return $results[0]; else return false;
	}
	
	function getRating()
	{	
		$sql = "SELECT AVG(rating) AS `rating` FROM ".DB_PREFIX."testimonials WHERE status = 'approved' AND rating > 0";
		$this->db->query($sql);
		$this->db->moveNext();
		$rating = $this->db->col['rating'];
		return $rating;
	}
	
	function saveTestimonial($data, $isAdminArea = false)
	{
		global $settings;
		$this->db->reset();
		$this->db->assignStr('name',$data['name']);
		$this->db->assignStr('company',$data['company']);
		$this->db->assign('rating',intval($data['rating']));
		$this->db->assignStr('testimonial',$data['testimonial']);
		$this->db->assignStr('status',$data['status']);
		
		if ((isset($data['id']) && is_numeric($data["id"]) && $data["id"] > 0) && $isAdminArea)
		{
			// update
			$this->db->update(DB_PREFIX.'testimonials',"WHERE `id` = ".intval($data['id']));
		}
		else
		{
			// insert
			$this->db->assign("date_created", "NOW()");
			$this->db->insert(DB_PREFIX.'testimonials');
			if ($settings['testimonials_Email_Notify'] != '')
			{
				require_once($settings['GlobalServerPath'].'/content/engine/engine_notifications.php');
				
				// ensure send_mail is defined
				$message = 'New testimonial received on '.$settings['CompanyName'].' website. Login to the admin to view it.';
				send_mail(
					$settings['testimonials_Email_Notify'],
					'New Testimonial for '.$settings['CompanyName'],
					$message,$settings["GlobalNotificationEmail"],
					'text'
				);
			}
		}
	}
	
	function setSetting($key,$value)
	{	
		$this->db->reset();
		$this->db->assignStr('value',$value);
		$this->db->update(DB_PREFIX.'settings',"WHERE `name` = '".addslashes($key)."'");
		return true;	
	}
	
	function setStatus($id,$status)
	{	
		$this->db->reset();
		$this->db->assignStr('status',$status);
		$this->db->update(DB_PREFIX.'testimonials',"WHERE `id` = ".intval($id));
		return true;	
	}
	
	function getImage($name)
	{	
		$possibleImages = array(
			$this->imageDirectory.$name.'.jpg',
			$this->imageDirectory.$name.'.png',
			$this->imageDirectory.$name.'.gif',
		);
		foreach ($possibleImages as $possibleImage)
		{
			if (file_exists($possibleImage)) return $possibleImage;
		}
	}
	
	private function uploadImage($file,$name)
	{
    	global $imageTypes;
    	if (!empty($file["tmp_name"]))
    	{
			$image_data = @getImageSize($file["tmp_name"]);
			if (in_array($image_data[2], $imageTypes))
			{
				$destination = $this->imageDirectory.$name;
				@unlink($destination."jpg");
				@unlink($destination."png");
				@unlink($destination."gif");
				//add new file extension
				switch ($image_data[2])
				{
					case IMAGETYPE_GIF: $destination.=".gif"; break;
					case IMAGETYPE_JPEG: $destination.=".jpg"; break;
					case IMAGETYPE_PNG: $destination.=".png"; break;
				}
				if (@copy($file["tmp_name"], $destination))
				{
					// image uploaded
					return $destination;
				}
				else
				{
					// unable to upload
					return false;
				}
			}
			else
			{
        		//incorrect file extension
				return false;
			}
		}
		else
		{
     		// not uploaded image?
			return false;
		}
	}
}
