<?php
/**
 * Class Admin_Controller_Exactor
 */
class Admin_Controller_Exactor extends Admin_Controller
{
	protected $exactorSettings = array(
		'ExactorEnableTax', 'exactor_merchant_id', 'exactor_user_id', 'exactor_password', 'exactor_full_name',
		'exactor_currency_code', 'exactor_commit_taxes', 'exactor_sku_source', 'EndiciaUspsAddressValidation',
	);

	/**
	 *
	 */
	public function aboutAction()
	{
		$this->renderExactorSettingsPage();
	}

	/**
	 *
	 */
	public function manageAction()
	{
		$errors = array();

		$lastActiveTab = 'settings';

		$request = $this->getRequest();
		$settings = $this->getSettings();

		// check do we have to save data
		if ($request->isPost())
		{
			// check nonce
			$this->nonceCheck($request->get('nonce'), 'tax_save_exactor');

			// get form data
			$formData = $request->getRequest()->get('exactor', array());

			// validate form data
			if (!in_array($formData['ExactorEnableTax'], array('Yes', 'No'))) $formData['ExactorEnableTax'] = 'No';

			if ($formData['ExactorEnableTax'] == 'Yes')
			{
				// enable address validation
				$_POST['exactor']['EndiciaUspsAddressValidation'] = 1;

				if ($settings->get('AvalaraEnableTax') == 'Yes')
				{
					$errors[] = 'Avalara Tax Service is enabled.  Please disable before enabling the Exactor Tax Service.';
					$formData['ExactorEnableTax'] = 'No';
				}
				else
				{
					// validate connection
					$exactorClient = Tax_Exactor_ExactorClient::getInstance(
						$this->getDb(),
						array_merge($this->getSettings()->getAll(), $formData)
					);

					$ret = $exactorClient->validate_exactor_info();
					$errors = $ret[0];

					if (count($errors) > 0)
					{
						$formData['ExactorEnableTax'] = 'No';
					}
					else
					{
						Admin_Flash::setMessage('Exactor Account was successfully setup.', Admin_Flash::TYPE_SUCCESS);
					}
				}
			}

			// save data if there no errors
			if (count($errors) == 0)
			{
				$formData['EndiciaUspsAddressValidation'] = 1;
				$this->getSettings()->save($formData, $this->exactorSettings);
			}
			else
			{
				$settings->save(array('ExactorEnableTax' => $formData['ExactorEnableTax']));
			}

			if ($formData['ExactorEnableTax'] == 'Yes')
			{
				Tax_Exactor_ExactorEUCHelper::updateEUCCodes();
			}
		}
		else
		{
			$formData = $this->getSettings()->getByKeys($this->exactorSettings);
		}

		$this->renderExactorSettingsPage($errors, $lastActiveTab, $formData);
	}

	/**
	 *
	 */
	public function manageTaxCodesAction()
	{
		$errors = array();
		$lastActiveTab = 'tax-codes';

		$request = $this->getRequest();

		if ($request->isPost())
		{
			//$this->nonceCheck($request->request->get('nonce'), 'tax_save_exactor');

			if (trim($request->get('tax_code_name')) != '' && trim($request->get('tax_code_code')) != '')
			{
				$this->addTaxCode($request->get('tax_code_code'), $request->get('tax_code_name'));
			}

			$taxCodeFields = $request->get('tax_code_fields', array());

			if (!empty($taxCodeFields) && is_array($taxCodeFields))
			{
				foreach ($taxCodeFields as $k => $code_item)
				{
					$this->db->reset();
					$this->db->assignStr('name', $code_item['name']);
					$this->db->assignStr('code', $code_item['code']);
					$this->db->update(DB_PREFIX.'exactor_tax_codes',"WHERE id = $k");
				}

				Admin_Flash::setMessage('Tax Codes updated');
			}
		}

		$this->renderExactorSettingsPage($errors, $lastActiveTab);
	}

	/**
	 *
	 */
	public function deleteTaxCode()
	{
		$errors = array();
		$lastActiveTab = 'tax-codes';

		$request = $this->getRequest();

		if ($request->isPost())
		{
			$taxCodeId = $request->request->getInt('delete_tax_code');

			$code = $this->db->selectOne('SELECT code FROM '.DB_PREFIX.'exactor_tax_codes WHERE id = '.$taxCodeId);

			if ($code)
			{
				$code = array_shift($code);

				$this->db->query('DELETE FROM '.DB_PREFIX.'exactor_tax_codes WHERE id = '.$taxCodeId);

				if ($this->db->numRows() > 0)
				{
					Admin_Flash::setMessage('Tax Code deleted');
				}

				$this->db->query("UPDATE ".DB_PREFIX."products SET exactor_euc_code = ''  WHERE exactor_euc_code = '".$this->db->escape($code)."'");
				$this->db->query("UPDATE ".DB_PREFIX."catalog SET exactor_euc_code = ''  WHERE exactor_euc_code = '".$this->db->escape($code)."'");
			}
		}

		$this->renderExactorSettingsPage($errors, $lastActiveTab);
	}

	/**
	 *
	 */
	public function addEucCode()
	{
		$errors = array();
		$submitSuccess = false;
		$eucCodeUpdatesSuccess = false;
		$lastActiveTab = 'tax-codes';

		$request = $this->getRequest();

		if ($request->isPost())
		{
			$eucCode = $request->get('euc_code');
			$eucName = $request->get('euc_name');

			if (trim($eucCode) != '' && trim($eucName) != '')
			{
				$this->addTaxCode($eucCode, $eucName);
			}
		}

		$this->renderExactorSettingsPage($errors, $lastActiveTab);
	}

	protected function addTaxCode($taxCode, $taxName)
	{
		$taxCode = trim($taxCode);
		$taxName = trim($taxName);

		$result = $this->db->selectOne('SELECT * FROM '.DB_PREFIX.'exactor_tax_codes WHERE code = \''.$this->db->escape($taxCode).'\'');
		if (!$result)
		{
			$this->db->reset();
			$this->db->assignStr('name', $taxName);
			$this->db->assignStr('code', $taxCode);
			$this->db->insert(DB_PREFIX.'exactor_tax_codes');

			Admin_Flash::setMessage('Tax Code added');
		}
		else
		{
			Admin_Flash::setMessage('Tax Code already exists', Admin_Flash::TYPE_ERROR);
		}
	}
	/**
	 *
	 */
	public function updateEucAction()
	{
		$errors = array();
		$lastActiveTab = 'tax-codes';

		$request = $this->getRequest();

		$this->nonceCheck($request->get('nonce'), 'tax_update_exactor_eucs');

		Tax_Exactor_ExactorEUCHelper::updateEUCCodes();

		Admin_Flash::setMessage('Exactor EUC Codes successfully updated.', Admin_Flash::TYPE_SUCCESS);

		$this->renderExactorSettingsPage($errors, $lastActiveTab);
	}

	/**
	 * @param array $errors
	 * @param string $lastActiveTab
	 */
	protected function renderExactorSettingsPage(array $errors = array(), $lastActiveTab = 'main', array $settingsFormData = array())
	{
		if (!count($settingsFormData)) $settingsFormData = $this->getSettings()->getByKeys($this->exactorSettings);

		$taxCodes = $this->db->selectAll('SELECT * FROM '.DB_PREFIX.'exactor_tax_codes ORDER BY name');

		$adminView = $this->getView();

		$settingsForm = new core_Form_FormBuilder('exactor_settings_form', array('label' => 'Properties', 'class' => 'ic-payment', 'first' => true));
		$settingsForm->add('action', 'hidden', array('label' => '', 'value' => 'save_exactor'));
		$settingsForm->add('nonce', 'hidden', array('value' => create_nonce('tax_save_exactor')));

		$settingsForm->add(
			'exactor[ExactorEnableTax]', 'choice',
			array('label' => 'Enable Exactor', 'value' => $settingsFormData['ExactorEnableTax'], 'options' => array('No' => 'No', 'Yes' => 'Yes'), 'attr' => array('class' => 'short'))
		);
		$settingsForm->add('exactor[exactor_merchant_id]', 'text', array('label' => 'Account #', 'value' => $settingsFormData['exactor_merchant_id'], 'required' => true));
		$settingsForm->add('exactor[exactor_user_id]', 'text', array('label' => 'User Name', 'value' => $settingsFormData['exactor_user_id'], 'required' => true));
		$settingsForm->add('exactor[exactor_password]', 'password', array('label' => 'Password', 'value' => $settingsFormData['exactor_password'], 'required' => true));
		$settingsForm->add('exactor[exactor_full_name]', 'text', array('label' => 'Company Full Name', 'value' => $settingsFormData['exactor_full_name'], 'required' => true));
		$settingsForm->add('exactor[exactor_currency_code]', 'text', array('label' => 'Currency Code', 'value' => $settingsFormData['exactor_currency_code'], 'attr' => array('class' => 'short'), 'required' => true,
			'note' => 'Use this field to override the Currency Code passed to Exactor for calculating taxes. It <strong>must</strong> match the default currency code for your store (e.g. USD or EUR).'
		));
		$settingsForm->add(
			'exactor[exactor_commit_taxes]', 'choice',
			array('label' => 'Commit taxes on completed orders?', 'value' => $settingsFormData['exactor_commit_taxes'], 'options' => array('0' => 'No', '1' => 'Yes'), 'attr' => array('class' => 'short'), 'note' => 'Yes - Commit taxes to Exactor for tax purposes<br/>No - Just calculate taxes. <strong>Do not</strong> use Exactor for tax reporting')
		);

		$settingsForm->add(
			'exactor[exactor_sku_source]', 'choice',
			array('label' => 'Source of SKU Information', 'value' => $settingsFormData['exactor_sku_source'], 'options' => array('SKU' => 'SKU Field', 'PID' => 'Product Id'), 'attr' => array('class' => 'short'))
		);

		$adminView->assign('settingsForm', $settingsForm->getForm());

		foreach ($errors as $error)
		{
			Admin_Flash::setMessage($error, Admin_Flash::TYPE_ERROR);
		}

		$adminView->assign('lastActiveTab', $lastActiveTab);
		//$adminView->assign('ExactorEnableTax', $this->settings['ExactorEnableTax']);

		$adminView->assign('taxCodes', $taxCodes);
		$adminView->assign('update_eucs_nonce', create_nonce('tax_update_exactor_eucs'));

		$adminView->assign('ExactorEnableTax', $settingsFormData['ExactorEnableTax']);

		$adminView->render('pages/taxes/exactor');
	}
}