<?php
	
class Admin_MenuItem
{
	public $id = null;
	public $parent_id = null;
	public $privileges = null;
	public $title = null;
	public $innerClass = null;
	public $outerClass = null;
	public $isActive = false;
	public $isOpen = false;
	public $hasChildren = false;
	public $url = null;
	public $urlTarget = null;
	public $alternativeUrls = null;
	public $html = null;
	public $level = 0; 
	
	protected $children = array();
	protected static $items = array();
	protected static $cc = 0;
	
	/**
	 * Class constructor
	 * @param string $id
	 * @param mixed $privileges
	 * @param string $title
	 * @param string $classInner
	 * @param string $classOuter
	 * @param string $url
	 * @param string $html
	 * @return AdminMenuItem 
	 * @return mixed $children
	 */
	public function __construct($id, $privileges, $title, $url = null, $alternativeUrls = null, $classInner = null, $classOuter = null, $html = null, $children = null)
	{
		$this->id = $id;
		$this->privileges = $privileges;
		$this->title = $title;
		$this->classInner = $classInner;
		$this->classOuter = $classOuter;
		$this->url = $url;
		$this->alternativeUrls = $alternativeUrls;
		$this->html = $html;
		
		if ($children != null)
		{
			$this->add($children);
		}
		
		return $this;
	}

	
	/**
	 * Add a menu item or items. 
	 * Accepts single item, multiple items or array
	 * @param mixed $menuItem 
	 */
	public function add()
	{
		if (func_num_args() > 0)
		{
			foreach (func_get_args() as $param)
			{
				if (is_object($param) && ($param instanceof Admin_MenuItem))
				{
					$param->parent_id = $this->id;
					
					self::$items[$param->id] = $param;
					
					$this->children[$param->id] = &self::$items[$param->id];
				}
				else if (is_array($param))
				{
					foreach ($param as $item)
					{
						$this->add($item);
					}
				}
			}
		}
	}
	
	/**
	 * Get item parent notes
	 */
	public function getParents()
	{
		$parents = array();
		if (!is_null($this->parent_id))
		{			
			$item = $this;
			while (!is_null($item) && is_object($item) && isset($item->parent_id) && !is_null($item->parent_id))
			{
				$item = isset(self::$items[$item->parent_id]) ? self::$items[$item->parent_id] : null;
				if (!is_null($item))
				{
					$parents[$item->id] = $item;
				}
			}
		}
		return count($parents) > 0 ? $parents : null;
	}
	
	/**
	 * Returns children
	 */
	public function getChildren()
	{
		return count($this->children) > 0 ? $this->children : null;
	}
	
	/**
	 * Check previleges required to view menu item
	 * @param type $privileges
	 * @return type 
	 */
	public function privilegeMatch($privileges)
	{
		// check some specific previleges 
		if (!is_null($this->privileges) && $this->privileges > 0)
		{
			return ($privileges & $this->privileges) > 0;
		}
		return true;
	}
	
	/**
	 *
	 * @param type $privileges 
	 */
	public function getChildrenByPrevileges($adminPrivileges)
	{
		$children = $this->getChildren();

		if (!is_null($children))
		{
			$result = array();
			
			foreach ($children as $item)
			{
				if ($item->privilegeMatch($adminPrivileges))
				{
					$result[$item->id] = &self::$items[$item->id];
				}
			}
			
			return count($result) > 0 ? $result : null;
		}
		
		return null;
	}
}