<?php
/**
 * Payment methods access
 */
class DataAccess_PaymentMethodRepository extends DataAccess_Repository
{
	/**
	 * Get ABS list of non-active payment gateways
	 * @param type $cache
	 * @return type 
	 */
	public function getPaymentMethods($cache = false)
	{
		$_methods = $this->getData('SELECT *, IF(`id` LIKE "paypal%", 1, 2) `paypals` FROM `'.DB_PREFIX.'payment_methods` ORDER BY `paypals`, `name`', $cache);
		$methods = array();
		foreach ($_methods as $method)
		{
			if ($this->methodAvailable($method['id']))
			{
				$methods[] = $method;
			}
		}
	}
	
	/**
	 * Check is method available
	 * @param string $id
	 * @return boolean 
	 */
	public function methodAvailable($id)
	{
		return
			$this->checkAccess('PAYMENT_ALL') || 
			($this->checkAccess('PAYMENT_PAYPAL_BASIC') && in_array($id, array('paypalec', 'paypalipn', 'paypalwps'))) ||
			($this->checkAccess('PAYMENT_PAYPAL_ALL') && substr($id, 0, 6) == 'paypal') ||
			($this->checkAccess('PAYMENT_WORLDPAY') && substr($id, 0, 8) == 'worldpay') ||
			($this->checkAccess('PAYMENT_AUTHORIZE') && substr($id, 0, 12) == 'authorizenet') ||
			($this->checkAccess('PAYMENT_CHASE') && substr($id, 0, 7) == 'orbital') ||
			($this->checkAccess('CCS') && substr($id, 0, 8) == 'cart_ccs');
	}
	
	/**
	 * Return payment method groups
	 * @param boolean $cache
	 * @return mixed
	 */
	public function getPaymentMethodsGroups($cache = false)
	{
		$groups = array(
			"US" => array("name"=>"United States", "methods"=>array()),
			"GB" => array("name"=>"Great Britain", "methods"=>array()),
			"CA" => array("name"=>"Canada", "methods"=>array()),
			"AU" => array("name"=>"Australia", "methods"=>array()),
			"global" => array("name"=>"Global / Other Countries", "methods"=>array()),
			"other" => array("name"=>"Other Options", "methods"=>array()),
		);
		
		$methods = $this->getData('SELECT * FROM `'.DB_PREFIX.'payment_methods` ORDER BY `name`', $cache);
		foreach ($methods as $method)
		{
			if (trim($method["countries_include"]) != '')
			{
				$countries = explode(",", $method["countries_include"]);
				foreach ($countries as $country)
				{
					if ($this->methodAvailable($method['id']))
					{
						$groups[trim($country)]["methods"][] = $method;
					}
				}
			}
		}
		return $groups;
	}
	
	/**
	 * Get payment method's data
	 * @param int $paymentMethodId
	 * @param boolean $cache
	 * @return mixed 
	 */
	public function getPaymentMethodById($paymentMethodId, $cache = false)
	{		
		$paymentMethod = $this->getOne($q = 'SELECT * FROM `'.DB_PREFIX.'payment_methods` WHERE id="'.self::$db->escape($paymentMethodId).'"', $cache);
		
		if (!is_null($paymentMethod) && $this->methodAvailable($paymentMethod['id']))
		{
			$paymentMethod['settings'] = $this->getData('SELECT * FROM `'.DB_PREFIX.'settings` WHERE group_name = "payment_'.self::$db->escape($paymentMethod['id']).'" ORDER BY `priority`', $cache);
			
			foreach ($paymentMethod['settings'] as $key=>$settingsOption)
			{
				if (trim($settingsOption["caption"]) == "")
				{
					$caption = substr($settingsOption["name"], strlen($paymentMethod["id"]), strlen($settingsOption["name"]) - strlen($paymentMethod["id"]));
					$caption = ucwords(strtolower(str_replace("_", " ", $caption)));
					$paymentMethod['settings'][$key]["caption"] = $caption;
				}
			}
			
			/**
			 * Get payment methods 
			 */
			$paymentMethod['supported_cards'] = $this->getData('SELECT * FROM `'.DB_PREFIX.'payment_accepted_cc` WHERE pid="'.intval($paymentMethod['pid']).'" ORDER BY `name`');
			
			/**
			 * Get logo 
			 */
			$path = 'content/admin/skins/default/templates/pages/payment/solutions/'.escapeFileName($paymentMethod['id']).'/';
			
			if (is_file($path.'logo.png'))
				$paymentMethod['logo'] = $path.'logo.png';
			else if (is_file($path.'logo.gif'))
				$paymentMethod['logo'] = $path.'logo.gif';
			else if (is_file($path.'logo.jpg'))
				$paymentMethod['logo'] = $path.'logo.jpg';
			else
				$paymentMethod['logo'] = false;
		}
		
		return $paymentMethod;
	}		
	
}
