<?php
/**
 * Class DataAccess_SettingsRepository
 */
class DataAccess_SettingsRepository
{
	protected static $db;
	protected static $settings;

	/**
	 * @param DB $db
	 * @param null $settings
	 */
	public function __construct(DB $db = null, &$settings = null)
	{
		if (is_null($db))
		{
			global $db;
		}

		self::$db = $db;

		// force pass by reference
		if (is_null($settings))
		{
			global $settings;
			self::$settings = &$settings;
		}
		else
		{
			self::$settings = $settings;
		}
	}

	/**
	 * @return DataAccess_SettingsRepository|null
	 */
	public static function getInstance()
	{
		static $instance = null;
		if (is_null($instance)) $instance = new self;
		return $instance;
	}

	/**
	 * @param $settings
	 * @param null $allowed
	 */
	public function save($settings, $allowed = null)
	{
		// filter settings, possibly may be optimized
		if (!is_null($allowed) && is_array($allowed))
		{
			$_settings = array();
			foreach ($settings as $key => $value)
			{
				if (in_array($key, $allowed))
				{
					$_settings[$key] = $value;
				}
			}
			$settings = $_settings;
		}

		foreach ($settings as $key => $value)
		{
			self::$db->query('UPDATE '.DB_PREFIX.'settings SET value="'.self::$db->escape($value).'" WHERE name="'.self::$db->escape($key).'"');
			self::$settings[$key] = $value;
		}
	}

	/**
	 * @param $key
	 * @param null $default
	 * @return null
	 */
	public function get($key, $default = null)
	{
		return isset(self::$settings[$key]) ? self::$settings[$key] : $default;
	}

	/**
	 * @param $keys
	 * @return array
	 */
	public function getByKeys($keys)
	{
		$result = array();

		foreach ($keys as $key)
		{
			$result[$key] = $this->get($key);
		}

		return $result;
	}

	/**
	 * @return array
	 */
	public function getAll()
	{
		return self::$settings;
	}
}