<?php

/**
 * Countries Repository
 */

class DataAccess_ShippingRepository implements DataAccess_ShippingRepositoryInterface
{
	protected $db;

	public function __construct($db = null)
	{
		if (is_null($db))
		{
			global $db;
		}

		$this->db = $db;
	}

	//TODO: Wrong place for these methods orders, orders_contents, countries, where should they go?
	/**
	 * @param int $orderId
	 * @return type
	 */
	public function getOrderItems($orderId)
	{
		return $this->db->selectAll("
			SELECT
				".DB_PREFIX."orders_content.*,
				'No' AS product_is_shipping_price,
				'0.00' AS product_shipping_price
			FROM ".DB_PREFIX."orders_content
			WHERE ".DB_PREFIX."orders_content.oid='".intval($orderId)."'
		");
	}

	/**
	 * Get order items with shipping price
	 *
	 * @param int $orderId
	 * @param int $productLevelMethodId
	 *
	 * @return array
	 */
	public function getOrderItemsWithShippingPrice($orderId, $productLevelMethodId)
	{
		return $this->db->selectAll("
			SELECT
				".DB_PREFIX."orders_content.*,
				if (".DB_PREFIX."products_shipping_price.is_price IS NULL, 'No', ".DB_PREFIX."products_shipping_price.is_price) AS product_is_shipping_price,
				if (".DB_PREFIX."products_shipping_price.price IS NULL, 0.00, ".DB_PREFIX."products_shipping_price.price) AS product_shipping_price
			FROM ".DB_PREFIX."orders_content
			LEFT JOIN ".DB_PREFIX."products_shipping_price ON 
			".DB_PREFIX."products_shipping_price.pid = ".DB_PREFIX."orders_content.pid AND 
			".DB_PREFIX."products_shipping_price.ssid='".intval($productLevelMethodId)."'
			WHERE ".DB_PREFIX."orders_content.oid='".intval($orderId)."'
		");
	}

	/**
	 * Get country by id
	 *
	 * @param int $countryId
	 *
	 * @return mixed
	 */
	public function getCountryById($countryId)
	{
		return $this->db->selectOne('SELECT * FROM '.DB_PREFIX.'countries WHERE coid='.intval($countryId), MYSQL_ASSOC);
	}

	/**
	 * Get state by country / state id
	 *
	 * @param $countryId
	 * @param $stateId
	 *
	 * @return mixed
	 */
	public function getStateById($countryId, $stateId)
	{
		return $this->db->selectOne('SELECT * FROM '.DB_PREFIX.'states WHERE coid='.intval($countryId).' AND stid='.intval($stateId), MYSQL_ASSOC);
	}

	/**
	 * Get shipping origin address
	 *
	 * @param $orderId
	 * @param $settings
	 *
	 * @return array
	 */
	public function getShippingOriginAddress($orderId, &$settings)
	{
		$originAddress = array(
			'address1' => $settings['ShippingOriginAddressLine1'],
			'address2' => $settings['ShippingOriginAddressLine2'],
			'city' => $settings['ShippingOriginCity'],
			'zip' => $settings['ShippingOriginCountry'] == '1' ? substr(trim($settings['ShippingOriginZip']), 0, 5) : trim($settings['ShippingOriginZip'])
		);

		$countryData = $this->getCountryById($settings['ShippingOriginCountry']);
		if ($countryData)
		{
			$originAddress['country_id'] = $countryData['coid'];
			$originAddress['country'] = $countryData['name'];
			$originAddress['country_iso_a2'] = $countryData['iso_a2'];
			$originAddress['country_iso_a3'] = $countryData['iso_a3'];
			$originAddress['country_iso_number'] = $countryData['iso_number'];
		}
		else
		{
			$originAddress['country_id'] = 1;
			$originAddress['country'] = 'United States';
			$originAddress['country_iso_a2'] = 'US';
			$originAddress['country_iso_a3'] = 'USA';
			$originAddress['country_iso_number'] = '840';
		}

		$stateData = $this->getStateById($originAddress['country_id'], $settings['ShippingOriginState']);
		if ($stateData)
		{
			$originAddress['state_id'] = $stateData['stid'];
			$originAddress['state_name'] = $stateData['name'];
			$originAddress['state_code'] = $stateData['short_name'];
		}
		else
		{
			$originAddress['state_id'] = 0;
			$originAddress['state_name'] = $originAddress['state_code'] = $settings['ShippingOriginProvince'];
		}

		return $originAddress;
	}

	/**
	 * Get order shipping address
	 *
	 * @param int $orderId
	 *
	 * @return type
	 */
	public function getOrderShippingAddress($orderId)
	{
		return $this->db->selectOne("
			SELECT
				".DB_PREFIX."orders.shipping_address_type,
				".DB_PREFIX."orders.shipping_name,
				".DB_PREFIX."orders.shipping_address1,
				".DB_PREFIX."orders.shipping_address2,
				".DB_PREFIX."orders.shipping_city,
				".DB_PREFIX."orders.shipping_province,
				".DB_PREFIX."orders.shipping_country,
				".DB_PREFIX."orders.shipping_zip,
				".DB_PREFIX."countries.name AS country_name,
				".DB_PREFIX."countries.iso_a2,
				".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
				".DB_PREFIX."countries.iso_a3,
				".DB_PREFIX."countries.iso_a3 AS country_iso_a3, 
				".DB_PREFIX."countries.iso_number,
				".DB_PREFIX."countries.iso_number AS country_iso_number,
				".DB_PREFIX."states.name AS state_name,
				".DB_PREFIX."states.short_name AS state_abbr,
				".DB_PREFIX."states.short_name AS state_code
			FROM ".DB_PREFIX."orders
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."orders.shipping_state
			WHERE oid='".intval($orderId)."'
		", MYSQL_ASSOC);
	}

	/**
	 * Update order items shipping price
	 *
	 * @param int $orderItemId
	 * @param float $shippingPrice
	 */
	public function updateOrderItemShippingPrice($orderItemId, $shippingPrice)
	{
		$db = $this->db;

		$db->reset();
		$db->assignStr("is_shipping_price", "Yes");
		$db->assignStr("shipping_price", $shippingPrice);
		$db->update(DB_PREFIX."orders_content", "WHERE ocid='".intval($orderItemId)."'");
	}

    /**
     * Returns shipping method data
     * 
     * @param int $shippingMethodId
     * 
     * @return mixed
     */
    public function getShippingMethodData($shippingMethodId)
    {
        return $this->db->selectOne("
            SELECT * FROM ".DB_PREFIX."shipping_selected 
            WHERE ssid='".intval($shippingMethodId)."'
        ", MYSQL_ASSOC);
    }

	/**
	 * Get custom shipping rates
     * 
	 * @param int $shippingMethodId
	 * @param string $shippingMethodType
	 * @param int $itemsCount
	 * @param float $itemsPriceBasedPrice
	 * @param float $weight
     * 
	 * @return mixed
	 */
	public function getCustomRate($shippingMethodId, $shippingMethodType, $itemsCount, $itemsPriceBasedPrice, $weight)
    {
		return $this->db->selectOne("
			SELECT * FROM ".DB_PREFIX."shipping_custom_rates 
			WHERE 
				ssid = '".intval($shippingMethodId)."' AND
				rate_min <= ".($shippingMethodType == "flat" ? intval($itemsCount) : floatval($shippingMethodType == "price" ? $itemsPriceBasedPrice : $weight))." AND 
				rate_max >= ".($shippingMethodType == "flat" ? intval($itemsCount) : floatval($shippingMethodType == "price" ? $itemsPriceBasedPrice : $weight))."
			ORDER BY rate_min LIMIT 1
		", MYSQL_ASSOC);
    }

	/**
	 * Make fake address for shipping quotes
	 *
	 * @param int $countryId
	 * @param int $stateId
	 * @param string $zip
	 *
	 * @return array
	 */
	public function makeShippingQuotesAddress($countryId, $stateId, $zip)
	{
		$address = $this->db->selectOne("
			SELECT
				'John Doe' AS name,
				'Some Company, LLC' AS company,
				'123 Some Street' AS address1,
				'' AS address2,
				'Bestcity' AS city,
				".DB_PREFIX."countries.coid AS country_id,
				".DB_PREFIX."countries.name AS country,
				".DB_PREFIX."countries.name AS country_name,
				".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
				".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
				".DB_PREFIX."countries.iso_number AS country_iso_number,
				".DB_PREFIX."states.short_name AS state_abbr,
				".DB_PREFIX."states.name AS state,
				".DB_PREFIX."states.name AS province,
				".DB_PREFIX."states.stid AS state_id
			FROM ".DB_PREFIX."countries, ".DB_PREFIX."states
			WHERE ".DB_PREFIX."countries.coid = ".intval($countryId)." AND ".DB_PREFIX."states.stid = ".intval($stateId)."
		", MYSQL_ASSOC);

		if (!$address)
		{
			$address = array(
				"country_id" => $countryId,
				"state_id" => $stateId,
			);
		}

		$address["shipping_zip"] = $zip;
		$address["zip"] = $zip;

		return $address;
	}

	/**
	 * Select active shipping methods
	 * 
	 * @return mixed
	 */
	public function getActiveShippingMethods()
	{
		return $this->db->selectAll("
			SELECT * 
			FROM ".DB_PREFIX."shipping_selected 
			WHERE ".DB_PREFIX."shipping_selected.hidden='No' 
			ORDER BY priority
		");
	}
}