<?php
/**
 * Class Framework_Request
 */
class Framework_Request
{
	/**
	 * @var Framework_ParameterBag
	 */
	public $request;

	/**
	 * @var Framework_ParameterBag
	 */
	public $query;

	/**
	 * @var Framework_ParameterBag
	 */
	public $server;

	/**
	 * Class constructor
	 *
	 * @param array $query
	 * @param array $request
	 * @param array $server
	 */
	public function __construct(array $query = array(), array $request = array(), array $server = array())
	{
		$this->query = new Framework_ParameterBag($query);
		$this->request = new Framework_ParameterBag($request);
		$this->server = new Framework_ParameterBag($server);
	}

	/**
	 * @return Framework_ParameterBag
	 */
	public function getRequest()
	{
		return $this->request;
	}

	/**
	 * @return Framework_ParameterBag
	 */
	public function getQuery()
	{
		return $this->query;
	}

	/**
	 * @return Framework_ParameterBag
	 */
	public function getServer()
	{
		return $this->server;
	}

	/**
	 * Create request from globals
	 *
	 * @return Framework_Request
	 */
	public static function createFromGlobals()
	{
		$request = new self($_GET, $_POST, $_SERVER);

		return $request;
	}

	/**
	 * Return request parameter - get first, then post
	 *
	 * @param string $key
	 * @param mixed $default
	 *
	 * @return mixed
	 */
	public function get($key, $default = null)
	{
		return $this->query->get($key, $this->request->get($key, $default));
	}

	/**
	 * Checks is request a post request
	 * @return bool
	 */
	public function isPost()
	{
		return strtolower($this->server->get('REQUEST_METHOD')) == 'post';
	}
}