<?php

class Listener_ProductGifts
{
	protected static $orderRepository;

	protected static function getOrderRepository()
	{
		if (is_null(self::$orderRepository))
		{
			self::$orderRepository = new DataAccess_OrderRepository();
		}

		return self::$orderRepository;
	}

	public static function setOrderRepository(DataAccess_OrderRepositoryInterface $orderRepository)
	{
		self::$orderRepository = $orderRepository;
	}

	public static function checkFreeProducts(Events_OrderEvent $event)
	{
		$orderRepository = self::getOrderRepository();

		$order = $event->getOrder();

		$items = array();
		$lineItemsIdsToDelete = array();
		$productQuantitiesMap = array();
		foreach ($order->lineItems as $id => $item)
		{
			/* @var $item Model_LineItem */
			if ($item->getIsGift() == 'Yes')
			{
				$lineItemsIdsToDelete[] = $id;
			}
			else
			{
				$items[] = $item->toArray();

				if (array_key_exists($item->getPid(), $productQuantitiesMap))
				{
					$productQuantitiesMap[$item->getPid()] += $item->getFinalQuantity();
				}
				else
				{
					$productQuantitiesMap[$item->getPid()] = $item->getFinalQuantity();
				}
			}
		}

		foreach ($lineItemsIdsToDelete as $id)
		{
			unset($order->lineItems[$id]);
		}
		$order->items = $items;

		$orderRepository->deleteLineItems($order->getId(), $lineItemsIdsToDelete);

		$productsGiftsData = $orderRepository->getProductsGiftsData($order->getId());

		foreach ($productsGiftsData as $productGiftData)
		{
			$originalPid = $productGiftData['original_pid'];

			if (array_key_exists($originalPid, $productQuantitiesMap) && $productGiftData['gift_min_quantity'] > 0)
			{
				$quantity = $productQuantitiesMap[$originalPid];
				if ($quantity >= $productGiftData['gift_min_quantity'])
				{
					$multiplier = floor($quantity / $productGiftData['gift_min_quantity']);
					$giftQuantity = $multiplier * $productGiftData['gift_quantity'];

					if ($giftQuantity > $productGiftData['gift_max_quantity'])
					{
						$giftQuantity = $productGiftData['gift_max_quantity'];
					}

					if ($giftQuantity > 0)
					{
						// Duplicates a bit of $order->addItem(
						$lineItem = Model_LineItem::createNewGiftLineItem($order->getId(), new Model_Product($productGiftData), $giftQuantity);

						$orderRepository->persistLineItem($order, $lineItem, true);

						$order->lineItems[$lineItem->getId()] = $lineItem;
						$order->items[] = $lineItem->toArray();
					}
				}
			}
		}
	}
}