<?php

class Listener_StockControl
{
	/**
	 * Handle on order status change event
	 * 
	 * @param Events_OrderEvent $event
	 */
	public static function onOrderStatusChange(Events_OrderEvent $event)
	{
		global $settings;

		$order = $event->getOrder();
		$oid = $order->getId();

		$oldOrderStatus = $event->getData('oldOrderStatus');
		$oldPaymentStatus = $event->getData('oldPaymentStatus');
		$newOrderStatus = $event->getData('newOrderStatus');
		$newPaymentStatus = $event->getData('newPaymentStatus');

		if (in_array($oldOrderStatus, array('Abandon', 'New')) && in_array($newOrderStatus, array('Process', 'Completed')) && $settings["InventoryStockUpdateAt"] == "Order Placed")
		{
			self::stockControl($order);
		}
		else if ($newPaymentStatus == "Received" && $oldPaymentStatus != "Received" && $settings["InventoryStockUpdateAt"] == "Payment Received")
		{
			self::stockControl($order);
		}
		else if ($newOrderStatus == 'Completed' && $oldOrderStatus != 'Completed' && $settings["InventoryStockUpdateAt"] == "Order Completed")
		{
			self::stockControl($order);
		}
		else if ($newOrderStatus == 'Canceled' && $oldOrderStatus != 'Canceled')
		{
			self::returnToInventory($order, true);
		}
	}

	/**
	 * Stock control
	 * at this point inventory control based on 2 "after checkout" conditions
	 * 1) stock_warning should be < then stock
	 * 2) min_order should be <= then stock
	 */
	protected static function stockControl(ORDER $order)
	{
		global $db;

		// select orders products with allowed stock control from database
		$db->query("
			SELECT ".DB_PREFIX."orders_content.*,
				".DB_PREFIX."products.stock,
				".DB_PREFIX."products.stock_warning,
				".DB_PREFIX."products.min_order,
				".DB_PREFIX."products.max_order,
				".DB_PREFIX."products.inventory_rule
			FROM ".DB_PREFIX."orders_content 
			LEFT JOIN ".DB_PREFIX."products ON 
				".DB_PREFIX."products.pid = ".DB_PREFIX."orders_content.pid AND
				(".DB_PREFIX."products.inventory_control = 'Yes' OR ".DB_PREFIX."products.inventory_control = 'AttrRuleExc' OR ".DB_PREFIX."products.inventory_control = 'AttrRuleInc')
			WHERE 
				".DB_PREFIX."orders_content.oid=".$order->getId()."
		");

		// store products in temporary array
		$products = array();
		while ($db->moveNext())
		{
			$products[$db->col["ocid"]] = array(
				"stock"=>$db->col["stock"],
				"stock_warning"=>$db->col["stock_warning"],
				"pid"=>$db->col["pid"],
				"quantity"=>$db->col["admin_quantity"],
				"min_order"=>$db->col["min_order"],
				"inventory_id"=>$db->col["inventory_id"],
				"inventory_rule"=>$db->col["inventory_rule"]
			);
		}

		// update stock for products
		reset($products);
		foreach ($products as $ocid=>$product)
		{
			$inventory_id = $product["inventory_id"];
			if ($inventory_id > 0)
			{
				//////////////////////////////////////////////////
				// stock on attribute level
				$db->query("SELECT * FROM ".DB_PREFIX."products_inventory WHERE pi_id='".$inventory_id."'");
				if ($db->moveNext())
				{
					$new_attr_stock = $db->col["stock"] - $product["quantity"];
					$new_attr_stock = $new_attr_stock < 0 ? 0 : $new_attr_stock;
					if ($new_attr_stock <= $db->col["stock_warning"])
					{
						emailOutOfStock($db, $product["pid"], $new_attr_stock, $inventory_id);
					}
					$db->reset();
					$db->assignStr("stock", $new_attr_stock);
					$db->update(DB_PREFIX."products_inventory", "WHERE pid='".$product["pid"]."' AND pi_id='".$inventory_id."'");
					$db->query("UPDATE ".DB_PREFIX."products SET stock = stock-".$product["quantity"]." WHERE pid='".$product["pid"]."' ");
				}
			}
			else
			{
				//////////////////////////////////////////////////
				// stock on product level
				$new_stock = $product["stock"] - $product["quantity"];
				$new_stock = $new_stock < 0 ? 0 : $new_stock;
				$db->reset();
				$db->assign("stock", $new_stock);
				$db->update(DB_PREFIX."products", "WHERE pid = '".$product["pid"]."'");

				if (
					(!is_null($product["stock_warning"]) && $new_stock <= $product["stock_warning"]) ||
					(!is_null($product["min_order"]) && ($product["min_order"]>0) && ($new_stock < $product["min_order"]))
				)
				{
					emailOutOfStock($db, $product["pid"], $new_stock, 0);
				}
			}
			$db->reset();
			$db->assignStr("is_stock_changed", "Yes");
			$db->update(DB_PREFIX."orders_content", "WHERE ocid='".$ocid."'");
		}
	}

	/**
	 * Public function return items to inventory
	 *
	 * @param bool $do_make_visible
	 */
	function returnToInventory(ORDER $order, $do_make_visible)
	{
		global $db;

		/**
		 * return on attribute level
		 */
		$db->query("
			SELECT
				".DB_PREFIX."orders_content.quantity AS total_quantity,
				".DB_PREFIX."orders_content.pid,
				".DB_PREFIX."orders_content.inventory_id,
				".DB_PREFIX."products_inventory.is_active,
				".DB_PREFIX."products_inventory.stock,
				".DB_PREFIX."products_inventory.stock_warning
			FROM ".DB_PREFIX."orders_content
			INNER JOIN ".DB_PREFIX."products_inventory ON ".DB_PREFIX."products_inventory.pi_id=".DB_PREFIX."orders_content.inventory_id
			WHERE
				".DB_PREFIX."orders_content.oid=".$order->getId()." AND
				".DB_PREFIX."orders_content.is_stock_changed='Yes' AND
				".DB_PREFIX."orders_content.inventory_id > 0
		");

		if ($db->numRows() > 0)
		{
			$products = $db->getRecords();

			foreach ($products as $product)
			{
				$new_attr_stock = $product["stock"] + $product["total_quantity"];
				$db->reset();
				$db->assignStr("stock", $new_attr_stock);

				if ($do_make_visible && $new_attr_stock >= $product["stock_warning"])
				{
					$db->assignStr("is_active", "1");
				}

				$db->update(DB_PREFIX."products_inventory", "WHERE pi_id=".intval($product["inventory_id"]));
			}
		}

		/**
		 * return on product level
		 * reason for grouping is there maybe be the same product but with different attributes
		 */
		$db->query("
			SELECT
				SUM(".DB_PREFIX."orders_content.quantity) AS total_quantity,
				".DB_PREFIX."orders_content.pid,
				".DB_PREFIX."products.is_visible,
				".DB_PREFIX."products.stock,
				".DB_PREFIX."products.stock_warning
			FROM ".DB_PREFIX."orders_content
			INNER JOIN ".DB_PREFIX."products ON ".DB_PREFIX."products.pid=".DB_PREFIX."orders_content.pid AND ".DB_PREFIX."products.inventory_control='Yes'
			WHERE
				".DB_PREFIX."orders_content.oid=".$order->getId()." AND
				".DB_PREFIX."orders_content.is_stock_changed='Yes'
			GROUP BY ".DB_PREFIX."products.pid
		");

		if ($db->numRows() > 0)
		{
			$products = $db->getRecords();

			foreach ($products as $product)
			{
				$new_stock = $product["stock"] + $product["total_quantity"];
				$db->reset();
				$db->assignStr("stock", $new_stock);

				if ($do_make_visible && $new_stock >= $product["stock_warning"])
				{
					$db->assignStr("is_visible", "Yes");
				}

				$db->update(DB_PREFIX."products", "WHERE pid=".intval($product["pid"]));
			}

			$db->query("UPDATE ".DB_PREFIX."orders_content SET is_stock_changed='No' WHERE oid=".$order->getId());
		}
	}
}