<?php

require_once dirname(dirname(__FILE__)).'/class.product_attribute.php';

class Model_LineItem
{
	protected $values;
	protected $product;

	/**
	 * Class constructor
	 *
	 * @param array $lineItemValues
	 */
	public function __construct(&$lineItemValues)
	{
		$this->values = &$lineItemValues;
		$this->parseValues();
	}

	/**
	 * Sets the admin price
	 *
	 * @param float $adminPrice
	 */
	public function setAdminPrice($adminPrice)
	{
		$this->values['admin_price'] = $adminPrice;

		// Update product_total
		// These values for backward support with themes / display
		$this->values['product_price'] = $adminPrice;
		$this->values['product_total'] = $this->getSubtotal();
	}

	/**
	 * Gets the admin price
	 *
	 * @return float
	 */
	public function getAdminPrice()
	{
		return $this->values['admin_price'];
	}

	/**
	 * Sets the price
	 *
	 * @param float $price
	 */
	public function setPrice($price)
	{
		$this->values['price'] = $price;
	}

	/**
	 * Gets the price
	 *
	 * @return float
	 */
	public function getPrice()
	{
		return $this->values['price'];
	}

	/**
	 * Sets the admin quantity
	 *
	 * @param int $adminQuantity
	 */
	public function setAdminQuantity($adminQuantity)
	{
		$this->values['admin_quantity'] = $adminQuantity;

		// Update product_total
		$this->values['product_total'] = $this->getSubtotal();
	}

	/**
	 * Gets the admin quantity
	 *
	 * @return int
	 */
	public function getAdminQuantity()
	{
		return $this->values['admin_quantity'];
	}

	/**
	 * Sets the quantity
	 *
	 * @param int $quantity
	 */
	public function setQuantity($quantity)
	{
		$this->values['quantity'] = $quantity;
	}

	/**
	 * Gets the quantity
	 *
	 * @return int
	 */
	public function getQuantity()
	{
		return $this->values['quantity'];
	}

	/**
	 * Sets the final price
	 *
	 * @param float $finalPrice
	 */
	public function setFinalPrice($finalPrice)
	{
		$this->setPrice($finalPrice);
		$this->setAdminPrice($finalPrice);
	}

	/**
	 * Gets the final price
	 *
	 * @return float
	 */
	public function getFinalPrice()
	{
		return $this->getAdminPrice();
	}

	/**
	 * Sets the final quantity
	 *
	 * @param int $finalQuantity
	 */
	public function setFinalQuantity($finalQuantity)
	{
		$this->setQuantity($finalQuantity);
		$this->setAdminQuantity($finalQuantity);
	}

	/**
	 * Gets the final quantity
	 *
	 * @return int
	 */
	public function getFinalQuantity()
	{
		return $this->getAdminQuantity();
	}

	/**
	 * Sets the discount amount
	 *
	 * @param float $discountAmount
	 */
	public function setDiscountAmount($discountAmount)
	{
		$this->values['discount_amount'] = $discountAmount;

		$this->calculatePriceWithTax();
	}

	/**
	 * Gets the discount amount
	 *
	 * @return float
	 */
	public function getDiscountAmount()
	{
		return $this->values['discount_amount'];
	}

	/**
	 * Sets the promo code discount amount
	 *
	 * @param float $promoDiscountAmount
	 */
	public function setPromoDiscountAmount($promoDiscountAmount)
	{
		$this->values['promo_discount_amount'] = $promoDiscountAmount;

		$this->calculatePriceWithTax();
	}

	/**
	 * Gets the promo code discount amount
	 *
	 * @return float
	 */
	public function getPromoDiscountAmount()
	{
		return $this->values['promo_discount_amount'];
	}

	/**
	 * Sets the line item attribute value
	 *
	 * @param float $attributeValue
	 */
	public function setAttributeValue($attributeValue)
	{
		$this->values['attribute_value'] = $attributeValue;
	}

	/**
	 * Gets the line item attribute modifier value
	 *
	 * @return float
	 */
	public function getAttributeValue()
	{
		return $this->values['attribute_value'];
	}

	/**
	 * Sets the line item attribute value type
	 *
	 * Valid values: amount or percentage
	 *
	 * @param string $attributeValueType
	 */
	public function setAttributeValueType($attributeValueType)
	{
		$this->values['attribute_value_type'] = $attributeValueType;
	}

	/**
	 * Gets the line item attribute value type
	 *
	 * @return string
	 */
	public function getAttributeValueType()
	{
		return $this->values['attribute_value_type'];
	}

	/**
	 * Sets the product cost
	 *
	 * @param float $cost
	 */
	public function setCost($cost)
	{
		$this->values['cost'] = $cost;
	}

	/**
	 * Gets the product cost
	 *
	 * @return float
	 */
	public function getCost()
	{
		return $this->values['cost'];
	}

	/**
	 * Sets whether this is a digital product
	 *
	 * Valid values: Yes or No
	 *
	 * @param string $digitalProduct
	 */
	public function setDigitalProduct($digitalProduct)
	{
		$this->values['digital_product'] = $digitalProduct;
	}

	/**
	 * Gets whether this is a digital product
	 *
	 * @return string
	 */
	public function getDigitalProduct()
	{
		return $this->values['digital_product'];
	}

	/**
	 * Sets the number of times the digital item has been downloaded
	 *
	 * @param int $digitalProductDownloads
	 */
	public function setDigitalProductDownloads($digitalProductDownloads)
	{
		$this->values['digital_product_downloads'] = $digitalProductDownloads;
	}

	/**
	 * Gets the number of times the digital item has been downloaded
	 *
	 * @return int
	 */
	public function getDigitalProductDownloads()
	{
		return $this->values['digital_product_downloads'];
	}

	/**
	 * Sets the digital product filename
	 *
	 * @param string $digitalProductFile
	 */
	public function setDigitalProductFile($digitalProductFile)
	{
		$this->values['digital_product_file'] = $digitalProductFile;
	}

	/**
	 * Gets the digital product filename
	 *
	 * @return string
	 */
	public function getDigitalProductFile()
	{
		return $this->values['digital_product_file'];
	}

	/**
	 * Sets the digital product key
	 *
	 * @param string $digitalProductKey
	 */
	public function setDigitalProductKey($digitalProductKey)
	{
		$this->values['digital_product_key'] = $digitalProductKey;
	}

	/**
	 * Gets the digital product key
	 *
	 * @return string
	 */
	public function getDigitalProductKey()
	{
		return $this->values['digital_product_key'];
	}

	/**
	 * Sets whether this line item has free shipping
	 *
	 * Valid values: Yes or No
	 *
	 * @param string $freeShipping
	 */
	public function setFreeShipping($freeShipping)
	{
		$this->values['free_shipping'] = $freeShipping;
	}

	/**
	 * Gets whether this line item has free shipping
	 *
	 * @return string
	 */
	public function getFreeShipping()
	{
		return $this->values['free_shipping'];
	}

	/**
	 * Sets the inventory id
	 *
	 * @param int $inventoryId
	 */
	public function setInventoryId($inventoryId)
	{
		$this->values['inventory_id'] = $inventoryId;
	}

	/**
	 * Gets the inventory id
	 *
	 * @return int
	 */
	public function getInventoryId()
	{
		return $this->values['inventory_id'];
	}

	/**
	 * Sets is doba item
	 *
	 * Valid values: Yes or No
	 *
	 * @param string $isDoba
	 */
	public function setIsDoba($isDoba)
	{
		$this->values['is_doba'] = $isDoba;
	}

	/**
	 * Gets is doba item
	 *
	 * @return string
	 */
	public function getIsDoba()
	{
		return $this->values['is_doba'];
	}

	/**
	 * Sets is dollar days
	 *
	 * Valid values: Yes or No
	 *
	 * @param string $isDollarDays
	 */
	public function setIsDollarDays($isDollarDays)
	{
		$this->values['is_dollar_days'] = $isDollarDays;
	}

	/**
	 * Gets is dollar days item
	 *
	 * @return string
	 */
	public function getIsDollarDays()
	{
		return $this->values['is_dollar_days'];
	}

	/**
	 * Sets is gift product item
	 *
	 * Valid values: Yes or No
	 *
	 * @param string $isGift
	 */
	public function setIsGift($isGift)
	{
		$this->values['is_gift'] = $isGift;
	}

	/**
	 * Gets is gift product item
	 *
	 * @return string
	 */
	public function getIsGift()
	{
		return $this->values['is_gift'];
	}

	/**
	 * Set is shipping price
	 *
	 * @param string $isShippingPrice
	 */
	public function setIsShippingPrice($isShippingPrice)
	{
		$this->values['is_shipping_price'] = $isShippingPrice;
	}

	/**
	 * Get is shipping price
	 *
	 * @return string
	 */
	public function getIsShippingPrice()
	{
		return $this->values['is_shipping_price'];
	}

	/**
	 * Set is stock changed
	 *
	 * @param string $isStockChanged
	 */
	public function setIsStockChanged($isStockChanged)
	{
		$this->values['is_stock_changed'] = $isStockChanged;
	}

	/**
	 * Get is stock changes
	 *
	 * @return string
	 */
	public function getIsStockChanged()
	{
		return $this->values['is_stock_changed'];
	}

	/**
	 * Set is taxable
	 *
	 * @param string $isTaxable
	 */
	public function setIsTaxable($isTaxable)
	{
		$this->values['is_taxable'] = $isTaxable;
	}

	/**
	 * Get is taxable
	 *
	 * @return string
	 */
	public function getIsTaxable()
	{
		return $this->values['is_taxable'];
	}

	/**
	 * Set line item id
	 * @param mixed $id
	 */
	public function setId($id)
	{
		$this->values['ocid'] = $id;
	}

	/**
	 * Get line item id
	 *
	 * @return mixed
	 */
	public function getId()
	{
		return $this->values['ocid'];
	}

	/**
	 * Set line item options
	 *
	 * @param mixed $options
	 */
	public function setOptions($options)
	{
		$this->values['options'] = $options;
	}

	/**
	 * Get line item options
	 *
	 * @return mixed
	 */
	public function getOptions()
	{
		return $this->values['options'];
	}

	/**
	 * Set options clean
	 *
	 * @param mixed $optionsClean
	 */
	public function setOptionsClean($optionsClean)
	{
		$this->values['options_clean'] = $optionsClean;
	}

	/**
	 * Get options clean
	 *
	 * @return mixed
	 */
	public function getOptionsClean()
	{
		return $this->values['options_clean'];
	}

	/**
	 * Set order id
	 *
	 * @param mixed $orderId
	 */
	public function setOrderId($orderId)
	{
		$this->values['oid'] = $orderId;
	}

	/**
	 * Get order id
	 *
	 * @return mixed
	 */
	public function getOrderId()
	{
		return $this->values['oid'];
	}

	/**
	 * Set product id
	 *
	 * @param mixed $pid
	 */
	public function setPid($pid)
	{
		$this->values['pid'] = $pid;
	}

	/**
	 * Get product id
	 *
	 * @return mixed
	 */
	public function getPid()
	{
		return $this->values['pid'];
	}

	/**
	 * Set price before quantity discount
	 *
	 * @param mixed $priceBeforeQuantityDiscount
	 */
	public function setPriceBeforeQuantityDiscount($priceBeforeQuantityDiscount)
	{
		$this->values['price_before_quantity_discount'] = $priceBeforeQuantityDiscount;
	}

	/**
	 * Get price before quantity discount
	 *
	 * @return mixed
	 */
	public function getPriceBeforeQuantityDiscount()
	{
		return $this->values['price_before_quantity_discount'];
	}

	/**
	 * Set product GTIN
	 *
	 * @param mixed $productGtin
	 */
	public function setProductGtin($productGtin)
	{
		$this->values['product_gtin'] = $productGtin;
	}

	/**
	 * Get product GTIN
	 *
	 * @return mixed
	 */
	public function getProductGtin()
	{
		return $this->values['product_gtin'];
	}

	/**
	 * Set product id
	 *
	 * @param mixed $productId
	 */
	public function setProductId($productId)
	{
		$this->values['product_id'] = $productId;
	}

	/**
	 * Get product id
	 *
	 * @return mixed
	 */
	public function getProductId()
	{
		return $this->values['product_id'];
	}

	/**
	 * Set product MPN
	 *
	 * @param mixed $productMpn
	 */
	public function setProductMpn($productMpn)
	{
		$this->values['product_mpn'] = $productMpn;
	}

	/**
	 * Get product MPN
	 *
	 * @return mixed
	 */
	public function getProductMpn()
	{
		return $this->values['product_mpn'];
	}

	/**
	 * Set product removed
	 *
	 * @param string $productRemoved
	 */
	public function setProductRemoved($productRemoved)
	{
		$this->values['product_removed'] = $productRemoved;
	}

	/**
	 * Get product remove
	 *
	 * @return string
	 */
	public function getProductRemoved()
	{
		return $this->values['product_removed'];
	}

	/**
	 * Set product SKU
	 *
	 * @param mixed $productSku
	 */
	public function setProductSku($productSku)
	{
		$this->values['product_sku'] = $productSku;
	}

	/**
	 * Get product SKU
	 *
	 * @return mixed
	 */
	public function getProductSku()
	{
		return $this->values['product_sku'];
	}

	/**
	 * Set product sub id
	 *
	 * @param mixed $productSubId
	 */
	public function setProductSubId($productSubId)
	{
		$this->values['product_sub_id']= $productSubId;
	}

	/**
	 * Get product sub id
	 *
	 * @return mixed
	 */
	public function getProductSubId()
	{
		return $this->values['product_sub_id'];
	}

	/**
	 * Set product Upc
	 *
	 * @param mixed $productUpc
	 */
	public function setProductUpc($productUpc)
	{
		$this->values['product_upc'] = $productUpc;
	}

	/**
	 * Get product Upc
	 *
	 * @return mixed
	 */
	public function getProductUpc()
	{
		return $this->values['product_upc'];
	}

	/**
	 * Set product location id
	 *
	 * @param mixed $productsLocationId
	 */
	public function setProductsLocationId($productsLocationId)
	{
		$this->values['products_location_id'] = $productsLocationId;
	}

	/**
	 * Get product location id
	 *
	 * @return mixed
	 */
	public function getProductsLocationId()
	{
		return $this->values['products_location_id'];
	}

	/**
	 * Set product shipping price
	 *
	 * @param mixed $shippingPrice
	 */
	public function setShippingPrice($shippingPrice)
	{
		$this->values['shipping_price'] = $shippingPrice;
	}

	/**
	 * Get shipping price
	 *
	 * @return mixed
	 */
	public function getShippingPrice()
	{
		return $this->values['shipping_price'];
	}

	/**
	 * Set tax description
	 *
	 * @param string $taxDescription
	 */
	public function setTaxDescription($taxDescription)
	{
		$this->values['tax_description'] = $taxDescription;
	}

	/**
	 * Get tax description
	 *
	 * @return string
	 */
	public function getTaxDescription()
	{
		return $this->values['tax_description'];
	}

	/**
	 * Set tax rate
	 *
	 * @param float $taxRate
	 */
	public function setTaxRate($taxRate)
	{
		$this->values['tax_rate'] = $taxRate;
	}

	/**
	 * Get tax rate
	 *
	 * @return float
	 */
	public function getTaxRate()
	{
		return $this->values['tax_rate'];
	}

	/**
	 * Set tax exemption
	 *
	 * @param string $taxExempt
	 */
	public function setTaxExempt($taxExempt)
	{
		$this->values['tax_exempt'] = $taxExempt;
	}

	/**
	 * Get tax rate
	 *
	 * @return string
	 */
	public function getTaxExempt()
	{
		return $this->values['tax_exempt'];
	}

	/**
	 * Set line total tax amount
	 *
	 * @param float $tax_amount
	 */
	public function setTaxAmount($tax_amount)
	{
		$this->values['tax_amount'] = $tax_amount;
	}

	/**
	 * Get line total tax amount
	 *
	 * @return float
	 */
	public function getTaxAmount()
	{
		return $this->values['tax_amount'];
	}

	/**
	 * Set product title (name)
	 *
	 * @param mixed $title
	 */
	public function setTitle($title)
	{
		$this->values['title'] = $title;
	}

	/**
	 * Get product title / name
	 *
	 * @return mixed
	 */
	public function getTitle()
	{
		return $this->values['title'];
	}

	/**
	 * Set product weight
	 *
	 * @param mixed $weight
	 */
	public function setWeight($weight)
	{
		$this->values['weight'] = $weight;
	}

	/**
	 * Get product weight
	 *
	 * @return mixed
	 */
	public function getWeight()
	{
		return $this->values['weight'];
	}

	/**
	 * @param $code
	 */
	public function setExactorEucCode($code)
	{
		$this->values['exactor_euc_code'] = $code;
	}

	/**
	 * @return mixed
	 */
	public function getExactorCategoryEucCode()
	{
		return $this->values['cat_exactor_euc_code'];
	}

	/**
	 * @param $code
	 */
	public function setExactorCategoryEucCode($code)
	{
		$this->values['cat_exactor_euc_code'] = $code;
	}

	/**
	 * @return mixed
	 */
	public function getExactorEucCode()
	{
		return $this->values['exactor_euc_code'];
	}

	/**
	 * Set line item attributes
	 *
	 * @param $attributes
	 */
	public function setAttributes($attributes)
	{
		$this->values['attributes'] = $attributes;
	}

	/**
	 * Get line item attributes
	 *
	 * @return mixed
	 */
	public function getAttributes()
	{
		return $this->values['attributes'];
	}

	/**
	 * Set inventory list
	 *
	 * @param $inventoryList
	 */
	public function setInventoryList($inventoryList)
	{
		$this->values['inventoryList'] = $inventoryList;
	}

	/**
	 * Get inventory list
	 *
	 * @return mixed
	 */
	public function getInventoryList()
	{
		return $this->values['inventoryList'];
	}

	/**
	 * Set price with tax
	 *
	 * @param $priceWithTax
	 */
	public function setPriceWithTax($priceWithTax)
	{
		if (strtolower($this->getIsGift()) == 'yes')
			$priceWithTax = 0;

		$this->values['price_withtax'] = $priceWithTax;
	}
	/**
	 * Get price with tax
	 *
	 * @return mixed
	 */
	public function getPriceWithTax()
	{
		if (strtolower($this->getIsGift()) == 'yes') return 0;

		return isset($this->values['price_withtax']) ? $this->values['price_withtax'] : $this->getFinalPrice();
	}

	/**
	 * Get subtotal
	 *
	 * @return float
	 */
	public function getSubtotal($withTax = false)
	{
		if (strtolower($this->getIsGift()) == 'yes') return 0;

		return PriceHelper::round(($withTax ? $this->getPriceWithTax() : $this->getFinalPrice()) * $this->getFinalQuantity());
	}

	/**
	 * Get total
	 *
	 * @return float
	 */
	public function getTotal()
	{
		return $this->getSubtotal() - $this->getDiscountTotal();
	}

	/**
	 * Get line item total discount
	 *
	 * @return float
	 */
	public function getDiscountTotal()
	{
		return $this->getDiscountAmount() + $this->getPromoDiscountAmount();
	}

	public function calculatePriceWithTax()
	{
		$taxProvider = Tax_ProviderFactory::getTaxProvider();

		if ($this->getIsTaxable() == 'Yes' && $taxProvider->getDisplayPricesWithTax())
		{
//			if ($this->getTaxRate() > 0)
//			{
//				$priceWithoutTax = $this->getFinalPrice() - $this->getDiscountTotal();
//				$priceWithTax = PriceHelper::round($priceWithoutTax + $priceWithoutTax * $this->getTaxRate() / 100);
//
//				$this->setPriceWithTax($priceWithTax);
//			}
		}
	}

	public function parseOptions()
	{
		$options = array();

		//parse options
		$s = explode("\n", $this->values["options"]);

		for ($j=0; $j<count($s); $j++)
		{
			$pos = strpos($s[$j], " : ");
			$pos2 = strpos($s[$j], "(");
			if (!$pos2 )
			{
				$pos2 = strlen($s[$j]);
			}
			else
			{
				$pos2 = $pos2 - $pos - 3;
			}

			if ($pos !== false)
			{
				$options[trim(substr($s[$j], 0, $pos))] = trim(substr($s[$j], $pos + 3, $pos2));
			}
		}

		return $options;
	}

	/**
	 * Parse values
	 *
	 */
	protected function parseValues()
	{
		if (!function_exists('getProductUrl'))
		{
			require_once dirname(dirname(dirname(__FILE__))).'/engine/engine_url_default.php';
		}

		$this->values["url"] = getProductUrl($this->values["product_url"], $this->values["pid"], $this->values["cid"]);
		$this->values["cat_url"] = getCatalogUrl($this->values["cat_url"], $this->values["cid"]);

		//inventory, min / max order
		if ($this->values["inventory_control"] == "Yes")
		{
			$this->values['allowed_max'] = $this->values['max_order'] > 0 && $this->values['max_order'] < $this->values['stock'] ? $this->values['max_order'] : $this->values['stock'];
		}
		else
		{
			$this->values["allowed_max"] = 0;
		}

		$options = $this->parseOptions();

		foreach ($options as $name => $value)
		{
			$this->values["options_parsed"][] = array(
					"name" => $name,
					"value" => $value
				);
		}

		if (strtolower($this->getIsGift()) == 'yes')
		{
			$this->setFinalPrice(0);
		}

		$this->values['product_total'] = $this->getSubtotal();
	}

	/**
	 * Return as array
	 *
	 * @return array
	 */
	public function &toArray()
	{
		return $this->values;
	}

	/**
	 * Set attributes data
	 *
	 * @param array $attributesData
	 */
	public function setAttributesData(array $attributesData)
	{
		$this->setWeight($this->getWeight() + $attributesData['weight']);
		$this->setOptions($attributesData['options']);
		$this->setOptionsClean($attributesData['options_clean']);
		$this->setAttributes($attributesData['attributes']);
		$this->setInventoryList($attributesData['inventoryList']);
	}

	public function setProduct(Model_Product $product)
	{
		$this->setFreeShipping($product->getFreeShipping());
		$this->setPriceWithTax($product->getPriceWithTax());
		$this->parseValues($product);
		$this->setPriceBeforeQuantityDiscount($product->getFinalPrice());
	}

	/**
	 * Create a new gift line item (product promotions)
	 *
	 * @param $orderId
	 * @param $product
	 * @param $quantity
	 * @return Model_LineItem
	 */
	public static function createNewGiftLineItem($orderId, Model_Product $product, $quantity)
	{
		$attributesData = array(
			'weight' => $product->getWeight(),
			'options' => '',
			'options_clean' => '',
			'attributes' => array(), // Value not needed on update
			'inventoryList' => '',
		);
		return self::createNewLineItem($orderId, $product, $quantity, $attributesData, 'Yes');
	}

	/**
	 * Create a new line item
	 *
	 * @param $orderId
	 * @param $product
	 * @param $quantity
	 * @param $attributesData
	 * @param string $isGift
	 * @return Model_LineItem
	 */
	public static function createNewLineItem($orderId, Model_Product $product, $quantity, &$attributesData, $isGift = 'No')
	{
		$values = array(
			'ocid' => 0,
			'oid' => $orderId,
			'pid' => $product->getPid(),
			'quantity' => $quantity,
			'admin_quantity' => $quantity,
			'cost' => $product->getCost(),
			'price_before_quantity_discount' => $product->getPrice(),
			'price' => $product->getPrice(),
			'admin_price' => $product->getFinalPrice(),
			'final_price' => $product->getFinalPrice(),
			'attribute_value' => 0,
			'attribute_value_type' => 'amount',
			'is_taxable' => $product->getIsTaxable(),
			'is_dollar_days' => $product->getIsDollarDays(),
			'is_doba' => $product->getIsDoba(),
			'is_stock_changed' => 'No',
			'weight' => $product->getWeight(),
			'free_shipping' => $product->getFreeShipping(),
			'is_shipping_price' => 'No',
			'shipping_price' => 0,
			'tax_rate' => 0,
			'tax_amount' => 0,
			'tax_description' => '',
			'tax_exempt' => 0,
			'digital_product' => $product->getDigitalProduct(),
			'digital_product_file' => $product->getDigitalProductFile(),
			'digital_product_key' => $product->getDigitalProduct() == 'Yes' ? md5(uniqid(rand(),1)) : '',
			'digital_product_downloads' => 0,
			'is_gift' => $isGift,
			'gift_parent' => 0,
			'product_removed' => 'No',
			'inventory_id' => 0,
			'products_location_id' => $product->getProductsLocationId(),
			'options' => '',
			'options_clean' => '',
			'product_id' => $product->getProductId(),
			'product_sub_id' => '',
			'product_sku' => $product->getProductSku(),
			'product_upc' => $product->getProductUpc(),
			'product_gtin' => $product->getProductGtin(),
			'product_mpn' => $product->getProductMpn(),
			'title' => $product->getTitle(),
			'discount_amount' => 0,
			'promo_discount_amount' => 0,
			'product_price' => $product->getPrice(),
			'price_withtax' => $product->getPrice(),
			'product_url' => '',
			'cat_url' => '',
			'inventory_control' => '',
			'cid' => $product->getCid(),
		);

		$lineItem = new self($values);

		$lineItem->setProduct($product);
		$lineItem->setAttributesData($attributesData);

		return $lineItem;
	}

	// Disable cloning for phpunit testing
	private function __clone() {}
}