<?php
/**
 * Class View_ShippingQuotesViewModel
 */
class View_ShippingQuotesViewModel extends View_Model
{
	/**
	 * Get shipping quotes view
	 *
	 * @param ORDER $order
	 * @param Tax_ProviderInterface $taxProvider
	 * @param $rates
	 * @param bool $shippingQuoteCall
	 * @param int $shippingQuoteCountryId
	 * @param int $shippingQuoteStateId
	 *
	 * @return mixed
	 */
	public function getShippingQuotesView(ORDER $order, Tax_ProviderInterface $taxProvider, $rates, $shippingQuoteCall = false, $shippingQuoteCountryId = 0, $shippingQuoteStateId = 0)
	{
		if ($rates && $this->showPricesWithTaxes())
		{
			$shippingTaxClassId = (int) $this->settings["ShippingTaxClassId"];

			// reread tax rates for shipping quote region
			if ($shippingQuoteCall)
			{
				$taxProvider->getTaxRates(intval($shippingQuoteCountryId), intval($shippingQuoteStateId), $order->getUserLevel());
			}

			if ($taxProvider->hasTaxRate($shippingTaxClassId))
			{
				$shippingTaxRate = $taxProvider->getTaxRate($shippingTaxClassId, false);

				$rates["shipping_price_at_product_level"] =
					$rates["shipping_price_at_product_level"] +
					round($rates["shipping_price_at_product_level"] / 100 * $shippingTaxRate, 2);

				if (is_array($rates["shipping_methods"]))
				{
					foreach ($rates["shipping_methods"] as $key => $value)
					{
						$rates["shipping_methods"][$key]['shipping_price'] =
							$rates["shipping_methods"][$key]['shipping_price'] +
							round($rates["shipping_methods"][$key]['shipping_price'] / 100 * $shippingTaxRate, 2);
					}
				}
			}

			// set taxes back to region based on order data
			if ($shippingQuoteCall)
			{
				$taxProvider->getTaxRates($order->getTaxCountryId(), $order->getTaxStateId(), $order->getUserLevel());
			}
		}

		$priceAtProductLevel = 0;

		if (isset($rates['shipping_price_at_product_level']))
		{
			$priceAtProductLevel = $rates['shipping_price_at_product_level'];
			$rates['shipping_price_at_product_level'] = $this->formatPrice($rates['shipping_price_at_product_level']);
		}

		if (isset($rates['shipping_methods']) && is_array($rates['shipping_methods']) && count($rates['shipping_methods']) > 0)
		{
			unset($rates['shipping_price_at_product_level']);

			foreach ($rates['shipping_methods'] as $key => $value)
			{
				$rates['shipping_methods'][$key]['shipping_price'] = $this->formatPrice($rates['shipping_methods'][$key]['shipping_price'] + $priceAtProductLevel);
			}
		}

		return $rates;
	}

	/**
	 * Return true is prices must be displayed with taxes
	 *
	 * @return bool
	 */
	protected function showPricesWithTaxes()
	{
		$taxProvider = Tax_ProviderFactory::getTaxProvider();

		return $taxProvider->getDisplayPricesWithTax() && (bool) $this->settings["ShippingTaxable"] && $this->settings["TaxAddress"] == "Shipping";
	}
}


