<?php 

/**
 * @class CustomFields
 */
class CustomFields
{
	public $db;
	public $is_error = false;
	public $errors = array();
	
	/**
	 * Class constructor
	 * @param DB $db
	 */
	public function CustomFields(&$db)
	{
		$this->db = $db;
		return $this;
	}
	
	/**
	 * Reset errors
	 */
	public function resetErrors()
	{
		$this->errors = array();
		$this->is_error = false;
	}
	
	/**
	 * Sets error
	 * @param $message
	 * @return unknown_type
	 */
	public function setError($message)
	{
		$this->errors[count($this->errors)] = $message;
		$this->is_error = true;
	}
	
	/**
	 * returns only definition structiure, + initial data
	 * @param string $location
	 * @param mixed $data
	 */
	public function getCustomFields($location, $data)
	{
		$data = is_array($data)?$data:array();
		$this->db->query("SELECT * FROM ".DB_PREFIX."custom_fields WHERE is_active=1 AND field_place = '".$location."' ORDER BY priority, field_caption");
		if ($this->db->numRows())
		{
			$f = $this->db->getRecords();
			for ($i=0; $i<count($f); $i++)
			{
				$f[$i]["value"] = array_key_exists($f[$i]["field_id"], $data)?$data[$f[$i]["field_id"]]:($f[$i]["field_type"] == "checkbox"?$f[$i]["value_unchecked"]:"");
				$f[$i]["parsed_options"] = explode("\n", $f[$i]["options"]);
			}
			return $f;
		}
		return false;
	}
	
	/**
	 * Validate custom fields
	 */
	public function validateCustomFields($location, $fields)
	{
		$e = count($this->errors);
		$r = $this->db->query("SELECT field_id, field_type, field_name FROM ".DB_PREFIX."custom_fields WHERE field_place='".$this->db->escape($location)."' AND is_active=1 AND is_required=1");
		while($c = $this->db->moveNext($r, MYSQL_ASSOC))
		{
			if (!isset($fields[$c["field_id"]]) || trim($fields[$c["field_id"]]) == "")
			{
				$this->setError("Please ".($c["field_type"]=="checkbox"?"check":"enter")." ".$c["field_name"]);
			}
		}
		return count($this->errors) == $e;
	}
	
	/**
	 * returns stored into database data
	 * @param string $location
	 * @param mixed $profile_id
	 * @param mixed $order_id
	 * @param mixed $address_id
	 */
	public function getCustomFieldsValues($location, $profile_id, $order_id, $address_id)
	{
		$fields = $this->getCustomFields($location, array());
		if ($fields)
		{
			foreach ($fields as $key=>$field)
			{
				$fields[$key]["value_translated"] = "";
				//check is value already in db
				$query = "SELECT * FROM ".DB_PREFIX."custom_fields_values WHERE profile_id='".$profile_id."' AND field_id='".$field["field_id"]."' ";
				switch ($location)
				{
					//billing, one user, one field, one value
					case "billing" : break;
					//shipping, the same field may be different for orders and addresses in address book
					case "shipping" : 
					{
						$query.=" AND order_id='".$order_id."' ";
						$query.=" AND address_id='".$address_id."' ";
						break;
					}
					//account, one user, one field, one value
					case "account" : break;
					//signup, one user, one field, one value
					case "signup" : break;
					//shipping, the same field may be different for orders
					case "invoice" : {
						$query.=" AND order_id=".$order_id." ";
						break;
					}
				}
				//echo "GetCustomFieldsValues Query: ".$query;
				$this->db->query($query);
				if ($this->db->moveNext())
				{
					$fields[$key]["value"] = $this->db->col["field_value"];
					if (in_array($fields[$key]["field_type"], array("text", "textarea", "checkbox")))
					{ 
						$fields[$key]["value_translated"] = $this->db->col["field_value"];
					}
					else
					{
						if (array_key_exists($this->db->col["field_value"],  $fields[$key]["parsed_options"]))
						{
							$fields[$key]["value_translated"] = $fields[$key]["parsed_options"][$this->db->col["field_value"]];
						}
						else
						{
							$fields[$key]["value_translated"] = "";
						}
					}
				}
				else
				{
					$fields[$key]["value"] = "";
				}
			}
		}
		
		return $fields;
		
	}
	
	//stores data into database
	public function storeCustomFields($location, $data, $profile_id=0, $order_id=0, $address_id=0)
	{
		$fields = $this->getCustomFields($location, $data);
		
		//echo "Order ID:".$order_id."<br>";
		if ($fields)
		{
			foreach($fields as $key=>$field)
			{
				//check is value already in db
				$query = "SELECT * FROM ".DB_PREFIX."custom_fields_values WHERE profile_id='".$profile_id."' AND field_id='".$field["field_id"]."' ";
				switch ($location)
				{
					//billing, one user, one field, one value
					case "billing" : break;
					//shipping, the same field may be different for orders and addresses in address book
					case "shipping" : {
						$query.=" AND order_id='".$order_id."' ";
						$query.=" AND address_id='".$address_id."' ";
						break;
					}
					//account, one user, one field, one value
					case "account" : break;
					//signup, one user, one field, one value
					case "signup" : break;
					//shipping, the same field may be different for orders
					case "invoice" : 
					{
						$query.=" AND order_id=".$order_id." ";
						break;
					}
				}

				$this->db->query($query);
				if ($this->db->moveNext())
				{
					$this->db->reset();
					$this->db->assignStr("field_value", $field["value"]);
					$this->db->update(DB_PREFIX."custom_fields_values", "WHERE profile_id='".$profile_id."' AND field_id='".$field["field_id"]."' AND order_id='".$order_id."' AND address_id='".$address_id."'");
				}
				else
				{
					$this->db->reset();
					$this->db->assign("profile_id", $profile_id);
					$this->db->assign("order_id", $order_id);
					$this->db->assign("address_id", $address_id);
					$this->db->assign("field_id", $field["field_id"]);
					$this->db->assignStr("field_value", $field["value"]);
					$this->db->insert(DB_PREFIX."custom_fields_values", "WHERE profile_id='".$profile_id."' AND field_id='".$field["field_id"]."' AND order_id='".$order_id."' AND address_id='".$address_id."'");
				}
			}
		}
	}
}

