<?php

class Ddm_Site_Feedback
{
    private $_db;
    private $_settings;

    /**
     * The constructor
     *
     * @param string $db pass database handler
     * @param string $settings pass global application settings
     * @return void
     */
    public function __construct()
    {
        $this->_db = db();
        $this->_settings = settings();
	
    }

    /**
     * Adds a product feedback to the database
     *
     * @param int $product_id The product being feedbacked
     * @param int $user_id The user id of the feedbacker
     * @param array $feedback_data Array with feedback specific keys ('rating','User Name','feedback')
     * @return int Returns feedback id on success, FALSE on failure
     */
    function addFeedback($feedback_data)
    {
		
		
      /*  $user_id = $feedback_data['uid'];*/

        // make sure we have feedback data
        if (empty($feedback_data))
        {
            return false;
        }

        // auto approve the feedback if the admin has enabled these settings
        $approved =	(
            $this->_settings['ReviewsAutoApprove']  &&
                intval($feedback_data['rating']) >= intval($this->_settings['ReviewsAutoApproveRating'])
        ) ? 1 : 0;

        // reset the database handler and prepare the insert
        $this->_db->reset();
       /* $this->_db->assign("uid", intval($user_id));*/
        $this->_db->assign("date_created", "NOW()");
        $this->_db->assignStr("firstname", htmlentities(strip_tags($feedback_data['firstname'])));
        $this->_db->assignStr("lastname", htmlentities(strip_tags($feedback_data['lastname'])));
		$this->_db->assignStr("company", htmlentities(strip_tags($feedback_data['company'])));		
		if(htmlentities(strip_tags($feedback_data['username'])))
		{									   
		$this->_db->assignStr("username", htmlentities(strip_tags($feedback_data['username'])));		
		}
		else
		{
		$this->_db->assignStr("username", 'off');		
		}
        $this->_db->assignStr("email", htmlentities(strip_tags($feedback_data['email'])));
        $this->_db->assignStr("feedback", htmlentities(strip_tags($feedback_data['feedback'])));
        $this->_db->assignStr("rating", $feedback_data['rating']);
        $this->_db->assignStr("approved", $approved);
        $this->_db->assignStr("is_visible", 1);
		
		

        // insert and return feedback id
        $feedbackId = $this->_db->insert(DB_PREFIX."site_feedback");

        return $feedbackId;
    }

 
    /**
     * Gets the feedback for the given product ids
     *
     * @param mixed $product_id Product ID or list of IDs in array
     * @return array Returns an array of feedback
     */
    function getFeedback()
    {
        $this->_db->query('
			SELECT firstname,lastname,rating,feedback,username,DATE_FORMAT(date_created,
			"'.$this->_settings["LocalizationDateTimeFormat"].'") AS date_created
			FROM '.DB_PREFIX.'site_feedback WHERE is_visible = 1	AND approved = 1 ORDER BY date_created DESC
		');

        return $this->_db->getRecords();
    }

    /**
     * Returns feedback coung by given product id
     *
     * @return int
     */
    function getFeedbackCount()
    {
        $this->_db->query('
			SELECT
				COUNT(*) AS c
			FROM '.DB_PREFIX.'site_feedback
			WHERE
				is_visible = 1
				AND approved = 1
		');
        return $this->_db->movenext() ? $this->_db->col["c"] : 0;
    }

    /**
     * Gets a product rating along with other useful information like total number of feedback
     *
     * @return array Rating details
     */
    public function getSiteRating()
    {
        $this->_db->query('
			SELECT
				AVG(rating) as rating,
				COUNT(0) as total_feedback
			FROM '.DB_PREFIX.'site_feedback
			WHERE approved = 1
		');

        return current($this->_db->getRecords());
    }

    /**
     * Checks if the user has written a product feedback already for the given product id
     *
     * @param int $user_id
     * @return void
     * @author Sebastian Nievas
     */
    public function hasWrittenFeedback($user_id)
    {
        $this->_db->query('
			SELECT COUNT(0)
			FROM '.DB_PREFIX.'site_feedback pr
			WHERE
				pr.uid = '.intval($user_id).'
		');

        return (bool)current($this->_db->moveNext());
    }
}
