<?php

/**
 * Doba API Class
 */
class DobaAPI
{
	public $test_mode = false;
	public $version = "610";
	protected $username = "";
	protected $password = "";
	public $retailer_id = "";
	public $api_url = "";
	
	public $price_option = "item_price_inc";
	public $price_modifier = 10;
	public $price_option99 = true;
	public $min_stock = 5;
	
	public $last_error = false;
	
	public $response_xml = null;
	public $response_error = null;
	
	private $_db = null;
	private $_settings = array();
	
	/**
	 * Class constructor
	 * @param array $settings
	 * @return BuySafeAPI
	 */
	public function __construct(&$settings, &$db = null)
	{
		$this->test_mode = $settings["doba_mode"] == "Test";
		$this->username = $settings["doba_username"];
		$this->password = $settings["doba_password"];
		$this->retailer_id = $settings["doba_retailer_id"];
		
		$this->api_url = $this->test_mode ? "https://www.sandbox.doba.com/api/20101012/xml_retailer_api.php" : "https://www.doba.com/api/20101012/xml_retailer_api.php";
		
		$this->price_option = $settings["doba_price_option"];
		$this->price_option99 = $settings["doba_price_option99"] == "Yes"; 
		$this->price_modifier = $settings["doba_price_modifier"];
		$this->min_stock = $settings["doba_min_stock"];
		
		$this->_db = $db;
		$this->_settings = $settings;
		
		return $this;
	}
	
	
	/**
	 * Connect to doba server and return data
	 * @param string $xml
	 * @return mixed
	 */
	public function call($body)
	{
		$xml = 
			'<dce>'."\n".
				'<request>'."\n".
					'<authentication>'."\n".
						'<username>'.($this->username).'</username>'."\n".
						'<password>'.($this->password).'</password>'."\n".
					'</authentication>'."\n".
					'<retailer_id>'.($this->retailer_id).'</retailer_id>'."\n".
					$body.
				'</request>'."\n".
			'</dce>'."\n";
					
		$c = curl_init($this->api_url);
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_POSTFIELDS, $xml);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_TIMEOUT, 300);
		
		set_time_limit(108000);
		
		$response = curl_exec($c);
		
		//file_put_contents("log/doba".rand(100, 10000).".xml", $response);
		
		if (curl_errno($c)) 
		{
			$this->last_error = curl_error($c);
			return false;
		}
		
		if (trim($response) == "")
		{
			$this->last_error = "Response is empty";
			$this->response_xml = null;
		}
		else
		{
			$this->response_xml = simplexml_load_string($response);
		}
		
		if ($this->response_xml != null)
		{
			if (isset($this->response_xml->response->outcome))
			{
				if ($this->response_xml->response->outcome == "success") return $this->response_xml;
				if ($this->response_xml->response->outcome == "failure")
				{
					$this->response_error = $this->response_xml->response->error;
					$this->last_error = 
						$this->response_xml->response->error->message." ".
						" (Level ".$this->response_xml->response->error->level.", Code ".
						$this->response_xml->response->error->code.")";
				}
				else
				{
					$this->last_error = "Unknown error. Please try again";
				}
			}
		}
		else
		{
			$this->last_error = "XML parse error";
		}
		
		return false;
	}
	
	/**
	 * Returns database instance
	 * @return unknown
	 */
	public function db()
	{
		return $this->_db;
	}
	
	/**
	 * Get settings
	 * @param string $name
	 */
	public function settings($name = "")
	{
		return $name == "" ? $this->_settings : $this->_settings[$name];
	}
}