<?php

if (!defined("ENV")) exit();

class QuickStartGuide
{
	public static $settings;
	public static $options;
	public static $db;
	public static $page;
	public static $category_layout = array();
	public static $view;
	
	public static function init()
	{
		global $settings, $db, $p;
		self::$settings = $settings;
		self::$db = $db;
		self::$page = $p;
		self::$view = Ddm_View::getInstance();
		self::$category_layout = array(
			'CatalogViewText' => self::$settings['CatalogViewText'],
			'CatalogViewThumb1' => self::$settings['CatalogViewThumb1'],
			'CatalogViewThumb1Clean' => self::$settings['CatalogViewThumb1Clean'],
			'CatalogViewThumb2' => self::$settings['CatalogViewThumb2'],
			'CatalogViewDisplay' => self::$settings['CatalogViewDisplay'],
			'CatalogViewDisplayBox' => self::$settings['CatalogViewDisplayBox'],
			'CatalogViewThumb3' => self::$settings['CatalogViewThumb3'],
			'CatalogViewThumb3Clean' => self::$settings['CatalogViewThumb3Clean'],
			'CatalogViewFlexible' => self::$settings['CatalogViewFlexible']
		);
		
		self::setOptions();
	}
	
	/**
	 * Builds and displays the overall layout of the guide
	 *
	 * @return void
	 */
	public static function buildLayout()
	{
		self::$view->assign('quickStartGuide', (is_array(self::$options) ? self::$options : array()));
		return self::$view->fetch('templates/admin/quick-start-guide/index.html');
	}
	
	public static function buildSectionData()
	{
		global $label_user_manual;
		QuickStartGuide::loadCompanyInformation();
		
		// section 2 - skin selection
		$themesFolderPath = 'content'.DS.'skins';
		$themes = DataAccess_Repository::get('SiteThemeRepository')->getInstalledThemes($themesFolderPath);

		$currentThemes = array();

		foreach ($themes as $theme)
		{
			$currentThemes[$theme->getName()] = array(
				'name' => $theme->getName(), // actual folder name
				'title' => $theme->getTitle(), // skin title
				'author' => $theme->getAuthor(), // who is the author
				'thumbnailImage' => $theme->getThumbnailImage(),
				'fullsizeImage' => $theme->getFullsizeImage(),
				'current' => $theme->getName() == self::$settings['DesignSkinName']
			);
		}
	
		self::$view->assign('currentThemes', json_encode($currentThemes));
		self::$view->assign('themesInstallable', is_writable($themesFolderPath) ? 'true' : 'false');
		
		self::$view->assign('LabelUserManual', $label_user_manual);
		self::$view->assign('LabelTutorials', (!IS_LABEL) ? '<li>Video tutorials &ndash; <a href="http://www.'.getp().'.com/shopping-cart-software-tutorials.htm" target="_blank">http://www.'.getp().'.com/shopping-cart-software-tutorials.htm</a></li>' : '');
		
		// section 7 - test products
		self::$db->query('SELECT COUNT(pid) AS testproducts_count FROM '.DB_PREFIX.'products WHERE product_id LIKE "test__%"');
		self::$db->movenext();
		self::$view->assign('hasTestProducts', (self::$db->col['testproducts_count'] > 0));
	}
	
	public static function getCategoryLayoutKeys()
	{
		return array_keys(self::$category_layout);
	}
	
	/**
	 * Populates the Company Information page with data from settings table
	 *
	 * @return void
	 */
	private static function loadCompanyInformation()
	{
		self::$view->assign('field_CompanyName', self::$settings['CompanyName']);
		self::$view->assign('field_CompanyAddressLine1', self::$settings['CompanyAddressLine1']);
		self::$view->assign('field_CompanyAddressLine2', self::$settings['CompanyAddressLine2']);
		self::$view->assign('field_CompanyCity', self::$settings['CompanyCity']);
		self::$view->assign('field_CompanyState', self::$settings['CompanyState']);
		self::$view->assign('field_CompanyZip', self::$settings['CompanyZip']);
		self::$view->assign('field_CompanyCountry', self::$settings['CompanyCountry']);
		self::$view->assign('field_CompanyPhone', self::$settings['CompanyPhone']);
		self::$view->assign('field_CompanyWebsite', self::$settings['CompanyWebsite']);
		self::$view->assign('field_CompanyEmail', self::$settings['CompanyEmail']);
	}
	
	/**
	 * Saves the Company Information section data into the settings table
	 *
	 * @param array $data Key-value array of the setting name in the settings table
	 * @return void
	 */
	public static function saveCompanyInformation($data = array())
	{
		if (!is_array($data) || count($data) == 0) return;
		
		foreach ($data as $key => $value)
		{
			if (strstr($key, 'Company'))
			{
				self::$db->reset();
				self::$db->assignStr('value', $value);
				self::$db->update(DB_PREFIX.'settings', 'WHERE name = "'.self::$db->escape($key).'" LIMIT 1');
			}
		}
	}
	
	
	/**
	 * Saves the Search Engine Optimization settings
	 *
	 * @param array $data Key-value array of the setting name in the settings table
	 * @return void
	 */
	public static function saveSeoInformation($data = array())
	{
		if (!is_array($data) || count($data) == 0) return;
		
		foreach ($data as $key => $value)
		{
			if (strstr($key, 'Search'))
			{
				self::$db->reset();
				self::$db->assignStr('value', $value);
				self::$db->update(DB_PREFIX.'settings', 'WHERE name = "'.self::$db->escape($key).'" LIMIT 1');
			}
		}
	}
	
	
	/**
	 * Saves the category layout
	 *
	 * @param array $data Key-value array of the setting name in the settings table
	 * @return void
	 */
	public static function saveCategoryLayout($data = array())
	{
		if (!is_array($data) || count($data) == 0) return;
		
		foreach ($data as $key => $value)
		{
			if (strstr($key, 'Catalog'))
			{
				self::$db->reset();
				self::$db->assignStr('value', $value);
				self::$db->update(DB_PREFIX.'settings', 'WHERE name = "'.self::$db->escape($key).'" LIMIT 1');
			}
		}
	}
	
	public static function saveCheckoutProcess($value)
	{
		switch ($value)
		{
			case 'choise': $account_mode = 'Allow user to decide'; break;
			case 'force': $account_mode = 'Yes'; break;
			case 'no-account': $account_mode = 'No'; break;
		}
		
		self::$db->reset();
		self::$db->assignStr('value', $account_mode);
		self::$db->update(DB_PREFIX.'settings', 'WHERE name = "AllowCreateAccount" LIMIT 1');	
	}
	
	public static function saveProductLayout($value)
	{
		if (!in_array($value, array('Style1','Style2','Style3','Style4','Style5'))) throw new Exception('Invalid value for product layout');
		self::$db->reset();
		self::$db->assignStr('value', $value);
		self::$db->update(DB_PREFIX.'settings', 'WHERE name = "CatalogProductPage" LIMIT 1');
	}
	
	/**
	 * Adds test items info the database. Will return a boolean value for whether the items were inserted successfully or not
	 *
	 * @return boolean Whether
	 */
	public static function addTestItems()
	{
		try
		{
			// attempt to open the test products fixture
			if (($fp = fopen('content/admin/fixtures/testproducts_default.sql', 'r')) !== false)
			{
				// queries must always be on a single line only loop over each line
				while ($query = fgets($fp, 8096*2))
				{
					if (trim($query) != "")
					{
						if (strstr($query,"INSERT INTO "))
							$query = str_replace("INSERT INTO ", "INSERT INTO ".DB_PREFIX, $query);
						if (strstr($query, "UPDATE "))
							$query = str_replace("UPDATE ", "UPDATE ".DB_PREFIX, $query);
						if (strstr($query," FROM "))
							$query = str_replace(" FROM ", " FROM ".DB_PREFIX, $query);
						
						self::$db->query($query);
					}
				}
				
				copy_dir('content/admin/fixtures/images', 'images/');
			}
		}
		catch (Exception $e)
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Removes test items from the database
	 *
	 * @return void
	 */
	public static function removeTestItems()
	{
		try
		{
			$product_tables = array(
				'products', 
				'products_attributes', 
				'products_categories', 
				'products_inventory', 
				'products_families_content', 
				'products_families_dependencies', 
				'products_images', 
				'products_quantity_discounts', 
				'products_reviews', 
				'products_shipping_price', 
				'users_wishlist_products', 
				'promo_products', 
				'products_groups_products', 
				'products_gifts'
			);
			
			$product_ids = array();
			self::$db->query('SELECT pid FROM '.DB_PREFIX.'products WHERE product_id LIKE "test__%"');
			while (self::$db->movenext())
			{
				$product_ids[] = self::$db->col['pid'];
			}
			
			$product_ids_string = implode(",", $product_ids);
			
			foreach ($product_tables as $table)
			{
				self::$db->query('DELETE FROM '.DB_PREFIX.$table.' WHERE pid IN ('.$product_ids_string.') LIMIT '.count($product_ids));
			}
			
			$catalog_tables = array(
				'catalog',
				'catalog_attributes',
				'promo_categories'
			);
			
			$category_ids = array();
			self::$db->query('SELECT cid FROM '.DB_PREFIX.'catalog WHERE key_name LIKE "test__%"');
			while (self::$db->movenext())
			{
				$category_ids[] = self::$db->col['cid'];
			}
			
			$category_ids_string = implode(",", $category_ids);
			foreach ($catalog_tables as $table)
			{
				self::$db->query('DELETE FROM '.DB_PREFIX.$table.' WHERE cid IN ('.$category_ids_string.') LIMIT '.count($category_ids));
			}
		}
		catch (Exception $e)
		{
			return false;
		}
		
		return true;
	}
	
	public static function setOptions()
	{
		// self options is an array
		self::$options = array();
		
		self::$options['activateSeoUrl'] = (self::$settings['USE_MOD_REWRITE'] == 'YES') ? 'checked="checked"' : '';
		
		// get the value for checkout process
		$checkout_mode = '';
		
		switch (self::$settings['AllowCreateAccount'])
		{
			case 'Allow user to decide': $checkout_mode = 'choise'; break;
			case 'Yes': $checkout_mode = 'force'; break;	
			case 'No': $checkout_mode = 'no-account'; break;
		}
		
		self::$options['checkoutProcess'] = $checkout_mode;
		
		// get the values for the catalog layout
		self::$options['categoryLayout'] = json_encode(self::$category_layout);
		
		// get the values for the product layout
		self::$options['productLayout'] = self::$settings['CatalogProductPage'];
	}
	
	public static function load()
	{
		self::buildSectionData();
		return self::buildLayout();
	}
	
	/**
	 * undocumented function
	 *
	 * @param string $value 
	 * @return void
	 */
	public static function setDisplayOnStartup($hide_quickstart)
	{
		// we reversed the logic due to the wording on the checkbox
		$display_quickstart = ($hide_quickstart == 'true' ? 0 : 1);
		self::$db->query('UPDATE '.DB_PREFIX.'admins SET display_quickstart = '.$display_quickstart.' WHERE aid = '.$_SESSION['admin_auth_id'].' LIMIT 1');
	}
	
	/**
	 * Sets the flag to activate search engine url's
	 *
	 * @param string $value 
	 * @return void
	 */
	public static function setActivateSearchEngineUrl($value)
	{
		// we reversed the logic due to the wording on the checkbox
		$activate_url = ($value == 'true') ? 'YES' : 'NO';
		self::$db->query('UPDATE '.DB_PREFIX.'settings SET value = "'.$activate_url.'" WHERE name = "USE_MOD_REWRITE" LIMIT 1');
		self::$settings['USE_MOD_REWRITE'] = $activate_url;
		
		
		// regenerate the htaccess file
		$rewrite_mode = ($activate_url == 'YES') ? 'On' : 'Off';
		Seo::generateHtaccessFile($rewrite_mode);
		
		
		// update SEO URL's if set to true
		if ($value == 'true')
			updateSeoURLs(self::$db, self::$settings);
	}
}