<?php

/**
* 
*/
class Seo
{
	/**
	 * Generates the .htaccess file
	 *
	 * @throws Exception On error when creating the .htaccess file
	 * @return void
	 * @author Sebi
	 */
	public static function generateHtaccessFile($rewrite_mode = 'Off')
	{
		global $settings;
		
		if (!in_array($rewrite_mode, array('Off', 'On'))) $rewrite_mode = 'Off';
		
		@unlink(".htaccess.back");
		@copy(".htaccess", ".htaccess.back");
		$f = @fopen(".htaccess", "w");
		if (!$f)
		{
			throw new Exception('Could not open the .htaccess file for writing', 100);
		}
		else
		{
			$pu = parse_url("http://foo.com/".$_SERVER["REQUEST_URI"]);
			$pi = pathinfo($pu["path"]);
			$rewriteBase = $pi["dirname"];
		
			$cond = '\#(.)*|\?(.)*';
			$d = dir($settings["GlobalServerPath"]);
			
			while (false !== ($entry = $d->read()))
			{
				if (!in_array($entry, array(".", "..")))
				{
	   				if (is_file($entry))
					{
						$cond .= ($cond == "" ? "" : "|").(preg_quote($entry)."(.)*");
					}
					elseif (is_dir($entry))
					{
						$cond .= ($cond == "" ? "" : "|").($entry."\\/(.)*");
					}
				}
			}
			
			$cond = str_replace(' ', '\ ', $cond);
			
			$d->close();
			
			$htaccess = 
				"<IfModule mod_headers.c>\n".
				"\tHeader unset ETag\n".
				"\tFileETag None\n".
				"\t<FilesMatch \"(?i)^.*\.(ico|flv|jpg|jpeg|png|gif|js|css)$\">\n".
				"\t\tHeader unset Last-Modified\n".
				"\t\tHeader set Expires \"Fri, 21 Dec 2020 00:00:00 GMT\"\n".
				"\t\tHeader set Cache-Control \"public, no-transform\"\n".
				"\t</FilesMatch>\n".
				"</IfModule>\n\n".
                 
				"<IfModule mod_rewrite.c>\n".
				"\tRewriteEngine\t".$rewrite_mode."\n".
				// RewriteBase is not necessary
				// "\tRewriteBase\t".str_replace("//", "/", $rewriteBase)."\n\n".
				//"RewriteCond\t$1 !^(content\/(.)*|images\/(.)*|install\/(.)*|admin\.php(.)*|download\.php(.)*|index\.php(.)*|login\.php(.)*|\#(.)*|\?(.)*)\n".
				
				"\tRewriteCond\t%{QUERY_STRING} ^\$\n". 
				"\tRewriteRule\t^((.)?)\$\tindex.php?p=home [L]\n\n".
				
				"\tRewriteCond\t\$1 ".$settings['GlobalServerPath']."\n".
				"\tRewriteRule\t^(.+)\$ / [L]\n\n".
			
				"\tRewriteCond\t$1 !^(".$cond.")\n".
				"\tRewriteRule\t^(.+)$ index.php?url=$1&%{QUERY_STRING} [L]\n".
				"</IfModule>\n\n".
			
				"<IfModule mod_deflate.c>\n".
				"\t<FilesMatch \"\.(js|css|ico|flv|jpg|jpeg|png|gif)$\">\n".
				"\t\tSetOutputFilter DEFLATE\n".
				"\t</FilesMatch>\n".
				"</IfModule>\n";
			
			if(!@fputs($f, $htaccess, strlen($htaccess))){
				throw new Exception('Could not write to the .htaccess file', 200);
			}
			fclose($f);
		}
	}
	
	/**
	 * Generate unique URL's for products and categories that may have duplicates
	 * @return integer Number of rows updated
	 */
	public static function generateUniqueUrls()
	{
		db()->reset();
		// Figure our which url_hashes have duplicates
		$sql = '
			SELECT SUM(1) AS totalcount, url_hash 
			FROM 
			(
				(SELECT url_hash FROM '.DB_PREFIX.'products)
				UNION ALL
				(SELECT url_hash FROM '.DB_PREFIX.'catalog)
				UNION ALL
				(SELECT url_hash FROM '.DB_PREFIX.'pages)
			) AS url_tables 
			GROUP BY url_hash HAVING totalcount > 1';
		
		db()->query($sql);
		
		// If we don't have any results, there aren't any duplicates to process
		if (db()->numRows() == 0) return 0;
		
		// Store the url hashes in an array to implode later
		$url_hashes = array();
		while (db()->moveNext())
		{
			$url_hashes[db()->col['url_hash']] = db()->col['url_hash'];
		}
				
		// A map of products and categories and their respective urls
		$url_map = array();
		
		// Compile a list of potential candidates based on the keywords that may conflict later for url suggestions
		$candidates = array();
		
		// Query all rows that had a duplicate by searching the url_hash
		$sql = '
			(SELECT pid as id, "products" as type, url_hash, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'products WHERE url_hash IN ("'.implode('","', $url_hashes).'"))
			UNION
			(SELECT cid as id, "catalog" as type, url_hash, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'catalog WHERE url_hash IN ("'.implode('","', $url_hashes).'"))
			UNION
			(SELECT pid as id, "pages" as type, url_hash, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'pages WHERE url_hash IN ("'.implode('","', $url_hashes).'"))
			ORDER BY url_hash, id';
		db()->query($sql);
		
		// Create our maps and add to conflicts array
		while (db()->moveNext())
		{
			$url_map[db()->col['type']][db()->col['id']] = db()->col['url'];
			$candidates[] = '^'.preg_replace('/\/?$|\.[a-z]+?$/','', db()->col['url']).'-[a-z]+([\./]?|\.[a-z]+)';
		}

		$candidates_chunks = array_chunk(array_unique($candidates), 100);

		$conflicts = array();
		
		foreach ($candidates_chunks as $candidates)
		{
			// Search for both products and categories
			$sql = '
				(SELECT pid as id, "products" as type, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'products WHERE (url_custom = "" AND url_default REGEXP "'.implode('|', $candidates).'") OR (url_custom REGEXP "'.implode('|', $candidates).'"))
				UNION
				(SELECT cid as id, "catalog" as type, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'catalog WHERE (url_custom = "" AND url_default REGEXP "'.implode('|', $candidates).'") OR (url_custom REGEXP "'.implode('|', $candidates).'"))
				UNION
				(SELECT pid as id, "pages" as type, IF(url_custom="",url_default,url_custom) AS url FROM '.DB_PREFIX.'pages WHERE (url_custom = "" AND url_default REGEXP "'.implode('|', $candidates).'") OR (url_custom REGEXP "'.implode('|', $candidates).'"))
			';
			db()->query($sql);
				
			// store each potential conflict
			while (db()->moveNext())
			{
				$conflicts[db()->col['url']] = array('id' => db()->col['id'], 'type' => db()->col['type'], 'update' => false);
			}
		}
		
		// An array of suggestions by type (product/category) and indexed by their native id (pid/cid)
		$suggestions = array();
		
		// url suffix
		$suffix = '-a';
		
		// sort the url_map manually so products have highest precedence
		$url_map = array(
			'products' => isset($url_map['products']) ? $url_map['products'] : array(),
			'catalog' => isset($url_map['catalog']) ? $url_map['catalog'] : array(),
			'pages' => isset($url_map['pages']) ? $url_map['pages'] : array()
		);
		
		// loop through each type (product/category)
		foreach ($url_map as $type => $data)
		{
			$current_url = null;
			
			if (empty($data)) continue;
			
			// loop through each duplicate entry
			foreach ($data as $key => $url)
			{
				
				if (is_null($current_url) || $url != $current_url)
				{
					$suffix = '-a';
				}
				
				// if (in_array($url, array_keys($conflicts)) && $conflicts[$url]['id'] == $key) continue;
				
				if (!in_array($url, array_keys($conflicts)) || (in_array($url, array_keys($conflicts)) && $conflicts[$url]['id'] == $key))
				{
					$suggestion = $url;
				}
				else
				{
					do
					{
						$suggestion = self::getUrlSuggestion($url, $suffix);
						$suffix++;
					}
					while (in_array($suggestion, array_keys($conflicts)));
				}
				
				$conflicts[$suggestion] = array(
					'id' => $key,
					'type' => $type,
					'update' => true
				);
				
				$current_url = $url;
			}
		}
		
		
		
		// Update each record with its suggestion
		$total_updated = 0;
		
		foreach ($conflicts as $url => $data)
		{
			if ($data['update'] === true)
			{
				$id = $data['id'];
				$type = $data['type'];
				
				switch ($type)
				{
					case 'catalog': $key_name = 'cid'; break;
					default: $key_name = 'pid';
				}
				
				db()->reset();
				db()->assignStr('url_hash', md5($url));
				db()->assignStr('url_custom', $url);
				db()->update(DB_PREFIX.$type, "WHERE {$key_name} = {$id} AND url_default != '{$url}' LIMIT 1");
				$total_updated++;
			}
		}
		
		return $total_updated;
	}
	
	/**
	 * Gets a URL string as a suggestion for the given URL and suffix
	 *
	 * @param string $url The URL that will be transformed
	 * @param string $suffix The suffix that wlll be appended to the given $url
	 * @return string The suggested URL string
	 */
	private static function getUrlSuggestion($url, $suffix)
	{
		return preg_replace('/(\/?$|\.[a-z]+?$)/',$suffix.'$1', $url, 1);
	}
}
