<?php
/**
 * Process one page checkout request
 */

	if (!defined("ENV")) exit();
	
	define('OPC_MSG_ERROR_DATA_MISSED', 1);
	
	/**
	 * OPC Response class
	 */
	class OnePageCheckoutResponse
	{
		private $_errors = array();
		private $_data = array();

		/**
		 * Saves error into array
		 * @param int, array, or string $msg
		 */
		public function setError($msg)
		{
			$a = is_array($msg) ? $msg : array($msg);
			foreach ($a as $msg)
			{ 
				switch ($msg)
				{
					case OPC_MSG_ERROR_DATA_MISSED : $this->_errors[] = ""; break;
					default : $this->_errors[] = $msg;
				}
			}
		}
		
		/**
		 * Return errors
		 */
		public function errors()
		{
			return count($this->_errors) > 0 ? $this->_errors : false;
		}
		
		/**
		 * Sets data value to be send in response
		 * @param string $varName
		 * @param mixed $varValue
		 */
		public function setData($varName, $varValue)
		{
			$this->_data[$varName] = $varValue;
		}
		
		/**
		 * Build OPC response
		 */
		public function getResponse()
		{
			$a = array(
				"result" => count($this->_errors) == 0,
				"errors" => $this->_errors,
				"data" => $this->_data
			);
			return json_encode($a);
		}
		
		/**
		 * @param string $data
		 */
		public function unserialize($data)
		{
			parse_str($data, $arr);
			return $arr;
		}
	}
	
	/**
	 * Init 
	 */
	$action = isset ($_POST["action"]) ? $_POST["action"] : false;
	$data = isset ($_POST["data"]) ? json_decode($_POST["data"]) : false;
	
	$opc = new OnePageCheckoutResponse();
	$getShippingMethods = false;
	
	if (!$action || !$data)
	{
		$opc->setError("Error processing form");
	}
	else
	{
		/**
	 	 * Process action
 	 	 */
		switch ($action)
		{
			case "doLogin" :
			{
				$_POST["login"] = $data->username;
				$_POST["password"] = $data->password;
				$opc_login = true;
				unset($_POST["remember_me"]); 
				require_once("content/engine/actions/action.user_login.php");
				if (!$login_successfull)
				{
					$opc->setError("Please enter valid username and password");
				}
				break;
			}
			/**
			 * Save billing data
			 */
			case "saveBillingData" : 
			{
				//billing form
				$billingForm = $opc->unserialize($data->billingForm);

				$customFieldsData = isset($billingForm["custom_field"]) ? $billingForm["custom_field"] : false;
				unset($billingForm["custom_field"]);
				unset($billingForm["custom_field_required"]);
				unset($billingForm["custom_field_name"]);
				unset($billingForm["custom_field_place"]);

				//check data to convert current express checkout to normal user
				$positiveResult = false;
				if (isset($billingForm["create_account"]) && $billingForm["create_account"])
				{
					//try to create account
					if (!$user->signUp($billingForm, $customFieldsData, $user->id))
					{
						$opc->setError($user->errors);		
					}
					else
					{
						emailNewUserRegistered($db, $user->id, $customFields);
						$positiveResult = true;
						$opc->setData("accountCreated", true);
					}
				}
				else
				{
					//try to store billing data
					if (!$user->updateBillingInfo($billingForm, $customFieldsData))
					{
						$opc->setError($user->errors);
					}
					else
					{
						$positiveResult = true;
						$opc->setData("accountCreated", false); 
					}
				}
				if ($positiveResult)
				{
					$billingData = $user->getBillingInfo();
					$opc->setData("billingData", $billingData);
	
					//update shipping address when needed
					if ($data->shippingAsBilling)
					{
						$order->updateShippingAddress($billingData);
						$shippingAddress = $order->getShippingAddress();
						$opc->setData("shippingAddress", $shippingAddress);
						
						$getShippingMethods = true;
					}
				}
				break;
			}
			
			/**
			 * Check promo code
			 */
			case "checkPromoCode" :
			{
				$promoResult = false;
				
				// Get form data
				$promoCode = $data->promo;
				
				// Check promo code
				if (trim($promoCode) != '' && validate($promoCode, VALIDATE_STRING))
				{
					$_SESSION["order_promo_code"] = $promoCode;
					$order->resetPromo();
					$promoResult = $order->checkPromoCode($promoCode);
				}
				
				if (!$promoResult)
				{
					$_SESSION["order_promo_code"] = '';
					$opc->setError("Entered promo code is expired, cannot be used with your order or is invalid");
				}
				
				// Set response
				$opc->setData("promoResult", $promoResult);
				break;
			}
			
			/**
			 * Save shipping data
			 */
			case "saveShippingAddress" : 
			{
				//update shipping address when needed
				$shippingForm = $opc->unserialize($data->shippingForm);
				
				//custom fields
				$customFieldsData = isset($shippingForm["custom_field"]) ? $shippingForm["custom_field"] : false;
				unset($shippingForm["custom_field"]);
				unset($shippingForm["custom_field_required"]);
				unset($shippingForm["custom_field_name"]);
				unset($shippingForm["custom_field_place"]);
				
				$_SESSION["opc-shipping-address-id"] = $shippingForm["shipping-address-id"];
				
				/**
				 * Shipping the same as billing
				 */
				if ($shippingForm["shipping-address-id"] == "billing")
				{
					$billingData = $user->getBillingInfo();
					$order->updateShippingAddress($billingData);		
				}
				
				/**
				 * New shipping address
				 */
				elseif ($shippingForm["shipping-address-id"] == "new")
				{
					$doUpdate = true;
					if (!$user->validateAddress($shippingForm))
					{
						$opc->setError($user->errors);
						$doUpdate = false;
					}
					if ($customFieldsData)
					{
						$order->customFields()->resetErrors();
						$order->customFields()->validateCustomFields("shipping", $customFieldsData);
						if ($order->customFields()->is_error)
						{
							$opc->setError($order->customFields()->errors);
							$doUpdate = false;
						} 
					}
					if ($doUpdate)
					{
						$order->updateShippingAddress($shippingForm, $customFieldsData);
					}
				}
				
				/**
				 * Shipping address from address book
				 */
				elseif (is_numeric($shippingForm["shipping-address-id"]))
				{
					$shipping_address = $user->getShippingAddressById(intval($shippingForm["shipping-address-id"]));
					$order->updateShippingAddress($shipping_address);
				}
				
				/**
				 * Set updated data
				 */
				$shippingAddress = $order->getShippingAddress();
				$opc->setData("shippingAddress", $shippingAddress);
				
				/**
				 * Assign quotes
				 */
				if (!$opc->errors())
				{
					$getShippingMethods = true;
				}
				break;
			}
			
			/**
			 * Get shipping methos
			 */
			case "getShippingMethods" :
			{
				$getShippingMethods = true;
				break;
			}
			
			/**
			 * Apply gift certificate
			 */
			case "applyGiftCertificate" :
			{
				// Check shipping method
				if (trim($data->shippingMethodId) != '')
				{
					$order->updateShippingMethod($data->shippingMethodId);
					$errorMessage = '';
					$order->getShippingAmount($errorMessage);
					if ($errorMessage != '')
					{
						$opc->setError($errorMessage);
					}
					else 
					{
						$order->closeShipping();	
					}
					$order->recalcOrderData();
				}
				
				// Check and apply gift certificate
				$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
				
				$giftError = true;
				
				$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
				$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
				$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
				if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
				{
					$giftCert = new GiftCertificates($db);
					$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
					// Note: we only validating gift certificate wishout applying
					if ($giftCertData) 
					{
						// gift certificate is correct
						$giftError = false;
						$opc->setData("giftCertificateAmount", $giftCertData['balance']);
					}
				}

				if ($giftError)
				{
					$opc->setError("Provided gift certificate is not valid");
				}	
			
				break;
			
			}
		
			/**
			 * Process payment
			 */
			case "processPayment" :
			{
				// Check additional information
				if ($data->additionalInformation)
				{
					$additionalInformation = $opc->unserialize($data->additionalInformation);
					//gift message
					if ($settings["GiftCardActive"] == "YES")
					{
						if (isset($additionalInformation["add_gift"]) && $additionalInformation["add_gift"] == "yes")
						{
							$order->storeGiftMessage(isset($additionalInformation["gift_message"]) ? $additionalInformation["gift_message"] : "");
						}
					}
					//custom fields
					$customFieldsData = isset($additionalInformation["custom_field"]) ? $additionalInformation["custom_field"] : false;
					//store custom fields
					if ($customFieldsData)
					{
						if (!$user->customFields()->validateCustomFields("invoice", $customFieldsData))
						{
							$opc->setError($user->customFields()->errors);
						}
						else
						{
							$customFields->storeCustomFields("invoice", $customFieldsData, $user->id, $order->oid, 0);
						}
					}
				}

				// Check shipping method
				if (trim($data->shippingMethodId) != '')
				{
					$order->updateShippingMethod($data->shippingMethodId);
					$errorMessage = '';
					$order->getShippingAmount($errorMessage);
					
					if ($errorMessage != '')
					{
						$opc->setError($errorMessage);
					}
					else 
					{
						$order->closeShipping();	
					}
					$order->recalcOrderData();
				}
				
				// Recheck and apply gift certificate
				if ($data->giftCertificateForm)
				{
					$giftError = true;
					
					$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
					$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
					$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
					$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
					if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
					{
						$giftCert = new GiftCertificates($db);
						$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
						if ($giftCertData) 
						{
							// gift certificate is correct
							$giftError = false;
							$giftCert->SetOrderGiftCert($order->oid, $gcFirstName, $gcLastName, $gcVoucher);
						
							if ($order->totalAmount > $giftCertData['balance'])
							{
								$giftCert->UpdateOrderGCAmount($order->oid, $giftCertData['balance']);
								$order->gift_cert_amount = $giftCertData['balance'];
							}
							else
							{
								$giftCert->UpdateOrderGCAmount($order->oid, $order->totalAmount);
								$order->gift_cert_amount = $order->totalAmount;
								$order->updatePaymentMethod("free");
							}
						}
					}
					
					if ($giftError)
					{
						$opc->setError("Please check gift certificate");
					}
				}
				
				// define the backurl var
				$backurl = getBackUrl();
				
				// extract data from post
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodForm = $opc->unserialize($data->paymentMethodForm);
				
				if ($paymentMethodsForm["payment-method-id"] == "free")
				{
					$order->updatePaymentMethod("free");
				}
				
				// Process payment
				if (!$opc->errors())
				{
					$order->getOrderData();
					
					if ($data->paymentMethodType != 'ipn')
					{
						foreach ($paymentMethodForm as $key=>$value)
						{
							$_REQUEST[$key] = $value;
							$_POST[$key] = $value;
						}
						if ($data->paymentMethodType != 'custom' && $data->paymentMethodType != 'free')
						{
							require_once("content/engine/actions/action.order_process_payment.php");
							if (isset($payment) && get_class($payment) == 'PAYMENT')
							{
								if (isset($is_payment_error) && $is_payment_error)
								{
									if (isset($account_blocked) && $account_blocked)
									{
										$opc->setData("backURL", $backurl);
									}
									$opc->setData("paymentErrors", $payment->error_message);
									$opc->setData("success", false);
								}
								else
								{
									$opc->setData("backURL", $backurl);
									$opc->setData("success", true);
								}
							}
						}
						else
						{
							require_once("content/engine/actions/action.order_complete_order.php");
							$opc->setData("backURL", $backurl);
							$opc->setData("success", true);
						}	
							
						break;	
					}
				}
				// NOTE: do not break here. Under some conditions next step may be executed
			}
			
			/**
			 * Get payment form
			 */
			case "getPaymentForm" : 
			{
				// Get form data
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodId = $paymentMethodsForm["payment-method-id"];
				$payment = isset($payment) && is_object($payment) && get_class($payment) != "PAYMENT" ? $payment : new PAYMENT($db);
				
				// Update payment method
				$order->updatePaymentMethod($paymentMethodId);
				
				$_SESSION["opc-payment-method-id"] = $_SESSION["pc_method_id"] = $order->paymentMethodId;

				// Free order
				if ($paymentMethodId == 0)
				{
					$opc->setData('paymentForm', false);
					$opc->setData('paymentFormValidatorJS', false);
				}
				
				// Render form based on gateway properties / order data
				else if ($payment->isMethod($paymentMethodId))
				{
					// Get payment method
					$paymentMethod = $payment->methods[$order->paymentMethodId];
					
					// Get data required to render form 
					$order->getOrderData();
					
					// ..and set template varialbes
					view()->assign("order_num", $order->order_num);
					view()->assign("order", $order);
					view()->assign("payment_method", $paymentMethod);
						
					if ($order->paymentIsRealtime == "Yes")
					{
						$order->getShippingAddress();
						
						// Create gateway class instance
						$gatewayFile = escapeFileName($paymentMethod->id);
						@require_once("content/engine/payment/payment_".$gatewayFile.".php");
						eval("\$paymentGateway = new ".$payment_processor_class."();"); // $payment_processor_class loaded from gateway file
						
						// Init gateway
						$paymentGateway->payment_page = "one_page_checkout";
						$paymentGateway->protocol = $paymentMethod->protocol;
						$paymentGateway->db_id = $order->paymentMethodId;
						$paymentGateway->getSettingsData($db);
						$paymentGateway->assignUser($user);
						$paymentGateway->assignOrder($order);
						$orderItems = $order->getOrderItems();
						$paymentGateway->assignOrderItems($orderItems);
						$paymentGateway->getCommonData($user, $order);
						
						// Check test/demo/sandbox mode
						$paymentTestMode = $paymentGateway->isTestMode();
						view()->assign("payment_test_mode", $paymentTestMode);
						
						// Get payment URL 
						$paymentURL = $paymentGateway->getPostUrl();
						if (trim($paymentURL) == "") $paymentURL = $url_https."p=one_page_checkout"; // Force HTTPS here
						
						view()->assign("auth_express", $user->express ? "yes" : "no");
						view()->assign("payment_form", $paymentGateway->getPaymentForm($db));
						view()->assign("show_cvv2_info", $paymentGateway->show_cvv2);
						
						$validatorJS = $paymentGateway->getValidatorJS();
					}
					else
					{
						// Custom payment method
						$paymentURL = $url_https."p=one_page_checkout";
						$validatorJS = '';
					}

					// TODO: gift cerificate
					/**
					if ($order->gift_cert_amount > 0 && $order->totalAmount > $order->gift_cert_amount)
					{
						//$objResponse->addAssign("gift_cert_error", "style.display", "none");
						//$objResponse->addAssign("gift_cert_alt_payment", "style.display", "block");
						//$objResponse->addScript("altPaymentGiftCert('$order->totalAmount', '$order->gift_cert_amount')");
					}
					**/
					
					// Render form
					view()->assign("payment_url", $paymentURL);
					$paymentForm = view()->fetch("templates/pages/checkout/opc/opc-payment-form.html");
					$opc->setData("paymentForm", $paymentForm);
					$opc->setData("paymentFormUrl", $paymentURL);
					$opc->setData("paymentFormValidatorJS", $validatorJS);
				}
				else
				{
					// Error: cannot find class by ID
					$opc->setError("Cannot find payment gateway by ID");
				}
					
				/**
				 * Set session variables
				 */
				$_SESSION["pc_order_id"] = $order->oid;
				$_SESSION["pc_order_amount"] = $order->totalAmount - $order->shippingAmount - $order->taxAmount - ($order->handlingSeparated ? $order->handlingAmount : 0.00);
				$_SESSION["pc_method_id"] = $order->paymentMethodId;
				$_SESSION["pc_order_quantity"] = $order->itemsCount;
				$_SESSION["pc_order_num"] = $order->order_num;

				break;
			}					
		}
	}
	
	//get shipping methods
	if ($getShippingMethods)
	{
		$rates = false;
		if ($order->shippingIsFree)
		{	
			if ($settings["ShippingShowAlternativeOnFree"] == "YES")
			{
				$ratesA = getShippingQuotes($db, $user, $order, false, false, true);
				$opc->setData("shippingMethodsAlternative", $ratesA);
			}
			else
			{
				$opc->setData("shippingMethodsAlternative", false);
			}
		}
		else
		{
			$rates = getShippingQuotes($db, $user, $order);
			
			// show alternative shipping methods for free shipping products
			if ($settings["ShippingShowAlternativeOnFree"] == "YES" && $order->shippingFreeProductsCount > 0)
			{
				$ratesA = getShippingQuotes($db, $user, $order, false, false, true);
				$ratesA = filterAlternativeShippingQuotes($rates, $ratesA);
				$opc->setData("shippingMethodsAlternative", $ratesA);
			}
		}
		
		$opc->setData("shippingMethods", $rates && is_array($rates) && count($rates) > 0 ? $rates : false);
	}
	
	$order->recalcOrderData();
	$order->getOrderData();
	
	//tax-related for shipping
	$shippingTaxable = $settings["ShippingTaxable"];
	$shippingTaxRate = 0;
	$shippingTaxDescription = "";
	if ($shippingTaxable)
	{
		$shippingTaxClassId = $settings["ShippingTaxClassId"];
		if (array_key_exists($shippingTaxClassId, $order->taxRates))
		{
			$shippingTaxRate = $order->taxRates[$shippingTaxClassId]["tax_rate"];
			$shippingTaxDescription = $order->taxRates[$shippingTaxClassId]["description"];
		}
	}
	
	$opc->setData("orderData", array(
		"subtotalAmount" => $order->subtotalAmount,
		"discountAmount" => $order->discountAmount,
		"promoDiscountAmount" => $order->promoDiscountAmount,
		// Promo Code calculations
		"promoDiscountValue" => $order->promoDiscountValue,
		"promoDiscountType" => $order->promoDiscountType,
		"promoType" => $order->promoType, // Global, Product, or Shipping
		"shippingAmount" => $order->shippingAmount,
		"handlingSeparated" => $order->handlingSeparated, 
		"handlingAmount" => $order->handlingAmount,
		"taxAmount" => $order->taxAmount - ($order->shippingTaxable ? $order->shippingTaxAmount : 0),
		"totalAmount" => $order->totalAmount,
		"shippingTaxable" => $shippingTaxable,
		"shippingTaxRate" => $shippingTaxRate,
		"shippingTaxDescription" => $shippingTaxDescription,
		"insureShipActive" => intval($order->insureShipActive),
		"insureShipAmount" => $order->insureShipAmount
	));
	
	echo $opc->getResponse();
	
	_session_write_close();
	$db->done();
	die();
	
	