<?php 

/**
 * @class CardinalCommerce 
 */
class CardinalCommerce
{
	public $is_error = false;
	public $error_message = "";
	
	public $version = "1.6";
	public $pan = "";
	//public  
	
	/**
	 * Class constructor
	 * @return this
	 */
	public function __construct()
	{
		global $settings;
		$this->is_error = false;
		$this->error_message = "";
		$_post_url = $settings["cardinalcommerce_URL"];
		$this->post_url = trim($_post_url)==""?"https://centineltest.cardinalcommerce.com/maps/txns.asp":trim($_post_url);
		return $this;
	}
	
	/**
	 * Processes XML request
	 * @param $xml
	 * @return unknown_type
	 */
	public function processRequest($xml)
	{
		global $settings;
		$ch = curl_init($this->post_url);
		
		if ($settings["ProxyAvailable"] == "YES")
		{
			//curl_setopt($ch, CURLOPT_VERBOSE, 1);
			if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
			{
				curl_setopt($ch, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
			}
			curl_setopt($ch, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
			if ($settings["ProxyRequiresAuthorization"] == "YES")
			{
				curl_setopt($ch, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
			}

			curl_setopt($ch, CURLOPT_TIMEOUT, 120);
		}
				
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, "cmpi_msg=".$xml);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,  2);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);

		curl_setopt($ch, CURLOPT_TIMEOUT, 300);
			
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($ch, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
		curl_setopt($ch, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
		$result = curl_exec($ch);
						
		if (curl_errno($ch) > 0)
		{
			$this->is_error = true;
			$this->error_message = curl_error($ch);
		}
		elseif ($result == "")
		{
			$this->is_error = true;
			$this->error_message = PAYMENT_TRANSACTION_ERROR_TEXT;
		}
		else
		{
			$xmlParser = new XMLParser();
			$parsed = $xmlParser->parse($result);
			
			if ($parsed["CardinalMPI"][0]["ErrorNo"][0] != "0")
			{
				$this->is_error = true;
				$this->error_message = "Paymen gateway response: ".$parsed["CardinalMPI"][0]["ErrorDesc"][0];
			}
			else
			{
				return $parsed;
			}
		}
		return false;
	}
		
	/**
	 * Get cmpi_lookup xml
	 * @param $card_number
	 * @param $card_exp_month
	 * @param $card_exp_year
	 * @return string
	 */
	public function getLookupXML($first_name, $last_name, $card_number, $card_exp_month, $card_exp_year, $currency)
	{
		global $settings;
		$currency = strtoupper($currency);
		$currencies = array("USD" => "840", "EUR" => "978", "JPY" => "392", "CAD" => "124", "GBP" => "826");
		if (!array_key_exists($currency, $currencies)) $currency = "USD";
		
		$xml = 
			"<CardinalMPI>".
				"<MsgType>cmpi_lookup</MsgType>".
				"<Version>".utf8_encode($this->version)."</Version>".
				"<ProcessorId>".utf8_encode($settings["cardinalcommerce_Processor_Id"])."</ProcessorId>".
				"<MerchantId>".utf8_encode($settings["cardinalcommerce_Merchant_Id"])."</MerchantId>".
				"<TransactionPwd>".utf8_encode($settings["cardinalcommerce_Password"])."</TransactionPwd>".
				"<TransactionType>C</TransactionType>".
				"<Amount>".utf8_encode(number_format($this->common_vars["order_total_amount"]["value"], 2, "", ""))."</Amount>".						
				"<CurrencyCode>".$currencies[$currency]."</CurrencyCode>".
				"<CardNumber>".utf8_encode($card_number)."</CardNumber>".
				"<CardExpMonth>".utf8_encode($card_exp_month)."</CardExpMonth>".
				"<CardExpYear>".utf8_encode($card_exp_year)."</CardExpYear>".
				"<OrderNumber>".utf8_encode($this->common_vars["order_id"]["value"])."</OrderNumber>".
				"<UserAgent>".utf8_encode($_SERVER["HTTP_USER_AGENT"])."</UserAgent>".
				"<BrowserHeader>".utf8_encode($_SERVER["HTTP_ACCEPT"])."</BrowserHeader>".
				"<Recurring>N</Recurring>".
				"<EMail>".utf8_encode($this->common_vars["billing_email"]["value"])."</EMail>".
				"<IPAddress>".utf8_encode($_SERVER["REMOTE_ADDR"])."</IPAddress>".
				"<BillingFirstName>".utf8_encode($first_name)."</BillingFirstName>".
				"<BillingLastName>".utf8_encode($last_name)."</BillingLastName>".
			"</CardinalMPI>";
		return $xml;
	}
		
	/**
	 * Get cmpi_authenticate xml
	 * @param $PAResPayload
	 * @return string
	 */
	public function getAuthenticateXML($PAResPayload)
	{
		global $settings;
		$xml = 
			"<CardinalMPI>".
				"<MsgType>cmpi_authenticate</MsgType>".
				"<Version>1.6</Version>".
				"<ProcessorId>".utf8_encode($settings["cardinalcommerce_Processor_Id"])."</ProcessorId>".
				"<MerchantId>".utf8_encode($settings["cardinalcommerce_Merchant_Id"])."</MerchantId>".
				"<TransactionPwd>".utf8_encode($settings["cardinalcommerce_Password"])."</TransactionPwd>".
				"<TransactionType>C</TransactionType>".
				"<TransactionId>".utf8_encode($_SESSION["CardinalTransactionId"])."</TransactionId>".
				"<PAResPayload>".urlencode($PAResPayload)."</PAResPayload>".
			"</CardinalMPI>";
		return $xml;
	}
			
	/**
	 * Get redirect HTML 
	 * @param $ACSUrl
	 * @param $Payload
	 * @param $NotifyURL
	 * @return string
	 */
	public function getRedirectHtml($ACSUrl, $Payload, $NotifyURL)
	{
		return 
			'<HTML>'."\n".
			'<BODY noonload="document.frmLaunch.submit();">'."\n".
			'<FORM name="frmLaunch" method="POST" action="'.$ACSUrl.'">'."\n".
			'<input type=hidden name="PaReq" value="'.$Payload.'">'."\n".
			'<input type=hidden name="TermUrl" value="'.$NotifyURL.'">'."\n".
			'<input type=hidden name="MD" value="">'."\n".
			'<input type="submit"/></FORM>'."\n".
			'</BODY>'."\n".
			'</HTML>';
	}
}
