<?php 

class paypalExpressCheckout
{
	public $ApiURL = "";
	public $Email = ""; //subject, or accelerated boarding email
	public $Username = "";
	public $Password = "";
	public $OrderTotal = "0.00";
	public $ReturnURL = "";
	public $CancelURL = "";
	public $currencyCode = "USD";
	public $PaymentAction = "";
	public $NoShipping = false;

	public $mode = "live"; // live or test
	public $is_error = false;
	public $error_messages = array();
	public $request = "";
	public $response = "";
	
	/**
	 * Returns API URL
	 * @return string
	 */
	public function getAPIUrl()
	{
		if ($this->ApiURL == "")
		{
			//$url = $this->mode == "live" ? "https://api.paypal.com/2.0/" : "https://api-aa.sandbox.paypal.com/2.0/";
			$url = $this->mode == "live" ? "https://api-3t.paypal.com/nvp" : "https://api-aa.sandbox.paypal.com/2.0/";
		}
		else
		{
			$url = $this->ApiURL;
		}
		return $url;
	}
	
	/**
	 * Returns express checkout URL
	 * @param $token
	 * @return unknown_type
	 */
	public function getExpressCheckoutUrl($token)
	{
		$url = $this->mode == "live" ? "https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout" : "https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout";
		return $url."&token=".$token;
	}
	
	/**
	 * Sends query to paypal website, adds headers, etc
	 * @param $xml
	 * @return unknown_type
	 */
	public function sendQuery($xml)
	{
		global $settings;
		
		$this->request = $xml;
		
		$_post_url = $this->getAPIUrl();
		$_cert_path = $settings["GlobalServerPath"]."/content/engine/payment/paypal/certificate.pem";		

		$c = curl_init();
		
		// check for proxy
		if ($settings["ProxyAvailable"] == "YES")
		{
			if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
			{
				curl_setopt($c, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
			}
			curl_setopt($c, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
			if ($settings["ProxyRequiresAuthorization"] == "YES")
			{
				curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
			}

			curl_setopt($c, CURLOPT_TIMEOUT, 120);
		}
		
		curl_setopt($c, CURLOPT_URL, $_post_url);
		
		//request certificate only when account setup already done
		if ($this->Username != "" && $this->Password != "")
		{
			curl_setopt($c, CURLOPT_SSLCERT, $_cert_path);
		}
		
		curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($c, CURLOPT_POSTFIELDS, $this->request);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($c, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
		curl_setopt($c, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
		
		if (!ini_get('safe_mode')) set_time_limit(3000);
		
		$buffer = curl_exec($c);
		
		$this->response = $buffer;
			
		if ($buffer && curl_errno($c) == 0)
		{
			$xmlParser = new XMLParser();
			$data = $xmlParser->parse($buffer);
			return $data;
		}
		
		$this->is_error = true;
		if (curl_errno($c))
		{
			$this->error_messages[] = curl_error($c);
		}
		else
		{
			$this->error_messages[] = "Unknown error (1)";
		}
		return false;
	}
	
	
	/**
	 * SetExpressCheckout API call
	 * @return boolean
	 */
	public function SetExpressCheckout()
	{
		//reset session var - Token
		global $_SESSION;
		unset($_SESSION['paypal_express_checkout_token']);
		
		//build XML
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents">'.
   							'<Username>'.utf8_encode($this->Username).'</Username>'.
   							'<Password>'.utf8_encode($this->Password).'</Password>'.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<SetExpressCheckoutReq xmlns="urn:ebay:api:PayPalAPI">'.
   						'<SetExpressCheckoutRequest xsi:type="ns:SetExpressCheckoutRequestType">'.
   							'<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
							'<SetExpressCheckoutRequestDetails xmlns="urn:ebay:apis:eBLBaseComponents">'.
								'<PaymentAction>'.utf8_encode($this->PaymentAction).'</PaymentAction>'.
   								'<OrderTotal xmlns="urn:ebay:apis:eBLBaseComponents" currencyID="'.$this->currencyCode.'" xsi:type="cc:BasicAmountType">'.number_format($this->OrderTotal, 2, ".", "").'</OrderTotal> '.
								'<ReturnURL xsi:type="xsd:string">'.utf8_encode(str_replace("&", "&amp;", $this->ReturnURL)).'</ReturnURL>'.
								'<CancelURL xsi:type="xsd:string">'.utf8_encode(str_replace("&", "&amp;", $this->CancelURL)).'</CancelURL>'.
								($this->NoShipping?'<NoShipping>1</NoShipping>':'').//'<ReqConfirmShipping>1</ReqConfirmShipping>').
							'</SetExpressCheckoutRequestDetails>'.
						'</SetExpressCheckoutRequest>'.
					'</SetExpressCheckoutReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		
		$response = $this->sendQuery($xml);
		//check response
		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["SetExpressCheckoutResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["SetExpressCheckoutResponse"][0];
				
				//check token
				if (isset($a["Token"][0]))
				{
					$_SESSION["paypal_express_checkout_token"] = $a["Token"][0];
					return $a["Token"][0];
				}
				//token is not set
				else
				{
					//store errors
					if (isset($a["Errors"]))
					{
						$this->is_error = true;
						foreach ($a["Errors"] as $key=>$value)
						{
							$this->error_messages[] = $value["ShortMessage"][0].($value["ShortMessage"][0]!=$value["LongMessage"][0]?(" - ".$value["LongMessage"][0]):"");
						}
					}
				}
			}
		}
		//check for unknown error
		if (!$this->is_error)
		{
			$this->is_error = true;
			$this->error_messages[] = "Unknown error";
		}
		return false;
	}
	
	/**
	 * GetExpressCheckoutDetails API call
	 * @param $token
	 * @return boolean
	 */
	function GetExpressCheckoutDetails($token = "")
	{
		//reset session var - PayerID
		global $_SESSION;
		session_unregister("paypal_express_checkout_payer_id");
		
		//get tocken value
		if ($token == "")
		{
			$token = isset($_SESSION["paypal_express_checkout_token"])?$_SESSION["paypal_express_checkout_token"]:"";
		}
		
		//check is token blank
		if ($token == "")
		{
			$this->is_error = true;
			$this->error_messages[] = "PayPal Express Checkout Token is Blank";
			return false;
		}
		
		//build XML
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<GetExpressCheckoutDetailsReq xmlns="urn:ebay:api:PayPalAPI">'.
	   					'<GetExpressCheckoutDetailsRequest xsi:type="ns:SetExpressCheckoutRequestType">'.
	   						'<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
	   						'<Token>'.utf8_encode($token).'</Token>'.
	   					'</GetExpressCheckoutDetailsRequest>'.
	   				'</GetExpressCheckoutDetailsReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
			
		//send query
		$response = $this->sendQuery($xml);
			
		//check response
		if ($response)
		{
			//chec XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["GetExpressCheckoutDetailsResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["GetExpressCheckoutDetailsResponse"][0];
				
				//check is token returned
				if (isset($a["GetExpressCheckoutDetailsResponseDetails"][0]["Token"][0]))
				{
					//check is token correct
					if ($a["GetExpressCheckoutDetailsResponseDetails"][0]["Token"][0] == $token)
					{ 
						$_SESSION["paypal_express_checkout_payer_id"] = $a["GetExpressCheckoutDetailsResponseDetails"][0]["PayerInfo"][0]["PayerID"][0];
						return ($a["GetExpressCheckoutDetailsResponseDetails"][0]);						
					}
					//token is not the same as sent
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect PayPal Express Checkout Token";
						return false;
					}
				}
				//token is not returned
				else
				{
					//store errors
					if (isset($a["Errors"]))
					{
						$this->is_error = true;
						foreach ($a["Errors"] as $key=>$value)
						{
							$this->error_messages[] = $value["ShortMessage"][0].($value["ShortMessage"][0]!=$value["LongMessage"][0]?(" - ".$value["LongMessage"][0]):"");
						}
					}
				}
			}
		}
		//check for unknown error
		if (!$this->is_error)
		{
			$this->is_error = true;
			$this->error_messages[] = "Unknown error";
		}

		return false;
	}
	
	/**
	 * DoExpressCheckoutPayment API call
	 * @param $token
	 * @param $payer_id
	 * @return boolean
	 */
	public function DoExpressCheckoutPayment($token = "", $payer_id = "")
	{
		global $_SESSION;
		
		//get tocken value
		if ($token == "")
		{
			$token = isset($_SESSION["paypal_express_checkout_token"])?$_SESSION["paypal_express_checkout_token"]:"";
		}
		
		//check is token blank
		if ($token == "")
		{
			$this->is_error = true;
			$this->error_messages[] = "PayPal Express Checkout Token is Blank";
			return false;
		}
		
		//get tocken value
		if ($payer_id == "")
		{
			$payer_id = isset($_SESSION["paypal_express_checkout_payer_id"])?$_SESSION["paypal_express_checkout_payer_id"]:"";
		}
		
		//check is payer id blank
		if ($payer_id == "")
		{
			$this->is_error = true;
			$this->error_messages[] = "PayPal Express Checkout Payer ID is Blank";
			return false;
		}
		
		//build XML
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<DoExpressCheckoutPaymentReq xmlns="urn:ebay:api:PayPalAPI">'.
	   					'<DoExpressCheckoutPaymentRequest xsi:type="ns:SetExpressCheckoutRequestType">'.
	   						'<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
							'<DoExpressCheckoutPaymentRequestDetails xmlns="urn:ebay:apis:eBLBaseComponents">'.
	   							'<Token>'.utf8_encode($token).'</Token>'.
	   							'<PaymentAction>'.utf8_encode($this->PaymentAction).'</PaymentAction>'.
		   						'<PayerID>'.utf8_encode($payer_id).'</PayerID>'.
		   						'<PaymentDetails>'.
	   								'<OrderTotal xmlns="urn:ebay:apis:eBLBaseComponents" currencyID="'.$this->currencyCode.'">'.number_format($this->OrderTotal, 2, ".", "").'</OrderTotal>'.
									'<ButtonSource>PCart_cart_EC_US</ButtonSource>'.
            	   						'</PaymentDetails>'.
            	   					'</DoExpressCheckoutPaymentRequestDetails>'.
	   					'</DoExpressCheckoutPaymentRequest>'.
	   				'</DoExpressCheckoutPaymentReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		
		//send query	
		$response = $this->sendQuery($xml);
		
		//check response
		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoExpressCheckoutPaymentResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoExpressCheckoutPaymentResponse"][0];
				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
				}
				//potentially all is good
				else
				{
					//check is payment token returned
					if (isset($a["DoExpressCheckoutPaymentResponseDetails"][0]["Token"][0]))
					{
						//check tocken is the same as sent
						if ($a["DoExpressCheckoutPaymentResponseDetails"][0]["Token"][0] == $token)
						{
							if (isset($a["DoExpressCheckoutPaymentResponseDetails"][0]["PaymentInfo"]["0"]))
							{
								//return payment data
								return $a["DoExpressCheckoutPaymentResponseDetails"][0]["PaymentInfo"]["0"];
							}
						}
						else
						{
							$this->is_error = true;
							$this->error_messages[] = "Incorrect PayPal Express Checkout Token";
							return false;
						}
					}
				}
				
			}
		}
		//check for unknown error
		if (!$this->is_error)
		{
			$this->is_error = true;
			$this->error_messages[] = "Unknown error";
		}
		return false;
	}
	
	/**
	 * DoCapture API call
	 * @param $auth_id
	 * @param $note
	 * @param $complete_type
	 * @return boolean
	 */
	public function DoCapture($auth_id,$note,$complete_type)
	{
		//build XML
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<DoCaptureReq xmlns="urn:ebay:api:PayPalAPI">'.
         				'<DoCaptureRequest xmlns="urn:ebay:api:PayPalAPI">'.
				            '<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
				            '<AuthorizationID>'.utf8_encode($auth_id).'</AuthorizationID>'.
				          //  '<InvoiceID>'.utf8_encode($invoice_id).'</InvoiceID>'.
				            '<Amount currencyID="'.$this->currencyCode.'" xsi:type="cc:BasicAmountType">'.number_format($this->OrderTotal, 2, ".", "").'</Amount>'.
				            '<CompleteType>'.utf8_encode($complete_type).'</CompleteType>'.
				            '<Note>'.utf8_encode($note).'</Note>'.
				         '</DoCaptureRequest>'.
				      '</DoCaptureReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		
		$response = $this->sendQuery($xml);
		
		if ($response)
		{
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoCaptureResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoCaptureResponse"][0];
				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
					return $this->error_messages;

				}
				//potentially all is good
				else
				{ 
					$this->PaymentAction = $a;
					
					//check tocken is the same as sent
					if (($a["Ack"][0] == "Success" || $a["Ack"][0] == "SuccessWithWarnings" || $a["Ack"][0] == "SuccessWithWarning") && $a["DoCaptureResponseDetails"][0]["AuthorizationID"][0] = $auth_id)
					{
						if (isset($a["DoCaptureResponseDetails"][0]["PaymentInfo"]["0"]))
						{
							//return payment data
							return $a["DoCaptureResponseDetails"][0]["PaymentInfo"]["0"];
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect transaction status";
						return false;
					}
				}			
			}
		}
		//check for unknown error
		if (!$this->is_error)
		{
			$this->is_error = true;
			$this->error_messages[] = "Unknown error";
		}
		return false;
	}

	/**
	 * DoAuthorization API call
	 * @param $auth_id
	 * @return boolean
	 */
	public function DoAuthorization($auth_id)
	{
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<DoAuthorizationReq xmlns="urn:ebay:api:PayPalAPI">'.
         				  '<DoAuthorizationRequest xmlns="urn:ebay:api:PayPalAPI">'.
				            '<Version xmlns="urn:ebay:apis:eBLBaseComponents">60.0</Version>'.
				            '<TransactionID>'.utf8_encode($auth_id).'</TransactionID>'.
				            '<Amount currencyID="'.$this->currencyCode.'" xsi:type="cc:BasicAmountType">'.number_format($this->OrderTotal, 2, ".", "").'</Amount>'.				           
				         '</DoAuthorizationRequest>'.
				      '</DoAuthorizationReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		//send query	
		$response = $this->sendQuery($xml);
		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoAuthorizationResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoAuthorizationResponse"][0];				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
					return $this->error_messages;
				}
				//potentially all is good
				else
				{ 
					$this->PaymentAction = $a;
					//check tocken is the same as sent
					if ($a["Ack"][0] == "Success")
					{
						if (isset($a["TransactionID"][0]))
						{
							//return payment data
							return $a["TransactionID"][0];
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect transaction status";
						return false;
					}
				}
			}
		}
	}

	/**
	 * DoReauthorization API call
	 * @param $auth_id
	 * @param $note
	 * @return boolean
	 */
	public function DoReauthorization($auth_id,$note)
	{
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<DoReauthorizationReq xmlns="urn:ebay:api:PayPalAPI">'.
         				  '<DoReauthorizationRequest xmlns="urn:ebay:api:PayPalAPI">'.
				            '<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
				            '<AuthorizationID>'.utf8_encode($auth_id).'</AuthorizationID>'.
				            '<Amount currencyID="'.$this->currencyCode.'" xsi:type="cc:BasicAmountType">'.number_format($this->OrderTotal, 2, ".", "").'</Amount>'.	
					     '<Note>'.utf8_encode($note).'</Note>'.			           
				         '</DoReauthorizationRequest>'.
				      '</DoReauthorizationReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		//send query	
		$response = $this->sendQuery($xml);
		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoReauthorizationResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoReauthorizationResponse"][0];				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
					return $this->error_messages;
				}
				//potentially all is good
				else
				{ 
					$this->PaymentAction = $a;
					//check tocken is the same as sent
					if ($a["Ack"][0] == "Success") 
					{
						if (isset($a["TransactionID"][0]))
						{
							//return payment data
							return $a["TransactionID"][0];
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect transaction status";
						return false;
					}
				}
			}
		}
	}


	/**
	 * DoVoid API call
	 * @param $auth_id
	 * @param $note
	 * @return boolean
	 */
	public function DoVoid($auth_id,$note)
	{
		$xml = 
			'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials '.
   						'xmlns="urn:ebay:api:PayPalAPI" '.
   						'SOAP-ENV:mustUnderstand="1"> '.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
	   				'<DoVoidReq xmlns="urn:ebay:api:PayPalAPI">'.
         				  '<DoVoidRequest xmlns="urn:ebay:api:PayPalAPI">'.
				            '<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
				            '<AuthorizationID>'.utf8_encode($auth_id).'</AuthorizationID>'.
				            '<Note>'.utf8_encode($note).'</Note>'.				           
				         '</DoVoidRequest>'.
				      '</DoVoidReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		//send query	
		$response = $this->sendQuery($xml);
		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoVoidResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoVoidResponse"][0];				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
					return $this->error_messages;
				}
				//potentially all is good
				else
				{ 
					$this->PaymentAction = $a;
					//check tocken is the same as sent
					if ($a["Ack"][0] == "Success") 
					{
						if (isset($a["AuthorizationID"][0]))
						{
							//return payment data
							return $a["AuthorizationID"][0];
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect transaction status";
						return false;
					}
				}
			}
		}
	}
	
	/**
	 * DoRefund API call
	 * @param $transaction_id
	 * @param $refund_type
	 * @param $note
	 * @param $amount
	 * @return boolean
	 */
	public function DoRefund($transaction_id, $refund_type, $note,$amount)
	{
		$refund_amount ="";
		if ($refund_type == "Partial")
		{
			$refund_amount = '<Amount currencyID="'.$this->currencyCode.'" xsi:type="cc:BasicAmountType">'.number_format($amount, 2, ".", "").'</Amount>';
		}
		
		$xml =	'<?xml version="1.0" encoding="UTF-8"?>'.
			'<SOAP-ENV:Envelope '.
   				'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
   				'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
	   			'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
   				'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
   				'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> '.
   				'<SOAP-ENV:Header>'.
	   				'<RequesterCredentials xmlns="urn:ebay:api:PayPalAPI" SOAP-ENV:mustUnderstand="1">'.
   						'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents"> '.
   							'<Username>'.utf8_encode($this->Username).'</Username> '.
   							'<Password>'.utf8_encode($this->Password).'</Password> '.
   							'<Subject>'.utf8_encode($this->Email).'</Subject>'.
	   					'</Credentials>'.
   					'</RequesterCredentials>'.
   				'</SOAP-ENV:Header>'.
   				'<SOAP-ENV:Body>'.
					'<RefundTransactionReq xmlns="urn:ebay:api:PayPalAPI">'.
						'<RefundTransactionRequest>'.
							'<Version xmlns="urn:ebay:apis:eBLBaseComponents" xsi:type="xsd:string">60.0</Version>'.
							'<TransactionID>'.utf8_encode($transaction_id).'</TransactionID>'.
							'<RefundType>'.utf8_encode($refund_type).'</RefundType>'.
							$refund_amount.
							'<Memo xsi:type="xsd:string">'.utf8_encode($note).'</Memo>'.
						'</RefundTransactionRequest>'.
					'</RefundTransactionReq>'.
				'</SOAP-ENV:Body>'.
			'</SOAP-ENV:Envelope>';
		$response = $this->sendQuery($xml);

		if ($response)
		{
			//check XML response data
			if (isset($response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["RefundTransactionResponse"][0]))
			{
				$a = $response["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["RefundTransactionResponse"][0];				
				//check is there transaction error
				if (isset($a["Errors"]))
				{
					$this->is_error = true;
					foreach ($a["Errors"] as $key=>$value)
					{
						$this->error_messages[] = $value["ShortMessage"][0].(trim($value["ShortMessage"][0])!=trim($value["LongMessage"][0])?(" - ".$value["LongMessage"][0]):"");
					}
					return $this->error_messages;
				}
				//potentially all is good
				else
				{ 
					$this->PaymentAction = $a;
					//check tocken is the same as sent
					if ($a["Ack"][0] == "Success")
					{
						if (isset($a["RefundTransactionID"][0]))
						{
							//return payment data
							return $a["RefundTransactionID"][0];
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_messages[] = "Incorrect transaction status";
						return false;
					}
				}
			}
		}
	}
	
	/**
	 * Creates paypal transaction record in DB
	 * @param $db
	 * @param $oid
	 * @param $uid
	 * @param $note
	 * @param $response
	 * @param $amount
	 * @param $gateway_details
	 */
	public function createPayPalTransaction($db,$oid,$uid,$note,$response,$amount,$gateway_details)
	{
		$db->reset();
		$db->assign("oid", $oid);
		$db->assign("uid", $uid);
		
		$db->assign("completed", "NOW()");
		$db->assignStr("extra", $note);
		
		$db->assignStr("payment_type", $gateway_details[0]);
		$db->assignStr("payment_gateway", $gateway_details[1]);
		$db->assignStr("payment_response", $response);
		
		$db->assignStr("order_total_amount", $amount);
		
		$db->assignStr("shipping_method", "");
		$db->assignStr("shipping_submethod", "");
		$db->assignStr("shipping_amount", "");
		
		$db->assignStr("tax_amount", "");			
		$db->insert(DB_PREFIX."payment_transactions");
	}
}
