<?php 
/**
 * Users actions
 * @var unknown_type
 */
define("USER_SIGNUP", "user_signup");
define("USER_LOGIN", "user_login");
define("USER_LOGOUT", "user_logout");
define("USER_PASSWORD_RESET", "password_reset");
define("USER_TEMP_PASSWORD", "temp_password");
define("USER_NEW_PASSWORD", "new_password");
define("USER_NEW_USERNAME", "new_username");
define("USER_UPDATE_PROFILE", "user_updateprofile");
define("USER_EXPRESS_CHECKOUT", "user_expresscheckout");
define("USER_START_CHECKOUT", "user_start_checkout");
define("USER_ADD_ADDRESS", "AddShippingAddress");
define("USER_DELETE_ADDRESS", "DeleteShippingAddress");
define("USER_UPDATE_ADDRESS", "UpdateShippingAddress");
define("USER_ADD_TO_WISHLIST", "UserAddToWishlist");
define("USER_FORM_MAIL", "FormMail");
define("USER_SET_COOKIE", "set_c");
define("USER_UNSET_COOKIE", "unset_c");
define("USER_ADD_REVIEW", "AddReview");

define("EXPRESS_CHECKOUT_USER", "ExpressCheckoutUser");

/**
 * 
 * @author Victor
 */
class USER
{
	public $id = 0;
	public $session_id = "";
	public $level = 0;
	public $fname = "";
	public $lname = "";
	public $login = "";
	public $auth_ok = false;
	public $data = array();
	public $messages = array();
	public $is_error = false;
	public $errors = array();
	public $blocked = false;
	public $cookie = "CartUserCookie";
	public $back_user = false;
	public $back_user_login = "";
	public $express = false;
	public $express_dmp = false;
	private $_db;
	private $_settings;
	private $_customFields = null;

	/**
	 * Class constructor
	 * @param $db
	 * @param $settings
	 * @return USER
	 */
	public function __construct(&$db, &$settings)
	{
		$this->_db = $db;
		$this->_settings = $settings;
			
		//set cookie prefix
		$this->cookie = $this->_settings["SecurityCookiesPrefix"];
			
		// init user, load cookies & session
		global $_COOKIE;
			
		// check returning customers
		$this->back_user = in_array($this->cookie."LoginUserName", array_keys($_COOKIE));
		$this->back_user_login = isset($_COOKIE[$this->cookie."LoginUserName"])?$_COOKIE[$this->cookie."LoginUserName"]:"";
			
		// check user session
		$this->id = isset($_SESSION["UserID"]) ? intval($_SESSION["UserID"]) : 0;
		$this->session_id = isset($_SESSION["SessionID"]) ? $_SESSION["SessionID"] : "";
			
		// check user registered (by user id)
		if ($this->id)
		{
			$this->db()->query("
				SELECT uid FROM ".DB_PREFIX."users 
				WHERE 
					uid='".intval($this->id)."' AND
					DATE_ADD(session_date, INTERVAL ".intval($this->_settings["SecurityUserTimeout"])." SECOND)>NOW() AND
					".($this->_settings["SecurityAccountBlocking"] == "YES" ? "block_date < NOW() AND" : "")."
					session_id='".$this->db()->escape($this->session_id)."'
			");
			if ($this->db()->moveNext())
			{
				// user is registered - user_id, session_id correct and not expired
				$this->auth_ok = true;
				$this->getUserData();
				
				// update session_timeout in db
				$this->db()->query("UPDATE ".DB_PREFIX."users SET session_date=NOW() WHERE uid='".intval($this->id)."'");
				
				// check express checkout users
				$this->express = $this->data["login"] == EXPRESS_CHECKOUT_USER;
				$this->express_dmp = false;
					
				// store user data to session
				$_SESSION["SessionID"] = $this->session_id;
				$_SESSION["UserID"] = $this->id;
			}
			else
			{
				// user is not registered - can't use existing user session
				$this->id = 0;
				$this->session_id = uniqid(md5(rand(1, 1000000)));
				$_SESSION["SessionID"] = $this->session_id;
				$_SESSION["UserID"] = $this->id;
			}
		}
		return $this;
	}
	/**
	 * Account Blocking on payment failures
	 * @param $hours
	 */
	public function block($hours)
	{
		$this->db()->query("UPDATE ".DB_PREFIX."users SET block_date=DATE_ADD(NOW(), INTERVAL ".intval($hours)." HOUR) WHERE uid='".intval($this->id)."'");
	}
	/**
	 * Sets error
	 * @param $message
	 * @return unknown_type
	 */
	public function setError($message)
	{
		if (is_array($message))
		{
			foreach($message as $m) $this->errors[] = $m;
		}
		else
		{
			$this->errors[] = $message;
		}
		$this->is_error = true;
	}
	
	/**
	 * Return database instance
	 * @return DB
	 */
	public function db()
	{
		return $this->_db;
	}

	/**
	 * Assign custom fields class
	 * @param $customFields
	 */
	public function customFields($customFields = false)
	{
		if ($customFields) $this->_customFields = $customFields;
		return $this->_customFields = $this->_customFields != null ? $this->_customFields : new CustomFields($this->_db);
	}
	
	/**
	 * Update / edit billing address on express checkout
	 * @param $address
	 */
	public function payPalExpressUserInfo($address)
	{
		//preselect country
		$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2 LIKE '".$this->db()->escape($address["PayerInfo"][0]["Address"][0]["Country"][0])."'");
		
		$country = $this->db()->moveNext() ? $this->db()->col["coid"] : 1;
		
		//preselect state
		$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".$country."' AND short_name LIKE '".$this->db()->escape($address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0])."'");
		
		if ($this->db()->moveNext())
		{
			$state = $this->db()->col["stid"];
			$province = $this->db()->col["name"];
		}
		else
		{
			$state = 0;
			$province = $address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0];
		}
		
		$this->db()->reset();
		$this->db()->assignStr("fname", 
			trim(
				$address["PayerInfo"][0]["PayerName"][0]["FirstName"][0]." ".
				$address["PayerInfo"][0]["PayerName"][0]["MiddleName"][0]
			)
		);
		$this->db()->assignStr("lname", $address["PayerInfo"][0]["PayerName"][0]["LastName"][0]);
		
		$this->db()->assignStr("address1", $address["PayerInfo"][0]["Address"][0]["Street1"][0]);
		$this->db()->assignStr("address2", $address["PayerInfo"][0]["Address"][0]["Street2"][0]);
		$this->db()->assignStr("city", $address["PayerInfo"][0]["Address"][0]["CityName"][0]);
		$this->db()->assignStr("state", $state);
		$this->db()->assignStr("province", $province);
		$this->db()->assignStr("country", $country);
		$this->db()->assignStr("phone", $address["PayerInfo"][0]["Address"][0]["Phone"][0]);
		$this->db()->assignStr("zip", $address["PayerInfo"][0]["Address"][0]["PostalCode"][0]);
		
		$this->db()->assignStr("email", $address["PayerInfo"][0]["Payer"][0]);
		$this->db()->update(DB_PREFIX."users", "WHERE uid='".intval($this->id)."'");
	}
		
	/**
	 * Add address from PayPal to User's Address Book
	 * @param $paypal_address
	 * @return boolean
	 */
	public function payPal2Shipping($paypal_address)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".intval($this->id)."'");
		if ($this->db()->moveNext())
		{
			$address = $this->db()->col;
			//check primary
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND is_primary='Yes'");
			$is_primary = $this->db()->moveNext()?"No":"Yes";
			//get country id
			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2 = '".$this->db()->escape($paypal_address["PayerInfo"][0]["Address"][0]["Country"][0])."'");
			if ($this->db()->moveNext())
			{
				$address_country = $this->db()->col["coid"];
				$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($address_country)."' AND short_name='".$this->db()->escape($paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0])."'");
				if ($this->db()->moveNext())
				{
					$address_state = $this->db()->col["stid"];
					$address_province = "";
				}
				else
				{
					$address_state = 0;
					$address_province = $paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0];
				}
			}
			else
			{
				$address_country = 1;
				$address_state = 0;
				$address_province = $paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0];
			}
			
			$this->db()->reset();
			$this->db()->assign("uid", $this->id);
			$this->db()->assign("country", intval($address_country));
			$this->db()->assign("state", intval($address_state));
			$this->db()->assignStr("is_primary", $is_primary);
			$this->db()->assignStr("name", $paypal_address["PayerInfo"][0]["Address"][0]["Name"][0]);
			$this->db()->assignStr("company", "");
			$this->db()->assignStr("address1", $paypal_address["PayerInfo"][0]["Address"][0]["Street1"][0]);
			$this->db()->assignStr("address2", $paypal_address["PayerInfo"][0]["Address"][0]["Street2"][0]);
			$this->db()->assignStr("city", $paypal_address["PayerInfo"][0]["Address"][0]["CityName"][0]);
			$this->db()->assignStr("province", $address_province);
			$this->db()->assignStr("zip", $paypal_address["PayerInfo"][0]["Address"][0]["PostalCode"][0]);
			$address_id = $this->db()->insert(DB_PREFIX."users_shipping");
			return $address_id;
		}
		return false;
	}

	/**
	 * Add shipping address to users account
	 * @param $address
	 * @return mixed
	 */
	public function addShippingAddress($address, $customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		
		/*if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($address["company"]) || !validate($address["company"], VALIDATE_STRING))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($address["address2"]) || !validate($address["address2"], VALIDATE_STRING))) $this->setError("Enter a valid address line 2");
		
		$this->validateAddress($address);*/
		
		//validate custom fields for shipping
		if ($customFieldsData)
		{
			$this->customFields()->resetErrors();
			$this->customFields()->validateCustomFields("shipping", $customFieldsData);
			if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
		}
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if( !$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["is_primary"]) && ($address["is_primary"] == "Yes"))
				{
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["is_primary"] = "Yes";
				}
				else
				{
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["is_primary"] = "No";
				}
			}
			//insert address values
			$dmp_name_name = $address["name"]?$address["name"]:"";
			$dmp_name = $address["fname"]?$address["fname"]:$dmp_name_name;
			$this->db()->reset();
			$this->db()->assign("uid", intval($this->id));
			$this->db()->assign("country", intval($address["country"]));
			$this->db()->assign("state", intval($address["state"]));
			$this->db()->assignStr("is_primary", $address["is_primary"]);
			$this->db()->assignStr("name", $dmp_name);
			$this->db()->assignStr("lname", $address["lname"]);
			$this->db()->assignStr("company", $address["company"]);
			$this->db()->assignStr("address1", $address["address1"]);
			$this->db()->assignStr("address2", $address["address2"]);
			$this->db()->assignStr("city", $address["city"]);
			$this->db()->assignStr("province", $address["province"]);
			$this->db()->assignStr("zip", $address["zip"]);
			$this->db()->assignStr("phone", $address["phone"]);
			$address_id = $this->db()->insert(DB_PREFIX."users_shipping");
			
			//store custom fields data
			if ($customFieldsData) $this->customFields()->storeCustomFields("shipping", $customFieldsData, $this->id, 0, $address_id);
			
			return $address_id;
		}
		return false;
	}
	
	/**
	 * Update shipping address
	 * @param $address_id
	 * @param $address
	 * @return boolean
	 */
	public function updateShippingAddress($address_id, $address, $customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		//$this->validateAddress($address);
		
		/*if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($address["company"]) || !validate($address["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($address["address2"]) || !validate($address["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");*/
		
		//validate custom fields for shipping
		if ($customFieldsData)
		{
			$this->customFields()->resetErrors();
			$this->customFields()->validateCustomFields("shipping", $customFieldsData);
			if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
		}
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if (!$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["set_is_primary"]) && ($address["set_is_primary"] == "Yes"))
				{
					//die("yes");
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["is_primary"] = "Yes";
				}
				else
				{
					//die("no");
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["is_primary"] = "No";
				}
			}
			
			//insert address values
			$this->db()->reset();
			$this->db()->assign("uid", intval($this->id));
			$this->db()->assign("country", intval($address["country"]));
			
			$this->db()->assignStr("is_primary", $address["is_primary"]);
			$this->db()->assignStr("name", $address["fname"]);
			$this->db()->assignStr("lname", $address["lname"]);
			$this->db()->assignStr("company", $address["company"]);
			$this->db()->assignStr("address1", $address["address1"]);
			$this->db()->assignStr("address2", $address["address2"]);
			$this->db()->assignStr("city", $address["city"]);
			if(intval($address["country"]) > 2)
			{
				$this->db()->assign("state", "");	
				$this->db()->assignStr("province", $address["province"]);
			}
			else
			{
				$this->db()->assign("state", intval($address["state"]));	
				$this->db()->assignStr("province", "");
			}			
			$this->db()->assignStr("zip", $address["zip"]);
			$this->db()->assignStr("phone", $address["phone"]);
			$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."' AND usid='".$address_id."'");
			
			//update custom shipping data
			if ($customFieldsData)
			{
				$this->customFields()->storeCustomFields("shipping", $customFieldsData, $this->id, 0, $address_id);	
			}
			
			return true;
		}
		return false;
	}
		
	/**
	 * Remove shipping address from user's profile
	 * @param $id
	 * @return boolean
	 */
	public function deleteShippingAddress($address_id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
		if ($this->db()->moveNext())
		{
			$this->db()->query("DELETE FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
			$this->db()->query("DELETE FROM ".DB_PREFIX."custom_fields_values WHERE profile_id='".intval($this->id)."' AND address_id='".intval($address_id)."'");
			return true;
		}
		return false;
	}
		
	/**
	 * Get Shipping Addresses
	 * @return array
	 */
	public function getShippingAddresses()
	{
		$this->db()->query("
			SELECT 
				".DB_PREFIX."users_shipping.*,
				".DB_PREFIX."countries.name AS country_name,
				".DB_PREFIX."states.name AS state_name
			FROM ".DB_PREFIX."users_shipping 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users_shipping.country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users_shipping.state
			WHERE uid='".intval($this->id)."'
			ORDER BY is_primary
		");
		return $this->db()->numRows()>0?$this->db()->getRecords():false; 
	}
		
	/**
	 * Get Shipping Address by ID
	 * @param $address_id
	 * @return mixed
	 */
	public function getShippingAddressById($address_id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
		return $this->db()->moveNext() ? $this->db()->col : false;
	}
	
	/**
	 * 
	 * @param $data
	 * @return boolean
	 */
	public function validateAddress(&$form)
	{
		$errCount = count($this->errors);
		if (isset($form["name"]))
		{
			if (!isset($form["name"]) || !validate($form["name"], VALIDATE_NAME)) $this->setError("Enter valid name");
		}
		else
		{
			if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
		}
		
		if (!isset($form["address1"]) || !validate($form["address1"], VALIDATE_STRING)) $this->setError("Enter valid address line 1");
		if (!isset($form["address2"])) $form["address2"] = "";
		if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_STRING))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_STRING))) $this->setError("Enter a valid address line 2");
		if (!isset($form["city"]) || !validate($form["city"], VALIDATE_STRING)) $this->setError("Enter city name");
			
		//validate country, state and province & zip
		if (!isset($form["country"]) || !validate($form["country"], VALIDATE_DECIMAL))
		{
			$this->setError("Select country");
		}
		else
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".intval($form["country"])."'");
			if ($country = $this->db()->moveNext())
			{
				$this->db()->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."'");
				$c = $this->db()->moveNext();
				if ($c["c"] > 0)
				{
					if (!isset($form["state"]) || !validate($form["state"], VALIDATE_DECIMAL))
					{
						$this->setError("Select state/province");
					}
					else
					{
						$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."' AND stid='".intval($form["state"])."'");
						if ($c = $this->db()->moveNext()) $form["province"] = $c["name"]; else $this->setError("Select state/province");	
					}
				}
				else
				{
					if (!isset($form["province"]) || !validate($form["province"], VALIDATE_STRING)) $this->setError("Enter state/province name"); else $form["state"] = 0;
				}
				
				if (isset($form["zip"]))
				{
					if ($country["iso_a3"] == "USA" && !validate($form["zip"], VALIDATE_ZIP_USA)) $this->setError("Enter valid zip code");
					
					//////hide by dharmin
					//elseif ($country["iso_a3"] == "CAN" && !validate($form["zip"], VALIDATE_ZIP_CANADA)) $this->setError("Enter valid postal code");
					//elseif ($country["iso_a3"] == "GBP" && !validate($form["zip"], VALIDATE_ZIP_UK)) $this->setError("Enter valid postal code");
					//elseif (!validate($form["zip"], VALIDATE_ZIP)) $this->setError("Enter valid zip/postal code");					
					//////
				}
				else
				{
					$this->setError("Enter valid zip/postal code");	
				}
			}
			else
			{
				$this->setError("Select country");
			}
		}
		return $errCount == count($this->errors);
	}
	
	/**
	 * Sign-up procedure
	 * @param $form
	 * @param $costomeFieldsData
	 * @param $guestUserId - used when need to convert guest user to normal one
	 * @return boolean
	 */
	public function signUp(&$form, &$customFieldsData = false, $guestUserId = false)
	{

		$this->is_error = false;
		$this->errors = array();
		
		
		//echo $cat_val;
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			//if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			//if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			//validate custom fields for billing
			if ($customFieldsData)
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			// Standard settings in the admin area
			if ($this->_settings["FormsBillingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
			if ($this->_settings["FormsBillingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");
			if ($this->_settings["FormsBillingPhone"] == 'Required' && (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE))) $this->setError("Enter valid phone number");
			//echo $form["login"];
			//die();
			if (!isset($form["login"]) || !validate($form["login"], VALIDATE_USERNAME))
			{
				$this->setError("Please enter username");	
			}
			else
			{
				$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE login LIKE '".$this->db()->escape(trim($form["login"]))."'");
				if ($this->db()->moveNext()) $this->setError("Login already exists");
			}
			
			if (!isset($form["password"]) || !validate($form["password"], VALIDATE_PASSWORD))
			{
				$this->setError("Please check your password");	
			}
			else
			{
				if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
				{
					$this->setError("Please check password confirmation");	
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			/*if($form['user_type']==1)
			{
				$form["level"] = 100;
				$str=strtotime("+1 year");
				$nextyear = date('Y-m-d', $str);
				$block_date = $nextyear." 00:00:00";
				
			}
			else
				$form["level"] = 0;
			// validate custom fields for account and signup
			*/
			if ($customFieldsData)
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("account", $customFieldsData);
				$this->customFields()->validateCustomFields("signup", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			//if()
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html"))?$form["email_mode"]:"text") : "text";
			$form["email_newsletters"] = isset($form["email_newsletters"]) ? "Yes" : "No";
			$form["email_updates"] = isset($form["email_updates"]) ? "Yes" : "No";
			//code for adding category and subcategory to DB by Hari
			$cat_count = $_REQUEST['new_cat_hid'];
			$cat_val = '';
			for($i=1;$i<=$cat_count;$i++)
			{
				if(isset($_REQUEST['new_cat_'.$i]))
				{
					if($cat_val=='')
						$cat_val = $_REQUEST['new_cat_'.$i];
					else
						$cat_val .= ",".$_REQUEST['new_cat_'.$i];
				}
			}
			if(isset($_REQUEST['notifyemail']))
				$notifyemail = 'Yes';
			else
				$notifyemail = 'No';
			if(isset($_REQUEST['notifysms']))
				$notifysms = 'Yes';
			else
				$notifysms = 'No';
				
			//$notify_cron_week = $_REQUEST['notify_week'];
			//$notify_cron_mnth = $_REQUEST['notify_mnth'];
			
			if(isset($_REQUEST['notify_week']))
				$notify_cron_week = 'Yes';
			else
				$notify_cron_week = 'No';
			if(isset($_REQUEST['notify_mnth']))
				$notify_cron_mnth = 'Yes';
			else
				$notify_cron_mnth = 'No';

			

		/*//code ends
		//echo $sub_cat_val;*/
			$shipping_thesame = isset($form["thesame"]);
			$_SESSION["shipping_thesame"] = $shipping_thesame;
			
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//account settings
				$this->db()->assign("created_date", "NOW()");
				//$this->db()->assignStr("block_date", $block_date);
				$this->db()->assignStr("used_equip_id", $cat_val);
				$this->db()->assignStr("notify_email", $notifyemail);
				$this->db()->assignStr("notify_sms", $notifysms);
				$this->db()->assignStr("notify_cron", $notify_cron_week);
				$this->db()->assignStr("notify_cron_mnth", $notify_cron_mnth);
				//$this->db()->assignStr("ptic_sub_cat_id", $sub_cat_val);
				//$this->db()->assign("level", $form["level"]);
				$this->db()->assignStr("newsletters", $form["email_newsletters"]);
				$this->db()->assignStr("updates", $form["email_updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				$this->db()->assignStr("login", $form["login"]);
				$this->db()->assignStr("password", getPasswordHash($form["password"]));
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				$this->db()->assignStr("fax", $form["fax"]);
				
				//check is it really new user
				if (!$guestUserId)
				{
				//echo $form["block_date"]."fdgfd";
			//exit;
					//create account
					$this->id = $this->db()->insert(DB_PREFIX."users");
					//echo $this->id;
					
				}
				else
				{
					//conver existing user
					$this->id = intval($guestUserId);
					$this->db()->update(DB_PREFIX."users", "WHERE uid='".$this->id."'");
				}
				
				if ($customFieldsData)
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, $this->id, 0, 0);
					$this->customFields()->storeCustomFields("account", $customFieldsData, $this->id, 0, 0);
					$this->customFields()->storeCustomFields("signup", $customFieldsData, $this->id, 0, 0);
				}
				
				//store shipping address if checked "the same"
				if ($shipping_thesame)
				{
					$shippingAddress = $form;
					$shippingAddress["is_primary"] = "Yes";
					$shippingAddress["name"] = $form["fname"]." ".$form["lname"];
					$this->addShippingAddress($shippingAddress);
				}
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Update profile procedure
	 * @param $form
	 * @return boolean
	 */
	public function updateProfile(&$form, &$customFieldsData = false)
	{
		
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
		//trim form

			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			//if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			//if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			/*if ($this->_settings["FormsBillingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
			if ($this->_settings["FormsBillingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");
			
			$this->validateAddress($form);
			
			if ($this->_settings["FormsBillingPhone"] == 'Required' && (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE))) $this->setError("Enter valid phone number"); 
			
			//validate custom fields for billing
			if ($customFieldsData)
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}*/
			
			if (isset($form["password"]) && trim($form["password"]) != "")
			{
				if (!validate($form["password"], VALIDATE_PASSWORD))
				{
					$this->setError("Please check your password");	
				}
				else
				{
					if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
					{
						$this->setError("Please check password confirmation");	
					}
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
						AND uid <> '".intval($this->id)."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
		
			//validate custom fields for account
			if ($customFieldsData)
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("account", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html"))?$form["email_mode"]:"text") : "text";
			$form["newsletters"] = isset($form["newsletters"]) ? "Yes" : "No";
			$form["updates"] = isset($form["updates"]) ? "Yes" : "No";
			/*if($form['user_type']==1)
			{
				$form["level"] = 100;
				$str=strtotime("+1 year");
				$nextyear = date('Y-m-d', $str);
				$form["block_date"] = $nextyear;
				
				//body for the mail which need to send when user change his mode to sales//
				$date = date('m/d/Y');
				$subject = 'New sales person has been added on '.$date ;
				$matter = "Please check the following link <br /> <a href='".ADMIN_URL."?p=user&uid=".$this->id."'>Click Here</a>";
				$matter.= "<br /> Regards,<br /> Pavementgroup.com";
				$send = send_cron_mail($subject, $matter, "", "html");				
			}
			else
			{	$form["level"] = 0; }*/
			$cat_count = $_REQUEST['new_cat_hid'];
			$cat_val='';
			for($i=1;$i<=$cat_count;$i++)
			{
				if(isset($_REQUEST['new_cat_'.$i]))
				{
					if($cat_val=='')
						$cat_val = $_REQUEST['new_cat_'.$i];
					else
						$cat_val .= ",".$_REQUEST['new_cat_'.$i];
				}
			}
			if(isset($_REQUEST['notifyemail']))
				$notifyemail = 'Yes';
			else
				$notifyemail = 'No';
			if(isset($_REQUEST['notifysms']))
				$notifysms = 'Yes';
			else
				$notifysms = 'No';
				
			if(isset($_REQUEST['notify_week']))
				$notify_cron_week = 'Yes';
			else
				$notify_cron_week = 'No';
			if(isset($_REQUEST['notify_mnth']))
				$notify_cron_mnth = 'Yes';
			else
				$notify_cron_mnth = 'No';
				
			if (!$this->is_error)
			{
				$this->db()->reset();
				//account settings
				$this->db()->assignStr("newsletters", $form["newsletters"]);
				//$this->db()->assignStr("block_date", $form["block_date"]);
				$this->db()->assignStr("used_equip_id", $cat_val);
				$this->db()->assignStr("notify_email", $notifyemail);
				$this->db()->assignStr("notify_sms", $notifysms);
				$this->db()->assignStr("notify_cron", $notify_cron_week);
				$this->db()->assignStr("notify_cron_mnth", $notify_cron_mnth);
				//$this->db()->assignStr("ptic_sub_cat_id", $sub_cat_val);
				//$this->db()->assign("level", $form["level"]);
				$this->db()->assignStr("updates", $form["updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				if (isset($form["password"]) && $form["password"] != "")
				{
					$this->db()->assignStr("password", getPasswordHash($form["password"]));
				}
				//billing info
				/*$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);*/
				
				$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
				
				if ($customFieldsData)
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, intval($this->id), 0, 0);
					$this->customFields()->storeCustomFields("account", $customFieldsData, intval($this->id), 0, 0);
				}
			}
		}
		else
		{
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Update billing info
	 * @param $form
	 * @return boolean
	 */
	public function updateBillingInfo(&$form, &$customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);		
			if (!$this->express)
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
						AND uid <> '".intval($this->id)."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			if ($customFieldsData)
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			$this->is_error = count($this->errors) > 0;
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				if(intval($form["country"]) > 2)
				{
					$this->db()->assignStr("state", "");	
					$this->db()->assignStr("province", $form["province"]);
				}
				else
				{
					$this->db()->assignStr("state", intval($form["state"]));	
					$this->db()->assignStr("province", "");
				}
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//email
				$this->db()->assignStr("email", $form["email"]);
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
				if ($customFieldsData)
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, $this->id, 0, 0);
				}
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Login procedure
	 * @param $login
	 * @param $password
	 * @param $remember_me
	 * @return boolean
	 */
	public function login($login, $password, $remember_me = false, $frompage, $section = '')
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE LCASE(login) = '".$this->db()->escape(strtolower($login))."' AND (password = '".getPasswordHash($password)."' OR password = MD5('".$this->db()->escape($password)."') OR password = PASSWORD('".$this->db()->escape($password)."') OR password = '".$password."')");
		if ($this->db()->moveNext())
		{
			$this->id = $this->db()->col["uid"];
			//die($this->_settings["SecurityAccountBlocking"]);
			if ($this->_settings["SecurityAccountBlocking"] == "YES")
			{
				// check is account blocked
				$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".($this->id)."' AND block_date > NOW()");
				if ($this->db()->moveNext())
				{
					$this->blocked = true;
					$this->auth_ok = false;
					return false;
				}
				else
				{
					//reselect user data
					$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".($this->id)."'");
				}
			}
			$this->auth_ok = true;
			$this->express = false;
			$this->express_dmp = false;
			$this->getUserData();
			$this->session_id = uniqid(md5(rand(1, 1000000)));
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->session_id."', session_date=NOW() WHERE uid='".intval($this->id)."'");
			
			$cookies = initCookies();
			
			if ($remember_me)
			{
				$cookies->set($this->cookie."LoginUserName", $login);
			}
			else
			{
				$cookies->remove($this->cookie."LoginUserName");
			}
			if($frompage!='')
				$_SESSION["frompage"] = $frompage;
			if($section!='')
				$_SESSION["section"] = $section;
				
			$_SESSION["SessionID"] = $this->session_id;
			$_SESSION["UserID"] = $this->id;
			
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Express Checkout Login procedure
	 * @return boolean
	 */
	public function expressLogin()
	{
		$this->db()->query("INSERT INTO ".DB_PREFIX."users (login, password, country, state) VALUES ('".EXPRESS_CHECKOUT_USER."', '".(md5(uniqid(rand())))."', '0', '0')");
		$this->id = $this->db()->insertId();
		$this->auth_ok = true;
		$this->express_dmp = false;
		$this->session_id = uniqid(md5(rand(1, 1000000)));
		$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->session_id."', session_date=NOW() WHERE uid='".intval($this->id)."'");
		$_SESSION["SessionID"] = $this->session_id;
		$_SESSION["UserID"] = $this->id;
		return true;
	}
	
	/**
	 * Password Reset
	 * @param $login
	 * @param $email
	 * @return boolean
	 */
	public function passwordReset($email)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE LOWER(email) = '".strtolower($this->db()->escape($email))."'");
		if ($user_data = $this->db()->moveNext())
		{
			$new_password = uniqid();
			$token = uniqid();
			$this->db()->query("UPDATE ".DB_PREFIX."users SET token = '".$token."', password='".$this->db()->escape(getPasswordHash($new_password))."' WHERE uid='".intval($user_data["uid"])."'");
			emailUserResetPassword($user_data, $this->db()->col["uid"], $new_password, $token);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function Temppassword($email,$password)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE LOWER(email) = '".strtolower($this->db()->escape($email))."' AND (password = '".getPasswordHash($password)."' OR password = MD5('".$this->db()->escape($password)."') OR password = PASSWORD('".$this->db()->escape($password)."'))");
		if ($this->db()->moveNext())
		{	
			$_SESSION['login'] = $this->db()->col["login"];
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public function ResetUser($token,$login)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE token = '".$token."' AND login = '".$login."'");
		if ($this->db()->moveNext())
		{	
			$_SESSION['login'] = $this->db()->col["login"];
			$_SESSION['pwd'] = $this->db()->col["password"];
			$this->auth_ok = true;
			return true;
		}
		else
		{
			return false;
		}
	}	
			
	public function NewpasswordReset($email,$token,$tmppass,$password2)
	{	
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE token = '".$token."' AND LOWER(email) = '".strtolower($this->db()->escape($email))."' AND (password = '".getPasswordHash($tmppass)."' OR password = MD5('".$this->db()->escape($tmppass)."') OR password = PASSWORD('".$this->db()->escape($tmppass)."')) ");
		//echo "SELECT * FROM ".DB_PREFIX."users WHERE token = '".$token."' AND LOWER(email) = '".strtolower($this->db()->escape($email))."' AND (password = '".getPasswordHash($tmppass)."' OR password = MD5('".$this->db()->escape($tmppass)."') OR password = PASSWORD('".$this->db()->escape($tmppass)."')) ";die;
	
		if ($user_data = $this->db()->moveNext())
		{
			//echo "UPDATE ".DB_PREFIX."users SET password='".getPasswordHash($password2)."' WHERE uid='".intval($user_data["uid"])."'";die;
			$token = uniqid();
			$this->db()->query("UPDATE ".DB_PREFIX."users SET token = '".$token."', password='".getPasswordHash($password2)."' WHERE uid='".intval($user_data["uid"])."'");
			return true;
		}
		else
		{
			return false;
		}
	}	
	public function NewUsernameReset($email,$token,$login,$password2)
	{	
		unset($_SESSION['exist_username']);
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE login = '".$password2."' ");
		if ($user_data = $this->db()->moveNext())
		{
			$_SESSION['exist_username'] = true;
			return false;	
		}
		else
		{	
			$this->db()->reset();
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE token = '".$token."' AND LOWER(email) = '".strtolower($this->db()->escape($email))."' AND login = '".$login."' ");
			//echo "SELECT * FROM ".DB_PREFIX."users WHERE token = '".$token."' AND LOWER(email) = '".strtolower($this->db()->escape($email))."' AND login = '".$login."'  ";
		
			if ($user_data = $this->db()->moveNext())
			{
				//echo $user_data["uid"]."UPDATE ".DB_PREFIX."users SET login='".$password2."' WHERE uid='".intval($user_data["uid"])."'";die;
				if($password2)
				{
				$token = uniqid();
				$this->db()->query("UPDATE ".DB_PREFIX."users SET login='".$password2."', token = '".$token."' WHERE uid='".intval($user_data["uid"])."'");
				return true;
				}
				else
				{
				return false;
				}
			}
			else
			{
				return false;
			}
		}
	}	
	
	
	
	
	
	/**
	 * Get Orders History List
	 * @return mixed
	 */
	public function getOrders()
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT *, DATE_FORMAT(status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date FROM ".DB_PREFIX."orders WHERE uid='".intval($this->id)."' AND status <> 'Abandon' AND status <> 'New' ORDER BY placed_date DESC");
			if ($this->db()->numRows() > 0)
			{
				return $this->db()->getRecords();
			}
		}
		return false;
	}
	
	/**
	 * Get Order Data
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderData($order_id)
	{
		if ($this->auth_ok)
		{
			$this->db()->query("
				SELECT ".DB_PREFIX."orders.*,
					DATE_FORMAT(".DB_PREFIX."orders.status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date,
					s_countries.name AS shipping_country_name,
					s_states.name AS shipping_state_name
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries AS s_countries ON s_countries.coid = ".DB_PREFIX."orders.shipping_country
				LEFT JOIN ".DB_PREFIX."states AS s_states ON s_states.stid = ".DB_PREFIX."orders.shipping_state
				WHERE oid='".intval($order_id)."' 
					AND uid='".intval($this->id)."' 
					AND status<>'Abandon' 
					AND status<>'New'
			");
			return $this->db()->moveNext() ? $this->db()->col : false;
		}
		return false;
	}

	/**
	 * Get Order Items
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderItems($order_id)
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".intval($order_id)."' ORDER BY product_id");
			if ($this->db()->numRows()>0)
			{
				$products = $this->db()->getRecords();
				for ($i=0; $i<count($products); $i++)
				{
					if ($this->_settings["DisplayPricesWithTax"] == "YES" && $products[$i]["is_taxable"] == "Yes")
					{
						$_price = $products[$i]["price"];
						$_tax_rate = -1;
						if ($products[$i]["tax_rate"] >= 0)
						{
							$_price = $_price + ($_price / 100 * $products[$i]["tax_rate"]);
						}
						$products[$i]["price"] = $_price;
					}
				}
			}
			return $products;
		}
		return false;
	}
	
	/**
	 * Logout procedure
	 */
	public function logOut()
	{
		if ($this->auth_ok)
		{
			// generate protect-on-logout session id
			$s = uniqid(md5(rand(1, 100000)));
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$s."' WHERE uid='".intval($this->id)."'");
		}
		
		unset($_SESSION["SessionID"]);
		unset($_SESSION["UserID"]);
		unset($_SESSION["login"]);
		$this->id = 0;
		$this->level = 0;
		$this->auth_ok = false;
		$this->session_id = "";
	}
	
	/**
	 * Return user data
	 * @return mixed
	 */
	public function getUserData()
	{
		if ($this->id)
		{
			$this->db()->query("
				SELECT 
					".DB_PREFIX."users.*,
					CONCAT(".DB_PREFIX."users.fname, ' ', ".DB_PREFIX."users.lname) AS name,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number,
					".DB_PREFIX."states.name AS state_name,
					".DB_PREFIX."states.short_name AS state_abbr
				FROM ".DB_PREFIX."users 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users.country
				LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users.state
				WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'
			");
			if ($this->db()->moveNext())
			{
				$this->data = $this->db()->col;
				$this->level = $this->db()->col["level"];
				$this->data["usid"] = "billing";
				return $this->data;
			}
		}
		return false;
	}
	
	/**
	 * Returns current user billing address, phone and email
	 */
	public function getBillingInfo()
	{
		if ($this->id)
		{
			$this->db()->query("
				SELECT 
					".DB_PREFIX."users.fname,
					".DB_PREFIX."users.lname,
					CONCAT(".DB_PREFIX."users.fname, ' ', ".DB_PREFIX."users.lname) AS name,
					".DB_PREFIX."users.company,
					".DB_PREFIX."users.address1,
					".DB_PREFIX."users.address2,
					".DB_PREFIX."users.city,
					".DB_PREFIX."users.state,
					".DB_PREFIX."users.province,
					".DB_PREFIX."users.country,
					".DB_PREFIX."users.zip,
					".DB_PREFIX."users.phone,
					".DB_PREFIX."users.email,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number,
					".DB_PREFIX."states.name AS state_name,
					".DB_PREFIX."states.short_name AS state_abbr
				FROM ".DB_PREFIX."users 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users.country
				LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users.state
				WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'
			");
			if ($this->db()->moveNext(false, MYSQL_ASSOC))
			{
				return $this->db()->col;
			}
		}
		return false;
	}
}

