<?php 
/**
 * Users actions
 * @var unknown_type
 */
define("USER_SIGNUP", "user_signup");
define("USER_LOGIN", "user_login");
define("USER_LOGOUT", "user_logout");
define("USER_PASSWORD_RESET", "password_reset");
define("USER_UPDATE_PROFILE", "user_updateprofile");
define("USER_EXPRESS_CHECKOUT", "user_expresscheckout");
define("USER_START_CHECKOUT", "user_start_checkout");
define("USER_ADD_ADDRESS", "AddShippingAddress");
define("USER_DELETE_ADDRESS", "DeleteShippingAddress");
define("USER_UPDATE_ADDRESS", "UpdateShippingAddress");
define("USER_ADD_TO_WISHLIST", "UserAddToWishlist");
define("USER_FORM_MAIL", "FormMail");

define("EXPRESS_CHECKOUT_USER", "ExpressCheckoutUser");

/**
 * 
 * @author Victor
 */
class USER
{
	public $id = 0;
	public $session_id = "";
	public $level = 0;
	public $fname = "";
	public $lname = "";
	public $login = "";
	public $auth_ok = false;
	public $data = array();
	public $messages = array();
	public $is_error = false;
	public $errors = array();
	public $blocked = false;
	public $cookie = "CartUserCookie";
	public $back_user = false;
	public $back_user_login = "";
	public $express = false;
	private $_db;
	private $_settings;

	/**
	 * Class constructor
	 * @param $db
	 * @param $settings
	 * @return USER
	 */
	public function __construct(&$db, &$settings)
	{
		$this->_db = $db;
		$this->_settings = $settings;
			
		//set cookie prefix
		$this->cookie = $this->_settings["SecurityCookiesPrefix"];
			
		// init user, load cookies & session
		global $_COOKIE;
			
		// check returning customers
		$this->back_user = in_array($this->cookie."LoginUserName", array_keys($_COOKIE));
		$this->back_user_login = isset($_COOKIE[$this->cookie."LoginUserName"])?$_COOKIE[$this->cookie."LoginUserName"]:"";
			
		// check user session
		$this->id = isset($_SESSION["UserID"]) ? intval($_SESSION["UserID"]) : 0;
		$this->session_id = isset($_SESSION["SessionID"]) ? $_SESSION["SessionID"] : "";
			
		// check user registered (by user id)
		if ($this->id)
		{
			$this->db()->query("
				SELECT uid FROM ".DB_PREFIX."users 
				WHERE 
					uid='".intval($this->id)."' AND
					DATE_ADD(session_date, INTERVAL ".intval($this->_settings["SecurityUserTimeout"])." SECOND)>NOW() AND
					".($this->_settings["SecurityAccountBlocking"] == "YES" ? "block_date < NOW() AND" : "")."
					session_id='".$this->db()->escape($this->session_id)."'
			");
			if ($this->db()->moveNext())
			{
				// user is registered - user_id, session_id correct and not expired
				$this->auth_ok = true;
				$this->getUserData();
				
				// update session_timeout in db
				$this->db()->query("UPDATE ".DB_PREFIX."users SET session_date=NOW() WHERE uid='".intval($this->id)."'");
				
				// check express checkout users
				$this->express = $this->data["login"] == EXPRESS_CHECKOUT_USER;
					
				// store user data to session
				$_SESSION["SessionID"] = $this->session_id;
				$_SESSION["UserID"] = $this->id;
			}
			else
			{
				// user is not registered - can't use existing user session
				$this->id = 0;
				$this->session_id = uniqid(md5(rand(1, 1000000)));
				$_SESSION["SessionID"] = $this->session_id;
				$_SESSION["UserID"] = $this->id;
			}
		}
		return $this;
	}
	
	/**
	 * Sets error
	 * @param $message
	 * @return unknown_type
	 */
	public function setError($message)
	{
		$this->errors[count($this->errors)] = $message;
		$this->is_error = true;
	}
	
	/**
	 * Return database instance
	 * @return DB
	 */
	public function db()
	{
		return $this->_db;
	}
		
	/**
	 * Update / edit billing address on express checkout
	 * @param $address
	 */
	public function payPalExpressUserInfo($address)
	{
		$this->db()->reset();
		$this->db()->assignStr("fname", 
			trim(
				$address["PayerInfo"][0]["PayerName"][0]["FirstName"][0]." ".
				$address["PayerInfo"][0]["PayerName"][0]["MiddleName"][0]
			)
		);
		$this->db()->assignStr("lname", $address["PayerInfo"][0]["PayerName"][0]["LastName"][0]);
		$this->db()->assignStr("email", $address["PayerInfo"][0]["Payer"][0]);
		$this->db()->update(DB_PREFIX."users", "WHERE uid='".intval($this->id)."'");
	}
		
	/**
	 * Add address from PayPal to User's Address Book
	 * @param $paypal_address
	 * @return boolean
	 */
	public function payPal2Shipping($paypal_address)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".intval($this->id)."'");
		if ($this->db()->moveNext())
		{
			$address = $this->db()->col;
			//check primary
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND is_primary='Yes'");
			$is_primary = $this->db()->moveNext()?"No":"Yes";
			//get country id
			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2 = '".$this->db()->escape($paypal_address["PayerInfo"][0]["Address"][0]["Country"][0])."'");
			if ($this->db()->moveNext())
			{
				$address_country = $this->db()->col["coid"];
				$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($address_country)."' AND short_name='".$this->db()->escape($paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0])."'");
				if ($this->db()->moveNext())
				{
					$address_state = $this->db()->col["stid"];
					$address_province = "";
				}
				else
				{
					$address_state = 0;
					$address_province = $paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0];
				}
			}
			else
			{
				$address_country = 1;
				$address_state = 0;
				$address_province = $paypal_address["PayerInfo"][0]["Address"][0]["StateOrProvince"][0];
			}
			
			$this->db()->reset();
			$this->db()->assign("uid", $this->id);
			$this->db()->assign("country", intval($address_country));
			$this->db()->assign("state", intval($address_state));
			$this->db()->assignStr("is_primary", $is_primary);
			$this->db()->assignStr("name", $paypal_address["PayerInfo"][0]["Address"][0]["Name"][0]);
			$this->db()->assignStr("company", "");
			$this->db()->assignStr("address1", $paypal_address["PayerInfo"][0]["Address"][0]["Street1"][0]);
			$this->db()->assignStr("address2", $paypal_address["PayerInfo"][0]["Address"][0]["Street2"][0]);
			$this->db()->assignStr("city", $paypal_address["PayerInfo"][0]["Address"][0]["CityName"][0]);
			$this->db()->assignStr("province", $address_province);
			$this->db()->assignStr("zip", $paypal_address["PayerInfo"][0]["Address"][0]["PostalCode"][0]);
			$address_id = $this->db()->insert(DB_PREFIX."users_shipping");
			return $address_id;
		}
		return false;
	}

	/**
	 * Add shipping address to users account
	 * @param $address
	 * @return mixed
	 */
	public function addShippingAddress($address)
	{
		$this->is_error = false;
		$this->errors = array();
		
		$this->validateAddress($address);
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if( !$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["is_primary"]) && ($address["is_primary"] == "Yes"))
				{
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["is_primary"] = "Yes";
				}
				else
				{
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["is_primary"] = "No";
				}
			}
			//insert address values
			$this->db()->reset();
			$this->db()->assign("uid", intval($this->id));
			$this->db()->assign("country", intval($address["country"]));
			$this->db()->assign("state", intval($address["state"]));
			$this->db()->assignStr("is_primary", $address["is_primary"]);
			$this->db()->assignStr("name", $address["name"]);
			$this->db()->assignStr("company", $address["company"]);
			$this->db()->assignStr("address1", $address["address1"]);
			$this->db()->assignStr("address2", $address["address2"]);
			$this->db()->assignStr("city", $address["city"]);
			$this->db()->assignStr("province", $address["province"]);
			$this->db()->assignStr("zip", $address["zip"]);
			return $this->db()->insert(DB_PREFIX."users_shipping");
		}
		return false;
	}
	
	/**
	 * Update shipping address
	 * @param $address_id
	 * @param $address
	 * @return boolean
	 */
	public function updateShippingAddress($address_id, $address)
	{
		$this->is_error = false;
		$this->errors = array();
		
		$this->validateAddress($address);
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if (!$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["set_is_primary"]) && ($address["set_is_primary"] == "Yes"))
				{
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["is_primary"] = "Yes";
				}
				else
				{
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["is_primary"] = "No";
				}
			}
			
			//insert address values
			$this->db()->reset();
			$this->db()->assign("uid", intval($this->id));
			$this->db()->assign("country", intval($address["country"]));
			$this->db()->assign("state", intval($address["state"]));
			$this->db()->assignStr("is_primary", $address["is_primary"]);
			$this->db()->assignStr("name", $address["name"]);
			$this->db()->assignStr("company", $address["company"]);
			$this->db()->assignStr("address1", $address["address1"]);
			$this->db()->assignStr("address2", $address["address2"]);
			$this->db()->assignStr("city", $address["city"]);
			$this->db()->assignStr("province", $address["province"]);
			$this->db()->assignStr("zip", $address["zip"]);
			return $this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."' AND usid='".$address_id."'");
		}
		return false;
	}
		
	/**
	 * Remove shipping address from user's profile
	 * @param $id
	 * @return boolean
	 */
	public function deleteShippingAddress($id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($id)."'");
		if ($this->db()->moveNext())
		{
			$this->db()->query("DELETE FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($id)."'");
			return true;
		}
		return false;
	}
		
	/**
	 * Get Shipping Addresses
	 * @return array
	 */
	public function getShippingAddresses()
	{
		$this->db()->query("
			SELECT 
				".DB_PREFIX."users_shipping.*,
				".DB_PREFIX."countries.name AS country_name
			FROM ".DB_PREFIX."users_shipping 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users_shipping.country 
			WHERE uid='".intval($this->id)."'
			ORDER BY is_primary
		");
		return $this->db()->numRows()>0?$this->db()->getRecords():false; 
	}
		
	/**
	 * Get Shipping Address by ID
	 * @param $address_id
	 * @return mixed
	 */
	public function getShippingAddressById($address_id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
		return $this->db()->moveNext() ? $this->db()->col : false;
	}
	
	/**
	 * 
	 * @param $data
	 * @return boolean
	 */
	public function validateAddress(&$form)
	{
		$errCount = count($this->errors);
		if (!isset($form["address1"]) || !validate($form["address1"], VALIDATE_STRING)) $this->setError("Enter valid address line 1");
		if (!isset($form["address2"])) $form["address2"] = "";
		if (!isset($form["city"]) || !validate($form["city"], VALIDATE_STRING)) $this->setError("Enter city name");
			
		//validate country, state and province & zip
		if (!isset($form["country"]) || !validate($form["country"], VALIDATE_DECIMAL))
		{
			$this->setError("Select country");
		}
		else
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".intval($form["country"])."'");
			if ($country = $this->db()->moveNext())
			{
				$this->db()->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."'");
				$c = $this->db()->moveNext();
				if ($c["c"] > 0)
				{
					if (!isset($form["state"]) || !validate($form["state"], VALIDATE_DECIMAL))
					{
						$this->setError("Select state/province");
					}
					else
					{
						$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."' AND stid='".intval($form["state"])."'");
						if ($c = $this->db()->moveNext()) $form["province"] = $c["name"]; else $this->setError("Select state/province");	
					}
				}
				else
				{
					if (!isset($form["province"]) || !validate($form["province"], VALIDATE_STRING)) $this->setError("Enter state/province name"); else $form["state"] = 0;
				}
				
				if (isset($form["zip"]))
				{
					if ($country["iso_a3"] == "USA" && !validate($form["zip"], VALIDATE_ZIP_USA)) $this->setError("Enter valid zip code");
					elseif ($country["iso_a3"] == "CAN" && !validate($form["zip"], VALIDATE_ZIP_CANADA)) $this->setError("Enter valid postal code");
					elseif ($country["iso_a3"] == "GBP" && !validate($form["zip"], VALIDATE_ZIP_UK)) $this->setError("Enter valid postal code");
					elseif (!validate($form["zip"], VALIDATE_ZIP)) $this->setError("Enter valid zip/postal code");					
				}
				else
				{
					$this->setError("Enter valid zip/postal code");	
				}
			}
			else
			{
				$this->setError("Select country");
			}
		}
		return $errCount == count($this->errors);
	}
	
	/**
	 * Sign-up procedure
	 * @param $form
	 * @return boolean
	 */
	public function signUp(&$form)
	{
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			if (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE)) $this->setError("Enter valid phone number");
			if (!isset($form["login"]) || !validate($form["login"], VALIDATE_USERNAME))
			{
				$this->setError("Please enter username");	
			}
			else
			{
				$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE login LIKE '".$this->db()->escape(trim($form["login"]))."'");
				if ($this->db()->moveNext()) $this->setError("Login already exists");
			}
			
			if (!isset($form["password"]) || !validate($form["password"], VALIDATE_PASSWORD))
			{
				$this->setError("Please check your password");	
			}
			else
			{
				if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
				{
					$this->setError("Please check password confirmation");	
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html"))?$form["email_mode"]:"text") : "text";
			$form["email_newsletters"] = isset($form["email_newsletters"]) ? "Yes" : "No";
			$form["email_updates"] = isset($form["email_updates"]) ? "Yes" : "No";
			
			$shipping_thesame = isset($form["thesame"]);
			$_SESSION["shipping_thesame"] = $shipping_thesame;
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//account settings
				$this->db()->assignStr("newsletters", $form["email_newsletters"]);
				$this->db()->assignStr("updates", $form["email_updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				$this->db()->assignStr("login", $form["login"]);
				$this->db()->assignStr("password", getPasswordHash($form["password"]));
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				
				$this->id = $this->db()->insert(DB_PREFIX."users");
				
				//store shipping address if checked "the same"
				if ($shipping_thesame)
				{
					$shippingAddress = $form;
					$shippingAddress["is_primary"] = "Yes";
					$shippingAddress["name"] = $form["fname"]." ".$form["lname"];
					$this->addShippingAddress($shippingAddress);
				}
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Update profile procedure
	 * @param $form
	 * @return boolean
	 */
	public function updateProfile(&$form)
	{
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			if (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE)) $this->setError("Enter valid phone number");
			
			if (isset($form["password"]) && trim($form["password"]) != "")
			{
				if (!validate($form["password"], VALIDATE_PASSWORD))
				{
					$this->setError("Please check your password");	
				}
				else
				{
					if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
					{
						$this->setError("Please check password confirmation");	
					}
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
						AND uid <> '".intval($this->id)."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html"))?$form["email_mode"]:"text") : "text";
			$form["newsletters"] = isset($form["newsletters"]) ? "Yes" : "No";
			$form["updates"] = isset($form["updates"]) ? "Yes" : "No";
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//account settings
				$this->db()->assignStr("newsletters", $form["newsletters"]);
				$this->db()->assignStr("updates", $form["updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				if (isset($form["password"]) && $form["password"] != "")
				{
					$this->db()->assignStr("password", getPasswordHash($form["password"]));
				}
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				
				$this->id = $this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
			}
		}
		else
		{
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Update billing info
	 * @param $form
	 * @return boolean
	 */
	public function updateBillingInfo(&$form)
	{
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			if (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE)) $this->setError("Enter valid phone number");
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
						AND uid <> '".intval($this->id)."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			$this->is_error = count($this->errors) > 0;
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//profile settings
				$this->db()->assignStr("email", $form["email"]);
				if (isset($form["password"]) && $form["password"] != "")
				{
					$this->db()->assignStr("password", getPasswordHash($form["password"]));
				}
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Login procedure
	 * @param $login
	 * @param $password
	 * @param $remember_me
	 * @return boolean
	 */
	public function login($login, $password, $remember_me = false)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE LCASE(login) = '".$this->db()->escape(strtolower($login))."' AND (password = '".getPasswordHash($password)."' OR password = MD5('".$this->db()->escape($password)."') OR password = PASSWORD('".$this->db()->escape($password)."'))");
		if ($this->db()->moveNext())
		{
			$this->id = $this->db()->col["uid"];
			if ($this->_settings["SecurityAccountBlocking"] == "YES")
			{
				// check is account blocked
				$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".($this->id)."' AND block_date > NOW()");
				if ($this->db()->moveNext())
				{
					$this->blocked = true;
					$this->auth_ok = false;
					return false;
				}
				else
				{
					//reselect user data
					$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".($this->id)."'");
				}
			}
			$this->auth_ok = true;
			$this->getUserData();
			$this->session_id = uniqid(md5(rand(1, 1000000)));
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->session_id."', session_date=NOW() WHERE uid='".intval($this->id)."'");
			
			$cookies = initCookies();
			
			if ($remember_me)
			{
				$cookies->set($this->cookie."LoginUserName", $login);
			}
			else
			{
				$cookies->remove($this->cookie."LoginUserName");
			}
			$_SESSION["SessionID"] = $this->session_id;
			$_SESSION["UserID"] = $this->id;
			
			session_regenerate_id(true);
			
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Express Checkout Login procedure
	 * @return boolean
	 */
	public function expressLogin()
	{
		$this->db()->query("INSERT INTO ".DB_PREFIX."users (login, password, country, state) VALUES ('".EXPRESS_CHECKOUT_USER."', '".(md5(uniqid(rand())))."', '0', '0')");
		$this->id = $this->db()->insertId();
		$this->auth_ok = true;
		$this->session_id = uniqid(md5(rand(1, 1000000)));
		$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->session_id."', session_date=NOW() WHERE uid='".intval($this->id)."'");
		$_SESSION["SessionID"] = $this->session_id;
		$_SESSION["UserID"] = $this->id;
		return true;
	}
	
	/**
	 * Password Reset
	 * @param $login
	 * @param $email
	 * @return boolean
	 */
	public function passwordReset($login, $email)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE login = '".$this->db()->escape($login)."' AND LOWER(email) = '".strtolower($this->db()->escape($email))."'");
		if ($user_data = $this->db()->moveNext())
		{
			$new_password = generatePassword();
			$this->db()->query("UPDATE ".DB_PREFIX."users SET password='".$this->db()->escape(getPasswordHash($new_password))."' WHERE uid='".intval($user_data["uid"])."'");
			emailUserResetPassword($user_data, $this->db()->col["uid"], $new_password);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Account Blocking on payment failures
	 * @param $hours
	 */
	public function block($hours)
	{
		$this->db()->query("UPDATE ".DB_PREFIX."users SET block_date=DATE_ADD(NOW(), INTERVAL ".intval($hours)." HOUR) WHERE uid='".intval($this->id)."'");
	}
	
	/**
	 * Get Orders History List
	 * @return mixed
	 */
	public function getOrders()
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT *, DATE_FORMAT(status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date FROM ".DB_PREFIX."orders WHERE uid='".intval($this->id)."' AND status <> 'Abandon' AND status <> 'New' ORDER BY create_date DESC");
			if ($this->db()->numRows() > 0)
			{
				return $this->db()->getRecords();
			}
		}
		return false;
	}
	
	/**
	 * Get Order Data
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderData($order_id)
	{
		if ($this->auth_ok)
		{
			$this->db()->query("
				SELECT ".DB_PREFIX."orders.*,
					DATE_FORMAT(".DB_PREFIX."orders.status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date,
					s_countries.name AS shipping_country_name,
					s_states.name AS shipping_state_name
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries AS s_countries ON s_countries.coid = ".DB_PREFIX."orders.shipping_country
				LEFT JOIN ".DB_PREFIX."states AS s_states ON s_states.stid = ".DB_PREFIX."orders.shipping_state
				WHERE oid='".intval($order_id)."' 
					AND uid='".intval($this->id)."' 
					AND status<>'Abandon' 
					AND status<>'New'
			");
			return $this->db()->moveNext() ? $this->db()->col : false;
		}
		return false;
	}

	/**
	 * Get Order Items
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderItems($order_id)
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".intval($order_id)."' ORDER BY product_id");
			if ($this->db()->numRows()>0)
			{
				$products = $this->db()->getRecords();
				for ($i=0; $i<count($products); $i++)
				{
					if ($this->_settings["DisplayPricesWithTax"] == "YES" && $products[$i]["is_taxable"] == "Yes")
					{
						$_price = $products[$i]["price"];
						$_tax_rate = -1;
						if ($products[$i]["tax_rate"] >= 0)
						{
							$_price = $_price + ($_price / 100 * $products[$i]["tax_rate"]);
						}
						$products[$i]["price"] = $_price;
					}
				}
			}
			return $products;
		}
		return false;
	}
	
	/**
	 * Logout procedure
	 */
	public function logOut()
	{
		if($this->auth_ok){
			// generate protect-on-logout session id
			$s = uniqid(md5(rand(1, 100000)));
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$s."' WHERE uid='".intval($this->id)."'");
		}
		
		session_unregister("SessionID");
		session_unregister("UserID");
		$this->id = 0;
		$this->level = 0;
		$this->auth_ok = false;
		$this->session_id = "";
	}
	
	/**
	 * Return user data
	 * @return mixed
	 */
	public function getUserData()
	{
		if ($this->id)
		{
			$this->db()->query("
				SELECT 
					".DB_PREFIX."users.*,
					CONCAT(".DB_PREFIX."users.fname, ' ', ".DB_PREFIX."users.lname) AS name,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number,
					".DB_PREFIX."states.name AS state_name,
					".DB_PREFIX."states.short_name AS state_abbr
				FROM ".DB_PREFIX."users 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users.country
				LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users.state
				WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'
			");
			if ($this->db()->moveNext())
			{
				$this->data = $this->db()->col;
				$this->level = $this->db()->col["level"];
				$this->data["usid"] = "billing";
				return $this->data;
			}
		}
		return false;
	}
}

