<?php
	/*
	 * File: content/engine/payment/payment_amazon_sp.php
	 * Description: The payment method implementation for amazon simple pay.
	 * Created: 1506 20/10/09
	 * Owner: Ricky Taylor
	 */
	
	// Details for the payment method loader.
	$payment_processor_id = "amazon_sp";
	$payment_processor_name = "Amazon Simple Pay";
	$payment_processor_version = "1.0a";
	$payment_processor_class = "PAYMENT_AMAZON_SP";
	$payment_processor_type = "ipn";
		
	// Implementation Class
	class PAYMENT_AMAZON_SP extends PAYMENT_PROCESSOR
	{
		/*
		 * Variables
		 */
		
		/*
		 * CTor/DTor
		 */
		
		// Constructor
		function PAYMENT_AMAZON_SP()
		{
			// Initialise base class
			parent::PAYMENT_PROCESSOR();
				
			// Set up integration details
			$this->id = "amazon_sp";
			$this->name = "Amazon Simple Pay";
			$this->version = "1.0a";
			$this->class_name = "PAYMENT_AMAZON_SP";
			$this->type = "ipn";
			$this->description = "";
			$this->steps = 2;
			$this->testMode = false;
		}
		
		/*
		 *  Methods
		 */
		
		// Turn an error code into a string
		function getErrorString($code)
		{
			$lookup = array(
				"PS"=>"Payment succeeded",
				"PF"=>"Payment failed",
				"RF"=>"Refund failed",
				"RS"=>"Refund succeeded",
				"ME"=>"Server error",
				"A"=>"Payment aborted",
				"SE"=>"Service error",
			);
			if(array_key_exists($code, $lookup))
				return $lookup[$code];
			
			return "Unknown error";
		}
		
		// Get the members of the payment form.
		function getPaymentForm($db)
		{
			global $settings;
			
			// Generate the default payment form. -- Ricky26
			$_pf = parent::getPaymentForm($db);
				
			// Work out URLs here, keeps code clean. -- Ricky26
			$mainUrl = ( $this->protocol=="https" ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"] ) . "/";
			$backUrl = $mainUrl.$settings["INDEX"]."?pcsid=".session_id()."&p=cart"; 
			$nextUrl = $mainUrl.$settings["INDEX"]."?pcsid=".session_id()."&p=completed";
			$notifyUrl = $mainUrl.$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT; 
			
			// Generate fields for amazon.
			$payFields = array();
			$payFields["abandonUrl"] = $backUrl;
			$payFields["accessKey"] = $this->settings_vars[$this->id."_access_key"]["value"];
			$payFields["amount"] = "USD " . number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "");
			$payFields["description"] = $settings["GlobalSiteName"]." Order ".$this->common_vars["order_id"]["value"];
			$payFields["immediateReturn"] = "1";
			$payFields["referenceId"] = $this->common_vars["order_id"]["value"];
			$payFields["ipnUrl"] = $notifyUrl;
			$payFields["returnUrl"] = $notifyUrl;
			$payFields["processImmediate"] = "1";
			
			// Sort the field names -- Ricky26
			$fieldNames = array_keys($payFields);
			natcasesort($fieldNames);
			
			// Generate a signature for the payment -- Ricky26
			$sigText = "";
			foreach($fieldNames as $k)
				$sigText = $sigText . $k . $payFields[$k];			
			
			$sig = $this->hashHMAC($sigText, $this->settings_vars[$this->id."_secret_key"]["value"]);
			
			// Generate fields -- Ricky26
			$fields = array();
			
			foreach($payFields as $k => $v)
				$fields[] = array("type"=>"hidden", "name"=>$k, "value"=>$v);
			
			$fields[] = array("type"=>"hidden", "name"=>"signature", "value"=>$sig);

			return $fields;
		}
		
		function isTestMode()
		{
			$this->testMode = $this->settings_vars[$this->id."_test_mode"]['value'] != "No";
			return $this->testMode;
		}
		
		// Get JS code for form validation
		function getValidatorJS()
		{
			// We don't need any validation as we're forwarding control to
			// Amazon... -- Ricky26
			return "";
		}
		
		// Get the url to forward to.
		function getPostUrl()
		{
			// We want to return either their custom URL, the sandbox URL or the
			// standard URL for amazon simple-pay. -- Ricky26
			
			$postUrl = "";
			if($this->url_to_gateway != "")
				$postUrl = trim($this->url_to_gateway);	
			elseif($this->isTestMode())
				$postUrl = "https://authorize.payments-sandbox.amazon.com/pba/paypipeline";
			else
				$postUrl = "https://authorize.payments.amazon.com/pba/paypipeline";
			
			$this->post_url = $postUrl;
			return $postUrl;
		}
		
		// Process the result from the AWS.
		function process($db, $user, $order, $post_form)
		{			
			// Variables for the signature -- Ricky26
			$sig = NULL;
			$sigText = "";
			$debug = FALSE;
			
			// Merge and sort the GET/POST variables -- Ricky26
			$vars = array_merge(array(), $_GET, $_POST);
			$varNames = array_keys($vars);
			natcasesort($varNames);
			
			// Generate the text for encoding -- Ricky26
			$logText = "\nAmazon vars:\n";
			foreach($varNames as $k)
			{
				$v = $vars[$k];
				
				$logText = $logText . ("\t" . $k . ":\t" . $v . "\n");
				
				// Check for signature, and signature details
				if($k == "signature")
					$sig = $v;
				else
					$sigText = $sigText . $k . $v;
			}
			
			// Check signature -- Ricky26
			$sigCheck = $this->hashHMAC($sigText, $this->settings_vars[$this->id."_secret_key"]["value"]);
			$OK = strcasecmp($sigCheck, $sig) == 0;
			
			// Logging
			if(!$OK || $debug)
			{	
				$this->logStart();
				$this->log($logText);
				
				if(!$OK)
					$this->log("Failed attempt.\n" . file_get_contents("php://input") . "\n" . $sigText . "\n" . $sigCheck);
				$this->logFinish();
			}
			
			// If the signature failed
			// we can't have received payment. -- Ricky26
			if(!$OK)
			{
				$this->is_error = TRUE;
				$this->error_message = "Payment signature failure, Amazon Payments gateway failure.";
				return !$this->is_error;
			}
			
			$status = $vars["status"];
			
			// Support for delayed transactions. -- Ricky26
			if($status == "PI")
			{
				$this->is_error = FALSE;
				$this->error_message = "Payment is pending, your order will be shipped once payment is received.";
				return "Pending";
			}
			
			// Return whether it was a success -- Ricky26
			if($status == "PS")
			{				
				$this->is_error = FALSE;
				$this->error_message = "";
				$this->createTransaction($db, $user, $order, $sigText, $vars["transactionId"]);
			}
			else
			{
				$this->logStart();
				$this->log("Failed payment:\n" . $logText);
				$this->logFinish();
				
				$this->is_error = TRUE;
				$this->error_message = $this->getErrorString($status) . ", please try again.";
			}
			
			return !$this->is_error;
		}
		
		// Generate a HMAC hash.
		function hashHMAC($data, $key)
		{
			$sig = hash_hmac("sha1", $data, $key, TRUE);
			$sig = base64_encode($sig);
			return $sig;
		}
	}
?>