<?php 
	$payment_processor_id = "ecsuite";
	$payment_processor_name = "EC Suite";
	$payment_processor_class = "PAYMENT_ECSUITE";
	$payment_processor_type = "cc";

	class PAYMENT_ECSUITE extends PAYMENT_PROCESSOR
	{
		function PAYMENT_ECSUITE()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "ecsuite";
			$this->name = "EC Suite";
			$this->class_name = "PAYMENT_ECSUITE";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->need_cc_codes = false;
			$this->support_ccs = false;
			return $this;
		}

		/**
		 *
		 * @param type $amount
		 * @return type 
		 */
		function formatAmountString($amount)
		{
			return str_replace(array(',','.'), '', number_format($amount, 2, '.', ''));
		}

		/**
		 * Processes Other Transactions
		 *
		 * @param string $db 
		 * @param string $custom_id 
		 * @param string $order_id 
		 * @return void
		 * @author Sebastian Nievas
		 */
		function processCustom($db, $custom_id, $order_id)
		{
			// Currently, this call is only made in from admin_order_process, globally load $order here
			global $settings, $order;

			$oldPaymentStatus = $order->getPaymentStatus();

			$order_id = intval(is_numeric($order_id) ? $order_id : 0);

			if (!$order_id) 
			{
				$this->is_error = true;
				$this->error_message = 'An Error Has Occurred - Invalid Order ID';
				return false;
			}

			$db->query("
				SELECT ".DB_PREFIX."orders.*,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
				WHERE 
					".DB_PREFIX."orders.oid='".$order_id."'
			");

			$db->moveNext();
			$data = $db->col;

			// set some environment variables
			$this->payment_status = $data['payment_status'];
			$this->custom = $data['custom1'];
			$this->transaction_id = $data['security_id'];

			$this->is_error = false;

			// transaction options
			$transaction_options = array(
				'transaction_id' => $this->transaction_id
			);

			// set the transaction type to perform based on the $custom_id sent
			switch($custom_id)
			{
				// capture
				case '1':
					$transactionType = 'CreditCardCapture';
					$order->setPaymentStatus('Received');
					$transaction_options['amount'] = $this->formatAmountString($data['total_amount']);
					break;

				// refund
				// case '2': $transactionType = 'CustomerCreditCardRefund';
				// 	$this->payment_status = 'Refunded';
				// 	break;

				// Void transaction
				case '3':
					$transactionType = 'CreditCardVoid';
					$order->setPaymentStatus('Canceled');
					break;

				default:
					$this->is_error = true;
					return false;
			}

			$buffer = $this->dispatch($transactionType, $transaction_options);

			if ($buffer)
			{
				$xmlParser = new XMLParser();

				$data = $xmlParser->parse($buffer);

				$response = $data['TGSMerchantResponse'][0]['Responses'][0];

				if (isset($response['ErrorResponse']))
				{
					// automatically decline transaction
					$this->error_message = $response['ErrorResponse'][0]['SystemCode'][0] . ': '.$response['ErrorResponse'][0]['ResponseText'][0];
					$this->is_error = true;
					return false;
				}
				elseif (isset($response['CCResponse']))
				{
					// We have received a proper Credit Card response
					$rsApproved = $response['CCResponse'][0]['Approved'][0];
					$rsBankCode = $response['CCResponse'][0]['BankingCode'][0];
					$rsSystemCode = $response['CCResponse'][0]['SystemCode'][0];
					$rsResponseText = $response['CCResponse'][0]['ResponseText'][0];
					$rsTransactionId = isset($response['CCResponse@']) ? $response['CCResponse@'][0]['transactionId'] : null;

					if ($rsSystemCode == '000' && $rsBankCode == '000' && $rsApproved == 'Y')
					{
						// Update the orders table status
						$db->reset();
						$db->assignStr('payment_status', $order->getPaymentStatus());
						$db->assignStr("security_id", $rsTransactionId);
						$db->assignStr("custom2", $buffer);
						$db->update(DB_PREFIX."orders", "WHERE oid='".$order_id."'");
					}
					else 
					{
						// there was an error
						$this->error_message = 'We are sorry, there was an error with your order: ' . $rsResponseText;
						$this->is_error = true;
						return false;
					}
				}
			}
			else 
			{
				$this->is_error = true;
			}

			$this->error_message = 'The gateway could not process this request';

			$this->fireOrderEvent($order, $order->getStatus(), $oldPaymentStatus);

			// will return false
			return false;
		}

		/**
		 * undocumented function
		 *
		 * @param string $custom1 
		 * @param string $custom2 
		 * @param string $custom3 
		 * @param string $order_status 
		 * @param string $payment_status 
		 * @return void
		 * @author Sebastian Nievas
		 */
		function getVarStatus($custom1="", $custom2="", $custom3="", $order_status="", $payment_status="")
		{
			$var_status = array(1 => false, 2 => false, 3 => false);
			$message = '';

			if ($custom1 == "CreditCardAuth" && $payment_status == "Pending")
			{
				switch ($payment_status)
				{
					case 'Pending':
						$message = "
						<div style='padding-top:7px;'>
							<input type=\"button\" name=\"txn_action\" value=\"Complete Transaction\" onclick=\"window.open('content/engine/payment/ecsuite/ecsuite_gateway.php?oid=".$_GET['oid']."&payment_status=Pending','PopupWind','titlebar=no, toolbar=no, menubar=no, status=no, directories=no, resizable=no, scrollbars=no, top=200, left=400, width=600, height=150')\" />
						</div>";
						break;
					// Authorization has been captured - Show Refund Link
					// case 'Received':
					// 	$message = "
					// 	<div style='padding-top:7px;'>
					// 		<a onclick=\"window.open('content/engine/payment/qbms/qbms_gateway.php?oid=".$_GET['oid']."','PopupWind','titlebar=no, toolbar=no, menubar=no, status=no, directories=no, resizable=no, scrollbars=no, top=200, left=400, width=320, height=240')\" href=\"#\">Refund Payment</a>
					// 	</div>";
					// 	break;
				}
			}
			elseif ($custom1 == 'CreditCardCapture' && $payment_status == 'Received')
			{
				// Refund Only
				// Funds have been captured - Show Refund Link
				// 	$message = "
				// 	<div style='padding-top:7px;'>
				// 		<a onclick=\"window.open('content/engine/payment/qbms/qbms_gateway.php?oid=".$_GET['oid']."','PopupWind','titlebar=no, toolbar=no, menubar=no, status=no, directories=no, resizable=no, scrollbars=no, top=200, left=400, width=320, height=240')\" href=\"#\">Refund Payment</a>
				// 	</div>";
				return array();
			}

			$var_status[1] = array(
					'message' => $message,
					'executable' => true,
					'link_caption' => ''
				);

			return $var_status;
		}

		public function replaceDataKeys()
		{
			return array('Transactions.CreditCardAuthRequest.CustomerAccountInfo.CardNumber', 'AuthKey');
		}

		/**
		 *
		 * @param type $transactionType
		 * @param type $postVars
		 * @return type 
		 */
		function dispatch($transactionType, $postVars)
		{
			global $settings;

			// production vars
			$post_url = (trim($this->url_to_gateway) == '')
				? 'https://tgs.ecsuite.com/merchant_interface/XMLServlet'
				: trim($this->url_to_gateway);

			// debug
			// $postVars['cc_cvv2'] = 100; // decline
			// $postVars['cc_cvv2'] = 350; // approved

			// create unique transaction request id
			$merchantTransactionId = mktime().rand(1000,9999);

			$input_array = array();
			switch ($transactionType)
			{
				case 'CreditCardAuthCapture':
					$capture = 'capture="1"';
				case 'CreditCardAuth':
					if (mktime(0,0,0,date('m'),1,date('y')) > mktime(0,0,0,$postVars['cc_expiration_month'],1,$postVars['cc_expiration_year']))
					{
						$this->error_message = 'This credit card has expired. Please check the expiration date or use a different credit card.';
						return false;
					}

					if (!isset($capture)) $capture = '';

					$transactionRequestXml = '
						<CreditCardAuthRequest merchantTransactionId="'.$merchantTransactionId.'" '.$capture.'>
							<CustomerAccountInfo>
								<CardPresent>false</CardPresent>
								<NameOnCard>'.$postVars["cc_first_name"].' '.$postVars["cc_last_name"].'</NameOnCard>
								<CardNumber cvv="'.$postVars['cc_cvv2'].'">'.$postVars["cc_number"].'</CardNumber>
								<ExpDate>'.$postVars['cc_expiration_month'].substr($postVars['cc_expiration_year'], -2).'</ExpDate>
							</CustomerAccountInfo>

							<MerchantInfo>
								<TerminalID>'.$this->settings_vars[$this->id."_terminal_id"]["value"].'</TerminalID>
							</MerchantInfo>

							<Currency>
								<Amount>'.$this->formatAmountString($this->common_vars["order_total_amount"]["value"]).'</Amount>
								<Code>'.$this->getCurrencyCode($this->settings_vars[$this->id."_currency"]["value"]).'</Code>
							</Currency>

							<AddressInfo>
								<IP>'.$_SERVER['REMOTE_ADDR'].'</IP>
								<StreetAddress>'.$this->common_vars["billing_address1"]["value"].' '.$this->common_vars["billing_address2"]["value"].'</StreetAddress>
								<City>'.$this->common_vars["billing_city"]["value"].'</City>
								<Province>'.$this->common_vars["billing_state"]["value"].'</Province>
								<PostalCode>'.$this->common_vars["billing_zip"]["value"].'</PostalCode>
								<CountryCode>'.$this->common_vars["billing_country_iso_a2"]["value"].'</CountryCode>
							</AddressInfo>

							<ACI>Y</ACI>
							<ECI>03</ECI>
						</CreditCardAuthRequest>
					';
					break;
				case 'CreditCardVoid':
					$transactionRequestXml = '
					<CreditCardVoidRequest merchantTransactionId="'.$merchantTransactionId.'" transactionId="'.$postVars['transaction_id'].'" />
					';
					break;
				case 'CreditCardCapture':
					$transactionRequestXml = '
					<CreditCardCaptureRequest merchantTransactionId="'.$merchantTransactionId.'" transactionId="'.$postVars['transaction_id'].'">
						<Currency>
							<Amount>'.$postVars['amount'].'</Amount>
						</Currency>
					</CreditCardCaptureRequest>
					';
					break;
				// not passing valid transactionType
				default: 
					return false;
			}

			$xml = '<?xml version="1.0"?>
				<TGSMerchantRequest version="0.1">
					<AuthKey merchantId="'.$this->settings_vars[$this->id."_merchant_id"]["value"].'">'.$this->settings_vars[$this->id."_auth_key"]["value"].'</AuthKey>
					<Transactions>
						'.$transactionRequestXml.'
					</Transactions>
				</TGSMerchantRequest>
			';

			// initialize curl
			$ch = curl_init();

			$headers = array();
			$headers[] = "Content-type: application/xml";
			$headers[] = "Content-length: ".strlen($xml);

			curl_setopt($ch, CURLOPT_TIMEOUT, 500);
			curl_setopt($ch, CURLOPT_URL, $post_url);

			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

			if (!safe_mode()) set_time_limit(3000);

			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
			curl_setopt($ch, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
			curl_setopt($ch, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);

			$requestData = xml2array($xml);
			$this->log("dispatch Request:\n".$post_url, $requestData);

			$buffer = curl_exec($ch) or die(curl_error($ch));

			$this->log("dispatch Response:\n".$buffer);

			if (!$buffer)
			{
				return false;
			}

			return $buffer;
		}

		function process($db, $user, $order, $post_form)
		{
			global $settings;

			if (isset($post_form) && is_array($post_form))
			{
				// get the correct transaction request type
				switch ($this->settings_vars[$this->id.'_transaction_type']['value'])
				{
					case 'Authorize Only':
						$transactionType = 'CreditCardAuth';
						break;
					case 'Authorize and Capture':
					default:
						$transactionType = 'CreditCardAuthCapture';
						break;
				}

				$buffer = $this->dispatch($transactionType, $post_form);

				// we received data, parse it
				if ($buffer)
				{
					$xmlParser = new XMLParser();
					$data = $xmlParser->parse($buffer);

					$response = $data['TGSMerchantResponse'][0]['Responses'][0];

					if (isset($response['ErrorResponse']))
					{
						// automatically decline transaction
						$this->error_message = $response['ErrorResponse'][0]['SystemCode'][0] . ': '.$response['ErrorResponse'][0]['ResponseText'][0];
						$this->is_error = true;
						return false;
					}
					elseif (isset($response['CCResponse']))
					{
						// We have received a proper Credit Card response
						$rsApproved = $response['CCResponse'][0]['Approved'][0];
						$rsBankCode = $response['CCResponse'][0]['BankingCode'][0];
						$rsSystemCode = $response['CCResponse'][0]['SystemCode'][0];
						$rsResponseText = $response['CCResponse'][0]['ResponseText'][0];

						// holds the error state of this block, defaults to false
						$found_error = false;

						if ($rsSystemCode == '000' && $rsBankCode == '000' && $rsApproved == 'Y')
						{
							$rsTransactionId = $response['CCResponse@'][0]['transactionId'];

							// log the transaction
							$this->createTransaction($db, $user, $order, $buffer, '', $transactionType, '', $charged_amount, $rsTransactionId);

							// store credit card info?
							if ($this->support_ccs && $this->enable_ccs)
							{
								// PA DSS compliance
								$card_number = str_pad(substr($post_form['cc_number'], -4), strlen($post_form['cc_number']), 'x', STR_PAD_LEFT);

								$card_data = array(
									"fn"=>$post_form["cc_first_name"],
									"ln"=>$post_form["cc_last_name"],
									"ct"=>$post_form["cc_type"],
									"cc"=>$card_number,
									"em"=>$post_form["cc_expiration_month"],
									"ey"=>$post_form["cc_expiration_year"]
								);
								$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
							}

							// transaction successful, return to not go past this point
							return ($transactionType == 'CreditCardAuth') ? 'Pending' : 'Received';
						}
						else
						{
							// there was an error
							$this->error_message = 'We are sorry, there was an error with your order: ' . $rsResponseText;
							$this->is_error = true;
							return false;
						}
					}

					// if script reaches this point, there was an error from the gateway
					$this->is_error = true;
				}
				else 
				{
					// no $buffer set, return error
					$this->is_error = true;
					$this->error_message = ($this->error_message) ? $this->error_message : PAYMENT_SERVER_ERROR_TEXT;
				}

				if ($this->is_error)
				{
					$this->storeTransaction($db, $user, $order, $buffer, '', false);
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administrator.";
			}
			return !$this->is_error;
		}

		function getCurrencyCode($key)
		{
			switch ($key)
			{
				case 'US Dollar': return '840'; break;
				case 'Afghani': return '971'; break;
				case 'Algerian Dinar': return '012'; break;
				case 'Argentine Peso': return '032'; break;
				case 'Armenian Dram': return '051'; break;
				case 'Aruban Guilder': return '533'; break;
				case 'Australian Dollar': return '036'; break;
				case 'Azerbaijanian Manat': return '031'; break;
				case 'Bahamian Dollar': return '044'; break;
				case 'Bahraini Dinar': return '048'; break;
				case 'Baht': return '764'; break;
				case 'Balboa': return '590'; break;
				case 'Barbados Dollar': return '052'; break;
				case 'Belarussian Ruble': return '974'; break;
				case 'Belize Dollar': return '084'; break;
				case 'Bermudian Dollar (Bermuda Dollar)': return '060'; break;
				case 'Bolivar': return '862'; break;
				case 'Boliviano': return '068'; break;
				case 'Brazilian Real': return '986'; break;
				case 'Brunei Dollar': return '096'; break;
				case 'Bulgarian Lev': return '100'; break;
				case 'Burandi Franc': return '108'; break;
				case 'Canadian Dollar': return '124'; break;
				case 'Cape Verde Escudo': return '132'; break;
				case 'Cayman Islands Dollar': return '126'; break;
				case 'Cedi': return '288'; break;
				case 'CFA Franc BCEAO': return '952'; break;
				case 'CFA Franc BEAC': return '950'; break;
				case 'CFP Franc': return '953'; break;
				case 'Chilean Peso': return '152'; break;
				case 'Colombian Peso': return '170'; break;
				case 'Comoro Franc': return '174'; break;
				case 'Convertible Marks': return '977'; break;
				case 'Cordoba Oro': return '558'; break;
				case 'Costa Rican Colon': return '188'; break;
				case 'Croatian Kuna': return '191'; break;
				case 'Cuban Peso': return '192'; break;
				case 'Cyprus Pound': return '196'; break;
				case 'Czech Koruna': return '203'; break;
				case 'Dalasi': return '270'; break;
				case 'Danish Krone': return '208'; break;
				case 'Denar': return '807'; break;
				case 'Djibouti Franc': return '262'; break;
				case 'Dobra': return '678'; break;
				case 'Dominican Peso': return '214'; break;
				case 'Dong': return '704'; break;
				case 'East Caribbean Dollar': return '951'; break;
				case 'Egyptian Pound': return '818'; break;
				case 'El Salvador Colon': return '222'; break;
				case 'Ethiopian Birr': return '230'; break;
				case 'Euro': return '978'; break;
				case 'Falldand Islands Pound': return '238'; break;
				case 'Fiji Dollar': return '242'; break;
				case 'Forint': return '348'; break;
				case 'Franc Congolais': return '976'; break;
				case 'Gibraltar Pound': return '292'; break;
				case 'Gourde': return '332'; break;
				case 'Guarani': return '600'; break;
				case 'Guinea Franc': return '324'; break;
				case 'Guinea-Bissau Peso': return '624'; break;
				case 'Guyana Dollar': return '328'; break;
				case 'Hong Kong Dollar': return '344'; break;
				case 'Hryvnia': return '980'; break;
				case 'Iceland Krona': return '352'; break;
				case 'Indian Rupee': return '356'; break;
				case 'Iranian Rial': return '364'; break;
				case 'Iraqi Dinar': return '368'; break;
				case 'Jamaican Dollar': return '388'; break;
				case 'Jordanian Dinar': return '400'; break;
				case 'Kenyan Shilling': return '404'; break;
				case 'Kina': return '598'; break;
				case 'Kip': return '418'; break;
				case 'Kroon': return '233'; break;
				case 'Kuwaiti Dinar': return '414'; break;
				case 'Kwacha': return '454'; break;
				case 'Kwacha': return '894'; break;
				case 'Kwanza': return '973'; break;
				case 'Kyat': return '104'; break;
				case 'Lari': return '981'; break;
				case 'Latvian Lats': return '428'; break;
				case 'Lebanese Pound': return '422'; break;
				case 'Lek': return '008'; break;
				case 'Lempira': return '340'; break;
				case 'Leone': return '694'; break;
				case 'Liberian Dollar': return '430'; break;
				case 'Libyan Dinar': return '434'; break;
				case 'Lilangeni': return '748'; break;
				case 'Lithuanian Litas': return '440'; break;
				case 'Loti': return '426'; break;
				case 'Malagasy Ariary': return '969'; break;
				case 'Malaysian Ringgit': return '458'; break;
				case 'Maltese Lira': return '470'; break;
				case 'Manat': return '795'; break;
				case 'Mauritius Peso': return '480'; break;
				case 'Merical': return '508'; break;
				case 'Mexican Peso': return '484'; break;
				case 'Mexican Unidad de Inversion (UID)': return '979'; break;
				case 'Moldovan Leu': return '498'; break;
				case 'Moroccan Dirham': return '504'; break;
				case 'Mvdol': return '984'; break;
				case 'Naira': return '566'; break;
				case 'Nakfa': return '232'; break;
				case 'Namibian Dollar': return '516'; break;
				case 'Nepalese Rupee': return '524'; break;
				case 'Netherlands Antillian Guikder': return '532'; break;
				case 'New Israeli Sheqel': return '376'; break;
				case 'New Leu': return '946'; break;
				case 'New Taiwan Dollar': return '091'; break;
				case 'New Turkish Lira': return '949'; break;
				case 'New Zealand Dollar': return '554'; break;
				case 'Ngultrum': return '064'; break;
				case 'North Korean Won': return '408'; break;
				case 'Norwegian Krone': return '578'; break;
				case 'Nuevo Sol': return '604'; break;
				case 'Old Leu': return '642'; break;
				case 'Ouguiya': return '478'; break;
				case 'Pa’anga': return '776'; break;
				case 'Pakistan Rupee': return '586'; break;
				case 'Pataca': return '446'; break;
				case 'Peso Uruguayo': return '858'; break;
				case 'Phillippine Peso': return '608'; break;
				case 'Pound Sterling': return '826'; break;
				case 'Pula': return '072'; break;
				case 'Qatari Rial': return '634'; break;
				case 'Quetzal': return '320'; break;
				case 'Rand': return '710'; break;
				case 'Rial Omani': return '512'; break;
				case 'Riel': return '116'; break;
				case 'Rufiyaa': return '462'; break;
				case 'Rupiah': return '360'; break;
				case 'Russian Ruble': return '643'; break;
				case 'Rwanda Franc': return '646'; break;
				case 'Saint Helena Pound': return '654'; break;
				case 'Saudi Riyal': return '682'; break;
				case 'Serbian Dinar': return '891'; break;
				case 'Seychelles Rupee': return '690'; break;
				case 'Singapore Dollar': return '702'; break;
				case 'Slovak Koruna': return '703'; break;
				case 'Solomon Islands Dollar': return '090'; break;
				case 'Som': return '417'; break;
				case 'Somali Shilling': return '706'; break;
				case 'Somoni': return '972'; break;
				case 'Sri Lanka Rupee': return '144'; break;
				case 'Sudanese Dinar': return '736'; break;
				case 'Surinam Dollar': return '740'; break;
				case 'Swedish Krona': return '752'; break;
				case 'Swiss Franc': return '756'; break;
				case 'Syrian Pound': return '760'; break;
				case 'Taka': return '050'; break;
				case 'Tala': return '882'; break;
				case 'Tanzanian Shilling': return '834'; break;
				case 'Tenge': return '398'; break;
				case 'Tolar': return '705'; break;
				case 'Trinidad and Tobago Dollar': return '780'; break;
				case 'Tugrik': return '496'; break;
				case 'Tunisian Dinar': return '788'; break;
				case 'UAE Dirham': return '784'; break;
				case 'Uganda Shilling': return '800'; break;
				case 'Unidad de Valor Real': return '970'; break;
				case 'Unidades de formento': return '990'; break;
				case 'Uzbekistan Sum': return '860'; break;
				case 'Vatu': return '548'; break;
				case 'WIR Euro': return '947'; break;
				case 'WIR Franc': return '948'; break;
				case 'Won': return '410'; break;
				case 'Yemeni Rial': return '886'; break;
				case 'Yen': return '392'; break;
				case 'Yuan Renminbi': return '156'; break;
				case 'Zimbabwe Dollar': return '716'; break;
				case 'Zloty': return '985'; break;

				default:
					$this->is_error = true;
					$this->error_message = 'Invalid currency set for gateway. Please contact administrator.';
			}
		}
	}
