<?php 
	$payment_processor_id = "paypalec";
	$payment_processor_name = "PayPal Exress Checkout";
	$payment_processor_class = "PAYMENT_PAYPALEC";
	$payment_processor_type = "cc";

	class PAYMENT_PAYPALEC extends PAYMENT_PROCESSOR
	{
		protected $api;
		protected $buttonSource = '';

		/**
		 * @return PAYMENT_PAYPALEC
		 */
		public function PAYMENT_PAYPALEC()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "paypalec";
			$this->name = "PayPal Express Checkout";
			$this->class_name = "PAYMENT_PAYPALEC";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->is_certificate = true;
			$this->certificate_optional = true;
			$this->certificate_path = "paypal/certificate.pem";
			$this->support_accepted_cc = false;
			$this->show_cvv2 = false;

			$this->buttonSource = getpp().'_Cart_EC';

			return $this;
		}

		/**
		 * Get payment form
		 * @param type $db
		 * @return string 
		 */
		public function getPaymentForm($db)
		{
			parent::getPaymentForm($db);

			$cc_options_codes = $this->cc_options_codes;
			$fields = array();
			$fields[] = array("type"=>"hidden", "name"=>"oa", "value"=>ORDER_PROCESS_PAYMENT);
			return $fields;
		}

		/**
		 * Get validation javascript
		 * @return string 
		 */
		public function getValidatorJS()
		{
			return "";
		}

		/**
		 * Test mode
		 * @return type 
		 */
		public function isTestMode()
		{
			if (strtolower($this->settings_vars[$this->id."_Mode"]["value"]) == "test")
			{
				$this->testMode = true;
			}
			else
			{
				$this->testMode = false;
			}
			return $this->testMode;
		}

		public function expressCheckout($order, $cartItems, $fromCart, $returnUrl, $cancelUrl)
		{
			global $settings, $db;

			$paypalLines = array();
							
			foreach ($cartItems as $cartItem)
			{
				$paypalLines[] = array(
					'name' => $cartItem['title'],
					'desc' => $cartItem['product_id']."\n".$cartItem['options_clean'],
					'amt' => number_format($cartItem['product_price'], 2, '.', ''),
					'qty' => $cartItem['quantity'],
					'tax' => $fromCart ? 0 : $cartItem['product_price'] * $cartItem['quantity'] * ($settings['DisplayPricesWithTax'] == 'YES' ? 0 : $cartItem['tax_rate']) / 100,
					'itemcategory' => $cartItem['digital_product'] == 'Yes' ? 'Digital' : 'Physical',
				);
			}

			if ($fromCart)
			{
				$paypalTotal = $settings['DisplayPricesWithTax'] == 'YES' ? $order->subtotalAmountWithTax : $order->subtotalAmount;
			}
			else
			{
				$paypalTotal = $order->totalAmount;
			}

			if ($order->discountAmount > 0)
			{
				$paypalLines[] = array(
					'name' => 'Discount amount',
					'desc' => '',
					'amt' => '-'.number_format($order->discountAmount, 2, '.', ''),
					'qty' => 1,
					'tax' => 0,
					'itemcategory' => 'Physical'
				);

				if ($fromCart)
				{
					$paypalTotal -= $order->discountAmount;
				}
			}

			if ($settings['DiscountsPromo'] == 'YES' && $order->promoDiscountAmount > 0)
			{
				$paypalLines[] = array(
					'name' => 'Promo discount',
					'desc' => '',
					'amt' => '-'.number_format($order->promoDiscountAmount, 2, '.', ''),
					'qty' => 1,
					'tax' => 0,
					'itemcategory' => 'Physical'
				);

				if ($fromCart)
				{
					$paypalTotal -= $order->promoDiscountAmount;
				}
			}

			$amountLinex = false;
			if (!$fromCart)
			{
				$amountLinex = array(
					"shipping_amount" => $order->shippingAmount,
					"tax_amount" => $settings['DisplayPricesWithTax'] == 'YES' ? 0 : $order->taxAmount,
					"handling_amount" => $order->handlingSeparated ? $order->handlingAmount : 0,
					"discount_amount" => $order->discountAmount,
					"promo_discount_amount" => $order->promoDiscountAmount,
				);
			}

			$giftCertificateData = isset($_SESSION["order_gift_certificate"]) ? $_SESSION["order_gift_certificate"] : false;
			if (!$fromCart && $giftCertificateData)
			{
				$giftCert = new GiftCertificates($db);
				$giftCertData = $giftCert->GetOrderedGiftCert($giftCertificateData['first_name'], $giftCertificateData['last_name'], $giftCertificateData['voucher']);

				if ($giftCertData)
				{
					$giftCertAmount = $giftCertData['balance'];
					$paypalLines[] = array(
						'name' => 'Gift Certificate',
						'desc' => '',
						'amt' => '-'.number_format($giftCertAmount, 2, '.', ''),
						'qty' => 1,
						'tax' => 0,
						'itemcategory' => 'Physical'
					);

					$paypalTotal = $paypalTotal - $giftCertAmount;
				}
			}

			$paypal_ec_token = $this->setExpressCheckout(
				$paypalTotal, 
				$returnUrl, 
				$cancelUrl, 
				$order->shippingDigital,
				$paypalLines,
				$amountLinex,
				$fromCart ? false : true
			);

			if (!$fromCart)
			{
				$_SESSION["paypal_express_checkout_token_opc"] = $paypal_ec_token;
			}

			return $this->getExpressCheckoutUrl($paypal_ec_token, $fromCart);
		}

		/**
		 * Get Express Checkout details
		 * @param type $token
		 * @return string 
		 */
		public function getExpressCheckoutDetails($token)
		{
			unset($_SESSION["paypal_express_checkout_payer_id"]);

			$api = $this->getClient();

			$response = $api->getExpressCheckoutDetails($this->settings_vars[$this->id.'_Transaction_Type']['value'], $token);

			if ((isset($response['RESULT']) && $response['RESULT'] == '0') || (isset($response['ACK']) && $response['ACK'] == 'Success'))
			{
				if (isset($response['PAYERID']))
				{
					$_SESSION["paypal_express_checkout_payer_id"] = $response["PAYERID"];
				}
				return $response;
			}
			else
			{
				$this->is_error = true;
				if (isset($response['RESPMSG']))
				{
					$this->error_message = $response['RESPMSG'];
				}
				else if (isset($response['L_SHORTMESSAGE0']))
				{
					$this->error_message = $response['L_SHORTMESSAGE0'] .': '.$response['L_LONGMESSAGE0'];
				}
			}

			return '';
		}

		/**
		 * Set express checkout
		 * @global type $db
		 * @global type $settings
		 * @param type $orderTotal
		 * @param type $returnUrl
		 * @param type $cancelUrl
		 * @param type $noShipping
		 * @param type $lines
		 * @return string 
		 */
		public function setExpressCheckout(
			$orderTotal, $returnUrl, $cancelUrl, $noShipping = false, $lines = false, 
			$orderAmounts = false, $noShippingAddress = false
		)
		{
			global $db, $settings, $order;

			unset($_SESSION['paypal_express_checkout_token']);

			$api = $this->getClient();

			$uxParams = array();

			if (substr($settings['GlobalHttpsUrl'], 0, strlen('https://')) == 'https://' || 1)
			{
				if (isset($this->settings_vars[$this->id.'_Logo']) && $this->settings_vars[$this->id.'_Logo']['value'] != '')
				{
					$uxParams['HDRIMG'] = $settings['GlobalHttpsUrl'].'/'.$this->settings_vars[$this->id.'_Logo']['value'];
				}
				else
				{
					$db->query('SELECT * FROM '.DB_PREFIX."design_elements WHERE name = 'image-logo'");
					
					if ($row = $db->moveNext())
					{
						$uxParams['HDRIMG'] = $settings['GlobalHttpsUrl'].'/'.$row['content'];
					}
				}
			}

			if (isset($this->settings_vars[$this->id.'_Border']))
				$uxParams['HDRBORDERCOLOR'] = $this->settings_vars[$this->id.'_Border']['value'];

			if (isset($this->settings_vars[$this->id.'_Background']))
				$uxParams['HDRBACKCOLOR'] = $this->settings_vars[$this->id.'_Background']['value'];

			$response = $api->setExpressCheckout(
				$this->settings_vars[$this->id.'_Transaction_Type']['value'],
				$orderTotal, 
				$this->settings_vars[$this->id."_Currency_Code"]["value"], 
				$returnUrl, 
				$cancelUrl, 
				$noShipping, 
				$lines, 
				$uxParams,
				$orderAmounts,
				$noShippingAddress,
				$order
			);

			if (
				(isset($response['RESULT']) && $response['RESULT'] == '0') || 
				(isset($response['ACK']) && in_array($response['ACK'], array('Success', 'SuccessWithWarning', 'SuccessWithWarnings')))
			)
			{
				if (isset($response['TOKEN']))
				{
					$_SESSION['paypal_express_checkout_token'] = $response['TOKEN'];
					return $response['TOKEN'];
				}
			}
			else
			{
				$this->is_error = true;
				if (isset($response['RESPMSG']))
				{
					$this->error_message = $response['RESPMSG'];
				}
				else if (isset($response['L_SHORTMESSAGE0']))
				{
					$this->error_message = $response['L_SHORTMESSAGE0'] .': '.$response['L_LONGMESSAGE0'];
				}
			}

			return '';
		}

		/**
		 * Get Express Checkout URL
		 * @param type $token
		 * @return type 
		 */
		public function getExpressCheckoutUrl($token, $fromCart = false)
		{
			$api = $this->getClient();

			return $api->getExpressCheckoutUrl($token, $fromCart);
		}

		/**
		 * Process transaction
		 * @global type $settings
		 * @param type $db
		 * @param type $user
		 * @param type $order
		 * @param type $post_form
		 * @return boolean|string 
		 */
		public function process($db, $user, $order, $post_form)
		{
			global $settings;
			if (isset($_SESSION["paypal_express_checkout_details"]) || isset($_SESSION["paypal_express_checkout_token_opc"]))
			{
				$api = $this->getClient();

				if (isset($_SESSION["paypal_express_checkout_token_opc"]))
				{
					$token = isset($_SESSION["paypal_express_checkout_token_opc"])?$_SESSION["paypal_express_checkout_token_opc"]:"";
					$this->getExpressCheckoutDetails($token);
				}
				else
				{
					$token = isset($_SESSION["paypal_express_checkout_token"])?$_SESSION["paypal_express_checkout_token"]:"";
				}
				
				$payer_id = isset($_SESSION["paypal_express_checkout_payer_id"])?$_SESSION["paypal_express_checkout_payer_id"]:"";

				if ($token == "")
				{
					$this->is_error = true;
					$this->error_message = "PayPal Express Checkout Token is Blank";
					return false;
				}
				
				if ($payer_id == '')
				{
					$this->is_error = true;
					$this->error_message = "PayPal Express Checkout Payer ID is Blank";
					return false;
				}

				$charged_amt = 0;

				$result = $api->doExpressCheckoutPayment(
					$this->settings_vars[$this->id."_Transaction_Type"]["value"], $token, $payer_id,
					$this->common_vars["order_total_amount"]["value"], $this->settings_vars[$this->id."_Currency_Code"]["value"]
				);

				if ((isset($result['RESULT']) && $result['RESULT'] == '0') || (isset($result['ACK']) && $result['ACK'] == 'Success'))
				{
					$response = "";
					foreach ($result as $var=>$val)
					{
						$response .= $var." = ".$val."\n";
					}

					$transaction_id = "";
					$payment_status = "Received";
					
					unset($_SESSION["paypal_express_checkout_token_opc"]);

					if (isset($result['ACK']))
					{
						//PayPal API
						$transaction_id = $result['TRANSACTIONID'];
						$PendingReason = $result['PENDINGREASON'];
						if ($result['PAYMENTSTATUS'] == 'Completed')
						{
							$payment_status = $this->settings_vars[$this->id."_Transaction_Type"]["value"] == 'Sale' ? 'Received' : 'Pending';
						}
						else
						{
							$payment_status = 'Pending';
						}
					}
					else
					{
						//Payflow API
						$transaction_id = $result['PNREF'];
						$PendingReason = $result['PENDINGREASON'];

						if ($PendingReason == 'completed')
						{
							$payment_status = $this->settings_vars[$this->id."_Transaction_Type"]["value"] == 'Sale' ? 'Received' : 'Pending';
						}
						else
						{
							$payment_status = 'Pending';
						}
					}
					
					if ($payment_status == "Received")
					{
						$charged_amt = $order->totalAmount;
					}
					
					$this->createTransaction($db, $user, $order, $response, "", strtolower($PendingReason), "", $charged_amt, $transaction_id);
					$this->is_error = false;

					return $payment_status;
				}
				else
				{
					$this->is_error = true;
					if (isset($result['RESPMSG']))
					{
						$this->error_message = $result['RESPMSG'];
					}
					else if (isset($result['L_SHORTMESSAGE0']))
					{
						$this->error_message = $result['L_SHORTMESSAGE0'] .': '.$result['L_LONGMESSAGE0'];
					}
					else
					{
						$this->error_message = "Unknown error occurred while processing your transaction";
					}

					$this->storeTransaction($db, $user, $order, $result, '', false);
				}
			}
			else
			{
				$this->error_message = "PayPal Express Checkout session expired or does not exist";
				$this->is_error = true;
			}
			
			return false;
		}

		/**
		 * Get client to be used to communicate EC
		 * @global type $settings
		 * @return type 
		 */
		public function getClient()
		{
			if (is_null($this->api))
			{
				global $settings;

				$this->api = Payment_PayPal_PayPalApi::getExpressCheckoutApi($settings, $this->buttonSource);

				$this->settings_vars['paypalec_Mode']['value'] = $settings['paypalec_Mode'];
				$this->settings_vars['paypalec_Currency_Code']['value'] = $settings['paypalec_Currency_Code'];
				$this->settings_vars['paypalec_Transaction_Type']['value'] = $settings['paypalec_Transaction_Type'];
			}

			return $this->api;
		}
	}
