<?php 
	$payment_processor_id = "paypalpfpro";
	$payment_processor_name = "PayPal Payflow Pro";
	$payment_processor_class = "PAYMENT_PAYPALPFPRO";
	$payment_processor_type = "cc";
	
	class PAYMENT_PAYPALPFPRO extends PAYMENT_PROCESSOR
	{
		protected $buttonSource = '';

		function PAYMENT_PAYPALPFPRO()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "paypalpfpro";
			$this->name = "PayPal Payflow Pro";
			$this->class_name = "PAYMENT_PAYPALPFPRO";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->support_ccs = true;
			$this->need_cc_codes = true;

			$this->buttonSource = getpp().'_Cart_PFP';

			return $this;
		}
		
		function isTestMode()
		{
			if ($this->settings_vars[$this->id."_Mode"]["value"] == "Test")
			{
				$this->testMode = true;
			}
			else
			{
				$this->testMode = false;
			}
			return $this->testMode;
		}

		function getClient()
		{
			global $settings;

			$liveMode = !$this->isTestMode();

			$api = new Payment_PayPal_PayflowApi($settings, $liveMode);

			$api->setApiPartner($this->settings_vars[$this->id."_Partner"]["value"]);
			$api->setApiVendor($this->settings_vars[$this->id."_Vendor"]["value"]);
			$api->setApiUsername($this->settings_vars[$this->id."_User"]["value"]);
			$api->setApiPassword($this->settings_vars[$this->id."_Password"]["value"]);

			$api->setButtonSource($this->buttonSource);

			return $api;
		}
			
		function process($db, $user, $order, $post_form)
		{ 
			global $settings, $order;

			if (isset($post_form) && is_array($post_form))
			{
				$tempstr = $post_form["cc_number"] . number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "") . date('YmdGis') . "1";
				$request_id = md5($tempstr);
				$tr_type = $this->settings_vars[$this->id."_Transaction_Type"]["value"] == "Sale" ? "S" : "A";	

				$api = $this->getClient();

				$response = $api->doTransaction($tr_type, $this->common_vars["order_id"]["value"],
					$this->common_vars["order_total_amount"]["value"],
					$post_form["cc_number"],
					$post_form["cc_expiration_month"].substr($post_form["cc_expiration_year"], 2, 2),
					$post_form["cc_cvv2"],
					$post_form["cc_first_name"]." ".$post_form["cc_last_name"],
					$this->common_vars["billing_zip"]["value"],
					$this->common_vars["billing_address"]["value"]
				);

				if ($response && isset($response['RESULT']))
				{
					if ($response["RESULT"] == "0" || $response["RESULT"] == "126")
					{
						foreach ($response as $var=>$val)
						{
							$buffer = $buffer.$var." = ".$val."\n";
						}

						$extra = '';

						$custom1 = $messageType == 'S' ? 'AUTH_CAPTURE' : 'AUTH_ONLY';
						//If Auth only, then store auth'd amount to custom2
						$custom2 = $messageType != "S" ? number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "") : "";
						//If Auth Capture, then store captured amount to custom3
						$custom3 = $messageType == "S" ? number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "") : 0;
						$security_id = $response['PNREF'];

						$this->createTransaction($db, $user, $order, $response, $extra, $custom1, $custom2, $custom3, $security_id);

						//$this->createTransaction($db, $user, $order, $response);
						
						if ($this->support_ccs && $this->enable_ccs)
						{
							$card_data = array(
								"fn"=>$post_form["cc_first_name"],
								"ln"=>$post_form["cc_last_name"],
								"ct"=>$post_form["cc_type"],
								"cc"=>$post_form["cc_number"],													
								"em"=>$post_form["cc_expiration_month"], 
								"ey"=>$post_form["cc_expiration_year"]
							);
							$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
						}
						$this->is_error = false;
						$this->error_message = "";
						
						return ($tr_type == 'S') ? 'Received' : 'Pending';
					}
					else
					{
						$this->is_error = true;
						$this->error_message = "Error code: ".$response["RESULT"]." ".$response["RESPMSG"];						
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = PAYMENT_SERVER_ERROR_TEXT;
				}

				if ($this->is_error)
				{
					$this->storeTransaction($db, $user, $order, $response, '', false);
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administartor.";
				//return false;
			}			
			return !$this->is_error;			
		}

		function supportsCapture($order_data = false)
		{
			if ($order_data)
			{
				if ($order_data['payment_status'] == 'Pending' || $order_data['payment_status'] == 'Partial')
				{
					if ($this->isTotalAmountCaptured($order_data))
						return false;

					return true;
				}
			}
			return false;
		}
		
		function supportsRefund($order_data = false)
		{
			if ($order_data)
			{
				if ($order_data['payment_status'] == 'Received' || $order_data['payment_status'] == 'Partial')
				{
					return true;
				}
				else
				if ($order_data['payment_status'] == 'Refunded')
				{
					return $this->_refundableAmount($order_data) > 0.0;
				}
			}
			return false;
		}
		
		function supportsVoid($order_data = false)
		{
			return $order_data && $order_data['custom1'] != 'AUTH_CAPTURE' && ($order_data['payment_status'] == 'Received' || $order_data['payment_status'] == 'Pending');
		}
		
		function isTotalAmountCaptured($order_data)
		{
			return $order_data['custom3'] != '' && $order_data['custom3'] == $order_data['total_amount'];
		}
		
		function capturableAmount($order_data=false)
		{
			if ($order_data)
			{
				if ($this->supportsCapture($order_data))
				{
					$capturedAmount = $order_data['custom3'];

					$total_amount = $order_data['total_amount'];
					$captured_amount =  $order_data['custom3'];
					
					$capturableAmount = $order_data['total_amount'] - $order_data['custom3'];
					if ($capturableAmount < 0)
						$capturableAmount = 0.0;
					return number_format($capturableAmount, 2, '.', '');
				}
			}
			return 0;
		}
		
		function getTotalCaptured($order_data)
		{
			return 0.00 + $order_data['custom3'];	
		}

		function capture($order_data=false,$capture_amount=false)
		{
			global $settings, $db;
			
			$this->is_error = false;

			if ($order_data)
			{
				$order_id = $order_data['order_num'];
				
				$client = $this->getClient();

				//check is transaction completed
				if ($this->isTotalAmountCaptured($order_data))
				{
					$this->is_error = true;
					$this->error_message = "Transaction already completed";
					return false;
				}
				
				//get captured total amount & x_trans_id
				$captured_total_amount = $capture_amount ? $capture_amount : $order_data["custom2"];
				$transaction_id = $order_data["security_id"];

				if ($transaction_id == '')
				{
					$this->is_error = true;
					$this->error_message = 'This order does not have a valid transaction id';
					return false;
				}
				else
				{
					if ($captured_total_amount <= 0)
					{
						$this->is_error = true;
						$this->error_message = "Capture amount must be greater than 0";
						return false;
					}
					else if ($captured_total_amount > $order_data['total_amount'])
					{
						$this->is_error = true;
						$this->error_message = "Capture amount cannot be more than total amount";
						return false;
					}
					else if ($order_data['custom3'] + $captured_total_amount > $order_data['total_amount'])
					{
						$this->is_error = true;
						$this->error_message = 'Captured amount cannot be more than total amount';
						return false;
					}

					$response = $client->doCapture($transaction_id); //, $captured_total_amount);
					
					$this->log("capture Response:\n".array2text($response));

					if (isset($response['RESULT']) && $response['RESULT'] == '0')
					{
						$totalCaptured = $this->getTotalCaptured($order_data) + $captured_total_amount;
						$custom3 = number_format($totalCaptured, 2, '.', '');

						$db->reset();
						$db->assignStr("custom1", "");
						$db->assignStr("custom2", "");
						$db->assignStr("custom3", $custom3);
						$payment_status = 'Received';
						
						if ($order_data['total_amount'] > $totalCaptured)
						{
							$payment_status = 'Partial';
						}
						
						$db->assignStr("payment_status", $payment_status);
						$db->update(DB_PREFIX."orders", "WHERE oid='".$order_data['oid']."'");

						$order_data['custom1'] = 'PRIOR_AUTH_CAPTURE';
						$order_data['custom2'] = '';
						$order_data['custom3'] = $custom3;
						$order_data['payment_status'] = $payment_status;

						$user = new tmp_object();
						$user->id = $order_data["uid"];
						
						$order = new tmp_object();
						$order->oid = $order_data["oid"];
						$order->subtotalAmount = $order_data["subtotal_amount"];
						$order->totalAmount = $order_data["total_amount"];
						$order->shippingAmount = $order_data["shipping_amount"];
						$order->taxAmount = $order_data["tax_amount"];
						$order->gift_cert_amount = $order_data["gift_cert_amount"];
						
						$custom1 = $order_data['custom1'];
						$custom2 = $order_data['custom2'];
						$custom3 = $custom3;
						$security_id = $transaction_id;
						
						$this->createTransaction($db, $user, $order, 
							"ORDER TYPE: PRIOR_AUTH_CAPTURE\n\n".array2text($response),
							"",
							$custom1,
							$custom2,
							$custom3,
							$security_id
						);

						return true;
					}
					else
					{
						$this->is_error = true;

						$this->storeTransaction($db, $user, $order, $response, '', false);

						$this->error_message = $response['RESPMSG'];
						return false;
					}
				}
			}
			return !$this->is_error;
		}
		
		function refundableAmount($order_data=false)
		{
			if ($order_data)
			{
				if ($this->supportsRefund($order_data))
				{
					return $this->_refundableAmount($order_data);
				}
			}
			return 0;
		}
		
		function _refundableAmount($order_data)
		{
			$refundableAmount = $this->getTotalCaptured($order_data);
			return number_format($refundableAmount, 2, '.', '');
		}

		function refund($order_data=false,$refund_amount=false)
		{
			global $settings, $db;
						
			$this->is_error = false;

			if ($order_data)
			{
				$order_id = $order_data['order_num'];

				if (!$this->supportsRefund($order_data))
				{
					$this->is_error = true;
					$this->error_message = "Order cannot be refunded";
					return false;
				}
				
				//get captured total amount & x_trans_id
				$refund_total_amount = $refund_amount;
				
				if (!$refund_total_amount)
				{
					$refund_total_amount = $order_data['custom3'];
				}

				if ($refund_total_amount < 0)
				{
					$this->is_error = true;
					$this->error_message = "Refund amount must be greater than 0";
					return false;
				}
				else if ($refund_total_amount > $order_data['total_amount'])
				{
					$this->is_error = true;
					$this->error_message = "Refund amount cannot be more than total amount";
					return false;
				}
				else if ($refund_total_amount > $order_data['custom3'])
				{
					$this->is_error = true;
					$this->error_message = 'Refund amount cannot be more than total captured amount';
					return false;
				}
				
				$transaction_id = $order_data["security_id"];

				if ($transaction_id == '')
				{
					$this->is_error = true;
					$this->error_message = 'This order does not have a valid transaction id';
					return false;
				}
				else
				{
					$currencyCode = $this->settings_vars[$this->id."_Currency_Code"]["value"];
					$client = $this->getClient();

					$response = $client->doCredit($transaction_id, $refund_total_amount);
					
					$this->log("refund Response:\n".array2text($response));

					if (isset($response['RESULT']) && $response['RESULT'] == '0')
					{
						$totalCaptured = $this->getTotalCaptured($order_data) - $refund_total_amount;
						$custom3 = number_format($totalCaptured, 2, '.', '');

						$db->reset();
						$db->assignStr("custom1", "");
						$db->assignStr("custom2", "");
						$db->assignStr("custom3", $custom3);
						$payment_status = 'Refunded';

						$db->assignStr("payment_status", $payment_status);
						$db->update(DB_PREFIX."orders", "WHERE oid='".$order_data['oid']."'");

						$order_data['custom1'] = 'REFUND';
						$order_data['custom2'] = '';
						$order_data['custom3'] = $custom3;
						$order_data['payment_status'] = $payment_status;

						$user = new tmp_object();
						$user->id = $order_data["uid"];
						
						$order = new tmp_object();
						$order->oid = $order_data["oid"];
						$order->subtotalAmount = $order_data["subtotal_amount"];
						$order->totalAmount = $order_data["total_amount"];
						$order->shippingAmount = $order_data["shipping_amount"];
						$order->taxAmount = $order_data["tax_amount"];
						$order->gift_cert_amount = $order_data["gift_cert_amount"];
						
						$custom1 = $order_data['custom1'];
						$custom2 = $order_data['custom2'];
						$custom3 = $custom3;
						$security_id = $transaction_id;
						
						$this->createTransaction($db, $user, $order, 
							"ORDER TYPE: REFUND\n\n".array2text($response),
							"",
							$custom1,
							$custom2,
							$custom3,
							$security_id
						);

						return true;
					}
					else
					{
						$this->is_error = true;

						$this->storeTransaction($db, $user, $order, $response, '', false);

						$this->error_message = $response['RESPMSG'];
						return false;
					}
				}
			}
			return !$this->is_error;
		}

		function void($order_data=false)
		{
			global $settings, $db;
			
			$this->is_error = false;

			if ($order_data)
			{
				$order_id = $order_data['order_num'];
				$client = $this->getClient();
				
				$total_amount = $order_data['total_amount'];
				$transaction_id = $order_data["security_id"];
				
				$response = $client->doVoid($transaction_id);

				$this->log("void Response:\n".array2text($response));
				
				if (isset($response['RESULT']) && $response['RESULT'] == '0')
				{
					$db->reset();
					$db->assignStr("custom1", "");
					$db->assignStr("custom2", "");
					$db->assignStr("custom3", '');
					
					$payment_status = 'Canceled';
					$status = 'Canceled';

					$db->assignStr("payment_status", $payment_status);
					$db->assignStr('status', $status);
					$db->update(DB_PREFIX."orders", "WHERE oid='".$order_data['oid']."'");

					$order_data['custom1'] = 'CANCELED';
					$order_data['custom2'] = '';
					$order_data['custom3'] = '';
					$order_data['payment_status'] = $payment_status;
					$order_data['status'] = $status;
					
					$this->createTransaction($db, $user, $order, 
						"ORDER TYPE: REVERSAL\n\n".array2text($response),
						"",
						$custom1,
						$custom2,
						$custom3,
						$transaction_id
					);
					return true;
				}
				else
				{
					$this->is_error = true;

					$this->storeTransaction($db, $user, $order, $response, '', false);

					$this->error_message = $response['RESPMSG'];
					return false;
				}
			}
			return !$this->is_error;
		}
	}
