<?php 
	$payment_processor_id = "paypalpro";
	$payment_processor_name = "PayPal Website Payments Pro";
	$payment_processor_version = "3.3";
	$payment_processor_class = "PAYMENT_PAYPALPRO";
	$payment_processor_type = "cc";
	
	class PAYMENT_PAYPALPRO extends PAYMENT_PROCESSOR
	{
		function PAYMENT_PAYPALPRO()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "paypalpro";
			$this->name = "PayPal Website Payments Pro";
			$this->version = "3.3";
			$this->class_name = "PAYMENT_PAYPALPRO";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->is_certificate = true;
			$this->certificate_path = "paypal/certificate.pem";
			$this->need_cc_codes = true;
			$this->support_ccs = true;
			$this->need_cc_codes = true;
			return $this;
		}
		
		/**
		 * Returns payment form fields
		 */
		function getPaymentForm($db)
		{
			global $settings;
			$fields = parent::getPaymentForm($db);
			if ($settings["cardinalcommerce_Enabled"] == "Yes" && $this->settings_vars[$this->id."_Enable_Cardinal_Centinel"]["value"] == "Yes")
			{
				$fields["do_cmpi_lookup"] = array("type" => "hidden", "name"=> "do_cmpi_lookup", "value" => 1);
			}
			return $fields;
		}
		
		/**
		 * 
		 */
		function isTestMode()
		{
			//if($this->settings_vars[$this->id."_Test_Request"]["value"] == "TRUE"){
			//	$this->testMode = true;
			//}
			//else{
				$this->testMode = false;
			//}
			return $this->testMode;
		}
		
		/**
		 * Process paypal transaction
		 * @param $db
		 * @param $user
		 * @param $order
		 * @param $post_form
		 * @param $EciFlag
		 * @param $Cavv
		 * @return unknown_type
		 */
		function processPayPal($db, $user, $order, $post_form, $cardinalData = false)
		{
			global $settings;
			
			/**
			 * Process paypal
			 */
			$paypal_currency = $this->settings_vars[$this->id."_code"]["value"];
			if (isset($post_form) && is_array($post_form))
			{
				
				$cardinalFieldsXML = "";
				if ($cardinalData)
				{
					foreach($cardinalData as $fieldName=>$fieldValue)
					{
						$cardinalFieldsXML .= ('<'.$fieldName.'>'.utf8_encode($fieldValue).'</'.$fieldName.'>');
					}
					if ($cardinalFieldsXML != "")
					{
						$cardinalFieldsXML = ''.$cardinalFieldsXML.'';
					}
				}
				
				//build post data for query
				$post_data = "";
				//get post_form, common, settings and custom vars
				$post_data = 
					'<?xml version="1.0" encoding="UTF-8"?>'.
					'<SOAP-ENV:Envelope '.
						'xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" '.
						'xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" '.
						'xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" '.
						'xmlns:xsd="http://www.w3.org/1999/XMLSchema" '.
						'SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'.
					'<SOAP-ENV:Header>'.
						'<RequesterCredentials xmlns="urn:ebay:api:PayPalAPI" SOAP-ENV:mustUnderstand="1">'.
							'<Credentials xmlns="urn:ebay:apis:eBLBaseComponents">'.
								'<Username>'.utf8_encode($this->settings_vars[$this->id."_Username"]["value"]).'</Username>'.
								'<Password>'.utf8_encode($this->settings_vars[$this->id."_Password"]["value"]).'</Password>'.
								'<Subject></Subject>'.
							'</Credentials>'.
						'</RequesterCredentials>'.
					'</SOAP-ENV:Header>'.
					'<SOAP-ENV:Body>'.
						'<DoDirectPaymentReq xmlns="urn:ebay:api:PayPalAPI">'.
							'<DoDirectPaymentRequest xmlns="urn:ebay:api:PayPalAPI">'.
								'<Version xmlns="urn:ebay:apis:eBLBaseComponents">60.0</Version>'.
								'<DoDirectPaymentRequestDetails xmlns="urn:ebay:apis:eBLBaseComponents">'.
									$cardinalFieldsXML.
									'<PaymentAction>'.$this->settings_vars[$this->id."_Payment_Action"]["value"].'</PaymentAction>'.
									'<PaymentDetails>';
									
									$order_total = $this->common_vars["order_total_amount"]["value"];
									$shipping_total = $this->common_vars["order_shipping_amount"]["value"];
									$handling_total = $this->common_vars["order_handling_separated"]=="yes" ? $this->common_vars["order_handling_amount"]["value"] : 0.00;
									$tax_total = $this->common_vars["order_tax_amount"]["value"];
									$item_total = $order_total - $shipping_total - $tax_total - $handling_total;
									
									if ($this->common_vars["shipping_country_iso_a2"]["value"]=="US" || $this->common_vars["shipping_country_iso_a2"]["value"]=="CA")
										$shipping_state = "shipping_state_abbr";
									else
										$shipping_state = "shipping_state";
									
									if (strlen(trim($this->common_vars["shipping_name"]["value"]))>0)
									{
										$shipping_data = '<ShipToAddress>'.
											'<Name>'.utf8_encode($this->common_vars["shipping_name"]["value"]).'</Name>'.
											'<Street1>'.utf8_encode($this->common_vars["shipping_address1"]["value"]).'</Street1>'.
											'<Street2>'.utf8_encode($this->common_vars["shipping_address2"]["value"]).'</Street2>'.
											'<CityName>'.utf8_encode($this->common_vars["shipping_city"]["value"]).'</CityName>'.
											// '<StateOrProvince>'.utf8_encode($this->common_vars["shipping_state".($this->common_vars["shipping_country_iso_a2"]["value"]=="US"?"_abbr":"")]["value"]).'</StateOrProvince>'.
											'<StateOrProvince>'.utf8_encode($this->common_vars[$shipping_state]["value"]).'</StateOrProvince>'.
											'<Country>'.utf8_encode($this->common_vars["shipping_country_iso_a2"]["value"]).'</Country>'.
											'<Phone></Phone>'.
											'<PostalCode>'.utf8_encode($this->common_vars["shipping_zip"]["value"]).'</PostalCode>'.
										'</ShipToAddress>';
									}
									else
									{
										$shipping_data = '';
									}
									
									
									if ($this->common_vars["billing_country_iso_a2"]["value"]=="US" || $this->common_vars["billing_country_iso_a2"]["value"]=="CA")
										$b_state = "billing_state_abbr";
									else
										$b_state = "billing_state";
										
									
									$post_data = $post_data.
										'<OrderTotal currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($order_total, 2, ".", "")).'</OrderTotal>'.
										// '<ItemTotal currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($item_total, 2, ".", "")).'</ItemTotal>'.
										// '<ShippingTotal currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($shipping_total, 2, ".", "")).'</ShippingTotal>'.
										// '<HandlingTotal currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($handling_total, 2, ".", "")).'</HandlingTotal>'.
										// '<TaxTotal currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($tax_total, 2, ".", "")).'</TaxTotal>'.
										'<OrderDescription>Order #'.utf8_encode($this->common_vars["order_id"]["value"]).'</OrderDescription>'.
										'<Custom>xxxx_custom</Custom>'.
										'<InvoiceID>'.utf8_encode($this->common_vars["order_id"]["value"]).'</InvoiceID>'.
    	  									$shipping_data.
										'<PaymentItem>'.
											'<Name>Order #'.utf8_encode($this->common_vars["order_id"]["value"]).'</Name>'.
											'<Number>'.utf8_encode($this->common_vars["order_id"]["value"]).'</Number>'.
											'<Quantity>'.utf8_encode('1').'</Quantity>'.
											'<SalesTax currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($this->common_vars["order_tax_amount"]["value"], 5, ".", "")).'</SalesTax>'.
											'<Amount currencyID="'.utf8_encode($paypal_currency).'">'.utf8_encode(number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "")).'</Amount>'.
										'</PaymentItem>'.
										'<ButtonSource>PCart_cart_DCC_US</ButtonSource>'.
									'</PaymentDetails>'.
									'<CreditCard>'.
										'<CreditCardType>'.utf8_encode($post_form["cc_type"]).'</CreditCardType>'.
										'<CreditCardNumber>'.utf8_encode($post_form["cc_number"]).'</CreditCardNumber>'.
										'<ExpMonth>'.utf8_encode($post_form["cc_expiration_month"]).'</ExpMonth>'.
										'<ExpYear>'.utf8_encode($post_form["cc_expiration_year"]).'</ExpYear>'.
										'<CardOwner>'.
											'<Payer>'.utf8_encode($this->common_vars["billing_email"]["value"]).'</Payer>'.
											'<PayerID>'.$this->common_vars["customer_id"]["value"].'</PayerID>'.
											'<PayerStatus>'.utf8_encode('unverified').'</PayerStatus>'.
											'<PayerName>'.
												'<Salutation></Salutation>'.
												'<FirstName>'.utf8_encode($post_form["cc_first_name"]).'</FirstName>'.
												'<MiddleName></MiddleName>'.
												'<LastName>'.utf8_encode($post_form["cc_last_name"]).'</LastName>'.
												'<Suffix></Suffix>'.
											'</PayerName>'.
											'<PayerCountry>'.utf8_encode($this->common_vars["billing_country"]["value"]).'</PayerCountry>'.
											'<PayerBusiness>xxxx_payer_business</PayerBusiness>'.
											'<Address>'.
												'<Name>'.utf8_encode($post_form["cc_first_name"].' '.$post_form["cc_last_name"]).'</Name>'.
												'<Street1>'.utf8_encode($this->common_vars["billing_address1"]["value"]).'</Street1>'.
												'<Street2>'.utf8_encode($this->common_vars["billing_address2"]["value"]).'</Street2>'.
												'<CityName>'.utf8_encode($this->common_vars["billing_city"]["value"]).'</CityName>'.
												// '<StateOrProvince>'.utf8_encode($this->common_vars["billing_state".($this->common_vars["billing_country_iso_a2"]["value"]=="US"?"_abbr":"")]["value"]).'</StateOrProvince>'.
												'<StateOrProvince>'.utf8_encode($this->common_vars[$b_state]["value"]).'</StateOrProvince>'.
												'<Country>'.utf8_encode($this->common_vars["billing_country_iso_a2"]["value"]).'</Country>'.
												'<CountryName>'.utf8_encode($this->common_vars["billing_country"]["value"]).'</CountryName>'.
												'<Phone>'.utf8_encode($this->common_vars["billing_phone"]["value"]).'</Phone>'.
												'<PostalCode>'.utf8_encode($this->common_vars["billing_zip"]["value"]).'</PostalCode>'.
												'<AddressID></AddressID>'.
												'<AddressOwner></AddressOwner>'.
												'<ExternalAddressID></ExternalAddressID>'.
												'<InternationalName></InternationalName>'.
												'<InternationalStateAndCity></InternationalStateAndCity>'.
												'<InternationalStreet></InternationalStreet>'.
												'<AddressStatus></AddressStatus>'.
											'</Address>'.
										'</CardOwner>'.
										'<CVV2>'.utf8_encode($post_form["cc_cvv2"]).'</CVV2>'.
										$cardinalFieldsXML.
									'</CreditCard>'.
									'<IPAddress>'.utf8_encode($_SERVER["REMOTE_ADDR"]).'</IPAddress>'.
									'<MerchantSessionId>'.utf8_encode(session_id()).'</MerchantSessionId>'.
								'</DoDirectPaymentRequestDetails>'.
							'</DoDirectPaymentRequest>'.
						'</DoDirectPaymentReq>'.
					'</SOAP-ENV:Body>'.
					'</SOAP-ENV:Envelope>';
				
				$_post_url = trim($this->url_to_gateway)==""?"https://api.paypal.com/2.0/":trim($this->url_to_gateway);
				
				$_cert_path = $settings["GlobalServerPath"]."/content/engine/payment/".$this->certificate_path;
							
				$c = curl_init();
				
				if ($settings["ProxyAvailable"] == "YES")
				{
					//curl_setopt($c, CURLOPT_VERBOSE, 1);
					if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
					{
						curl_setopt($c, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
					}
					curl_setopt($c, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
					if ($settings["ProxyRequiresAuthorization"] == "YES")
					{
						curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
					}

					curl_setopt($c, CURLOPT_TIMEOUT, 120);
				}
				
				curl_setopt($c, CURLOPT_URL, $_post_url);
				curl_setopt($c, CURLOPT_SSLCERT, $_cert_path);

				curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
				curl_setopt($c, CURLOPT_POSTFIELDS, $post_data);
				curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
				
				$this->log($post_data);

				if (!ini_get('safe_mode')) set_time_limit(3000);
				curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 1);
				curl_setopt($c, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
				curl_setopt($c, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
				$buffer = curl_exec($c);
				
				$this->log($buffer);
				//echo htmlspecialchars($post_data);
				//die();
				
				if ($buffer)
				{
					$this->logStart();
					$this->log($buffer);
					$this->logFinish();
					
					$xmlParser = new XMLParser();
					
					$data = $xmlParser->parse($buffer);
					
					if (isset($data["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoDirectPaymentResponse"][0]))
					{
						
						$a = $data["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["DoDirectPaymentResponse"][0];
						if ($a["Ack"][0] == "Success" || $a["Ack"][0] == "SuccessWithWarnings" || $a["Ack"][0] == "SuccessWithWarning")
						{
							$response = "";
							$payment_status = "";
							$charged_amount = 0;
							foreach ($a as $var=>$value)
							{
								$response.= $var."  =  ".$value[0]."\n";
								if($var == "TransactionID")
									$transaction_id = $value[0];
							}
							if ($this->settings_vars[$this->id."_Payment_Action"]["value"] == "Sale")
							{										
								$charged_amount = $this->common_vars["order_total_amount"]["value"];												
							}
							$custom1 = strtolower($this->settings_vars[$this->id."_Payment_Action"]["value"]);
							$this->createTransaction($db, $user, $order, $response, "", $custom1, "", $charged_amount, $transaction_id);
							if ($this->support_ccs && $this->enable_ccs)
							{
								$card_data = array(
									"fn"=>$post_form["cc_first_name"],
									"ln"=>$post_form["cc_last_name"],
									"ct"=>$post_form["cc_type"],
									"cc"=>$post_form["cc_number"],				
									"em"=>$post_form["cc_expiration_month"], 
									"ey"=>$post_form["cc_expiration_year"]
								);
								$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
							}
							$this->is_error = false;
							if ($this->settings_vars[$this->id."_Payment_Action"]["value"] == "Authorization" )
							{
								$payment_status = "Pending";
								return $payment_status;
							}
						}
						elseif (trim($a["Ack"][0]) == "Failure")
						{
							$this->is_error = true;
							$this->error_message = $a["Errors"][0]["ShortMessage"][0]." ".$a["Errors"][0]["LongMessage"][0];
						}
						else
						{
							$this->is_error = true;
							$this->error_message = PAYMENT_TRANSACTION_ERROR_TEXT;
						}
					}
					elseif (isset($data["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["SOAP-ENV:Fault"][0]["faultcode"][0]))
					{
						$this->error_message = $data["SOAP-ENV:Envelope"][0]["SOAP-ENV:Body"][0]["SOAP-ENV:Fault"][0]["faultstring"][0];
						$this->is_error = true;
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = PAYMENT_SERVER_ERROR_TEXT;
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administartor.";
			}
			return !$this->is_error;
		}
		
		/**
		 * Process transactoin
		 * @param $db
		 * @param $user
		 * @param $order
		 * @param $post_form
		 * @return boolean
		 */
		function process($db, $user, $order, $post_form)
		{
			global $settings;
			global $backurl;
			
			//check cardinal centinel is enabled
			if ($settings["cardinalcommerce_Enabled"] == "Yes" && $this->settings_vars[$this->id."_Enable_Cardinal_Centinel"]["value"] == "Yes")
			{
				//include module
				require_once("content/engine/engine_cardinal_commerce.php");
				//create object and set it up
				$cardinal = new CardinalCommerce();
				$cardinal->version ="1.7";
				$cardinal->common_vars = $this->common_vars;
				if (isset($_REQUEST["do_cmpi_lookup"]))
				{
					$cardinal_lookup_xml = $cardinal->getLookupXML(
						$post_form["cc_first_name"],
						$post_form["cc_last_name"],	
						$post_form["cc_number"], 
						$post_form["cc_expiration_month"], 
						$post_form["cc_expiration_year"],
						$this->currency["code"]
					);
					
					$cardinal_lookup_result = $cardinal->processRequest($cardinal_lookup_xml);
					if ($cardinal_lookup_result)
					{
						if ($cardinal_lookup_result["CardinalMPI"][0]["Enrolled"][0] == "Y")
						{
							//change back url
							$backurl = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=payment_validation";
							
							$_SESSION["CardinalPostForm"] = $post_form;
							$_SESSION["CardinalTransactionId"] = $cardinal_lookup_result["CardinalMPI"][0]["TransactionId"][0];							
							$_SESSION["CardinalACSUrl"] = $cardinal_lookup_result["CardinalMPI"][0]["ACSUrl"][0];
							$_SESSION["CardinalPayload"] = $cardinal_lookup_result["CardinalMPI"][0]["Payload"][0];
							//validator will replace ORDER_PROCESS_PAYMENT_VALIDATION with ORDER_PROCESS_PAYMENT
							$_SESSION["CardinalNotifyUrl"] =($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?pcsid=".session_id()."&p=".$this->payment_page."&oa=".ORDER_PROCESS_PAYMENT_VALIDATION."&do_cmpi_authenticate=true";
							$_SESSION["CardinalRedirectUrl"] = ($this->protocol=="https"?$settings["GlobalHttpsUrl"]:$settings["GlobalHttpUrl"])."/".$settings["INDEX"];
							return;
						}
						else
						{
							//do not redirect, process simple paypal transaction
							$this->processPayPal($db, $user, $order, $post_form);
						}
					}
					else
					{
						$this->is_error = true; 
						$this->error_message = $cardinal->error_message;
					}
				}
				
				if (isset($_REQUEST["do_cmpi_authenticate"]))
				{
					$post_form = $_SESSION["CardinalPostForm"];
					unset($_SESSION["CardinalPostForm"]);
					$cardinal_auth_xml = $cardinal->getAuthenticateXML($_REQUEST["PaRes"]);
					$cardinal_auth_result = $cardinal->processRequest($cardinal_auth_xml);
					if ($cardinal_auth_result)
					{
						$EciFlag = $cardinal_auth_result["CardinalMPI"][0]["EciFlag"][0];
						$PAResStatus = $cardinal_auth_result["CardinalMPI"][0]["PAResStatus"][0];
						$Cavv = $cardinal_auth_result["CardinalMPI"][0]["Cavv"][0];
						$SignatureVerification = $cardinal_auth_result["CardinalMPI"][0]["SignatureVerification"][0];
						$Xid = $cardinal_auth_result["CardinalMPI"][0]["Xid"][0];
						$ErrorNo = $cardinal_auth_result["CardinalMPI"][0]["ErrorNo"][0];
						if ($SignatureVerification == "Y" && $PAResStatus != "N")
						{
							switch ($PAResStatus)
							{
								case "Y" : 
								case "A" : 
								{
									$cardinalData = array(
										"AuthStatus3ds" => $PAResStatus,
										"MpiVendor3ds" => "Y",
										"Cavv" => $Cavv,
										"Eci3ds" => $EciFlag,
										"Xid" => $Xid
									);	
									$this->processPayPal($db, $user, $order, $post_form, $cardinalData);
									break;
								}
								case "U" : 
								{
									$this->processPayPal($db, $user, $order, $post_form);
									break;
								}
							}
						}
						else 
						{
							$this->is_error = true;
							$this->error_message = "Your card can not be validated. Please try other card or other payment method";
						}
					}
					else
					{
						$this->is_error = true;
						$this->error_message = $cardinal->error_message;
					}
				}
			}
			else
			{
				$this->processPayPal($db, $user, $order, $post_form);
			}
			
			return !$this->is_error;
		}
	}
