<?php 

	require_once("content/engine/payment/securenet/class_gateway.php");

	$payment_processor_id = "secure_net";
	$payment_processor_name = "SecureNet";
	$payment_processor_class = "PAYMENT_SECURE_NET";
	$payment_processor_type = "cc";

	class PAYMENT_SECURE_NET extends PAYMENT_PROCESSOR
	{
		private $_gateway = null;
		/**
		 *
		 * @return PAYMENT_SECURE_NET 
		 */
		public function __construct()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "secure_net";
			$this->name = "SecureNet";
			$this->class_name = "PAYMENT_SECURE_NET";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->support_ccs = true;
			$this->need_cc_codes = true;
			
			$this->lastRequest = '';
			$this->lastResponse = '';
			
			/**
			 * Create gateway class instance
			 */
			$this->_gateway = new SecureNetGateway();
			
			return $this;
		}

		public function replaceDataKeys()
		{
			return array('CARD.CARDNUMBER', 'CARD.CARDCODE', 'MERCHANT_KEY.GROUPID', 'MERCHANT_KEY.SECURENETID', 'MERCHANT_KEY.SECUREKEY');
		}

		/**
		 *
		 * @global type $settings
		 * @param type $db
		 * @return string 
		 */
		public function getPaymentForm($db)
		{
			global $settings;
			$fields = parent::getPaymentForm($db);
			if ($settings["cardinalcommerce_Enabled"] == "Yes" && $this->settings_vars[$this->id."_Enable_Cardinal_Centinel"]["value"] == "Yes")
			{
				$fields["do_cmpi_lookup"] = array("type" => "hidden", "name"=> "do_cmpi_lookup", "value" => 1);
			}
			return $fields;
		}

		/**
		 *
		 * @return type 
		 */
		public function isTestMode()
		{
			if (strtoupper($this->settings_vars[$this->id."_Test_Request"]["value"]) == "TRUE")
			{
				$this->testMode = true;
			}
			else
			{
				$this->testMode = false;
			}
			return $this->testMode;
		}

		/**
		 * Send request for processing
		 * @param array $request
		 * @return mixed
		 */
		private function _process($request)
		{
			/**
			 * Set URL
			 */
			if (trim($this->url_to_gateway) == "")
			{
				$this->_gateway->set_base_url('https://gateway.securenet.com/API/Gateway.svc/webHttp');
			}
			else
			{
				$this->_gateway->set_base_url($this->url_to_gateway);
			}

			/**
			 * Process transaction
			 */
			$response = $this->_gateway->ProcessTransaction($request);

			$this->log("_process Request:", $request);

			$this->lastRequest = $request;

			/**
			 * Hnadle response
			 */
			if ($response)
			{
				//parse response
				$xml = xml2array(@simplexml_load_string($response));
				$this->log("_process Response:", is_array($xml) ? $xml : array($xml));

				$this->lastResponse = $xml;

				if (isset($xml['TRANSACTIONRESPONSE']['RESPONSE_CODE']))
				{
					if ($xml['TRANSACTIONRESPONSE']['RESPONSE_CODE'] == '1')
					{
						return $xml;
					}
					else
					{
						$this->is_error = true;
						$this->error_message = 
							$xml['TRANSACTIONRESPONSE']['RESPONSE_REASON_TEXT'].
							' (Reason code: '.$xml['TRANSACTIONRESPONSE']['RESPONSE_REASON_CODE'].')';
					}
				}
				else
				{
					$this->is_error = true;
					$this->error_message = $this->_gateway->error_message;
				}
			}

			return false;
		}

		/**
		 * Transforms SecureNet datetime to unix timestamp
		 * @param string $datetime
		 * @return timestamp
		 */
		private function _dateTimeToTimestamp($datetime)
		{
			//mmddyyyyhhmmss
			$mm = intval(substr($datetime, 0, 2)); // month
			$dd = intval(substr($datetime, 2, 2)); // day
			$yyyy = intval(substr($datetime, 4, 4)); // year
			$hh = intval(substr($datetime, 8, 2)); // hour
			$ii = intval(substr($datetime, 10, 2)); // minute
			$ss = intval(substr($datetime, 12, 2)); // second
			
			return mktime($hh, $ii, $ss, $mm, $dd, $yyyy);
		}

		/**
		 * 
		 * @global type $settings
		 * @global type $backurl
		 * @param type $db
		 * @param type $user
		 * @param type $order
		 * @param string $post_form
		 * @return type 
		 */
		public function process($db, $user, $order, $post_form)
		{
			$request = array();

			$request['AMOUNT'] = number_format($this->common_vars["order_total_amount"]["value"], 2, ".", "");

			$group_id = trim($this->settings_vars[$this->id."_Group_Id"]["value"]);
			$request['MERCHANT_KEY']['GROUPID'] = (trim($group_id) != '') ? $group_id : 0;
			$request['MERCHANT_KEY']['SECURENETID'] = $this->settings_vars[$this->id."_SecureNet_Id"]["value"];
			$request['MERCHANT_KEY']['SECUREKEY'] = $this->settings_vars[$this->id."_SecureNet_Key"]["value"];

			$request['CODE'] = $this->settings_vars[$this->id."_Auth_Type"]["value"] == "Auth-Capture" ? "0100" : "0000"; //"AUTH_CAPTURE" : "AUTH_ONLY";
			$request['METHOD'] = "CC";
			$request['CARD']['CARDCODE'] = $post_form["cc_cvv2"];
			$request['CARD']['CARDNUMBER'] = $post_form["cc_number"];
			$request['CARD']['EXPDATE'] = $post_form["cc_expiration_month"].substr($post_form["cc_expiration_year"], 2, 2);

			$request['ORDERID'] = $this->common_vars["order_id"]["value"];
			$request['DCI'] = 0;
			$request['TEST'] = $this->settings_vars[$this->id."_Test_Request"]["value"];
			$request['TRANSACTION_SERVICE'] = 0;
			$request['CUSTOMERID'] = $this->common_vars["customer_id"]["value"];
			$request['PAYMENTID'] = $this->common_vars["order_id"]["value"];
			$request['CUSTOMERIP'] = $_SERVER['REMOTE_ADDR'];
			$request['INVOICEDESC'] = '#'.$this->common_vars["order_id"]["value"];
			$request['INVOICENUM'] = urlencode($this->common_vars["order_id"]["value"]);

			$request['INSTALLMENT_SEQUENCENUM'] = 0;
			$request['OVERRIDE_FROM'] = 0;
			$request['RETAIL_LANENUM'] = 0;
			$request['TOTAL_INSTALLMENTCOUNT'] = 0;
			$request['INDUSTRYSPECIFICDATA'] = $order->shippingDigital ? "D" : "P";

			$request['NOTE'] = '';
			$request['CUSTOMER_BILL']['ADDRESS'] = $this->common_vars["billing_address"]["value"];
			$request['CUSTOMER_BILL']['CITY'] = $this->common_vars["billing_city"]["value"];
			$request['CUSTOMER_BILL']['ZIP'] = $this->common_vars["billing_zip"]["value"];
			$request['CUSTOMER_BILL']['STATE'] = $this->common_vars["billing_state"]["value"];
			$request['CUSTOMER_BILL']['COMPANY'] = $this->common_vars["billing_company"]["value"];
			$request['CUSTOMER_BILL']['COUNTRY'] = $this->common_vars["billing_country"]["value"];
			$request['CUSTOMER_BILL']['EMAIL'] = $this->common_vars["billing_email"]["value"];
			$request['CUSTOMER_BILL']['EMAILRECEIPT'] = 'TRUE';
			$request['CUSTOMER_BILL']['FIRSTNAME'] = $post_form["cc_first_name"];
			$request['CUSTOMER_BILL']['LASTNAME'] = $post_form["cc_last_name"];
			$request['CUSTOMER_BILL']['PHONE'] = $this->common_vars["billing_phone"]["value"];

			//shipping data
			if (!$this->common_vars["shipping_digital"]["value"])
			{
				$request['CUSTOMER_SHIP']['ADDRESS'] = $this->common_vars["shipping_address"]["value"];
				$request['CUSTOMER_SHIP']['CITY'] = $this->common_vars["shipping_city"]["value"];
				$request['CUSTOMER_SHIP']['COMPANY'] = '';
				$request['CUSTOMER_SHIP']['COUNTRY'] = $this->common_vars["shipping_country"]["value"];
				$request['CUSTOMER_SHIP']['FIRSTNAME'] = '-';
				$request['CUSTOMER_SHIP']['LASTNAME'] = $this->common_vars["shipping_name"]["value"];
				$request['CUSTOMER_SHIP']['STATE'] = $this->common_vars["shipping_state"]["value"];
				$request['CUSTOMER_SHIP']['ZIP'] = $this->common_vars["shipping_zip"]["value"];
			}

			$request['USERDEFINED'] = array();

			$response = $this->_process($request);

			if ($response)
			{
				$custom1 = array(
					"AUTHCODE" => $response['TRANSACTIONRESPONSE']['AUTHCODE'],
					"CODE" => $response['TRANSACTIONRESPONSE']['CODE'],
					"TRANSACTIONID" => $response['TRANSACTIONRESPONSE']['TRANSACTIONID'],
					"TRANSACTIONAMOUNT" => $response['TRANSACTIONRESPONSE']['TRANSACTIONAMOUNT'],
					"SETTLEMENTAMOUNT" => $response['TRANSACTIONRESPONSE']['SETTLEMENTAMOUNT'],
					"SETTLEMENTDATETIME" => $response['TRANSACTIONRESPONSE']['SETTLEMENTDATETIME'],
					"CARD" => array(
						"LAST4DIGITS" => $response['TRANSACTIONRESPONSE']['LAST4DIGITS'],
						"CARDCODE" => $request['CARD']['CARDCODE'],
						"EXPDATE" => $request['CARD']['EXPDATE']
					)
				);

				$this->createTransaction($db, $user, $order, $this->settings_vars[$this->id."_Auth_Type"]["value"]." Transaction\n".array2text($response), '', serialize($custom1));
				$this->is_error = false;
				$this->error_message = "";
				return $this->settings_vars[$this->id."_Auth_Type"]["value"] == "Auth-Capture" ? "Received" : "Pending";
			}
			else
			{
				$this->storeTransaction($db, $user, $order, $this->lastResponse, '', false);
			}

			return false;
		}

		/**
		 * Returns current order payment status
		 * @param string $order_data 
		 */
		public function currentOrderPaymentStatus($order_data = array())
		{
			$custom1 = unserialize($order_data["custom1"]);
			switch($custom1["CODE"])
			{
				case '0000' : return 'Authorized';
				case '0100' : return 'Captured';
				case '0200' : return 'Captured';
				case '0400' : return 'Void';
			}
			return '';
		}

		/**
		 * Check is there capture supported.
		 * Capture is supported when current status is 0000 (AUTH_ONLY) and transaction is not older then 30 days
		 * @param type $order_data
		 * @return type 
		 */
		public function supportsCapture($order_data = array())
		{
			$custom1 = unserialize($order_data["custom1"]);
			$settlement_date_time = $this->_dateTimeToTimestamp($custom1["SETTLEMENTDATETIME"]);
			return $custom1["CODE"] == "0000" && ($settlement_date_time + 60*60*24*30 > time());
		}

		/**
		 * Check is there capture supported
		 * Void is supporter when current status is 0000 (AUTH_ONLY), 0100 (AUTH_CAPTURE), 0500 (CREDIT/REFUND)
		 * @param type $order_data
		 * @return type 
		 */
		public function supportsVoid($order_data = array())
		{
			$custom1 = unserialize($order_data["custom1"]);
			return in_array($custom1["CODE"], array("0000", "0100", "0500", "0200"));
		}

		/**
		 * Check is there refund supported
		 * @param type $order_data
		 * @return type 
		 */
		public function supportsRefund($order_data = array())
		{
			$custom1 = unserialize($order_data["custom1"]);
			return in_array($custom1["CODE"], array("0100", "0200"));
		}

		/**
		 * Returns amount that may be refunded
		 * @param type $order_data 
		 */
		public function refundableAmount($order_data = false)
		{
			$custom1 = unserialize($order_data["custom1"]);
			return $custom1["SETTLEMENTAMOUNT"];
		}

		/**
		 * Returns amount that may be captured
		 * @param type $order_data 
		 */
		public function capturableAmount($order_data = false)
		{
			$custom1 = unserialize($order_data["custom1"]);
			return $custom1["SETTLEMENTAMOUNT"];
		}

		/**
		 * Do capture
		 * @param type $order_data
		 * @param type $capture_amount 
		 */
		public function capture($order_data = false, $capture_amount = false)
		{
			global $db;

			$custom1 = unserialize($order_data["custom1"]);

			// limit amount to be captured
			$settlement_amount = $custom1['SETTLEMENTAMOUNT'];
			$amount = $capture_amount > $settlement_amount ? $settlement_amount : $capture_amount;
			$group_id = trim($this->settings_vars[$this->id."_Group_Id"]["value"]);

			$request = array();
			$request['AMOUNT'] = number_format($amount, 2, ".", "");
			$request['MERCHANT_KEY']['GROUPID'] = (trim($group_id) != '') ? $group_id : 0;
			$request['MERCHANT_KEY']['SECURENETID'] = $this->settings_vars[$this->id."_SecureNet_Id"]["value"];
			$request['MERCHANT_KEY']['SECUREKEY'] = $this->settings_vars[$this->id."_SecureNet_Key"]["value"];
			$request['CODE'] = '0200';
			$request['METHOD'] = 'CC';

			$request['CARD']['CARDNUMBER'] = $custom1['CARD']['LAST4DIGITS'];
			$request['CARD']['CARDCODE'] = $custom1['CARD']['CARDCODE'];
			$request['CARD']['EXPDATE'] = $custom1['CARD']['EXPDATE'];

			$request['ORDERID'] = $order_data['order_num'];
			$request['DCI'] = 0;
			$request['TEST'] = $this->settings_vars[$this->id."_Test_Request"]["value"];
			$request['TRANSACTION_SERVICE'] = 0;
			$request['REF_TRANSID'] = $custom1['TRANSACTIONID'];
			$request['CUSTOMERID'] = $order_data['uid'];
			$request['PAYMENTID'] = $order_data['uid'];
			$request['INSTALLMENT_SEQUENCENUM'] = 0;
			$request['OVERRIDE_FROM'] = 0;
			$request['RETAIL_LANENUM'] = 0;
			$request['TOTAL_INSTALLMENTCOUNT'] = 0;

			$response = $this->_process($request);

			if ($response)
			{
				$custom1["AUTHCODE"] = $response['TRANSACTIONRESPONSE']['AUTHCODE'];
				$custom1["CODE"] = $response['TRANSACTIONRESPONSE']['CODE'];
				$custom1["TRANSACTIONID"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONID'];
				$custom1["TRANSACTIONAMOUNT"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONAMOUNT'];
				$custom1["SETTLEMENTAMOUNT"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTAMOUNT'];
				$custom1["SETTLEMENTDATETIME"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTDATETIME'];

				$user = new tmp_object();
				$user->id = $order_data['uid'];

				$order = new tmp_object();
				$order->oid = $order_data['oid'];
				$order->subtotalAmount = $order_data['subtotal_amount'];
				$order->totalAmount = $order_data['total_amount'];
				$order->gift_cert_amount = $order_data['gift_cert_amount'];
				$order->shippingAmount = $order_data['shipping_amount'];
				$order->taxAmount = $order_data['tax_amount'];

				$this->createTransaction($db, $user, $order, "Capture Transaction\n".array2text($response), '', serialize($custom1));

				$this->is_error = false;
				$this->error_message = "";
				return true;
			}

			return false;
		}

		/**
		 * Do capture
		 * @param type $order_data
		 * @param type $capture_amount 
		 */
		public function refund($order_data = false, $refund_amount = false)
		{
			global $db;

			$custom1 = unserialize($order_data["custom1"]);
			$group_id = trim($this->settings_vars[$this->id."_Group_Id"]["value"]);
			$settlement_amount = $custom1['SETTLEMENTAMOUNT'];
			$amount = $refund_amount > $settlement_amount? $settlement_amount : $refund_amount;

			$request = array();

			$request['AMOUNT'] = number_format($amount, 2, ".", "");
			$request['MERCHANT_KEY']['GROUPID'] = (trim($group_id) != '') ? $group_id : 0;
			$request['MERCHANT_KEY']['SECURENETID'] = $this->settings_vars[$this->id."_SecureNet_Id"]["value"];
			$request['MERCHANT_KEY']['SECUREKEY'] = $this->settings_vars[$this->id."_SecureNet_Key"]["value"];
			$request['CODE'] = '0500';
			$request['METHOD'] = 'CC';

			$request['CARD']['CARDNUMBER'] = $custom1['CARD']['LAST4DIGITS'];
			$request['CARD']['CARDCODE'] = $custom1['CARD']['CARDCODE'];
			$request['CARD']['EXPDATE'] = $custom1['CARD']['EXPDATE'];

			$request['ORDERID'] = $order_data['order_num'];
			$request['DCI'] = 0;
			$request['TEST'] = $this->settings_vars[$this->id."_Test_Request"]["value"];
			$request['TRANSACTION_SERVICE'] = 0;
			$request['REF_TRANSID'] = $custom1['TRANSACTIONID'];
			$request['CUSTOMERID'] = $order_data['uid'];
			$request['PAYMENTID'] = $order_data['uid'];
			$request['INSTALLMENT_SEQUENCENUM'] = 0;
			$request['OVERRIDE_FROM'] = 0;
			$request['RETAIL_LANENUM'] = 0;
			$request['TOTAL_INSTALLMENTCOUNT'] = 0;

			$response = $this->_process($request);

			if ($response)
			{
				$custom1["AUTHCODE"] = $response['TRANSACTIONRESPONSE']['AUTHCODE'];
				$custom1["CODE"] = $response['TRANSACTIONRESPONSE']['CODE'];
				$custom1["TRANSACTIONID"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONID'];
				$custom1["TRANSACTIONAMOUNT"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONAMOUNT'];
				$custom1["SETTLEMENTAMOUNT"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTAMOUNT'];
				$custom1["SETTLEMENTDATETIME"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTDATETIME'];

				$user = new tmp_object();
				$user->id = $order_data['uid'];

				$order = new tmp_object();
				$order->oid = $order_data['oid'];
				$order->subtotalAmount = $order_data['subtotal_amount'];
				$order->totalAmount = $order_data['total_amount'];
				$order->gift_cert_amount = $order_data['gift_cert_amount'];
				$order->shippingAmount = $order_data['shipping_amount'];
				$order->taxAmount = $order_data['tax_amount'];

				$this->createTransaction($db, $user, $order, "Refund Transaction\n".array2text($response), '', serialize($custom1));

				$this->is_error = false;
				$this->error_message = "";
				return true;
			}

			return false;
		}

		/**
		 * Do capture
		 * @param type $order_data
		 * @param type $capture_amount 
		 */
		public function void($order_data = false)
		{
			global $db;

			$custom1 = unserialize($order_data["custom1"]);

			$group_id = trim($this->settings_vars[$this->id."_Group_Id"]["value"]);

			$request = array();

			$request['AMOUNT'] = number_format($order_data['total_amount'], 2, ".", "");
			$request['MERCHANT_KEY']['GROUPID'] = (trim($group_id) != '') ? $group_id : 0;
			$request['MERCHANT_KEY']['SECURENETID'] = $this->settings_vars[$this->id."_SecureNet_Id"]["value"];
			$request['MERCHANT_KEY']['SECUREKEY'] = $this->settings_vars[$this->id."_SecureNet_Key"]["value"];
			$request['CODE'] = '0400';
			$request['METHOD'] = 'CC';

			$request['CARD']['CARDNUMBER'] = $custom1['CARD']['LAST4DIGITS'];
			$request['CARD']['CARDCODE'] = $custom1['CARD']['CARDCODE'];
			$request['CARD']['EXPDATE'] = $custom1['CARD']['EXPDATE'];

			$request['ORDERID'] = $order_data['order_num'];
			$request['DCI'] = 0;
			$request['TEST'] = $this->settings_vars[$this->id."_Test_Request"]["value"];
			$request['TRANSACTION_SERVICE'] = 0;
			$request['REF_TRANSID'] = $custom1['TRANSACTIONID'];
			$request['CUSTOMERID'] = $order_data['uid'];
			$request['PAYMENTID'] = $order_data['uid'];
			$request['INSTALLMENT_SEQUENCENUM'] = 0;
			$request['OVERRIDE_FROM'] = 0;
			$request['RETAIL_LANENUM'] = 0;
			$request['TOTAL_INSTALLMENTCOUNT'] = 0;

			$response = $this->_process($request);

			if ($response)
			{
				$custom1["AUTHCODE"] = $response['TRANSACTIONRESPONSE']['AUTHCODE'];
				$custom1["CODE"] = $response['TRANSACTIONRESPONSE']['CODE'];
				$custom1["TRANSACTIONID"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONID'];
				$custom1["TRANSACTIONAMOUNT"] = $response['TRANSACTIONRESPONSE']['TRANSACTIONAMOUNT'];
				$custom1["SETTLEMENTAMOUNT"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTAMOUNT'];
				$custom1["SETTLEMENTDATETIME"] = $response['TRANSACTIONRESPONSE']['SETTLEMENTDATETIME'];

				$user = new tmp_object();
				$user->id = $order_data['uid'];

				$order = new tmp_object();
				$order->oid = $order_data['oid'];
				$order->subtotalAmount = $order_data['subtotal_amount'];
				$order->totalAmount = $order_data['total_amount'];
				$order->gift_cert_amount = $order_data['gift_cert_amount'];
				$order->shippingAmount = $order_data['shipping_amount'];
				$order->taxAmount = $order_data['tax_amount'];

				$this->createTransaction($db, $user, $order, "Void Transaction\n".array2text($response), '', serialize($custom1));

				$this->is_error = false;
				$this->error_message = "";
				
				return true;
			}
			
			return false;
		}
	}
