<?php 
	$payment_processor_id = "securehosting";
	$payment_processor_name = "Secure Hosting";
	$payment_processor_class = "PAYMENT_SECUREHOSTING";
	$payment_processor_type = "cc";
	
	class PAYMENT_SECUREHOSTING extends PAYMENT_PROCESSOR{
		
		function PAYMENT_SECUREHOSTING()
		{
			parent::PAYMENT_PROCESSOR();
			$this->id = "securehosting";
			$this->name = "Secure Hosting";
			$this->class_name = "PAYMENT_SECUREHOSTING";
			$this->type = "cc";
			$this->description = "";
			$this->steps = 1;
			$this->testMode = false;
			$this->requires_cc_start_date = true;
			$this->has_issue_number = true;
			$this->need_cc_codes = false;
			$this->support_ccs = false;
			return $this;
		}
		
		function getPaymentForm($db)
		{
			$fields = parent::getPaymentForm($db);
			
			$fields['cc_start_year']['hint'] = 'Not required for Switch cards only';
			array_unshift($fields['cc_start_month']['options'], array('value'=>'','option'=>''));
			array_unshift($fields['cc_start_year']['options'], array('value'=>'','option'=>''));
			
			return $fields;
		}
		
		function isTestMode()
		{
			static $testMode = null;
			
			if (!is_null($testMode)) return $testMode;
			$testMode = ($this->settings_vars[$this->id."_test_mode"]["value"] == "On");
			$this->testMode = $testMode;
			
			return $this->testMode;
		}
		
		function formatAmountString($amount)
		{
			return str_replace(array(',','.'), '', number_format($amount, 2, '.', ''));
		}
		
		/**
		 * Processes Other Transactions
		 *
		 * @param string $db 
		 * @param string $custom_id 
		 * @param string $order_id 
		 * @return void
		 * @author Sebastian Nievas
		 */
		function processCustom($db, $custom_id, $order_id)
		{
			global $settings;
			
			$order_id = intval(is_numeric($order_id) ? $order_id : 0);

			if (!$order_id) 
			{
				$this->is_error = true;
				$this->error_message = 'An Error Has Occurred - Invalid Order ID';
				return false;
			}

			$db->query("
				SELECT ".DB_PREFIX."orders.*,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."orders.shipping_country
				WHERE 
					".DB_PREFIX."orders.oid='".$order_id."'
			");

			$db->moveNext();
			$data = $db->col;

			// set some environment variables
			$this->payment_status = $data['payment_status'];
			$this->custom = $data['custom1'];
			$this->transaction_id = $data['security_id'];
			
			$this->is_error = false;
			
			// transaction options
			$transaction_options = array(
					'transaction_id' => $this->transaction_id
				);
			
			// set the transaction type to perform based on the $custom_id sent
			switch($custom_id)
			{
				// capture
				case '1':
					$transactionType = 'capture';
					$this->payment_status = 'Received';
					$transaction_options['amount'] = $data['total_amount'];
					break;
				
				// Void transaction
				case '3': $transactionType = 'CreditCardVoid'; break;
					$this->payment_status = 'Canceled';
				
				default:
					$this->is_error = true;
					return false;
			}
			
			$buffer = $this->dispatch($transactionType, $transaction_options);
			
			$this->log("Process custom ".$transactionType." Request:\n".array2text($transaction_options));
			$this->log("Process custom ".$transactionType." Response:\n".$buffer);
			
			if ($buffer)
			{
				$xmlParser = new XMLParser();
				
				$data = $xmlParser->parse($buffer);
				
				$response = $data['result'][0];
				
				$rsStatus = $response['status'][0];
				
				if ($rsStatus == 'ERROR')
				{
					// automatically decline transaction
					$this->error_message = $response['statustext'][0];
					
					if ($response['statustext'][0] == 'TRANSACTION_ERROR') $this->error_message .= ': '.$response['reason'][0];
					
					$this->is_error = true;
					
					return false;
				}
				elseif ($rsStatus == 'OK')
				{
					// Update the orders table status
					$db->reset();
					$db->assignStr('payment_status', $this->payment_status);
					$db->assignStr("security_id", $this->transaction_id);
					$db->assignStr("custom2", $buffer);
					$db->update(DB_PREFIX."orders", "WHERE oid='".$order_id."'");
					
					return true;
				}
			}
			else 
			{
				$this->is_error = true;
			}
			
			$this->error_message = 'The gateway could not process this request';
			
			// will return false
			return false;
		}
		
		/**
		 * Returns order payment status to handle any additional transactions on the order
		 *
		 * @param string $custom1 
		 * @param string $custom2 
		 * @param string $custom3 
		 * @param string $order_status 
		 * @param string $payment_status 
		 * @return void
		 */
		function getVarStatus($custom1="", $custom2="", $custom3="", $order_status="", $payment_status="")
		{
			$var_status = array(1 => false, 2 => false, 3 => false);
			$message = '';
			
			if ($custom1 == "pre_auth" && $payment_status == "Pending")
			{
				switch ($payment_status)
				{
					case 'Pending':
						$message = "
							<div style='padding-top:7px;'>
								<input type=\"button\" name=\"txn_action\" value=\"Complete Transaction\" onclick=\"window.open('content/engine/payment/securehosting/securehosting_gateway.php?oid=".$_GET['oid']."&payment_status=Pending','PopupWind','titlebar=no, toolbar=no, menubar=no, status=no, directories=no, resizable=no, scrollbars=no, top=200, left=400, width=600, height=150')\" />
							</div>";
						break;
				}
			}
			
			$var_status[1] = array(
				'message' => $message,
				'executable' => true,
				'link_caption' => ''
			);
			
			return $var_status;
		}
		
		function dispatch($transactionType, $postVars)
		{
			// production vars
			$post_url = (trim($this->url_to_gateway) == '')
				? 'https://www.secure-server-hosting.com/secutran/api.php'
				: trim($this->url_to_gateway);
			
			// sandbox vars
			if ($this->isTestMode()) 
			{
				$post_url =  'https://test.secure-server-hosting.com/secutran/api.php';
			}
			
			$input_array = array();
			switch ($transactionType)
			{
				case 'authorise':
				{
					$auth_type = 'authorise';
				}
				
				case 'pre_auth':
				{
					if (mktime(0,0,0,date('m'),1,date('y')) > mktime(0,0,0,$postVars['cc_expiration_month'],1,$postVars['cc_expiration_year']))
					{
						$this->error_message = 'This credit card has expired. Please check the expiration date or use a different credit card.';
						return false;
					}
					
					if (!isset($auth_type)) $auth_type = 'pre_auth';
					
					$request_type = 'transaction';
					
					if ($this->settings_vars[$this->id."_gateway_configuration"]["value"] == 'Default')
					{
						$transactionRequestXml = '
							<authtype>'.$auth_type.'</authtype>
							<authentication>
								<shreference>'.$this->settings_vars[$this->id."_sh_reference"]["value"].'</shreference>
								<checkcode>'.$this->settings_vars[$this->id."_checkcode"]["value"].'</checkcode>
							</authentication>
							<transaction>
								<currency>'.$_SESSION['current_currency']['code'].'</currency>
								<cardnumber>'.$postVars["cc_number"].'</cardnumber>
								<cardstartmonth>'.$postVars['cc_start_month'].'</cardstartmonth>
								<cardstartyear>'.substr($postVars['cc_start_year'], -2).'</cardstartyear>
								<cardexpiremonth>'.$postVars['cc_expiration_month'].'</cardexpiremonth>
								<cardexpireyear>'.substr($postVars['cc_expiration_year'], -2).'</cardexpireyear>
								<cv2>'.$postVars['cc_cvv2'].'</cv2>
								<switchnumber>'.$postVars['cc_issue_number'].'</switchnumber>
								<cardholdersname>'.$postVars["cc_first_name"].' '.$postVars["cc_last_name"].'</cardholdersname>
								<cardholdersemail>'.$this->common_vars["billing_email"]["value"].'</cardholdersemail>
								<cardholderaddr1>'.$this->common_vars["billing_address1"]["value"].'</cardholderaddr1>
								<cardholderaddr2>'.$this->common_vars["billing_address2"]["value"].'</cardholderaddr2>
								<cardholdercity>'.$this->common_vars["billing_city"]["value"].'</cardholdercity>
								<cardholderstate>'.$this->common_vars["billing_state"]["value"].'</cardholderstate>
								<cardholderpostcode>'.$this->common_vars["billing_zip"]["value"].'</cardholderpostcode>
								<cardholdercountry>'.$this->common_vars["billing_country"]["value"].'</cardholdercountry>
								<cardholdertelephonenumber>'.$this->common_vars["billing_phone"]["value"].'</cardholdertelephonenumber>
								<transactionamount>'.number_format($this->common_vars["order_total_amount"]["value"], 2, '.', '').'</transactionamount>
								<transactiontax>'.number_format($this->common_vars["order_tax_amount"]["value"],2, '.', '').'</transactiontax>
								<shippingcharge>'.number_format($this->common_vars["order_shipping_amount"]["value"], 2, '.', '').'</shippingcharge>
							</transaction>
						';
					}
					else
					{
						$transactionRequestXml = '
							<authtype>'.$auth_type.'</authtype>
							<authentication>
								<shreference>'.$this->settings_vars[$this->id."_sh_reference"]["value"].'</shreference>
								<checkcode>'.$this->settings_vars[$this->id."_checkcode"]["value"].'</checkcode>
							</authentication>
							<transaction>
								<transactioncurrency>'.$_SESSION['current_currency']['code'].'</transactioncurrency>
								<txtcardnum>'.$postVars["cc_number"].'</txtcardnum>
								<optvalidfrommonths>'.$postVars['cc_start_month'].'</optvalidfrommonths>
								<optvalidfromyears>'.substr($postVars['cc_start_year'], -2).'</optvalidfromyears>
								<optexpirymonths>'.$postVars['cc_expiration_month'].'</optexpirymonths>
								<optexpiryyears>'.substr($postVars['cc_expiration_year'], -2).'</optexpiryyears>
								<cv2>'.$postVars['cc_cvv2'].'</cv2>
								<switchnumber>'.$postVars['cc_issue_number'].'</switchnumber>
								<cardholdersname>'.$postVars["cc_first_name"].' '.$postVars["cc_last_name"].'</cardholdersname>
								<txtemail>'.$this->common_vars["billing_email"]["value"].'</txtemail>
								<cardholderaddr1>'.$this->common_vars["billing_address1"]["value"].'</cardholderaddr1>
								<cardholderaddr2>'.$this->common_vars["billing_address2"]["value"].'</cardholderaddr2>
								<cardholdercity>'.$this->common_vars["billing_city"]["value"].'</cardholdercity>
								<cardholderstate>'.$this->common_vars["billing_state"]["value"].'</cardholderstate>
								<cardholderpostcode>'.$this->common_vars["billing_zip"]["value"].'</cardholderpostcode>
								<cardholdercountry>'.$this->common_vars["billing_country"]["value"].'</cardholdercountry>
								<cardholdertelephonenumber>'.$this->common_vars["billing_phone"]["value"].'</cardholdertelephonenumber>
								<txtprice>'.number_format($this->common_vars["order_total_amount"]["value"], 2, '.', '').'</txtprice>
								<transactiontax>'.number_format($this->common_vars["order_tax_amount"]["value"],2, '.', '').'</transactiontax>
								<shippingcharge>'.number_format($this->common_vars["order_shipping_amount"]["value"], 2, '.', '').'</shippingcharge>
							</transaction>
						';
					}
					
					break;
				}
				
				case 'capture':
				{
					$request_type = 'authorisation';
					
					$transactionRequestXml = '
						<authentication>
							<shreference>'.$this->settings_vars[$this->id."_sh_reference"]["value"].'</shreference>
							<password>'.$this->settings_vars[$this->id."_password"]["value"].'</password>
						</authentication>
						<transaction>
							<reference>'.$postVars['transaction_id'].'</reference>
							<amount>'.number_format($postVars['amount'], 2, '.', '').'</amount>
						</transaction>
					';
					
					break;
				}
				// not passing valid transactionType
				default: 
				{
					return false;
				}
			}
			
			$xml = '<?xml version="1.0"?>
				<request>
					<type>'.$request_type.'</type>
					'.$transactionRequestXml.'
				</request>
			';
			
			// url encode xml and pass as xmldoc post var
			$post_vars = "xmldoc=".urlencode($xml);
			
			// initialize curl
			$ch = curl_init();
			
			curl_setopt($ch, CURLOPT_TIMEOUT, 500);
			curl_setopt($ch, CURLOPT_URL, $post_url);

			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 1);
			curl_setopt ($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $post_vars);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt ($ch, CURLOPT_HEADER, 0);
			
			if (!safe_mode()) set_time_limit(3000);
			
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
			global $settings;
			curl_setopt($ch, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
			curl_setopt($ch, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
			$buffer = curl_exec($ch) or die(curl_error($ch));
			
			$this->log("dispatch Request:\n".array2text($post_vars));
			$this->log("dispatch Response:\n".$buffer);
			
			if (!$buffer)
			{
				return false;
			}
			
			return $buffer;
		}
		
		function process($db, $user, $order, $post_form)
		{
			global $settings;
			
			if (isset($post_form) && is_array($post_form))
			{
				
				// get the correct transaction request type
				switch ($this->settings_vars[$this->id.'_transaction_type']['value'])
				{
					case 'Authorize Only': $transactionType = 'pre_auth'; break;
					case 'Authorize and Capture': $transactionType = 'authorise'; break;
				}
				
				$buffer = $this->dispatch($transactionType, $post_form);
				
				$this->log("process Request:\n".array2text($post_form));
				$this->log("process Response:\n".$buffer);
				
				// we received data, parse it
				if ($buffer)
				{
					$xmlParser = new XMLParser();
					$data = $xmlParser->parse($buffer);

					$response = $data['result'][0];
					
					$rsStatus = $response['status'][0];
					
					if ($rsStatus == 'ERROR')
					{
						// automatically decline transaction
						$this->error_message = $response['statustext'][0];
						
						if ($response['statustext'][0] == 'TRANSACTION_ERROR') $this->error_message .= ': '.$response['reason'][0];
						
						$this->is_error = true;
						return false;
					}
					elseif ($rsStatus == 'OK')
					{
						// Transaction Valid
						$rsTransactionId = $response['reference'][0];
						
						// log the transaction
						$this->createTransaction($db, $user, $order, $buffer, '', $transactionType, '', number_format($this->common_vars["order_total_amount"]["value"], 2, '.', ''), $rsTransactionId);
						
						// store credit card info?
						if ($this->support_ccs && $this->enable_ccs)
						{
							
							// PA DSS compliance
							$card_number = str_pad(substr($post_form['cc_number'], -4), strlen($post_form['cc_number']), 'x', STR_PAD_LEFT);
							
							$card_data = array(
								"fn"=>$post_form["cc_first_name"],
								"ln"=>$post_form["cc_last_name"],
								"ct"=>$post_form["cc_type"],
								"cc"=>$card_number,
								"em"=>$post_form["cc_expiration_month"],
								"ey"=>$post_form["cc_expiration_year"]
							);
							$this->saveCCdata($db, $order, $card_data, base64_decode($settings["SecurityCCSCertificate"]));
						}
						
						// transaction successful, return to not go past this point
						return ($transactionType == 'pre_auth') ? 'Pending' : 'Received';
					}
					
					// if script reaches this point, there was an error from the gateway
					$this->is_error = true;
					$this->error_message = PAYMENT_SERVER_ERROR_TEXT;
					
				}
				else 
				{
					// no $buffer set, return error
					$this->is_error = true;
					$this->error_message = ($this->error_message) ? $this->error_message : PAYMENT_SERVER_ERROR_TEXT;
				}

				if ($this->is_error)
				{
					$this->storeTransaction($db, $user, $order, $buffer, '', false);
				}
			}
			else
			{
				$this->is_error = true;
				$this->error_message = "Incorrect form params. Please contact site administrator.";
			}
			
			return !$this->is_error;
		}
	}
