<?php

class core_Form_CheckboxField extends core_Form_Field
{
	protected $checked = false;

	public function __construct($name, $title, $value = null, $required = false, $lockable = false, $lockName = null, $tooltip = null, $note = null, $attr = array(), $wrapperClass = '', $current_value = null, $readonly = false)
	{
		parent::__construct($name, $title, $value, $required, $lockable, $lockName, $tooltip, $note, $attr, $wrapperClass, $readonly);

		$this->setChecked($current_value);
	}

	public function getType()
	{
		return 'checkbox';
	}

	public function checked()
	{
		return $this->checked;
	}
	
	public function setChecked($current_value)
	{
		if (!is_null($this->value) && trim($this->value) != '')
		{
			$this->checked = ($current_value == $this->value);
		}
		else
		{
			$this->checked = ($current_value ? true : false);
		}
	}

	public function toArray()
	{
		$vals = parent::toArray();

		$vals['checked'] = $this->checked;

		return $vals;
	}

	public function bind($request, $files)
	{
		if (isset($request[$this->name]))
		{
			$val = $request[$this->name];
			$this->bindData($val);
		}
	}

	public function bindData($value)
	{
		if (trim($value) == 'on')
		{
			$this->setChecked(true);
		}
		else
		{
			$this->setChecked($value);
		}
	}

	public function getData()
	{
		if (is_null($this->value) || trim($this->value) == '')
			return $this->checked;

		if ($this->checked)
			return $this->value;
		else
			return null;
	}

	public static function createFromFormBuilder($name, $options = array())
	{
		core_Form_Field::setDefaults($options);
		$options['current_value'] = (isset($options['current_value']) ? $options['current_value'] : null);

		return new core_Form_CheckboxField(
			$name, $options['label'], $options['value'], $options['required'], 
			$options['lockable'], $options['lockName'], $options['tooltip'], $options['note'], 
			$options['attr'], $options['wrapperClass'], $options['current_value'], $options['readonly']
		);
	}
}