<?php

abstract class core_Form_Field
{
	protected $name;
	protected $label;
	protected $value;
	protected $required;
	protected $lockable;
	protected $lockName = null;
	protected $locked = false;
	protected $tooltip;
	protected $note;
	protected $readonly = false;

	protected $attr;
	protected $wrapperClass;
	protected $errors = array();

	protected $rendered = false;

	public function __construct($name, $label, $value = null, $required = false, $lockable = false, $lockName = null, $tooltip = null, $note = null, $attr = array(), $wrapperClass = '', $readonly = false)
	{
		$this->name = $name;
		$this->label = $label;
		$this->value = $value;
		$this->required = $required;
		$this->lockable = $lockable;
		$this->lockName = $lockName;
		$this->tooltip = $tooltip;
		$this->note = $note;
		$this->attr = $attr;
		$this->wrapperClass = $wrapperClass;
		$this->readonly = $readonly;
	}

	public function getRendered()
	{
		return $this->rendered;
	}
	
	public function setRendered($val = true)
	{
		$this->rendered = $val;
	}

	public function getName()
	{
		return $this->name;
	}
	
	public function setName($name)
	{
		$this->name;

		return $this;
	}

	public abstract function getType();

	public function getLabel()
	{
		return $this->label;
	}

	public function setLabel($label)
	{
		$this->label = $label;

		return $this;
	}

	public function getValue()
	{
		return $this->value;
	}
	
	public function setValue($value)
	{
		$this->value = $value;

		return $this;
	}

	public function getErrors()
	{
		return $this->errors;
	}
	public function setErrors($errors)
	{
		$this->errors = $errors;
	}

	public function isRequired()
	{
		return $this->required;
	}
	
	public function setRequired($required)
	{
		$this->required = $required;

		return $this;
	}

	public function isLockable()
	{
		return $this->lockable;
	}
	
	public function setLockable($lockable)
	{
		$this->lockable = $lockable;

		return $this;
	}

	public function getLockName()
	{
		if ($this->lockName)
			return $this->lockName;

		return $this->name;
	}
	
	public function setLockName($lockName)
	{
		$this->lockName = $lockName;

		return $this;
	}

	public function isLocked()
	{
		return $this->locked;
	}
	
	public function setLocked($locked)
	{
		$this->locked = $locked;
	}

	public function getTooltip()
	{
		return $this->tooltip;
	}
	
	public function setTooltip($tooltip)
	{
		$this->tooltip = $tooltip;

		return $this;
	}

	public function getNote()
	{
		return $this->note;
	}
	
	public function setNote($note)
	{
		$this->note = $note;

		return $this;
	}

	public function isReadonly()
	{
		return $this->readonly;
	}
	
	public function setReadonly($readonly)
	{
		$this->readonly = $readonly;
	}

	public function getElementId()
	{
		$id = $this->getAttribute('id');
		if (is_null($id))
			return 'field-'.str_replace(']', '', str_replace('[', '-', $this->getName()));

		return $id;
	}
	
	public function getAttributes()
	{
		$attributes = array();
		
		if (is_array($this->attr))
		{
			foreach ($this->attr as $key => $value)
			{
				if ($key !== 'class' && $key !== 'id')
					$attributes[] = $key.'="'.$value.'"';
			}
		}
		
		if (!empty($attributes))
		{
			return implode(' ', $attributes);
		}
		
		return null;
	}

	public function getAttribute($name)
	{
		if (is_array($this->attr))
		{
			if (array_key_exists($name, $this->attr))
				return $this->attr[$name];
		}
		return null;
	}

	public function getCssClass()
	{
		$class = $this->getElementId();
		if ($this->getName() != $this->getLockName())
		{
			$class .= ' field-'.$this->getLockName();
		}
		if ($this->locked)
		{
			$class .= ' readonly';
		}

		if (!is_null($this->getAttribute('class')))
		{
			$class .= ' '.$this->getAttribute('class');
		}

		return $class;
	}
	
	public function getWrapperClass()
	{
		return $this->wrapperClass;
	}
	
	public function setWrapperClass($wrapperClass)
	{
		$this->wrapperClass = $wrapperClass;
		return $this;
	}

	public function toArray()
	{
		$vals = array();

		$vals['value'] = $this->getValue();
		$vals['required'] = $this->isRequired();
		$vals['class'] = $this->getCssClass(); // What happens if they define a class argument in attr?
		$vals['id'] = $this->getElementId();
		$vals['read_only'] = $this->isLocked();
		$vals['attr'] = $this->getAttributes();
		$vals['wrapperClass'] = $this->getWrapperClass();
		$vals['errors'] = $this->getErrors();

		//TODO: Refactor missed named / extra field values
		$vals['name'] = $this->getName(); // => full_name?
		$vals['title'] = $this->getLabel(); // => label
		$vals['label'] = $this->getLabel();
		$vals['locked'] = $this->isLocked(); // Replace with read_only
		$vals['readonly'] = $this->isReadonly();

		//TODO: What to do with app specific values
		$vals['lockable'] = $this->isLockable();
		$vals['lockName'] = $this->getLockName();
		$vals['tooltip'] = $this->getTooltip();
		$vals['note'] = $this->getNote();

		//TODO: Implement missing field values
		$vals['max_length'] = null;
		$vals['pattern'] = null;

		return $vals;
	}

	public function getEditorName()
	{
		return 'field-'.$this->getType();
	}

	public function getTemplate()
	{
		return 'templates/elements/form/'.$this->getEditorName().'.html';
	}

	public function bind($request, $files)
	{
		$this->bindData(isset($request[$this->getName()]) ? $request[$this->getName()] : $this->value);
	}

	public function bindData($value)
	{
		$this->value = $value;
	}

	public function getData()
	{
		return $this->value;
	}

	public function getDataMapper()
	{
		return null;
	}

	public static function setDefaults(&$options)
	{
		$defaults = array(
			'name' => '',
			'label' => '',
			'value' => '',
			'required' => false,
			'lockable' => false,
			'lockName' => null,
			'tooltip' => null,
			'note' => null,
			'locked' => false,
			'attr' => array(),
			'wrapperClass' => '',
			'readonly' => false,
		);

		$options = array_merge($defaults, $options);

		return $options;
	}
}