<?php
/**
 * 
 */
class core_Form_FieldGroup
{
	protected $id;
	protected $title;
	protected $class;
	protected $collapsable = false;
	protected $hasWysiwyg = false;
	protected $fields = array();
	protected $rendered = false;
	protected $first = false;

	/**
	 * Class constructor
	 * @param type $id
	 * @param type $title
	 * @param type $icon
	 * @param type $collapsable
	 * @param type $hasWysiwyg
	 * @param type $first 
	 */
	public function __construct($id, $title, $class = null, $collapsable = false, $hasWysiwyg = false, $first = false)
	{
		$this->id = $id;
		$this->title = $title;
		$this->class = $class;
		$this->collapsable = $collapsable;
		$this->hasWysiwyg = $hasWysiwyg;
		$this->first = $first;
	}

	/**
	 *
	 * @return type 
	 */
	public function getRendered()
	{
		return $this->rendered;
	}
	
	/**
	 *
	 * @param type $val 
	 */
	public function setRendered($val = true)
	{
		$this->rendered = $val;
	}

	/**
	 *
	 * @return type 
	 */
	public function getType()
	{
		return 'group';
	}

	/**
	 *
	 * @return type 
	 */
	public function getFields()
	{
		return $this->fields;
	}

	/**
	 *
	 * @param core_Form_Field $field 
	 */
	public function addField(core_Form_Field $field)
	{
		if ($field != null)
			$this->fields[] = $field;
	}
	
	/**
	 *
	 * @param type $fields 
	 */
	public function setFields($fields)
	{
		$this->fields = $fields;
	}

	/**
	 *
	 * @param type $name
	 * @return type 
	 */
	public function getField($name)
	{
		foreach ($this->fields as $item)
		{
			if (is_a($item, 'core_Form_Field') && $item->getName() == $name)
				return $item;
		}
		return null;
	}

	/**
	 *
	 * @param type $lock_fields 
	 */
	public function lockFields($lock_fields)
	{
		foreach ($this->fields as $field)
		{
			if (is_a($field, 'core_Form_Field'))
			{
				$field->setLocked(in_array($field->getLockName(), $lock_fields));
			}
			else if (is_a($field, 'core_Form_FieldGroup'))
			{
				$field->lockFields(in_array($field->getLockName(), $lock_fields));
			}
		}
	}

	public function setTooltips($tooltips)
	{
		foreach ($this->fields as $field)
		{
			if (is_a($field, 'core_Form_Field'))
			{
				if (isset($tooltips[$field->getName()]))
				{
					$field->setTooltip($tooltips[$field->getName()]);
				}
			}
			else if (is_a($field, 'core_Form_FieldGroup'))
			{
				$field->setTooltips($tooltips);
			}
		}
	}

	/**
	 *
	 * @param type $request
	 * @param type $files 
	 */
	public function bind($request, $files)
	{
		foreach ($this->fields as $field)
			$field->bind($request, $files);
	}

	protected function setFieldErrors($messages)
	{
		foreach ($this->items as $fieldName => $field)
		{
			if (is_a($field, 'core_Form_Field'))
			{
				if (isset($messages[$fieldName]))
				{
					$field->setErrors($messages[$fieldName]);
				}
			}
			else if (is_a($field, 'core_Form_FieldGroup'))
			{
				$field->setFieldErrors($messages);
			}
		}
	}

	// /**
	//  *
	//  * @param type $messages
	//  * @return boolean 
	//  */
	// public function isValid(&$messages = array())
	// {
	// 	$isValid = true;

	// 	foreach ($this->fields as $field)
	// 	{
	// 		$ret = $field->isValid($messages);
	// 		if (!$ret) $isValid = false;
	// 	}

	// 	return $isValid;
	// }

	/**
	 * 
	 */
	public function toArray()
	{
		return array(
			'id' => $this->id,
			'title' => $this->title,
			'class' => $this->class,
			'collapsable' => $this->collapsable,
			'fields' => $this->fields,
			'hasWysiwyg' => $this->hasWysiwyg,
			'first' => $this->first,
		);
	}

	/**
	 *
	 * @param type $name
	 * @param type $options
	 * @return core_Form_FieldGroup 
	 */
	public static function createFromFormBuilder($name, $options = array())
	{
		$defaults = array(
			'label' => null,
			'class' => null,
			'collapsable' => false,
			'hasWysiwyg' => false,
			'first' => false
		);
		
		if (is_null($options))
		{
			$options = array();
		}
		
		$options = array_merge($defaults, $options);

		return new core_Form_FieldGroup($name, $options['label'], $options['class'], $options['collapsable'], $options['hasWysiwyg'], $options['first']);
	}
}