<?php
	/**
	 * Do not cache output
	 */
	header("Cache-Control: private");
	header("Content-Type: text/html; charset=UTF-8");

	define("ENV", true);

	if (file_exists(dirname(__FILE__).'/app_maintenance.htm'))
	{
		echo file_get_contents(dirname(__FILE__).'/app_maintenance.htm');
		exit();
	}

	require_once("content/engine/engine_functions.php");
	
	loadApplicationConfig();

	require_once("content/engine/engine_config.php");
	require_once("content/engine/engine_label.php");
	require_once("content/engine/engine_mysql.php");
	require_once("content/engine/engine_tree.php");
	require_once("content/engine/engine_wishlist.php");
	require_once("content/classes/class.json.php");
	require_once("content/classes/class.xml.php");
	require_once("content/engine/engine_statistics.php");

	/**
	 * Create main database connection
	 */
	$db = new DB;
	$db->init(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
	
	/**
	 * Load site settings in $settings array
	 */
	require_once("content/engine/engine_settings.php");
	
	setApplicationDefaultTimezone($db, $settings);

	require_once("content/classes/Events/Register.php");

	/**
	 * Mailchimp stuff
	 */
	if ($settings["mailchimp_e360"] == "1")
	{
		$thirty_days = time()+60*60*24*30;
		if (isset($_REQUEST['mc_cid']))
		{
			setcookie(
				'mailchimp_campaign_id', trim($_REQUEST['mc_cid']), $thirty_days,
				"/", isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]
			);
		}
		if (isset($_REQUEST['mc_eid']))
		{
			setcookie(
				'mailchimp_email_id', trim($_REQUEST['mc_eid']), $thirty_days,
				"/", isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]
			);
		}
	}
 	
	/**
	 * Resolve magic quotes
	 */
	if (get_magic_quotes_gpc() || $settings["ForceStripSlashes"] == "Yes")
	{
		strips($_POST);
		strips($_GET);
		strips($_REQUEST);
	}
	
	if (isset($_GET["url"])) require_once("content/engine/engine_url_parser.php");

	/**
	 * Start session
	 */
	$userCookie = false;

	if (isset($_COOKIE["_pcud"]))
	{
		$userCookie = unserialize(base64_decode($_COOKIE["_pcud"]));
		if (!$userCookie || !is_array($userCookie)) $userCookie = false;
		if ($userCookie)
		{
			$db->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".$db->escape($userCookie["i"])."' AND removed='No' AND level='".$db->escape($userCookie["l"])."' AND MD5(session_id)='".(isset($userCookie["s"]) ? $db->escape($userCookie["s"]) : '')."'");
			if (!$db->moveNext()) $userCookie = false;
		}
	}
	
	setSessionSavePath();
	
	/**
	 * Handle session in HTTPS mode
	 */
	$isDesignMode = false;
	if ((isset($_COOKIE["_admc"]) && $_COOKIE["_admc"] == "true") || isset($_GET["_admc"]) || isset($_POST["_admc"]))
	{
		$isDesignMode = true;
		$_COOKIE["_admc"] = "true";
		setcookie(
			"_admc", "true", time() + 60*60, "/",
			isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]
		);
	}
	else
	{
		$isDesignMode = false;
	}

	/**
	 * Handle session
	 */
	$pcsid = isset($_REQUEST["pcsid"]) ? $_REQUEST["pcsid"] : false;
	
	if ($pcsid && preg_match("/[0-9a-zA-Z]{1,}/", $pcsid))
	{
		session_id($pcsid);
	}
	
	if (isset($_GET["_pcod"]))
	{
		setcookie(
			"_pcod", $_GET["_pcod"], time() + 60*60*24*30*6, "/",
			isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]
		);
		$_COOKIE["_pcod"] = $_GET["_pcod"]; 
	}

	if ($isDesignMode || (defined('TRIAL_MODE') && TRIAL_MODE))
	{
		if (shouldBeSecure())
		{
			header('Location: '.$settings['GlobalHttpsUrl'].getRequestUri());
			exit();
		}

		Admin_SessionHandler::designModeInit($settings);

		if (!(isset($_SESSION['admin_auth_id']) && isset($_SESSION['admin_session_id'])))
		{
			$isDesignMode = false;
			if (array_key_exists('_admc', $_COOKIE))
			{
				setcookie('_admc', 'false', time() - 31536000, '/', isSslMode() ? $settings["GlobalHttpsCookieDomain"] : $settings["GlobalHttpCookieDomain"]);
			}
		}

		define('DESIGN_MODE', $isDesignMode ? true : false);
	}
	else
	{
		if (isSslMode())
		{
			$host = $settings['GlobalHttpsCookieDomain'] != '' ? $settings['GlobalHttpsCookieDomain'] : parse_url($settings["GlobalHttpsUrl"], PHP_URL_HOST);
		}
		else
		{
			$host = $settings['GlobalHttpCookieDomain'] != '' ? $settings['GlobalHttpCookieDomain'] : parse_url($settings["GlobalHttpUrl"], PHP_URL_HOST);
		}

		if ($host == 'localhost')
		{
			$host = '';
		}

		if (floatval(phpversion()) >= 5.2)
		{
			session_set_cookie_params($settings["SecurityUserTimeout"], '/', $host, false, true);
		}
		else
		{
			session_set_cookie_params($settings["SecurityUserTimeout"], '/', $host);
		}

		session_cache_expire(round($settings["SecurityUserTimeout"]/60));
		session_name("ShoppingCartSession");
		session_start();

		if (isset($_COOKIE['ShoppingCartSession']))
		{
			if (floatval(phpversion()) >= 5.2)
			{
				setcookie('ShoppingCartSession', session_id(), time() + $settings["SecurityUserTimeout"], '/', $host, false, true);
			}
			else
			{
				setcookie('ShoppingCartSession', session_id(), time() + $settings["SecurityUserTimeout"], '/', $host);
			}
		}

	}
	
	/**
	 * Mobile site
	 */
	$isMobile = false;

	require_once("content/classes/class.mobile.php");
	$mobileHelper = new MobileHelper($settings);
	
	if ($settings['MobileSiteEnabled'] == 'Yes')
	{	
		$isMobile = $mobileHelper->isMobile();
		
		if ($isMobile) $mobileHelper->setSettings($settings);
		if ($settings['GlobalMobileHttpUrl'] == '') $settings['GlobalMobileHttpUrl'] = $settings['GlobalHttpUrl'];
		if ($settings['GlobalMobileHttpsUrl'] == '') $settings['GlobalMobileHttpsUrl'] = $settings['GlobalHttpsUrl'];
		if ($isMobile)
		{
			$settings['GlobalHttpUrl'] = $settings['GlobalMobileHttpUrl'];
			$settings['GlobalHttpsUrl'] = $settings['GlobalMobileHttpsUrl'];
			$settings['CatalogShowNested'] = strtoupper($settings["MobileShowNested"]);
			$settings['CatalogItemsOnPage'] = $settings["MobileItemsOnPage"];
			
			$mobileThemes = array('Black'=>'a', 'Blue'=>'b', 'Light Gray'=>'c', 'Dark Gray'=>'d', 'Yellow'=>'e');
			view()->assign('MobileSiteThemeID', $mobileThemes[$settings['MobileSiteTheme']]);
		}
	}

	view()->assign('ToggleMobileUrl', $mobileHelper->getToggleMobileUrl(currentPageURL(), $isMobile));

	define("MOBILE", $isMobile ? true : false);
	
	/**
	 * DEMO Mode
	 */
	if ($settings["FrontEndDemo"] == "Yes")
	{
		if (!defined("DEMO_MODE")) define("DEMO_MODE", true);
		if (!defined("DEMO_LOCK")) define("DEMO_LOCK", true);

		$demoModeSkins = Ddm_View::getDemoModeSkins();

		//TODO: How to handle this?
		// $demoModeSkins = array(
		// 	"BigDesign"=>"BigBlue Design",
		// 	"BlueGreen"=>"Blue Green",
		// 	"ModernVista"=>"Modern Vista",
		// 	"NewOrange"=>"New Orange",
		// 	"SimpleGreen"=>"Simple Green",
		// 	"Template3700"=>"Template 3700",
		// 	"Template3701"=>"Template 3701",
		// 	"Template3702"=>"Template 3702",
		// 	"Template3703"=>"Template 3703",
		// 	"Template3704"=>"Template 3704",
		// 	"Template3705"=>"Template 3705",
		// 	"Template3706"=>"Template 3706",
		// 	"Template3707"=>"Template 3707",
		// 	"Template3708"=>"Template 3708",
		// 	"Template3709"=>"Template 3709"
		// );
		
		//Set new skin
		if (isset($_REQUEST["demoModeSetSkin"]) && array_key_exists($_REQUEST["demoModeSetSkin"], $demoModeSkins))
		{
			$settings['DesignSkinName'] = $_REQUEST["demoModeSetSkin"];
			$_SESSION['demoModeSkin'] = $_REQUEST["demoModeSetSkin"];

			view()->setTemplateDir("content/cache/skins/".escapeFileName($settings["DesignSkinName"])."/");
			view()->setCompileDir("content/cache/smarty/skins/".escapeFileName($settings["DesignSkinName"])."/");

			view()->assemble(
				'content'.DS.'cache'.DS.'skins'.DS.escapeFileName($settings['DesignSkinName']).DS, 
				null, false
			);
		}
		
		//Get current skin
		$settings['DesignSkinName'] = $sn = isset($_SESSION["demoModeSkin"]) ? $_SESSION["demoModeSkin"] : $settings['DesignSkinName'];

		view()->setTemplateDir("content/cache/skins/".escapeFileName($settings["DesignSkinName"])."/");
		view()->setCompileDir("content/cache/smarty/skins/".escapeFileName($settings["DesignSkinName"])."/");
		view()->loadSkinSettings($settings["DesignSkinName"]);

		//set skin settings
		
		view()->assign("demoModeSkins", $demoModeSkins);
		view()->assign("demoModeCurrentSkin", $settings['DesignSkinName']);
	}
	else
	{
		if (!defined("DEMO_MODE")) define("DEMO_MODE", false);
		if (!defined("DEMO_LOCK")) define("DEMO_LOCK", false);	
	}
	
	/**
	 * Currency
	 */
	require_once("content/classes/class.currencies.php");
	$currencies = new Currencies($db);
	if (isset($_POST["set_current_currency"]))
	{
		$currencies->setCurrentCurrency(intval($_POST["set_current_currency"]));
	}
	else
	{
		$currencies->getCurrentCurrency();
	}
	
	if (!isset($_SESSION["default_currency"]))
	{
		$_SESSION["default_currency"] = $currencies->getDefaultCurrency();
	}

	// load localization procedures
	require_once("content/engine/engine_localization.php");
	require_once("content/engine/engine_url_".($settings["USE_MOD_REWRITE"]=="YES" ? "rewrite" : "default").".php");
	require_once("content/engine/engine_pages.php");
	require_once("content/engine/engine_notifications.php");
	require_once("content/engine/engine_payment.php");
	require_once("content/engine/engine_shipping.php");

	/**
	 * Current page
	 */
	$p = isset($_REQUEST["p"]) ? trim($_REQUEST["p"]) : (isset($_REQUEST['P']) ? trim($_REQUEST['P']) : "home");
	$p = preg_match("/^([a-z0-9_-])+$/", $p) ? $p : "home";
	
	
	// FIXME: do NOT commit the following line
	// if (array_key_exists('ddmskin', $_REQUEST)) $settings['DesignSkinName'] = $_REQUEST['ddmskin'];
	/**
	 * Init smarty
	 */
	view()->debugging = false;
	view()->use_sub_dirs = false;
	view()->assign("isDemoMode", DEMO_MODE);
	view()->assign("isDevMode", defined("DEVMODE") && DEVMODE);
	
	// FIXME: Assemble through backend only
	if (
		!is_dir('content/cache/skins/'.escapeFileName($settings['DesignSkinName'])) || 
		(isset($_SESSION['admin_auth_id']) && $_SESSION['admin_auth_id'] && isset($_SESSION['admin_design_mode']) && $_SESSION['admin_design_mode'])
		|| (defined("DEVMODE") && DEVMODE)
	)
	{
		view()->assemble(
			'content'.DS.'cache'.DS.'skins'.DS.escapeFileName($settings['DesignSkinName']).DS, 
			null, $settings['DesignSkinName'] != '_mobile', isset($_REQUEST['_admc']) && $_REQUEST['_admc'] == 'true'
		);
	}

	/**
	 * Languages
	 */
	require_once("content/classes/class.languages.php");
	$languages = new Language($db);
	
	view()->assign("languages", $active_languages = $languages->getActiveLanguages());
	view()->assign("languages_count", count($active_languages));
	
	// check / set language for a first time
	if (!isset($_SESSION["current_language_id"]))
	{
		$def_lang = $languages->getDefaultLanguage();
		$_SESSION["current_language_id"] = $def_lang["language_id"];
	}
	//change current active language
	if (isset($_POST["set_language"]))
	{
		if (array_key_exists(intval($_POST["set_language"]), $active_languages))
		{
			$_SESSION["current_language_id"] = intval($_POST["set_language"]);
		}
	}	
	
	//check / set current active language
	$current_language = $languages->getActiveLanguageById($_SESSION["current_language_id"]);
	view()->assign("current_language", $current_language);
	view()->assign("msg", $msg = $languages->getDictionary($current_language["code"]));
	view()->prependJavascript($languages->getJavascriptCache($current_language["code"]));
		
	// pass settings to smarty
	foreach ($settings as $key=>$value) view()->assign($key, $value);

	if (isset($settings['GoogleAnalytics']))
	{
		$trackingCode = $settings['GoogleAnalytics'];
		if (strstr($trackingCode, '_getTracker(') !== false)
		{
			view()->assign('GoogleAnalyticsAsync', 'No');
		}
		else
		{
			view()->assign('GoogleAnalyticsAsync', 'Yes');
		}
	}
	
	// by default user is unknown
	view()->assign("user_cookie", false);//$userCookie);
	view()->assign("auth_ok", "no");
	view()->assign("current_page", $p);
	
	view()->assign("url_admin", $settings["GlobalHttpsUrl"]);
	
	// Added url request uri
	view()->assign("url_request_uri", htmlentities($_SERVER['REQUEST_URI']));
	
	// load user and order
	require_once("content/engine/engine_user.php");
	require_once("content/engine/engine_order.php");

	// assign most used user and cart actions
	view()->assign("USER_EXPRESS_CHECKOUT", USER_EXPRESS_CHECKOUT);
	view()->assign("USER_START_CHECKOUT", USER_START_CHECKOUT);
	view()->assign("USER_LOGIN", USER_LOGIN);
	view()->assign("USER_LOGOUT", USER_LOGOUT);
	view()->assign("USER_PASSWORD_RESET", USER_PASSWORD_RESET);
	view()->assign("USER_UPDATE_ADDRESS", USER_UPDATE_ADDRESS);
	view()->assign("USER_DELETE_ADDRESS", USER_DELETE_ADDRESS);
	view()->assign("USER_ADD_TO_WISHLIST", USER_ADD_TO_WISHLIST);
	view()->assign("USER_UPDATE_PROFILE", USER_UPDATE_PROFILE);
	view()->assign("USER_ADD_ADDRESS", USER_ADD_ADDRESS);
	view()->assign("USER_ADD_PAYMENT_PROFILE", USER_ADD_PAYMENT_PROFILE);
	view()->assign("USER_DELETE_PAYMENT_PROFILE", USER_DELETE_PAYMENT_PROFILE);
	view()->assign("ORDER_ADD_ITEM", ORDER_ADD_ITEM);
	view()->assign("ORDER_REMOVE_ITEM", ORDER_REMOVE_ITEM);
	view()->assign("ORDER_CLEAR_ITEMS", ORDER_CLEAR_ITEMS);
	view()->assign("ORDER_UPDATE_ITEMS", ORDER_UPDATE_ITEMS);
	view()->assign("ORDER_COMPLETE_ORDER", ORDER_COMPLETE_ORDER);
	view()->assign("ORDER_WISHLIST_TO_CART", ORDER_WISHLIST_TO_CART);
	view()->assign("ORDER_COMPLETE_ORDER", ORDER_COMPLETE_ORDER);
	view()->assign("ORDER_PROCESS_PAYMENT", ORDER_PROCESS_PAYMENT);
	view()->assign("ORDER_PROCESS_PAYMENT_VALIDATION", ORDER_PROCESS_PAYMENT_VALIDATION);
	view()->assign("ORDER_ADD_GIFT_CERT", ORDER_ADD_GIFT_CERT);

	view()->assign('IS_TRIAL', defined('TRIAL_MODE') && TRIAL_MODE);
	view()->assign('IS_TRIAL_BAR', defined('TRIAL_MODE') && TRIAL_MODE && isset($_SESSION['admin_trial_token']));

	view()->assign('isDesignMode', $isDesignMode);

	if (defined('TRIAL_MODE') && TRIAL_MODE && isset($_SESSION['admin_auth_id']) && isset($_SESSION['admin_session_id']) && is_file('trial/index-include.php'))
	{
		include('trial/index-include.php');
	}

	/**
	 * Define page zones
	 */
	$pages_public = array(
		'home', 'page', 'catalog', 'product', 'cart', 'site_map',
		'login', 'signup', 'subscribe', 'unsubscribe', 'password_reset', 'blocked', 'completed',
		'email2friend', 'shipping_quote', 'international', 'wishlist', 'manage_wishlist', 'gift_certificate',
		'testimonials', 'smartsuggest', '404', 'paypal_callback'
	);
	$pages_public_only = array('!login', '!signup', '!password_reset', 'blocked');

	$pages_protected = array(
		'account', 'profile', 'orders', 'order', 'address_book', 'address_edit',
		'one_page_checkout', 'payment_validation', 'payment_profiles', 'payment_profile_edit'
	);

	$pages_protected_express = array('one_page_checkout', 'payment_validation');

	$pages_for_https = array(
		'account', 'profile', 'orders', 'order', 
		'payment_validation', 'completed', 'address_book', 'address_edit', 'one_page_checkout',
		'cart',	'login', 'signup', 'password_reset', 'paypal_callback'
	);

	/**
	 * Prepare dynamic URLs engine
	 */
	if (isSslMode())
	{
		if ($isDesignMode)
		{
			$url_dynamic = $url_http = $url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?_admc=true&";
		}
		else
		{
			$url_http = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?";
			$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
			$url_dynamic = (isset($_SESSION["UserID"]) && intval($_SESSION["UserID"]) > 0 ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"])."/".$settings["INDEX"]."?";
		}
		$url_base = $settings["GlobalHttpsUrl"]."/";
	} 
	else
	{
		if ($isDesignMode)
		{
			$url_http = $url_dynamic = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
		
			if (parse_url($settings["GlobalHttpUrl"], PHP_URL_HOST) != parse_url($settings["GlobalHttpsUrl"], PHP_URL_HOST))
			{
				$_pcod = isset($_COOKIE["_pcod"]) ? ("&_pcod=".$_COOKIE["_pcod"]) : "";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id().$_pcod."&_admc=true&";
			}
			else
			{
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?_admc=true&";	
			}
			$url_base = $settings["GlobalHttpsUrl"]."/";
		}
		else
		{
			$url_http = $url_dynamic = $settings["GlobalHttpUrl"]."/".$settings["INDEX"]."?";

			// transfer session from http to https when domains are different for http and https
			if (parse_url($settings["GlobalHttpUrl"], PHP_URL_HOST) != parse_url($settings["GlobalHttpsUrl"], PHP_URL_HOST))
			{
				$_pcod = isset($_COOKIE["_pcod"]) ? ("&_pcod=".$_COOKIE["_pcod"]) : "";
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?pcsid=".session_id().$_pcod."&";
			}
			else
			{
				$url_https = $settings["GlobalHttpsUrl"]."/".$settings["INDEX"]."?";
			}
			$url_base = $settings["GlobalHttpUrl"]."/";
		}
	}
	
	$url_rel = $settings["INDEX"]."?";
	$url_checkout = $url_https."ua=".USER_START_CHECKOUT."&checkout_start_page=".$p;
	
	view()->assign("url_http", $url_http);
	view()->assign("url_https", $url_https);
	view()->assign("url_dynamic", $url_dynamic);
	view()->assign("url_dinamic", $url_dynamic); // backwards compatibility
	view()->assign("url_base", $url_base);
	view()->assign("url_rel", $url_rel);
	view()->assign("url_checkout", $url_checkout);
	view()->assign("url_ajax", $url_ajax = (isSslMode() ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"]) . '/'.$settings['INDEX'].'?pcsid='.session_id());
	view()->assign("url_ajax_admin", $url_ajax_admin = $settings["GlobalHttpsUrl"]);
	
	// gift certificates
	require_once("content/classes/class.gift_certificates.php");
	
	// custom fields
	require_once("content/classes/class.custom_fields.php");
	$customFields = new CustomFields($db);

	// load and start shopping cart engine
	require_once("content/engine/engine_logic.php");

	Payment_Google_GoogleCheckout::initializeButton($db, $settings, $order);

	view()->assign('paypalEnabled', Payment_PayPal_PayPalApi::isPayPalEnabled());

	// check redirects and generate global back URL
	$backurl = isset($backurl) ? $backurl : "";
	
	//complete here in case of ajax or backurl
	if ($backurl && $backurl != "")
	{
		$_SESSION["HTTP_REFERER"] = (isSslMode() ? "https://" :"http://").$_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
		
		$_mysession = $_SESSION;
		foreach ($_mysession as $_mysessionvar=>$_mysessionkey)
		{
			$_SESSION[$_mysessionvar] = $_mysessionkey;
		}
		
		_session_write_close();
		
		$db->done();
		
		header("Location: ".$backurl."\n");
		die();
	}
	
	// assign user security / order vars
	view()->assign("auth_ok", $user->auth_ok?"yes":"no");
	view()->assign("user_level", $user->auth_ok ? $user->level : ($userCookie ? $userCookie["l"] : 0));
	view()->assign("auth_express", $user->express?"yes":"no");

	view()->assign("order", $order);
	view()->assign("order_items", $order->items);

	foreach ($_GET as $_name => $_value)
	{
		if ($_name != "set_language" && !is_array($_name) && !is_array($_value))
		{
			$backurl .= urlencode($_name)."=".urlencode($_value)."&";
		}
	}
	view()->assign("backurl", $backurl);

	//////////////////////////////
	// HERE STARTS BLACK WOOD :)
	// I started to develop this application 5 years ago.
	// 5 years. It is pretty big kid now.
	// Well. 3 more years. It is 2011 already. Future here.
	
	//////////////////////////////////////////////////////////
	// DISPLAY PART

	//catalog menu	
	if (isset($_REQUEST["parent"]))
	{
		$parent = intval($_REQUEST["parent"]);
	}
	elseif ($settings["USE_MOD_REWRITE"] == "YES" && isset($_REQUEST["catalog_code"]))
	{
		$parts = explode("-", $_REQUEST["catalog_code"]);
		$parent = count($parts) == 3 ? intval($parts[1]) : "0";
	}
	else
	{
		$parent = "0";
	}

	require_once("content/classes/class.catalog.php");
	
	$categories = new ShoppingCartCatalog($db);
	$categories_tree = $categories->getParsedTree($parent, true);
	view()->assign("parent", $parent);
	view()->assign("categories", $categories->tree);
	
	//manufacturers
	if ($settings["CatalogManufacturers"] == "YES")
	{
		require_once("content/classes/class.manufacturers.php");
		
		$manufacturers = new Manufacturers($db, $settings);
		$manufacturers_list = $manufacturers->getManufacturers(true);
		
		if ($manufacturers_list)
		{
			view()->assign("manufacturers", $manufacturers_list);
			$is_manufacturers = true;
		}
		else
		{
			view()->assign("manufacturers", false);
			$is_manufacturers = false;
		}
	}
	else
	{
		view()->assign("manufacturers", false);
		$is_manufacturers = false;
	}

	view()->assign("current_currency", $_SESSION["current_currency"]);
    view()->assign("default_currency", $_SESSION["default_currency"]);

	$currencies_list = $currencies->getCurrencies();
	
	if ($currencies_list)
	{
		view()->assign("currencies", $currencies_list);
		view()->assign("currencies_count", count($currencies_list));
		$is_currencies = true;
	}
	else
	{
		view()->assign("currencies", array());
		view()->assign("currencies_count", 1);
		$is_currencies = false;
	}

	//bestsellers
	if (
		(isset($settings["CatalogBestsellersAvailable"]) && $settings["CatalogBestsellersAvailable"] == "YES")
		||
		(isset($settings["CatalogBestSellersAvailable"]) && $settings["CatalogBestSellersAvailable"] == "YES")
	)
	{
		require_once("content/classes/class.products.php");
		$params = $settings;
		
		if ($parent)
		{
			if (in_array($parent, array_keys($categories->tree)))
			{
				$params["categories"][] = $parent;
				if (is_array($categories->tree[$parent]["children"]))
				{
					reset($categories->tree[$parent]["children"]);
					foreach ($categories->tree[$parent]["children"] as $id=>$node)
					{
						$params["categories"][] = $id;
					}
				}
			}
		}
		else
		{
			$params["categories"] = array();
		}
		
		$bs = new ShoppingCartProducts($db, $settings);

		view()->assign("bestsellers", $bs->getItemsBestSellers($params, $user->auth_ok ? $user->level : ($userCookie ? $userCookie["l"] : 0)));
		view()->assign("bestsellers_view", $settings["CatalogBestsellersView"]);		
		$is_bestsellers = $bs->bestsellersCount > 0;
	}
	else
	{
		view()->assign("bestsellers", false);//
		$is_bestsellers = false;
	}

	if ($user->auth_ok)
	{
		view()->assign("user_name", gs($user->data["fname"]." ".$user->data["lname"]));
	}

	//top & bottom links
	$page_id = isset($_REQUEST["page_id"]) ? $_REQUEST["page_id"] : "";
	view()->assign("pages_links", $pages_links = getPagesLinks($db, $p=="page"?$page_id:""));

	//recent items
	require_once("content/classes/class.recent.php");
	$recentItems = new ShoppingCartRecentItems($settings);
	view()->assign("recent_items", $recentItems->getItems());

	view()->assign("designImages", view()->getImages());

	view()->assign("label_app_name", $label_app_name);
	view()->assign("label_home_site", $label_home_site);

	$meta_keywords = $settings["SearchMetaKeywords"];
	$meta_title = $settings["SearchTitle"];
	$meta_description = $settings["SearchMetaDescription"];

	// check page exists and page protection
	$isPublicPage = in_array($p, $pages_public);
	$isProtectedPage = in_array($p, $pages_protected);

	if ($isPublicPage || $isProtectedPage)
	{
		$body = "content/public/public_".$p.".php";

		$userAuthOk = $user->auth_ok;
		$userExpress = $user->express;

		if ($isPublicPage || ($userAuthOk && !$userExpress && $isProtectedPage) || ($userAuthOk && $userExpress && in_array($p, $pages_protected_express)))
		{
			$body = "content/public/public_".$p.".php";
		}
		else
		{
			$_SESSION['after_login_url'] = currentPageURL();
			
			$_mysession = $_SESSION;
			foreach ($_mysession as $_mysessionvar=>$_mysessionkey) $_SESSION[$_mysessionvar] = $_mysessionkey;
			_session_write_close();

			$db->done();
			
			header('Location: '.$url_https.'p=login'."\n");
			
			die();
		}
	}
	else
	{
		$p = "404";
		$body = "content/public/public_404.php";
	}

	if ($user->auth_ok && in_array($p, $pages_public_only))
	{
		header("Location: ".$url_dynamic."p=cart");
	}

	view()->assign("site_map_url", getSiteMapUrl());

	//TODO: Is this the best place for this code?
	$paymentProfilesEnabled = false;
	$db->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE active='Yes'");
	
	while ($db->moveNext())
	{
		if ($db->col['class'] != '')
		{
			if (file_exists('content/engine/payment/payment_'.$db->col['id'].'.php'))
			{
				require_once('content/engine/payment/payment_'.$db->col['id'].'.php');

				$gw_temp = new $db->col['class'];

				if ($gw_temp->supportsPaymentProfiles())
				{
					$paymentProfilesEnabled = true;
					break;
				}
			}
		}
	}
	view()->assign("enable_payment_profiles", $paymentProfilesEnabled);
	
	/**
	 * Log statistics
	 */
	if (($p != "catalog") && ($p != "product"))
	{
		LogStatistics($db, $user->id, "other", 0, "");
	}

	view()->assign("canonical_url", getCanonicalUrl($p));

	// load executable module (by $p variable)
	require($body);
	
	// assign $meta_xxx vars
	view()->assign("meta_title", $meta_title);
	view()->assign("meta_keywords", $meta_keywords);
	view()->assign("meta_description", $meta_description);
	
	// meta pre-assign
	view()->appendMetatag('Content-Type', 'text/html; charset='.$msg['common']['charset'], true, 'http-equiv');
	view()->appendMetatag('keywords', $meta_keywords);
	view()->appendMetatag('description', $meta_description);
	view()->setTitle($meta_title);
	
	// forces IE7 compatibility mode for IE8 browsers
	if (!MOBILE)
	{
		view()->appendMetatag('X-UA-Compatible', 'IE=EmulateIE7', true, 'http-equiv');
	}
	
	// sets the metatags for use with designmode
	view()->appendMetatag('ddm-page', htmlentities($p));
	view()->appendMetatag('ddm-lang', $current_language["code"]);
	
	$doRender = ((array_key_exists('norender', $_GET) && $_GET['norender'] == 'true') === false);
	
	require_once("content/engine/engine_design_mode.php");

	if (in_array($p, array("completed", "invoice", "payment_validation", "cart", "billing", "one_page_checkout")))
	{
		view()->force_compile = true;
	}

	/**
	 * Render output
	 */
	if ($doRender)
	{
		$wrapperToRender = "default.html";
		if ($settings["StoreClosed"] == "Yes" && !$isDesignMode)
		{
			view()->assign("ClosedMessage", $settings["StoreClosedMessage"]);
			$wrapperToRender = "store_closed.html";
		}
		else
		{		
			$is_print_version = isset($_REQUEST["is_print_version"])?$_REQUEST["is_print_version"]:false;
			view()->assign("is_print_version", $is_print_version);
		
			if ($is_print_version == "true")
			{
				view()->assign("site_title", $settings["GlobalSiteName"]);
				$wrapperToRender = "print.html";
			}
		}
		
		view()->render("templates/wrappers/{$wrapperToRender}");
	}
	
	/**
	 * Done
	 */
	$db->done();
	
	if ($p != "email2friend" && $p != "404")
	{
		$_SESSION["HTTP_REFERER"] = (isSslMode() ? "https://" : "http://").$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
	}

	/**
	 * Save data from HTTPS to HTTP session
	 */
	
	_session_write_close();
	
/**
 * The End. To be continued. See you next time.
 */